************************************************************************
      SUBROUTINE SEARCH(XX,FX,F,XMIN,XMAX,DX,ICUT,FLMT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C*I   XX   = Argument of function                                     **
C*I   FX   = Function value of XX, F(XX)                              **
C*F   F(X) = Function to calculate F(X)                               **
C*I   XMIN,XMAX,DX = Search limits and increment for root             **
C*I   ICUT = Max number of iterations                                 **
C*I   FLMT = Limited function values of F(XX)                         **
C**   ============================================================    **
      FN= 1.0E30
      FP=-1.0E30
      XX=XMIN
      IR=0
C**   +----------------------------------------------------------+    **
C**   | Compute function value                                   |    **
C**   +----------------------------------------------------------+    **
   30 FX=F(XX)
      IR=IR+1
      IF(FX.EQ.0.0) RETURN
C*    WRITE(*,'('' IR,FX,XX'',I5,1P2E15.6)') IR,FX,XX
C**   +----------------------------------------------------------+    **
C**   | Return on discontinuous point                            |    **
C**   +----------------------------------------------------------+    **
      IF(DABS(FX).GE.FLMT) RETURN
C**   +----------------------------------------------------------+    **
C**   | Push down old values, and put the new one on top         |    **
C**   +----------------------------------------------------------+    **
      X1=X2
      X2=XX
      F1=F2
      F2=FX
C**   +----------------------------------------------------------+    **
C**   | Set limits for next new root                             |    **
C**   +----------------------------------------------------------+    **
      IF(FX.LT.0.0) THEN
        XN=XX
        IF(FX.LT.FN) IE=IE-1
        FN=FX
      ELSE
        XP=XX
        IF(FX.GT.FP) IE=IE-1
        FP=FX
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Search interval for changing sign                        |    **
C**   +----------------------------------------------------------+    **
      IF(FN*FP.GT.0.0) THEN
        IF(XX.GT.XMAX) THEN
          WRITE(*,'('' NO ROOT BETWEEN'',1P2E15.6)') XMIN,XMAX
          RETURN
        ENDIF
        XX=XX+DX
        IE=IR+ICUT
        GO TO 30
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Get new root by secant method                            |    **
C**   +----------------------------------------------------------+    **
      IF(F1.NE.F2) THEN
        XX=X1-F1*(X1-X2)/(F1-F2)
C**     +--------------------------------------------------------+    **
C**     | Check if the new root goes outside the limits          |    **
C**     +--------------------------------------------------------+    **
        IF((XX-XP)*(XX-XN).GT.0.0) THEN
C**       +------------------------------------------------------+    **
C**       | YES: Get new root by false position method           |    **
C**       +------------------------------------------------------+    **
          XX=XP-FP*(XP-XN)/(FP-FN)
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Check the iteration numbers                              |    **
C**   +----------------------------------------------------------+    **
      IF(IR.LE.IE) GO TO 30
      WRITE(*,*) ' THE NUMBER OF ITERATIONS EXCEEDS',ICUT
      RETURN
      END
************************************************************************
