************************************************************************
      FUNCTION XZERO(X,F,DX,ISTEP,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(11),B(11)
      EQUIVALENCE (A(1),X0),(A(2),X1),(A(3),X2)
     *           ,(A(4),F0),(A(5),F1),(A(6),F2)
     * ,(A(7),XP),(A(8),XN),(A(9),FP),(A(10),FN),(A(11),ST)
C**   ============================================================    **
C*I   X     = Argument of function                                    **
C*I   F     = Function value of X, F(X)                               **
C*O   XZERO = A better guess for F(XZERO)=0                           **
C*I   DX    = In case no better formula to guess XZERO,               **
C**           This function will return :                             **
C**           Either XZERO = X + DX, if DX .NE. 0                     **
C**           Or     XZERO = X + F,  if DX .EQ. 0                     **
C**                  this will simulate XZERO = G(X)                  **
C**                  as in the direct substitution method             **
C**                  if set F(X) = G(X) - X                           **
C*I   ISTEP = 1 : For the first call of a new function                **
C**         > 1 : Otherwise                                           **
C**                                                                   **
C**   For ISTEP = 1 :                                                 **
C**   XZERO = X + DX  or  XZERO = X + F                               **
C**                                                                   **
C**   For ISTEP > 1 :                                                 **
C**   XZERO by secant method or                                       **
C**         by false position method (if applicable)                  **
C**         if XZERO by secant method goes outside (XP,XN)            **
C**   ============================================================    **
      IF (ISTEP.EQ.1) THEN
        DO 10 I=1,11
   10   A(I)=0.0
        FP=-1.0
        FN=+1.0
      ELSE
        DO 20 I=1,11
   20   A(I)=B(I)
      ENDIF
      ST=ST+1
C**   +----------------------------------------------------------+    **
C**   | Save the most newly 3 points                             |    **
C**   +----------------------------------------------------------+    **
      X0=X1
      X1=X2
      X2=X
      F0=F1
      F1=F2
      F2=F
C**   +----------------------------------------------------------+    **
C**   | Find new points (XP,XN) that bracket the root            |    **
C**   +----------------------------------------------------------+    **
      IF (F.LT.0.0) THEN
        XN=X
        FN=F
      ELSE
        XP=X
        FP=F
      ENDIF
      IGET=0
      XX=0.0
C**   +----------------------------------------------------------+    **
C**   | For slow convergence due to multiple roots               |    **
C**   | Then use secant method on U(X)=F(X)/F'(X)=0              |    **
C**   | After 13 iterations (13 in statement below, >=3)         |    **
C**   +----------------------------------------------------------+    **
      IF (ST.GE.13) THEN
        IF ((F2-F1).NE.0.0.AND.(F1-F0).NE.0.0) THEN
          U1=F1*(X1-X0)/(F1-F0)
          U2=F2*(X2-X1)/(F2-F1)
          IF ((U2-U1).NE.0.0) THEN
            XX=X2-U2*(X2-X1)/(U2-U1)
            IGET=4
          ENDIF
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | If we have at least 2 functions and no XX get            |    **
C**   | Then use secant method on F(X)=0                         |    **
C**   +----------------------------------------------------------+    **
      IF (ST.GE.2.AND.IGET.EQ.0) THEN
        IF ((F2-F1).NE.0.0) THEN
          XX=X2-F2*(X2-X1)/(F2-F1)
          IGET=3
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | If root is bracketed by (XP,XN)                          |    **
C**   | And if no XX get or XX goes outside (XP,XN)              |    **
C**   | Then use false position method by (XP,XN)                |    **
C**   +----------------------------------------------------------+    **
      IF (FP.GE.0.0.AND.FN.LT.0.0) THEN
        IF (IGET.EQ.0.OR.(XX-XP)*(XX-XN).GT.0.0) THEN
          XX=XP-FP*(XP-XN)/(FP-FN)
          IGET=2
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | If secant method & false position method cannot be used: |    **
C**   | If DX.NE.0 : Arbitrary set XX=X+DX                       |    **
C**   |    Else    : Set XX=X+F(X)                               |    **
C**   |              To simulate direct substitution method      |    **
C**   |              If set F(X)=G(X)-X in calling program       |    **
C**   +----------------------------------------------------------+    **
      IF (IGET.EQ.0) THEN
        IF (DX.NE.0.0) THEN
          XX=X+DX
        ELSE
          XX=X+F
        ENDIF
        IGET=1
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | For debug                                                |    **
C**   +----------------------------------------------------------+    **
      X0=IGET
      F0=XX
C**   ------------------------------------------------------------    **
      DO 30 I=1,11
   30 B(I)=A(I)
      XZERO=XX
      RETURN
      END
************************************************************************
