************************************************************************
      PROGRAM ROMQMN
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON/ROMQ00/ LTMAX,LSMAX,LYMIN,IDEBG,KFUNC,KSUBC,KLVLM,KDIMM
      COMMON R,S,R1,S1,RS,SR
      EXTERNAL F0,F1,G0
C**   ============================================================    **
C**   Find the integral of exp(-Y**2) from YA to YB.                  **
C**   The change of variable Y=sqrt(-ln(Z)) transforms                **
C**     the integral into (1/2)sqrt(-ln(Z))                           **
C**     from ZB=exp(-YB**2) to ZA=exp(-YA**2).                        **
C**   Then divide the interval into [ZB,1/e] and [1/e,ZA].            **
C**   For Z:[ZB,1/e] change variable to X by Z^{1/3-1}dZ=dX           **
C**   For Z:[1/e,ZA] change variable to V by Z=1-V                    **
C**   and V:[1-ZA,1-1/e] change var. to X by V^{1/2-1}dV=dX           **
C**   ============================================================    **
   10 READ (*,'(3F10.0,4I5)') YA,YB,EPS,LTMAX,LSMAX,LYMIN,IDEBG
      IF(YA.EQ.0.0)  YA= 1.0D-6
      IF(YB.EQ.0.0)  YB= 1.0D+6
      IF(EPS.EQ.0.0) EPS=1.0D-7
      IF(LTMAX.EQ.0) LTMAX=41
      IF(LSMAX.EQ.0) LSMAX=7
      IF(LYMIN.EQ.0) LYMIN=1
C**   +----------------------------------------------------------+    **
C**   | AREA = Series expansions + Asymptotic expansion          |    **
C**   +----------------------------------------------------------+    **
      YA2=YA**2
      YB2=YB**2
      AREA=DSQRT(3.14159265358979D0)/2-YA
     *    *(1-YA2/(3*1)+YA2**2/(5*2*1)-YA2**3/(7*3*2*1))
      IF(YB.LT.7.0D0) AREA=AREA-EXP(-YB2)/(2*YB)
     *    *(1-1/(2*YB2)+1*3/(2*YB2)**2-1*3*5/(2*YB2)**3)
      WRITE(*,'(1X,1P3E11.4,E22.14)') YA,YB,DEXP(-1.0D0),AREA
C**   +----------------------------------------------------------+    **
C**   | Integration by the change of variable:                   |    **
C**   | Y = sqrt(-ln(Z)),  Z = exp(-Y**2)                        |    **
C**   | Z:[ZA,ZB] divided into Z:[ZA,1/e] and Z:[1/e,ZB]         |    **
C**   +----------------------------------------------------------+    **
      ZA=DEXP(-YA**2)
      ZB=DEXP(-YB**2)
      ZC=DEXP(-1.0D0)
      S=1.0D0
C**   +----------------------------------------------------------+    **
C**   | For Z:[ZB,1/e] change variable by Z^{R-1}dZ=dX           |    **
C**   |                F0(X) = G0(Z)*Z**(1-R)                    |    **
C**   |                G0(Z) = sqrt(-ln(Z))                      |    **
C**   +----------------------------------------------------------+    **
      R=1.0D0/3.0D0
      CALL ROMR (ZB,ZC,F0,ARA0,ARB0,EPS)
      IF(IDEBG.LE.0) WRITE(*,'(2X,4I3,4I5,1PE22.14,E12.4)')
     *   LTMAX,LSMAX,LYMIN,IDEBG,KFUNC,KSUBC,KLVLM,KDIMM
     *   ,ARA0,ARB0-ARA0
C**   +----------------------------------------------------------+    **
C**   | For Z:[1/e,ZA] change variable by Z=1-V to V             |    **
C**   | V:[1-ZA,1-1/e] change variable by V^{R-1}dV=dX           |    **
C**   |                F1(X) = G1(V)*V**(1-R)                    |    **
C**   |                G1(V) = G0(1-V) = sqrt(-ln(Z))            |    **
C**   +----------------------------------------------------------+    **
      R=1.0D0/2.0D0
      CALL ROMR (1.0D0-ZA,1.0D0-ZC,F1,ARA1,ARB1,EPS)
      IF(IDEBG.LE.0) WRITE(*,'(2X,4I3,4I5,1PE22.14,E12.4)')
     *   LTMAX,LSMAX,LYMIN,IDEBG,KFUNC,KSUBC,KLVLM,KDIMM
     *   ,ARA1,ARB1-ARA1
C**   ------------------------------------------------------------    **
      WRITE(*,'(1X,1P3E11.4,E22.14,E12.4)')
     *   ZB,1-ZA,EPS,ARA0+ARA1,AREA-ARA0-ARA1
C**   +----------------------------------------------------------+    **
C**   | ARAA = value by Romberg integration tree                 |    **
C**   +----------------------------------------------------------+    **
      CALL ROMQ (ZB,ZA,G0,ARAA,ARBB,EPS)
      IF(IDEBG.LE.0) WRITE(*,'(2X,4I3,4I5,1PE22.14,E12.4)')
     *   LTMAX,LSMAX,LYMIN,IDEBG,KFUNC,KSUBC,KLVLM,KDIMM
     *   ,ARAA,ARBB-ARAA
C**   ------------------------------------------------------------    **
      WRITE(*,'(1X,1P3E11.4,E22.14,2E12.4)')
     *   ZB,ZA,EPS,ARAA,ARBB-ARAA,AREA-ARAA
      GO TO 10
      END
************************************************************************
      SUBROUTINE ROMR(ZA,ZB,F,ARAA,ARBB,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON R,S,R1,S1,RS,SR
      EXTERNAL F
C**   ============================================================    **
C**      ZB          ZB              XB              XB               **
C**     /           /      R-1      /      S-1      /                 **
C**    | G(Z) dZ = | g(Z) Z   dZ = | f(X) X   dX = | F(X) dX          **
C**   /           /               /               /                   **
C**   ZA          ZA              XA              XA                  **
C**                                      1-R                          **
C**                  f(X) = g(Z) = G(Z) Z                             **
C**                                                                   **
C**                S-1        1-R S-1            1/R-1 S/R-1          **
C**   F(X) = f(X) X   = G(Z) Z   X   = G(Z) (R/S)     X               **
C**                                                                   **
C**                      R-1      S-1                                 **
C**                     Z   dZ = X   dX                               **
C**    -----------------------------------------------------------    **
C**    IF R .NE. 0 (.AND. S .NE. 0 ) :                                **
C**                                                                   **
C**                1/R  S/R                     1/S  R/S              **
C**       Z = (R/S)    X       and     X = (S/R)    Z                 **
C**    -----------------------------------------------------------    **
C**    IF R .EQ. 0 (.AND. S .NE. 0 ) :                                **
C**                                                                   **
C**                      S                           1/S              **
C**       Z = EXP((1/S) X )    and     X = (S LOG(Z))                 **
C**   ============================================================    **
C**   This program requires the conditions (0) and ((1) or (2))       **
C**   (0) ZA > 0 and ZB > 0                                           **
C**   (1) 1 > R > 0 and S > R (S = 1 is the best)                     **
C**   (2)     R = 0 and S = 1                                         **
C**   ============================================================    **
      IF(R.NE.0.0D0) THEN
         R1=1.D0/R
         S1=1.D0/S
         RS=R*S1
         SR=S*R1
         R1=RS**R1
         S1=SR**S1
         XA=S1*ZA**RS
         XB=S1*ZB**RS
      ELSE
         XA=DLOG(ZA)
         XB=DLOG(ZB)
      ENDIF
      CALL ROMQ (XA,XB,F,ARAA,ARBB,EPS)
      RETURN
      END
************************************************************************
      FUNCTION F0(X)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON R,S,R1,S1,RS,SR
C**   ============================================================    **
      IF(R.NE.0.0D0) THEN
         Z=R1*X**SR
         IF(S.EQ.1.0D0) THEN
            F0=G0(Z)*Z**(1-R)
         ELSE
            F0=G0(Z)*R1*SR*X**(SR-1)
         ENDIF
      ELSE
         Z=DEXP(X)
         F0=G0(Z)*Z
      ENDIF
      RETURN
      END
************************************************************************
      FUNCTION F1(X)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON R,S,R1,S1,RS,SR
C**   ============================================================    **
      IF(R.NE.0.0D0) THEN
         Z=R1*X**SR
         IF(S.EQ.1.0D0) THEN
            F1=G1(Z)*Z**(1-R)
         ELSE
            F1=G1(Z)*R1*SR*X**(SR-1)
         ENDIF
      ELSE
         Z=DEXP(X)
         F1=G1(Z)*Z
      ENDIF
      RETURN
      END
************************************************************************
      FUNCTION G0(Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
      IF(Z.LE.0.0D0) THEN
         G0=0.0D0
      ELSE IF(Z.GE.1.0D0) THEN
         G0=1.0D7
      ELSE
         G0=0.5D0/DSQRT(-DLOG(Z))
      ENDIF
      RETURN
      END
************************************************************************
      FUNCTION G1(Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C**   G1=G0(1.0D0-Z)
      IF(Z.LE.0.0D0) THEN
         G1=1.0D30
      ELSE IF(Z.GE.1.0D0) THEN
         G1=0.0D0
      ELSE IF(Z.LE.1.D-3) THEN
         G1=0.5D0/DSQRT(Z+Z**2/2+Z**3/3+Z**4/4+Z**5/5)
      ELSE
         G1=0.5D0/DSQRT(-DLOG(1.0D0-Z))
      ENDIF
      RETURN
      END
************************************************************************
