************************************************************************
      SUBROUTINE CBDECP(A,M,N,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM) or A((N-1)*MM+N)                     **
C**   ------------------------------------------------------------    **
C**   Find [D] & [U] for given [A],        | a11 a12 a13  0   0  |    **
C**   Such that  [A] = [U]'[D] [U],        | a12 a22 a23 a24  0  |    **
C**   where [U]'= transpose of [U].  [A] = | a13 a23 a33 a34 a35 |    **
C**                                        |  0  a24 a34 a44 a45 |    **
C**   For example: N=5, M=2, MM=2:         |  0   0  a35 a45 a55 |    **
C**                                                                   **
C**         | d11  0   0   0   0  |        |  1  u12 u13  0   0  |    **
C**         |  0  d22  0   0   0  |        |  0   1  u23 u24  0  |    **
C**   [D] = |  0   0  d33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         |  0   0   0  d44  0  |        |  0   0   0   1  u45 |    **
C**         |  0   0   0   0  d55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C**   INPUT  data in A(2,7) =  |\a11 a12 a13\a33 a34 a35\a55 |        **
C**                            |  ? \a22 a23 a24\a44 a45  ? \|        **
C**   or in 1-D array:                                                **
C**   A(13) = (a11 ? a12 a22 a13 a23 a33 a24 a34 a44 a35 a45 a55);    **
C**                                                                   **
C**   OUTPUT data in A(2,7) =  |\d11 u12 u13\d33 u34 u35\d55 |        **
C**                            |  ? \d22 u23 u24\d44 u45  ? \|        **
C**   or in 1-D array:                                                **
C**   A(13) = (d11 ? u12 d22 u13 u23 d33 u24 u34 d44 u35 u45 d55).    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | D(I,I)U(I,J)=A(I,J)-\sum_{K=JM}^{I-1} D(K,K)U(K,I)U(K,J) |    **
C**   | A(K+IA)=U(K,I),     A(K+JA)=D(K,K)*U(K,J)                |    **
C**   | A(I+JA)=AIJ=A(I,J) --> D(I,I)*U(I,J)                     |    **
C**   +----------------------------------------------------------+    **
      JA=0
      DO 40 J=1,N
      JM=MAX0(J-M,1)
        IA=JM*MM
        DO 20 I=JM+1,J-1
          AIJ=A(I+JA)
          DO 10 K=JM,I-1
          AIJ=AIJ-A(K+IA)*A(K+JA)
   10     CONTINUE
          A(I+JA)=AIJ
   20   IA=IA+MM
C**   +----------------------------------------------------------+    **
C**   | D(J,J) =A(J,J) - \sum_{K=JM}^{J-1} D(K,K)*U(K,J)*U(K,J)  |    **
C**   | A(K+JA)=D(K,K)*U(K,J) --> U(K,J),  AKJ=U(K,J)            |    **
C**   | A(J+JA)=AJJ=A(J,J) --> D(J,J),     A(K+KA)=D(K,K)        |    **
C**   +----------------------------------------------------------+    **
        AJJ=A(J+JA)
        KA=(JM-1)*MM
        DO 30 K=JM,J-1
          AKJ=A(K+JA)/A(K+KA)
          AJJ=AJJ-AKJ*A(K+JA)
          A(K+JA)=AKJ
   30   KA=KA+MM
        A(J+JA)=AJJ
        IF(A(J+JA).EQ.0.0) GO TO 50
   40 JA=JA+MM
      RETURN
C**   +----------------------------------------------------------+    **
C**   | Return with error : D(J,J)=0                             |    **
C**   +----------------------------------------------------------+    **
   50 WRITE(*,'('' A('',I3,'','',I3,'')=0''/)') J,J
      RETURN
      END
************************************************************************
      SUBROUTINE CBSOLX(A,B,M,N,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(N)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM),B(N) or A((N-1)*MM+N),B(N)           **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={B} for given {B} & [A]=[U]'[D][U].        **
C**   [D] & [U] are given & put in A(*) (Get by subroutine CBDECP)    **
C**   [U]'[D][U]{X}={B} ===> [U]'{Z}={B},  [D]{Y}={Z},  [U]{X}={Y}    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Z} from {B} by forward substitution of [U]'{Z}={B} |    **
C**   | Z(I) = BI = B(I) - \sum_{K=IM}^{I-1} U'(I,K)*Z(K)        |    **
C**   | A(K+IA)=U(K,I)=U'(I,K), B(K)=Z(K), B(I)=B(I) --> Z(I)    |    **
C**   +----------------------------------------------------------+    **
      IA=0
      DO 70 I=1,N
      IM=MAX0(I-M,1)
        BI=B(I)
        DO 60 K=IM,I-1
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI
   70 IA=IA+MM
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {Z} from the equation [D]{Y}={Z}           |    **
C**   | Y(I) = Z(I) / D(I,I)                                     |    **
C**   | B(I) = Z(I) --> Y(I),     A(I+IA)=D(I,I)                 |    **
C**   +----------------------------------------------------------+    **
      IA=0
      DO 75 I=1,N
        B(I)=B(I)/A(I+IA)
   75 IA=IA+MM
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=N:2  Compute  B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N,2,-1
      IA=IA-MM
      IM=MAX0(I-M,1)
        BI=B(I)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
   90 CONTINUE
      RETURN
      END
************************************************************************
