************************************************************************
      SUBROUTINE CBDECP(A,M,N,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM) or A((N-1)*MM+N)                     **
C**   ------------------------------------------------------------    **
C**   Find [U] for given [A],              | a11 a12 a13  0   0  |    **
C**   Such that  [A] = [U]'[U],            | a12 a22 a23 a24  0  |    **
C**   where [U]'= transpose of [U].  [A] = | a13 a23 a33 a34 a35 |    **
C**                                        |  0  a24 a34 a44 a45 |    **
C**   For example: N=5, M=2, MM=2:         |  0   0  a35 a45 a55 |    **
C**                                                                   **
C**         | u11  0   0   0   0  |        | u11 u12 u13  0   0  |    **
C**         | u12 u22  0   0   0  |        |  0  u22 u23 u24  0  |    **
C**   [U]'= | u13 u23 u33  0   0  |, [U] = |  0   0  u33 u34 u35 |    **
C**         |  0  u24 u34 u44  0  |        |  0   0   0  u44 u45 |    **
C**         |  0   0  u35 u45 u55 |        |  0   0   0   0  u55 |    **
C**                                                                   **
C**   INPUT  data in A(2,7) =  |\a11 a12 a13\a33 a34 a35\a55 |        **
C**                            |  ? \a22 a23 a24\a44 a45  ? \|        **
C**   or in 1-D array:                                                **
C**   A(13) = (a11 ? a12 a22 a13 a23 a33 a24 a34 a44 a35 a45 a55);    **
C**                                                                   **
C**   OUTPUT data in A(2,7) =  |\u11 u12 u13\u33 u34 u35\u55 |        **
C**                            |  ? \u22 u23 u24\u44 u45  ? \|        **
C**   or in 1-D array:                                                **
C**   A(13) = (u11 ? u12 u22 u13 u23 u33 u24 u34 u44 u35 u45 u55).    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | AIJ = A(I,J) - \sum_{K=JM}^{I-1} U(K,I)*U(K,J)           |    **
C**   | I<J : U(I,J)=AIJ/U(I,I);        I=J : U(I,J)=sqrt(AIJ)   |    **
C**   | A(K+IA)=U(K,I), A(K+JA)=U(K,J), A(I+JA)=A(I,J) --> U(I,J)|    **
C**   +----------------------------------------------------------+    **
      JA=0
      DO 40 J=1,N
      JM=MAX0(J-M,1)
        IA=(JM-1)*MM
        DO 30 I=JM,J
          AIJ=A(I+JA)
          DO 10 K=JM,I-1
          AIJ=AIJ-A(K+IA)*A(K+JA)
   10     CONTINUE
          IF(I.NE.J) THEN
            A(I+JA)=AIJ/A(I+IA)
          ELSE IF(AIJ.GT.0.0) THEN
            A(I+JA)=SQRT(AIJ)
          ELSE
            WRITE(*,'('' A('',I3,'','',I3,'') <= 0''/)') J,J
            RETURN
          ENDIF
   30   IA=IA+MM
   40 JA=JA+MM
      RETURN
      END
************************************************************************
      SUBROUTINE CBSOLX(A,B,M,N,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(N)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM),B(N) or A((N-1)*MM+N),B(N)           **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={B} for given {B} & [A]=[U]'[U].           **
C**   [U] is given & put in A(*) (Get by subroutine CBDECP)           **
C**   [U]'[U]{X}={B} ===> [U]'{Y}={B},  [U]{X}={Y}                    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {B} by forward substitution of [U]'{Y}={B} |    **
C**   | U(I,I)*Y(I) = BI = B(I) - \sum_{K=IM}^{I-1} U'(I,K)*Y(K) |    **
C**   | A(K+IA)=U(K,I)=U'(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)    |    **
C**   +----------------------------------------------------------+    **
      IA=0
      DO 70 I=1,N
      IM=MAX0(I-M,1)
        BI=B(I)
        DO 60 K=IM,I-1
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI/A(I+IA)
   70 IA=IA+MM
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=N:1  Compute  B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I), B(K)=Y(K) --> U(K,K)*X(K) ==> BI=X(I)    |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N,1,-1
      IA=IA-MM
      IM=MAX0(I-M,1)
        BI=B(I)/A(I+IA)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
      RETURN
      END
************************************************************************
