************************************************************************
      SUBROUTINE CUDECP(A,N,MA,MB,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MM,1)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM) or A((N-1)*MM+N)                     **
C**   ------------------------------------------------------------    **
C**   Find [L] & [U] for given [A],        | a11 a12 a13  0   0  |    **
C**   Such that  [A] = [L] [U],            | a21 a22 a23 a24  0  |    **
C**                                  [A] = |  0  a32 a33 a34 a35 |    **
C**   For example:                         |  0   0  a43 a44 a45 |    **
C**   N=5, MA=1, MB=2, MM=3 (>=MA+MB):     |  0   0   0  a54 a55 |    **
C**                                                                   **
C**         | l11  0   0   0   0  |        |  1  u12 u13  0   0  |    **
C**         | l21 l22  0   0   0  |        |  0   1  u23 u24  0  |    **
C**   [L] = |  0  l32 l33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         |  0   0  l43 l44  0  |        |  0   0   0   1  u45 |    **
C**         |  0   0   0  l54 l55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C**   INPUT  data in A(3,6) =  | a11 a12 a13\a43 a44 a45 |            **
C**                            |\a21 a22 a23 a24\a54 a55 |            **
C**                            |  ? \a32 a33 a34 a35\ ?  |            **
C**   or in 1-D array:                                                **
C**   A(17) = (a11 a21 ? a12 a22 a32 a13 a23 a33 a43 a24
C**                                      a34 a44 a54 a35 a45 a55);    **
C**                                                                   **
C**   OUTPUT data in A(3,6) =  | l11 u12 u13\l43 l44 u45 |            **
C**                            |\l21 l22 u23 u24\l54 l55 |            **
C**                            |  ? \l32 l33 u34 u35\ ?  |            **
C**   or in 1-D array:                                                **
C**   A(17) = (l11 l21 ? u12 l22 l32 u13 u23 l33 l43 u24
C**                                      u34 l44 l54 u35 u45 l55).    **
C**   ============================================================    **
      DO 50 J=1,N
      JMB=MAX0(J-MB,1)
      JMA=MAX0(J-MA,1)
C**     +--------------------------------------------------------+    **
C**     | AIJ = A(I,J) - \sum_{K=IMA}^{I-1} L(I,K)*U(K,J)        |    **
C**     | A(I,K)=L(I,K),    A(K,J)=U(K,J)                        |    **
C**     | A(I,J)=A(I,J) --> U(I,J)=AIJ/L(I,I)                    |    **
C**     +--------------------------------------------------------+    **
        DO 20 I=JMB,J-1
        IMA=MAX0(I-MA,JMB)
          AIJ=A(I,J)
          DO 10 K=IMA,I-1
          AIJ=AIJ-A(I,K)*A(K,J)
   10     CONTINUE
          A(I,J)=AIJ/A(I,I)
   20   CONTINUE
C**     +--------------------------------------------------------+    **
C**     | AJI = A(J,I) - \sum_{K=IMB}^{I-1} L(J,K)*U(K,I)        |    **
C**     | A(J,K)=L(J,K),    A(K,I)=U(K,I)                        |    **
C**     | A(J,I)=A(J,I) --> L(J,I)=AJI                           |    **
C**     +--------------------------------------------------------+    **
        DO 40 I=JMA,J
        IMB=MAX0(I-MB,JMA)
          AJI=A(J,I)
          DO 30 K=IMB,I-1
          AJI=AJI-A(J,K)*A(K,I)
   30     CONTINUE
          A(J,I)=AJI
   40   CONTINUE
C**     ----------------------------------------------------------    **
        IF(A(J,J).EQ.0.0) GO TO 55
   50 CONTINUE
      RETURN
C**   +----------------------------------------------------------+    **
C**   | Return with error : L(J,J)=0                             |    **
C**   +----------------------------------------------------------+    **
   55 WRITE(*,'('' A('',I3,'','',I3,'')=0''/)') J,J
      RETURN
      END
************************************************************************
      SUBROUTINE CUSOLX(A,B,N,MA,MB,MM)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MM,1),B(N)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM),B(N) or A((N-1)*MM+N),B(N)           **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={B} for given {B} & [A]=[L][U].            **
C**   [L] & [U] are given & put in A(*) (Get by subroutine CUDECP)    **
C**   [L][U]{X}={B} ===> [L]{Y}={B},  [U]{X}={Y}                      **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {B} by forward substitution of [L]{Y}={B}  |    **
C**   | BI = B(I) - \sum_{K=IMA}^{I-1} L(I,K)*Y(K)               |    **
C**   | A(I,K)=L(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)=BI/L(I,I)   |    **
C**   +----------------------------------------------------------+    **
      B(1)=B(1)/A(1,1)
      DO 70 I=2,N
      IMA=MAX0(I-MA,1)
        BI=B(I)
        DO 60 K=IMA,I-1
        BI=BI-A(I,K)*B(K)
   60   CONTINUE
        B(I)=BI/A(I,I)
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | X(I) = BI = Y(I) - \sum_{K=I+1}^{IMB} U(I,K)*X(K)        |    **
C**   | A(I,K)=U(I,K), B(K)=X(K), B(I)=Y(I) --> X(I)=BI          |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N-1,1,-1
      IMB=MIN0(I+MB,N)
        BI=B(I)
        DO 80 K=I+1,IMB
        BI=BI-A(I,K)*B(K)
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
      RETURN
      END
************************************************************************
