************************************************************************
      SUBROUTINE CVPPDC(A,LI,LJ,N,MA,MB,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MM,1),LI(N),LJ(N)
C**   ============================================================    **
C**   DIMENSION A(MM,N+(N-1)/MM) or A((N-1)*MM+N) : MM >= MA+MB+MA    **
C**   ------------------------------------------------------------    **
C**   Find [L] & [U] for given [A],        | a11 a12  0   0   0  |    **
C**   Such that  [R][A] = [L][U],          | a21 a22 a23  0   0  |    **
C**   [R] is ROW-exchanges Matrix.   [A] = |  0  a32 a33 a34  0  |    **
C**   For example:                         |  0   0  a43 a44 a45 |    **
C**   N=5, MA=1, MB=1, MM=3:               |  0   0   0  a54 a55 |    **
C**                                                                   **
C**         | l11  0   0   0   0  |        |  1  u12 u13  0   0  |    **
C**         | l21 l22  0   0   0  |        |  0   1  u23 u24  0  |    **
C**   [L] = |  0  l32 l33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         |  0   0  l43 l44  0  |        |  0   0   0   1  u45 |    **
C**         |  0   0   0  l54 l55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C**   INPUT  data in A(3,6) =  | a11 a12  0 \a43 a44 a45 |            **
C**                            |\a21 a22 a23  0 \a54 a55 |            **
C**                            |  ? \a32 a33 a34  0 \ ?  |            **
C**   or in 1-D array:                                                **
C**   A(17) = (a11 a21 ? a12 a22 a32  0  a23 a33 a43  0               **
C**                                      a34 a44 a54  0  a45 a55);    **
C**                                                                   **
C**   OUTPUT data in A(3,6) =  | l11 u12 u13\l43 l44 u45 |            **
C**                            |\l21 l22 u23 u24\l54 l55 |            **
C**                            |  ? \l32 l33 u34 u35\ ?  |            **
C**   or in 1-D array:                                                **
C**   A(17) = (l11 l21 ? u12 l22 l32 u13 u23 l33 l43 u24              **
C**                                      u34 l44 l54 u35 u45 l55).    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Initialize LJ(J) : Top_Non_Zero_Row_No. of J_th col.     |    **
C**   +----------------------------------------------------------+    **
      DO 10 J=1,N
   10 LJ(J)=MAX0(J-MB,1)
C**   +----------------------------------------------------------+    **
C**   | MC  : New Upper_Band_Width = max of (KMB-K)              |    **
C**   | KMB : Right_Non_Zero_Col_No. of K_th row (Pivoting_row)  |    **
C**   +----------------------------------------------------------+    **
      MC=MB
      KMB=1
      DO 80 K=1,N
      KMA=MIN0(K+MA,N)
C**     +--------------------------------------------------------+    **
C**     | Find Pivoting element A(I,K) for I=K,KMA               |    **
C**     +--------------------------------------------------------+    **
        AKK=0.0
        DO 20 I=K,KMA
        IF(AKK.LT.ABS(A(I,K))) AKK=ABS(A(I,K))
   20   CONTINUE
        IF(AKK.EQ.0.0) GO TO 90
        AKK=AKK*0.25
        DO 30 I=K,KMA
        IF(AKK.LE.ABS(A(I,K))) GO TO 35
   30   CONTINUE
C**     +--------------------------------------------------------+    **
C**     | KMB : Right_Non_Zero_Col_No of K_th row (Pivoting_row) |    **
C**     | Update MC=max(MC,KMB-K)                                |    **
C**     +--------------------------------------------------------+    **
   35   KMB=MIN0(MAX0(MB+I,KMB),N)
        MC=MAX0(KMB-K,MC)
        IF(MA+MC.GT.MM) STOP
C**     +--------------------------------------------------------+    **
C**     | Set LI(K)=I  &  Exchange K_th_row & I_th_row           |    **
C**     | Update LJ(J)=min(LJ(J),K)                              |    **
C**     +--------------------------------------------------------+    **
        LI(K)=I
        IF(K.NE.I) THEN
          DO 40 J=K,KMB
          IF(LJ(J).GT.K) LJ(J)=K
          AKK=A(I,J)
          A(I,J)=A(K,J)
          A(K,J)=AKK
   40     CONTINUE
        ENDIF
C**     +--------------------------------------------------------+    **
C**     | A(I,J)=A(I,J) - L(I,K)*U(K,J) for I=K+1,KMA & J=K+1,KMB|    **
C**     | A(I,K)=L(I,K),  A(K,J)=L(K,K)*U(K,J) --> U(K,J)=AKJ    |    **
C**     +--------------------------------------------------------+    **
        IF(K.EQ.N) RETURN
        DO 60 J=K+1,KMB
          AKJ=A(K,J)/A(K,K)
          DO 50 I=K+1,KMA
          A(I,J)=A(I,J)-A(I,K)*AKJ
   50     CONTINUE
          A(K,J)=AKJ
   60   CONTINUE
   80 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Return on singular matrix : L(K,K)=0                     |    **
C**   +----------------------------------------------------------+    **
   90 WRITE(*,'(/'' Matrix is singular: A('',I3,'','',I3,'')=0''/)') K,K
      RETURN
      END
************************************************************************
      SUBROUTINE CVPPSB(A,B,LI,LJ,N,MA,MB,MM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MM,1),B(N),LI(N),LJ(N)
C**   ============================================================    **
C**   Find {X} from [A]{X}={B} for given {B} & [R][A]=[L][U].         **
C**   [L] & [U] are given & put in A(NN,N),  [R] is given by LI(N)    **
C**   [L][U]{X}=[R]{B}  ==>  [L]{Y}=[R]{B},  [U]{X}={Y}               **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from [R]{B} by forward subst. of [L]{Y}=[R]{B}  |    **
C**   | For K=1:N  Compute B(I)=B(I)-L(I,K)*Y(K) for I=(K+1):KMA |    **
C**   | A(I,K)=L(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)=BK/L(K,K)   |    **
C**   +----------------------------------------------------------+    **
      DO 70 K=1,N
      KMA=MIN0(K+MA,N)
        I=LI(K)
        BK=B(I)/A(K,K)
        B(I)=B(K)
        DO 60 I=K+1,KMA
        B(I)=B(I)-A(I,K)*BK
   60   CONTINUE
        B(K)=BK
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For K=N:2  Compute B(I)=B(I)-U(I,K)*X(K) for I=KMB:(K-1) |    **
C**   | A(I,K)=U(I,K), B(K)=X(K), B(I)=Y(I) ==> X(K)=BK          |    **
C**   +----------------------------------------------------------+    **
      DO 90 K=N,2,-1
      KMB=LJ(K)
        BK=B(K)
        DO 80 I=KMB,K-1
        B(I)=B(I)-A(I,K)*BK
   80   CONTINUE
   90 CONTINUE
      RETURN
      END
************************************************************************
