************************************************************************
      SUBROUTINE LUFPDC(A,LI,LJ,N,NSD,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),LI(N),LJ(N)
      DATA EPS/1.0D-7/
C**   ============================================================    **
C**   Find [L] & [U] for given [A], Such that  [R][A][C] = [L][U].    **
C**   where [R] is ROW_exchanges Matrix indicated by LI(K)=IK         **
C**   where [C] is COL_exchanges Matrix indicated by LJ(K)=JK         **
C**   LI(K)=IK : ROW_exchange between K_th_row & IK_th_row            **
C**   LJ(K)=JK : COL_exchange between K_th_col & JK_th_col            **
C*O   NSD = Number of singular degrees of the given matrix            **
C**   ============================================================    **
      NSD=0
      DO 80 K=1,N
C**   +----------------------------------------------------------+    **
C**   | Find IK, JK, AKK=|A(IK,JK)|=max(|A(I,J)|, I=K,N & J=K,N) |    **
C**   | A(I,J)=A(I,J)-\sum_{k=1)^{K-1} L(I,k)*U(k,J)             |    **
C**   +----------------------------------------------------------+    **
        AKK=0.0
        DO 20 J=K,N
        DO 10 I=K,N
          IF(AKK.GE.DABS(A(I,J))) GO TO 10
          AKK=DABS(A(I,J))
          IK=I
          JK=J
   10   CONTINUE
   20   CONTINUE
        IF(K.EQ.1) ADD=AKK
        IF(AKK.LE.ADD*EPS) GO TO 90
C**     +--------------------------------------------------------+    **
C**     | Set LI(K)=IK  & Exchange K_th_row & IK_th_row          |    **
C**     | Set LJ(K)=JK  & Exchange K_th_col & JK_th_col          |    **
C**     | To save time No exchange to L(I,1:(K-1)), U(1:(K-1),J) |    **
C**     +--------------------------------------------------------+    **
        LI(K)=IK
        IF(K.NE.IK) THEN
          I=IK
          DO 30 J=K,N
          AKJ=A(I,J)
          A(I,J)=A(K,J)
          A(K,J)=AKJ
   30     CONTINUE
        ENDIF
        LJ(K)=JK
        IF(K.NE.JK) THEN
          J=JK
          DO 40 I=K,N
          AKJ=A(I,J)
          A(I,J)=A(I,K)
          A(I,K)=AKJ
   40     CONTINUE
        ENDIF
C**     +--------------------------------------------------------+    **
C**     | A(I,J)=A(I,J) - L(I,K)*U(K,J)  for I=K+1,N & J=K+1,N   |    **
C**     | A(I,K)=L(I,K),  A(K,J)=L(K,K)*U(K,J) --> U(K,J)=AKJ    |    **
C**     +--------------------------------------------------------+    **
        IF(K.EQ.N) RETURN
        DO 60 J=K+1,N
          AKJ=A(K,J)/A(K,K)
          DO 50 I=K+1,N
          A(I,J)=A(I,J)-A(I,K)*AKJ
   50     CONTINUE
          A(K,J)=AKJ
   60   CONTINUE
   80 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Return on singular matrix : L(K,K)=0                     |    **
C**   +----------------------------------------------------------+    **
   90 NSD=N-K+1
      DO 95 K=N-NSD+1,N
      LI(K)=K
      LJ(K)=K
      A(K,K)=DMAX1(DABS(A(K,K)),ADD*EPS*EPS)
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPSB(A,B,LI,LJ,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(N),LI(N),LJ(N)
C**   ============================================================    **
C**   Find {X} from [A]{X}={B} for given {B} & [R][A][C]=[L][U].      **
C**   [R] is given by LI(N),   [C] is given by LJ(N),                 **
C**   [L] & [U] are given & put in A(NN,N),    [C]'=inverse of [C]    **
C**   [L][U][C]'{X}=[R]{B} ===> [L]{Y}=[R]{B}, [U][C]'{X}={Y}         **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from [R]{B} by forward subst. of [L]{Y}=[R]{B}  |    **
C**   | B(K) <==> B(LI(K));           Y(K)=BK=B(LI(K))/L(K,K)    |    **
C**   | Subst. Y(K) to Eq.I=(K+1):N : B(I)=B(I)-L(I,K)*Y(K)      |    **
C**   | A(I,K)=L(I,K), A(K,K)=L(K,K), B(I)=B(I) --> L(I,I)*Y(I)  |    **
C**   +----------------------------------------------------------+    **
      DO 70 K=1,N
        IK=LI(K)
        BK=B(IK)/A(K,K)
        B(IK)=B(K)
        DO 60 I=K+1,N
        B(I)=B(I)-A(I,K)*BK
   60   CONTINUE
        B(K)=BK
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward subst. of [U][C]'{X}={Y}   |    **
C**   | X(K) = BK = Y(K) - \sum_{J=K+1}^{N} U(K,J)*X(J)          |    **
C**   | A(K,J)=U(K,J), B(J)=X(J);           X(K) <==> X(LJ(K))   |    **
C**   +----------------------------------------------------------+    **
      DO 90 K=N-1,1,-1
        JK=LJ(K)
        BK=B(K)
        B(K)=B(JK)
        DO 80 J=K+1,N
        BK=BK-A(K,J)*B(J)
   80   CONTINUE
        B(JK)=BK
   90 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPBS(A,B,LI,LJ,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(N),LI(N),LJ(N)
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward subst. of [U][C]'{X}={Y}   |    **
C**   | X(K) = BK = Y(K) - \sum_{J=K+1}^{N} U(K,J)*X(J)          |    **
C**   | A(K,J)=U(K,J), B(J)=X(J);           X(K) <==> X(LJ(K))   |    **
C**   +----------------------------------------------------------+    **
      DO 90 K=N-1,1,-1
        JK=LJ(K)
        BK=B(K)
        B(K)=B(JK)
        DO 80 J=K+1,N
        BK=BK-A(K,J)*B(J)
   80   CONTINUE
        B(JK)=BK
   90 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPSG(A,B,LI,LJ,N,NSD,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(NN,NSD),LI(N),LJ(N)
C**   ============================================================    **
C**   Find the solution vectors [B] of the singular matrix [A]        **
C**   ------------------------------------------------------------    **
C*O   B(N,NSD) = Solution vectors of singular matrix A(N,N)           **
C**   ============================================================    **
      DO 50 J=1,NSD
        DO 30 I=1,N
        B(I,J)=0.0
   30   CONTINUE
        B(N-NSD+J,J)=1.0
        CALL LUFPBS(A,B(1,J),LI,LJ,N,NN)
   50 CONTINUE
      RETURN
      END
************************************************************************
