************************************************************************
      SUBROUTINE LUFPDC(A,LL,N,NSD,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),LL(2,N)
      DATA EPS/1.0D-7/
C**   ============================================================    **
C**   Find [L] & [U] for given [A], Such that  [R][A][C] = [L][U].    **
C**   where [C] is COL_exchanges Matrix indicated by LL(1,K)=JK       **
C**   where [R] is ROW_exchanges Matrix indicated by LL(2,K)=IK       **
C**   LL(1,K)=JK : COL_exchange between K_th_col & JK_th_col          **
C**   LL(2,K)=IK : ROW_exchange between K_th_row & IK_th_row          **
C*O   NSD = Number of singular degrees of the given matrix            **
C**   ============================================================    **
      NSD=0
      DO 80 K=1,N
C**   +----------------------------------------------------------+    **
C**   | Find IK, JK, AKK=|A(IK,JK)|=max(|A(I,J)|, I=K,N & J=K,N) |    **
C**   | A(I,J)=A(I,J)-\sum_{k=1)^{K-1} L(I,k)*U(k,J)             |    **
C**   +----------------------------------------------------------+    **
        AKK=0.0
        DO 20 J=K,N
        DO 10 I=K,N
          IF(AKK.GE.DABS(A(I,J))) GO TO 10
          AKK=DABS(A(I,J))
          IK=I
          JK=J
   10   CONTINUE
   20   CONTINUE
        IF(K.EQ.1) ADD=AKK
        IF(AKK.LE.ADD*EPS) GO TO 90
C**     +--------------------------------------------------------+    **
C**     | Set LL(1,K)=JK  & Exchange K_th_col & JK_th_col        |    **
C**     | Set LL(2,K)=IK  & Exchange K_th_row & IK_th_row        |    **
C**     +--------------------------------------------------------+    **
        LL(1,K)=JK
        IF(K.NE.JK) THEN
          J=JK
          DO 30 I=1,N
          AKJ=A(I,J)
          A(I,J)=A(I,K)
          A(I,K)=AKJ
   30     CONTINUE
        ENDIF
        LL(2,K)=IK
        IF(K.NE.IK) THEN
          I=IK
          DO 40 J=1,N
          AKJ=A(I,J)
          A(I,J)=A(K,J)
          A(K,J)=AKJ
   40     CONTINUE
        ENDIF
C**     +--------------------------------------------------------+    **
C**     | A(I,J)=A(I,J) - L(I,K)*U(K,J)  for I=K+1,N & J=K+1,N   |    **
C**     | A(I,K)=L(I,K),  A(K,J)=L(K,K)*U(K,J) --> U(K,J)=AKJ    |    **
C**     +--------------------------------------------------------+    **
        IF(K.EQ.N) RETURN
        DO 60 J=K+1,N
          AKJ=A(K,J)/A(K,K)
          DO 50 I=K+1,N
          A(I,J)=A(I,J)-A(I,K)*AKJ
   50     CONTINUE
          A(K,J)=AKJ
   60   CONTINUE
   80 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Return on singular matrix : L(K,K)=0                     |    **
C**   +----------------------------------------------------------+    **
   90 NSD=N-K+1
      IF(ADD.EQ.0.0) ADD=1.0
      DO 95 K=N-NSD+1,N
      LL(1,K)=K
      LL(2,K)=K
      A(K,K)=DMAX1(DABS(A(K,K)),ADD*EPS*EPS)
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPSB(A,B,LL,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(N),LL(2,N)
C**   ============================================================    **
C**   Find {X} from [A]{X}={B} for given {B} & [R][A][C]=[L][U].      **
C**   [C] is given by LL(1,N),   [R] is given by LL(2,N),             **
C**   [L] & [U] are given & put in A(NN,N),  [C]'=inverse of [C]      **
C**   [L][U][C]'{X}=[R]{B} => [L]{Z}=[R]{B}, [U]{Y}={Z}, {X}=[C]{Y}   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Z} from [R]{B} by forward subst. of [L]{Z}=[R]{B}  |    **
C**   | B(I) <==> B(LL(2,I))                                     |    **
C**   | BI = B(I) - \sum_{K=1}^{I-1} L(I,K)*Z(K)                 |    **
C**   | A(I,K)=L(I,K), B(K)=Z(K), B(I)=B(I) --> Z(I)=BI/L(I,I)   |    **
C**   +----------------------------------------------------------+    **
      DO 70 I=1,N
        M=LL(2,I)
        BI=B(M)
        B(M)=B(I)
        DO 60 K=1,I-1
        BI=BI-A(I,K)*B(K)
   60   CONTINUE
        B(I)=BI/A(I,I)
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {Z} by backward substitution of [U]{Y}={Z} |    **
C**   | Y(I) = BI = Z(I) - \sum_{K=I+1}^{N} U(I,K)*Y(K)          |    **
C**   | A(I,K)=U(I,K), B(K)=Y(K), B(I)=Z(I) --> Y(I)=BI          |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N-1,1,-1
        BI=B(I)
        DO 80 K=I+1,N
        BI=BI-A(I,K)*B(K)
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by ROW_exchanges : {X}=[C]{Y}          |    **
C**   +----------------------------------------------------------+    **
      DO 95 I=N-1,1,-1
        M=LL(1,I)
        BI=B(I)
        B(I)=B(M)
        B(M)=BI
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPBS(A,B,LL,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(N),LL(2,N)
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {Z} by backward substitution of [U]{Y}={Z} |    **
C**   | Y(I) = BI = Z(I) - \sum_{K=I+1}^{N} U(I,K)*Y(K)          |    **
C**   | A(I,K)=U(I,K), B(K)=Y(K), B(I)=Z(I) --> Y(I)=BI          |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N-1,1,-1
        BI=B(I)
        DO 80 K=I+1,N
        BI=BI-A(I,K)*B(K)
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by ROW_exchanges : {X}=[C]{Y}          |    **
C**   +----------------------------------------------------------+    **
      DO 95 I=N-1,1,-1
        M=LL(1,I)
        BI=B(I)
        B(I)=B(M)
        B(M)=BI
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE LUFPSG(A,B,LL,N,NSD,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(NN,NSD),LL(2,N)
C**   ============================================================    **
C**   Find the solution vectors [B] of the singular matrix [A]        **
C**   ------------------------------------------------------------    **
C*O   B(N,NSD) = Solution vectors of singular matrix A(N,N)           **
C**   ============================================================    **
      DO 50 J=1,NSD
        DO 30 I=1,N
        B(I,J)=0.0
   30   CONTINUE
        B(N-NSD+J,J)=1.0
        CALL LUFPBS(A,B(1,J),LL,N,NN)
   50 CONTINUE
      RETURN
      END
************************************************************************
