************************************************************************
      SUBROUTINE LUPPDC(A,LL,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),LL(N)
C**   ============================================================    **
C**   Find [L] & [U] for given [A], Such that  [R][A] = [L][U].       **
C**   where [R] is ROW_exchanges Matrix indicated by LL(J)=M          **
C**   LL(J)=M : ROW_exchange between J_th_row & M_th_row              **
C**   ============================================================    **
      DO 50 J=1,N
C**     +--------------------------------------------------------+    **
C**     | AIJ = A(I,J) - \sum_{K=1}^{min(I,J)-1} L(I,K)*U(K,J)   |    **
C**     | A(I,K)=L(I,K),    A(K,J)=U(K,J)                        |    **
C**     +--------------------------------------------------------+    **
        AJJ=0.0
        DO 30 I=1,N
          AIJ=A(I,J)
          DO 10 K=1,MIN0(I-1,J-1)
          AIJ=AIJ-A(I,K)*A(K,J)
   10     CONTINUE
C**       +------------------------------------------------------+    **
C**       | I <  J : A(I,J)=A(I,J) --> U(I,J)=AIJ/L(I,I)         |    **
C**       | I >= J : A(I,J)=A(I,J) --> L(I,J)=AIJ                |    **
C**       +------------------------------------------------------+    **
          IF(I.LT.J) THEN
            A(I,J)=AIJ/A(I,I)
          ELSE
            A(I,J)=AIJ
C**         +----------------------------------------------------+    **
C**         | Find M & AJJ=abs(L(M,J))=max(abs(L(I,J)), I=J,N)   |    **
C**         +----------------------------------------------------+    **
            IF(AJJ.LT.ABS(AIJ)) THEN
              AJJ=ABS(AIJ)
              M=I
            ENDIF
          ENDIF
   30   CONTINUE
C**     ----------------------------------------------------------    **
        IF(AJJ.EQ.0.0) GO TO 55
C**     +--------------------------------------------------------+    **
C**     | Set LL(J)=M   &   Exchange J_th_row & M_th_row         |    **
C**     +--------------------------------------------------------+    **
        LL(J)=M
        IF(M.NE.J) THEN
          DO 40 I=1,N
          AIJ=A(M,I)
          A(M,I)=A(J,I)
          A(J,I)=AIJ
   40     CONTINUE
        ENDIF
   50 CONTINUE
      RETURN
C**   +----------------------------------------------------------+    **
C**   | Return on singular matrix : L(J,J)=0                     |    **
C**   +----------------------------------------------------------+    **
   55 WRITE(*,'(/'' Matrix is singular: A('',I3,'','',I3,'')=0''/)') J,J
      RETURN
      END
************************************************************************
      SUBROUTINE LUPPSB(A,B,LL,N,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NN,N),B(N),LL(N)
C**   ============================================================    **
C**   Find {X} from [A]{X}={B} for given {B} & [R][A]=[L][U].         **
C**   [L] & [U] are given & put in A(NN,N),  [R] is given by LL(N)    **
C**   [L][U]{X}=[R]{B}  ==>  [L]{Y}=[R]{B},  [U]{X}={Y}               **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from [R]{B} by forward subst. of [L]{Y}=[R]{B}  |    **
C**   | BI = B(I) - \sum_{K=1}^{I-1} L(I,K)*Y(K)                 |    **
C**   | A(I,K)=L(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)=BI/L(I,I)   |    **
C**   +----------------------------------------------------------+    **
      DO 70 I=1,N
        M=LL(I)
        BI=B(M)
        B(M)=B(I)
        DO 60 K=1,I-1
        BI=BI-A(I,K)*B(K)
   60   CONTINUE
        B(I)=BI/A(I,I)
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | X(I) = BI = Y(I) - \sum_{K=I+1}^{N} U(I,K)*X(K)          |    **
C**   | A(I,K)=U(I,K), B(K)=X(K), B(I)=Y(I) --> X(I)=BI          |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N-1,1,-1
        BI=B(I)
        DO 80 K=I+1,N
        BI=BI-A(I,K)*B(K)
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
      RETURN
      END
************************************************************************
