************************************************************************
      SUBROUTINE VBPTDC(A,LL,LM,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),LL(N),LM(2,N)
C**   ============================================================    **
C**   DIMENSION A(N+LL(N)),LL(N),LM(2,N)                              **
C**   ------------------------------------------------------------    **
C**   Find [D] & [U] for given [A],        | a11  0  a13  0   0  |    **
C**   Such that  [A] = [U]'[D] [U],        |  0  a22 a23  0  a25 |    **
C**   where [U]'= transpose of [U].  [A] = | a13 a23 a33 a34 a35 |    **
C**   For example:                         |  0   0  a34 a44 a45 |    **
C**   N=5  and  LL(5)=(0 0 2 3 6):         |  0  a25 a35 a45 a55 |    **
C**                                                                   **
C**         | d11  0   0   0   0  |        |  1   0  u13  0   0  |    **
C**         |  0  d22  0   0   0  |        |  0   1  u23  0  u25 |    **
C**   [D] = |  0   0  d33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         |  0   0   0  d44  0  |        |  0   0   0   1  u45 |    **
C**         |  0   0   0   0  d55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C**   Given LL(5) = (  0   0           2       3               6),    **
C**     I + LL(J) :  1+0 2+0 1+2 2+2 3+2 3+3 4+3 2+6 3+6 4+6 5+6      **
C**   Location    :    1   2   3   4   5   6   7   8   9  10  11      **
C**   INPUT A(11) = (a11 a22 a13 a23 a33 a34 a44 a25 a35 a45 a55);    **
C**   OUTPU A(11) = (d11 d22 u13 u23 d33 u34 d44 u25 u35 u45 d55).    **
C**   ============================================================    **
      CALL VBPTST(A,LL,LM,N)
C**   +----------------------------------------------------------+    **
C**   | D(I,I)U(I,J)=A(I,J)-\sum_{K=IM}^{I-1} D(K,K)U(K,I)U(K,J) |    **
C**   | A(K+IA)=U(K,I),     A(K+JA)=D(K,K)*U(K,J)                |    **
C**   | A(I+JA)=AIJ=A(I,J) --> D(I,I)*U(I,J)                     |    **
C**   +----------------------------------------------------------+    **
      JB=0
      DO 40 J=1,N
      JA=LL(J)-LL(1)
      JM=LM(2,J)
        DO 20 I=JM+1,J-1
        IA=LL(I)-LL(1)
        IM=MAX0(LM(2,I),JM)
          AIJ=A(I+JA)
          DO 10 K=IM,I-1
          AIJ=AIJ-A(K+IA)*A(K+JA)
   10     CONTINUE
          A(I+JA)=AIJ
   20   CONTINUE
C**   +----------------------------------------------------------+    **
C**   | D(J,J) =A(J,J) - \sum_{K=JM}^{J-1} D(K,K)*U(K,J)*U(K,J)  |    **
C**   | A(K+JA)=D(K,K)*U(K,J) --> U(K,J),  AKJ=U(K,J)            |    **
C**   | A(J+JA)=AJJ=A(J,J) --> D(J,J),     A(K+KA)=D(K,K)        |    **
C**   +----------------------------------------------------------+    **
        AJJ=A(J+JA)
        DO 30 K=JM,J-1
        KA=LL(K)-LL(1)
          AKJ=A(K+JA)/A(K+KA)
          AJJ=AJJ-AKJ*A(K+JA)
          A(K+JA)=AKJ
   30   CONTINUE
        A(J+JA)=AJJ
        IF(A(J+JA).EQ.0.0) GO TO 50
   40 JB=JA
      RETURN
C**   +----------------------------------------------------------+    **
C**   | Return with error : D(J,J)=0                             |    **
C**   +----------------------------------------------------------+    **
   50 WRITE(*,'('' A('',I3,'','',I3,'')=0''/)') J,J
      RETURN
      END
************************************************************************
      SUBROUTINE VBPTSB(A,B,T,LL,LM,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(N),T(3,N),LL(N),LM(2,N)
      DATA EPS/1.0D-3/
C**   ============================================================    **
C**   DIMENSION A(N+LL(N)),B(N),T(3,N),LL(N),LM(2,N)                  **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={B} for given {B} & [A]=[U]'[D][U].        **
C**   [D] & [U] are given & put in A(*) (Get by subroutine VBDECP)    **
C**   [U]'[D][U]{X}={B} ===> [U]'{Z}={B},  [D]{Y}={Z},  [U]{X}={Y}    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Save given B(I) in T(1,I)                                |    **
C**   | Set Initial solution T(2,I)=0, Number of iterations NI=0 |    **
C**   +----------------------------------------------------------+    **
      NI=0
      DO 10 I=1,N
        T(1,I)=B(I)
        T(2,I)=0.0
   10 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | T(1,I) = Given constant vector                           |    **
C**   | T(2,I) = Initial solution vector                         |    **
C**   | B(I) = Modified constant vector = T(1,I) - A(I,J)*T(2,J) |    **
C**   +----------------------------------------------------------+    **
   20 B(1)=T(1,1)
      DO 50 I=2,N
      IA=LL(I)-LL(1)
      IM=I-LL(I)+LL(I-1)
      IN=LM(1,I)
        BI=T(1,I)
        XI=T(2,I)
        DO 40 K=IM,IN
        B(K)=B(K)-A(K+IA)*XI
        BI=BI-A(K+IA)*T(2,K)
   40   CONTINUE
        B(I)=BI
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Z} from {B} by forward substitution of [U]'{Z}={B} |    **
C**   | Z(I) = BI = B(I) - \sum_{K=IM}^{I-1} U'(I,K)*Z(K)        |    **
C**   | A(K+IA)=U(K,I)=U'(I,K), B(K)=Z(K), B(I)=B(I) --> Z(I)    |    **
C**   +----------------------------------------------------------+    **
      DO 70 I=1,N
      IA=LL(I)-LL(1)
      IM=LM(2,I)
        BI=B(I)
        DO 60 K=IM,I-1
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {Z} from the equation [D]{Y}={Z}           |    **
C**   | Y(I) = Z(I) / D(I,I)                                     |    **
C**   | B(I) = Z(I) --> Y(I),     A(I+IA)=D(I,I)                 |    **
C**   +----------------------------------------------------------+    **
      DO 75 I=1,N
      IA=LL(I)-LL(1)
        B(I)=B(I)/A(I+IA)
   75 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=N:2  Compute  B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N,2,-1
      IA=LL(I)-LL(1)
      IM=LM(2,I)
        BI=B(I)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
   90 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Convergence test                                         |    **
C**   +----------------------------------------------------------+    **
      EPSM=0.0
      DO 95 I=1,N
      IF(B(I).NE.0.0) EPSM=DMAX1(DABS((B(I)-T(2,I))/B(I)),EPSM)
   95 CONTINUE
      NI=NI+1
      WRITE(*,'('' NI,EPSM:'',I5,F10.6)') NI,EPSM
      IF(EPSM.LE.EPS.OR.NI.GE.1000) RETURN
C**   +----------------------------------------------------------+    **
C**   | Save old solution in T(2,N) for next iteration           |    **
C**   +----------------------------------------------------------+    **
      DO 99 I=1,N
      T(2,I)=B(I)
   99 CONTINUE
      GO TO 20
      RETURN
      END
************************************************************************
      SUBROUTINE VBPTST(A,LL,LM,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),LL(N),LM(2,N)
C**   ============================================================    **
C**   DIMENSION A(N+LL(N)),LL(N),LM(2,N)                              **
C**   +----------------------------------------------------------+    **
C**   |  0   0   X   X   X   0   0   0   0   X   X   X   X   X   |    **
C**   |  1      JM     LM(1)               LM(2)             J   |    **
C**   +----------------------------------------------------------+    **
C**   |  LM(2)-LM(1)-1 = Max. No. of continuous zeros            |    **
C**   +----------------------------------------------------------+    **
      LM(1,1)=0
      LM(2,1)=1
      K=0
      DO 30 J=2,N
      JA=LL(J)-LL(1)
      JM=J-LL(J)+LL(J-1)
      LM(1,J)=JM-1
      LM(2,J)=JM
      DO 20 I=JM+1,J
      IF(K.EQ.0) THEN
        IF(A(I+JA).EQ.0.0) K=I-1
      ELSE
        IF(A(I+JA).NE.0.0) THEN
          IF(I-K.GE.LM(2,J)-LM(1,J)) THEN
            LM(1,J)=K
            LM(2,J)=I
          ENDIF
          K=0
        ENDIF
      ENDIF
   20 CONTINUE
   30 CONTINUE
      RETURN
      END
************************************************************************
