************************************************************************
      SUBROUTINE VBDECP(A,LL,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),LL(N)
C**   ============================================================    **
C**   DIMENSION A(N+LL(N)),LL(N)                                      **
C**   ------------------------------------------------------------    **
C**   Find [U] for given [A],              | a11  0  a13  0   0  |    **
C**   Such that  [A] = [U]'[U],            |  0  a22 a23  0  a25 |    **
C**   where [U]'= transpose of [U].  [A] = | a13 a23 a33 a34 a35 |    **
C**   For example:                         |  0   0  a34 a44 a45 |    **
C**   N=5  and  LL(5)=(0 0 2 3 6):         |  0  a25 a35 a45 a55 |    **
C**                                                                   **
C**         | u11  0   0   0   0  |        | u11  0  u13  0   0  |    **
C**         |  0  u22  0   0   0  |        |  0  u22 u23  0  u25 |    **
C**   [U]'= | u13 u23 u33  0   0  |, [U] = |  0   0  u33 u34 u35 |    **
C**         |  0   0  u34 u44  0  |        |  0   0   0  u44 u45 |    **
C**         |  0  u25 u35 u45 u55 |        |  0   0   0   0  u55 |    **
C**                                                                   **
C**   Given LL(5) = (  0   0           2       3               6),    **
C**     I + LL(J) :  1+0 2+0 1+2 2+2 3+2 3+3 4+3 2+6 3+6 4+6 5+6      **
C**   Location    :    1   2   3   4   5   6   7   8   9  10  11      **
C**   INPUT A(11) = (a11 a22 a13 a23 a33 a34 a44 a25 a35 a45 a55);    **
C**   OUTPU A(11) = (u11 u22 u13 u23 u33 u34 u44 u25 u35 u45 d55).    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | AIJ = A(I,J) - \sum_{K=IM}^{I-1} U(K,I)*U(K,J)           |    **
C**   | I<J : U(I,J)=AIJ/U(I,I);        I=J : U(I,J)=sqrt(AIJ)   |    **
C**   | A(K+IA)=U(K,I), A(K+JA)=U(K,J), A(I+JA)=A(I,J) --> U(I,J)|    **
C**   +----------------------------------------------------------+    **
      JB=0
      DO 40 J=1,N
      JA=LL(J)-LL(1)
      JM=J-JA+JB
        IB=0
        IF(JM.GT.1) IB=LL(JM-1)-LL(1)
        DO 30 I=JM,J
        IA=LL(I)-LL(1)
        IM=MAX0(I-IA+IB,JM)
          AIJ=A(I+JA)
          DO 10 K=IM,I-1
          AIJ=AIJ-A(K+IA)*A(K+JA)
   10     CONTINUE
          IF(I.NE.J) THEN
            A(I+JA)=AIJ/A(I+IA)
          ELSE IF(AIJ.GT.0.0) THEN
            A(I+JA)=SQRT(AIJ)
          ELSE
            WRITE(*,'('' A('',I3,'','',I3,'') <= 0''/)') J,J
            RETURN
          ENDIF
   30   IB=IA
   40 JB=JA
      RETURN
      END
************************************************************************
      SUBROUTINE VBSOLX(A,B,LL,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(N),LL(N)
C**   ============================================================    **
C**   DIMENSION A(N+LL(N)),B(N),LL(N)                                 **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={B} for given {B} & [A]=[U]'[U].           **
C**   [U] is given & put in A(*) (Get by subroutine VBDECP)           **
C**   [U]'[U]{X}={B} ===> [U]'{Y}={B},  [U]{X}={Y}                    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {B} by forward substitution of [U]'{Y}={B} |    **
C**   | U(I,I)*Y(I) = BI = B(I) - \sum_{K=IM}^{I-1} U'(I,K)*Y(K) |    **
C**   | A(K+IA)=U(K,I)=U'(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)    |    **
C**   +----------------------------------------------------------+    **
      IB=0
      DO 70 I=1,N
      IA=LL(I)-LL(1)
      IM=I-IA+IB
        BI=B(I)
        DO 60 K=IM,I-1
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI/A(I+IA)
   70 IB=IA
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=N:2  Compute  B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I), B(K)=Y(K) --> U(K,K)*X(K) ==> BI=X(I)    |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=N,2,-1
      IA=LL(I)-LL(1)
      IM=I-LL(I)+LL(I-1)
        BI=B(I)/A(I+IA)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
        B(I)=BI
   90 CONTINUE
      B(1)=B(1)/A(1)
      RETURN
      END
************************************************************************
