************************************************************************
      SUBROUTINE USDECP(A,LL,NOT,NNT,NSD,MAXA,MAXN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MAXA),LL(2,MAXN)
      REAL*8 EPS,ADD
      DATA EPS/1.0D-7/
C**   ============================================================    **
C**   LU decomposition [A]=[L][U]                                     **
C**   ------------------------------------------------------------    **
C*I   A(NOT+L*(NOT))  = The given original matrix A(NOT,NOT)          **
C*O   A(NNT+L*(NNT))  = Decomposed matrix L+U-I : L\U(NNT,NNT)        **
C*I   LL(2,1:NOT)     = Original storage index of A(NOT,NOT)          **
C*O   LL(2,NOT+1:NNT) = Expanded storage index of L\U(NNT,NNT)        **
C*I   NOT     = The order of the original matrix A(NOT,NOT)           **
C*O   NNT     = The order of the expanded matrix A(NNT,NNT)           **
C*O   NSD     = Number of singular degrees of the given matrix        **
C*I   MAXA    = The maximum allowed dimension of A(MAXA)              **
C*I   MAXN    = The maximum allowed dimension of LL(2,MAXN)           **
C**   EPS     = Epsilon for accuracy control                          **
C**   ------------------------------------------------------------    **
C**   Find [L] & [U] for given [A], Such that  [A] => [L] [U]         **
C**   For example:                                                    **
C**   Input : NOT=3, LR(3)=(0 1 4),      LL(3)=(0 2 4)                **
C**   Output: NNT=5, LR(5)=(0 1 4 7 13), LL(5)=(0 2 4 10 16)          **
C**                                                                   **
C**         | a11 a12 a13 |           | a11+p1 a12   a13 -p1  0  |    **
C**   [A] = | a21 a22 a23 |           |  a21  a22+p2 a23  0  -p2 |    **
C**         |  0   0  a33 |           |   0     0    a33  0   0  |    **
C**   If it is expanded to ---> [A] = |  -p1    0     0  +p1  0  |    **
C**   Then decomposed matrices are:   |   0    -p2    0   0  +p2 |    **
C**                                                                   **
C**         | l11  0   0   0   0  |        |  1  u12 u13 u14  0  |    **
C**         | l21 l22  0   0   0  |        |  0   1  u23 u24 u25 |    **
C**   [L] = |  0   0  l33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         | l41 l42 l43 l44  0  |        |  0   0   0   1  u45 |    **
C**         |  0  l52 l53 l54 l55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C*I    A(7):(a11 a12 a21 a22 a13 a23 a33)                             **
C*O   A(21):(l11 u12 l21 l22 u13 u23 l33 u14 u24 u34                  **
C*O                  l41 l42 l43 l44 u25 u35 u45 l52 l53 l54 d55)     **
C**   ============================================================    **
C**   LL(1,J)=Storage index of upper triangular part W/O  diagonal    **
C**           A(I,J) is referred as A(I+LL(1,J))                      **
C**   LL(2,I)=Storage index of lower triangular part with diagonal    **
C**           A(I,J) is referred as A(J+LL(2,I))                      **
C**   ------------------------------------------------------------    **
C**   The storage Indexes/Locations of the matrix elements:           **
C**   LL(1,J) 0  1  4  7 13                                           **
C**   LL(2,I)                (I) For examples:                        **
C**      0    1\ 2  5  8      1  2+LL(1,5)=15 for A(2,5) in Loc.15    **
C**      2    3  4\ 6  9 15   2  3+LL(1,5)=16 for A(3,5) in Loc.16    **
C**      4          7\10 16   3  4+LL(1,5)=17 for A(4,5) in Loc.17    **
C**     10   11 12 13 14\17   4  2+LL(2,5)=18 for A(5,2) in Loc.18    **
C**     16      18 19 20 21\  5  3+LL(2,5)=19 for A(5,3) in Loc.19    **
C**                              4+LL(2,5)=20 for A(5,4) in Loc.20    **
C**     (J)   1  2  3  4  5      5+LL(2,5)=21 for A(5,5) in Loc.21    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Initial set NSD=0 : this also allows matrix to expend    |    **
C**   | Set to decompose the original given matrix A(NOT,NOT)    |    **
C**   +----------------------------------------------------------+    **
      NSD=0
      NAT=0
      NBT=NOT
      NNT=NOT
C**   +----------------------------------------------------------+    **
C**   | Decompose the sub-matrix A((NAT+1):NBT,(NAT+1):NBT)      |    **
C**   +----------------------------------------------------------+    **
      JC=0
   10 DO 80 J=NAT+1,NBT
      JA=LL(2,J)-LL(1,1)
      JB=LL(1,J)-LL(1,1)
      JMA=J-JA+JB
      JMB=J-JB+JC
      JC=JA
C**     +--------------------------------------------------------+    **
C**     | AIJ = A(I,J) - \sum_{K=IMA}^{I-1} L(I,K)*U(K,J)        |    **
C**     | A(K+IA)=L(I,K),    A(K+JB)=U(K,J)                      |    **
C**     | A(I+JB)=A(I,J) --> U(I,J)=AIJ/L(I,I)                   |    **
C**     +--------------------------------------------------------+    **
        DO 20 I=JMB,J-1
        IA=LL(2,I)-LL(1,1)
        IMA=MAX0(I-LL(2,I)+LL(1,I),JMB)
          AIJ=A(I+JB)
          DO 15 K=IMA,MIN0(I-1,NOT)
          AIJ=AIJ-A(K+IA)*A(K+JB)
   15     CONTINUE
          A(I+JB)=AIJ
          IF(I.LE.NOT) A(I+JB)=AIJ/A(I+IA)
   20   CONTINUE
C**     +--------------------------------------------------------+    **
C**     | AJI = A(J,I) - \sum_{K=IMB}^{I-1} L(J,K)*U(K,I)        |    **
C**     | A(K+JA)=L(J,K),    A(K+IB)=U(K,I)                      |    **
C**     | A(I+JA)=A(J,I) --> L(J,I)=AJI                          |    **
C**     +--------------------------------------------------------+    **
        DO 30 I=JMA+1,J-1
        IB=LL(1,I)-LL(1,1)
        IMB=MAX0(I-LL(1,I)+LL(2,I-1),JMA)
          AJI=A(I+JA)
          DO 25 K=IMB,MIN0(I-1,NOT)
          AJI=AJI-A(K+JA)*A(K+IB)
   25     CONTINUE
          A(I+JA)=AJI
   30   CONTINUE
C**     +--------------------------------------------------------+    **
C**     | Compute L(J,J)                                         |    **
C**     | AJJ = A(J,J) - \sum_{K=JM*}^{J-1} L(J,K)*U(K,J)        |    **
C**     | A(K+JA)=L(J,K),    A(K+JB)=U(K,J)                      |    **
C**     | A(J+JA)=A(J,J) --> L(J,J)=AJJ                          |    **
C**     +--------------------------------------------------------+    **
        AJJ=A(J+JA)
        ADD=DABS(AJJ)
        DO 40 K=MAX0(JMA,JMB),MIN0(J-1,NOT)
        AJJ=AJJ-A(K+JA)*A(K+JB)
        ADD=DMAX1(ADD,DABS(AJJ))
   40   CONTINUE
        A(J+JA)=AJJ
        IF(J.GT.NOT) GO TO 80
C**     +--------------------------------------------------------+    **
C**     | IF the pivoting element A(J,J) > ADD*EPS ?             |    **
C**     | Yes:80 : OK and Continue                               |    **
C**     | No     : Add pivoting by ADD and Expand one column     |    **
C**     +--------------------------------------------------------+    **
        IF(DABS(A(J+JA)).GT.ADD*EPS) GO TO 80
C**     +--------------------------------------------------------+    **
C**     | IF ADD.EQ.0 ?                                          |    **
C**     | Yes: No reference value available, get from diagonal   |    **
C**     +--------------------------------------------------------+    **
        IF(ADD.EQ.0.0) THEN
          DO 45 K=NAT+1,NBT
          KA=LL(2,K)-LL(1,1)
          ADD=DMAX1(ADD,DABS(A(K+KA)))
   45     CONTINUE
        ENDIF
        IF(ADD.EQ.0.0) ADD=1.0
C**     +--------------------------------------------------------+    **
C**     | If J.EQ.NNT.AND.NSD.EQ.0                               |    **
C**     | Then it is the Only & the Last ZERO_diagonal,          |    **
C**     | So set NSD=1 to prevent from any further expanding     |    **
C**     +--------------------------------------------------------+    **
        IF(J.EQ.NNT.AND.NSD.EQ.0) NSD=1
C**     +--------------------------------------------------------+    **
C**     | If NSD.GT.0 : NO MORE expanding is allowed             |    **
C**     +--------------------------------------------------------+    **
        IF(NSD.GT.0) THEN
          A(J+JA)=DMAX1(DABS(A(J+JA)),ADD*EPS*EPS)
C**     +--------------------------------------------------------+    **
C**     | If NSD.EQ.0 : Expand the matrix by                     |    **
C**     | Modify the diagonal : A(J,J)=A(J,J)+ADD                |    **
C**     | Add column : A(J,NNT)=-ADD, A(NNT,NNT)=ADD, A(*,NNT)=0 |    **
C**     +--------------------------------------------------------+    **
        ELSE
          A(J+JA)=A(J+JA)+ADD
          NNT=NNT+1
          IF(NNT.GT.MAXN) STOP 'USDECP : NNT.GT.MAXN'
          LL(1,NNT)=LL(2,NNT-1)+NNT-J
          LL(2,NNT)=LL(1,NNT)  +NNT-J
          JB=LL(1,NNT)-LL(1,1)
          JA=LL(2,NNT)-LL(1,1)
          IF(NNT+JA.GT.MAXA) STOP 'USDECP : NNT+JA.GT.MAXA'
          A(J+JB)=-ADD
          A(J+JA)=-ADD
          DO 60 I=J+1,NNT-1
          A(I+JB)=0.0
          A(I+JA)=0.0
   60     CONTINUE
          A(NNT+JA)=ADD
        ENDIF
   80 CONTINUE
C**   +----------------------------------------------------------+    **
c**   | If NNT.GT.NBT Then decompose the newly expand matrix     |    **
C**   | Set NSD=NNT-NBT to prevent from further expanding        |    **
C**   +----------------------------------------------------------+    **
      IF(NNT.GT.NBT) THEN
        NSD=NNT-NBT
        NAT=NBT
        NBT=NNT
        GO TO 10
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | If NNT.EQ.NOT Then return, Else:                         |    **
C**   | Form a sub-system [A]{X}={B} to solve by full pivoting   |    **
C**   +----------------------------------------------------------+    **
      IF(NNT.EQ.NOT) RETURN
      IJ=NNT+LL(2,NNT)
      IF(NNT+NSD.GT.MAXN) STOP 'USDECP : MAXN TOO SMALL'
      IF(IJ+NSD*NSD.GT.MAXA) STOP 'USDECP : MAXA TOO SMALL'
      DO 95 J=NOT+1,NNT
      DO 90 I=NOT+1,NNT
      IJ=IJ+1
      IF(I.LT.J) A(IJ)=A(I+LL(1,J)-LL(1,1))
      IF(I.GE.J) A(IJ)=A(J+LL(2,I)-LL(1,1))
   90 CONTINUE
   95 CONTINUE
      NAD=NSD
      CALL LUFPDC(A(NNT+LL(2,NNT)+1),LL(1,NNT+1),NAD,NSD,NAD)
      RETURN
      END
************************************************************************
      SUBROUTINE USSOLX(A,P,LL,NOT,NNT,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),P(NOT),LL(2,NNT),B(NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(2,NNT)),P(NOT),LL(2,NNT),B(NNT)              **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={P} for given {P} & [A]=>[L][U].           **
C**   [L] & [U] are given & put in A(*)  (Expanded by sub. USDECP)    **
C**   [L][U]{X}={B} ===> [L]{Y}={B},  [U]{X}={Y}                      **
C**   ------------------------------------------------------------    **
C*I   P(NOT) = The given constant vector                              **
C*O   P(NOT) = The solution vector                                    **
C*W   B(NNT) = The expanded vector (form by USSOLX)                   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Expand P(NOT) to B(NNT)                                  |    **
C**   +----------------------------------------------------------+    **
      DO 40 I=1,NOT
      B(I)=P(I)
   40 CONTINUE
      DO 45 I=NOT+1,NNT
      B(I)=0.0
   45 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {B} by forward substitution of [L]{Y}={B}  |    **
C**   | BI = B(I) - \sum_{K=IMA}^{I-1} L(I,K)*Y(K)               |    **
C**   | A(K+IA)=L(I,K), B(K)=Y(K), B(I)=B(I) --> Y(I)=BI/L(I,I)  |    **
C**   +----------------------------------------------------------+    **
      DO 70 I=1,NNT
      IA=LL(2,I)-LL(1,1)
      IMA=I-LL(2,I)+LL(1,I)
        BI=B(I)
        DO 60 K=IMA,MIN0(I-1,NOT)
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI
        IF(I.LE.NOT) B(I)=BI/A(I+IA)
   70 CONTINUE
C**   ------------------------------------------------------------    **
      NAD=NNT-NOT
      IF(NAD.GT.0) CALL LUFPSB
     *(A(NNT+LL(2,NNT)+1),B(NOT+1),LL(1,NNT+1),NAD,NAD)
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=NNT:2 Compute B(K)=B(K)-U(K,I)*X(I) for K=IMB:(I-1)|    **
C**   | A(K+IB)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=NNT,2,-1
      IB=LL(1,I)-LL(1,1)
      IMB=I-LL(1,I)+LL(2,I-1)
        BI=B(I)
        DO 80 K=IMB,MIN0(I-1,NOT)
        B(K)=B(K)-A(K+IB)*BI
   80   CONTINUE
   90 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Move solution vector B(NOT) to P(NOT)                    |    **
C**   +----------------------------------------------------------+    **
      DO 95 I=1,NOT
      P(I)=B(I)
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE USBKSB(A,B,LL,NOT,NNT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(NNT),LL(2,NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(2,NNT)),B(NNT),LL(2,NNT)                     **
C**   ------------------------------------------------------------    **
C**   Find {X} from {Y} by backward substitution of [U]{X}={Y}        **
C**   ------------------------------------------------------------    **
C*I   B(NNT) = The given vector {Y}                                   **
C*O   B(NNT) = The solution vector {X}                                **
C**   ============================================================    **
      NAD=NNT-NOT
      IF(NAD.GT.0) CALL LUFPBS
     *(A(NNT+LL(2,NNT)+1),B(NOT+1),LL(1,NNT+1),NAD,NAD)
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=NNT:2 Compute B(K)=B(K)-U(K,I)*X(I) for K=IMB:(I-1)|    **
C**   | A(K+IB)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=NNT,2,-1
      IB=LL(1,I)-LL(1,1)
      IMB=I-LL(1,I)+LL(2,I-1)
        BI=B(I)
        DO 80 K=IMB,MIN0(I-1,NOT)
        B(K)=B(K)-A(K+IB)*BI
   80   CONTINUE
   90 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE USSNGX(A,S,LL,NOT,NNT,NSD,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),S(1),LL(2,NNT),B(NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(2,NNT)),S(NOT,NSD),LL(2,NNT),B(NNT)          **
C**   ------------------------------------------------------------    **
C**   Find the solution vectors [S] of the singular matrix [A]        **
C**   ------------------------------------------------------------    **
C*I   A(NNT+L*(NNT)) = Decomposed matrix L+U-I : L\U(NNT,NNT)         **
C*I   LL(2,NNT)      = Expanded storage index of L\U(NNT,NNT)         **
C*O   S(NOT,NSD) = Solution vectors of singular matrix A(NOT,NOT)     **
C*I   NOT     = The order of the original matrix A(NOT,NOT)           **
C*I   NNT     = The order of the expanded matrix A(NNT,NNT)           **
C*I   NSD     = Number of singular degrees of the given matrix        **
C*W   B(NNT)  = The working array                                     **
C**   ============================================================    **
      DO 50 J=NNT-NSD+1,NNT
      JS=(J-NNT+NSD-1)*NOT
        DO 30 I=1,NNT
        B(I)=0.0
   30   CONTINUE
        B(J)=1.0
        CALL USBKSB(A,B,LL,NOT,NNT)
        DO 45 I=1,NOT
        S(I+JS)=B(I)
   45   CONTINUE
   50 CONTINUE
      RETURN
      END
************************************************************************
