************************************************************************
      SUBROUTINE VSDECP(A,LL,NOT,NNT,NSD,MAXA,MAXN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MAXA),LL(MAXN)
      REAL*8 EPS,ADD
      DATA EPS/1.0D-7/
C**   ============================================================    **
C**   Modified Cholesky decomposition [A]=[U'][D][U]                  **
C**   ------------------------------------------------------------    **
C*I   A(NOT+LL(NOT)) = The given original matrix A(NOT,NOT)           **
C*O   A(NNT+LL(NNT)) = Decomposed matrix D+U-I : U(NNT,NNT),D(NNT)    **
C*I   LL(1:NOT)      = Original storage index of A(NOT,NOT)           **
C*O   LL(NOT+1:NNT)  = Expanded storage index OF U(NNT,NNT)           **
C*I   NOT     = The order of the original matrix A(NOT,NOT)           **
C*O   NNT     = The order of the expanded matrix A(NNT,NNT)           **
C*O   NSD     = Number of singular degrees of the given matrix        **
C*I   MAXA    = The maximum allowed dimension of A (MAXA)             **
C*I   MAXN    = The maximum allowed dimension of LL(MAXN)             **
C**   EPS     = Epsilon for accuracy control                          **
C**   ------------------------------------------------------------    **
C**   Find [D] & [U] for given [A], Such that  [A] = [U]'[D] [U]      **
C**   where [U]'= transpose of [U], [D] is a diagonal matrix.         **
C**   For example:                                                    **
C**   Input : NOT=3, LL(3)=(0 1 3), [A] is a NOT*NOT matrix           **
C**   Output: NNT=5, LL(5)=(0 1 3 6 9), [D][U] are NNT*NNT matrices   **
C**                                                                   **
C**         | a11 a12 a13 |           | a11+p1 a12   a13 -p1  0  |    **
C**   [A] = | a12 a22 a23 |           |  a12  a22+p2 a23  0  -p2 |    **
C**         | a13 a23 a33 |           |  a13   a23   a33  0   0  |    **
C**   If it is expanded to ---> [A] = |  -p1    0     0  +p1  0  |    **
C**   Then decomposed matrices are:   |   0    -p2    0   0  +p2 |    **
C**                                                                   **
C**         | d11  0   0   0   0  |        |  1  u12 u13 u14  0  |    **
C**         |  0  d22  0   0   0  |        |  0   1  u23 u24 u25 |    **
C**   [D] = |  0   0  d33  0   0  |, [U] = |  0   0   1  u34 u35 |    **
C**         |  0   0   0  d44  0  |        |  0   0   0   1  u45 |    **
C**         |  0   0   0   0  d55 |        |  0   0   0   0   1  |    **
C**                                                                   **
C*O  LL(5):( 0       1           3 |             6               9)   **
C**I+LL(J):1+0 1+1 2+1 1+3 2+3 3+3 1+6 2+6 3+6 4+6 2+9 3+9 4+9 5+9    **
C**   Loc : 1   2   3   4   5   6   7   8   9   10  11  12  13  14    **
C*I  A(6):(a11 a12 a22 a13 a23 a33)                                   **
C*O A(14):(d11 u12 d22 u13 u23 d33 u14 u24 u34 d44 u25 u35 u45 d55)   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Initial set NSD=0 : this also allows matrix to expend    |    **
C**   | Set to decompose the original given matrix A(NOT,NOT)    |    **
C**   +----------------------------------------------------------+    **
      NSD=0
      NAT=0
      NBT=NOT
      NNT=NOT
C**   +----------------------------------------------------------+    **
C**   | Decompose the sub-matrix A((NAT+1):NBT,(NAT+1):NBT)      |    **
C**   +----------------------------------------------------------+    **
C**   | D(I,I)U(I,J)=A(I,J)-\sum_{K=IM}^{I-1} U(K,I)D(K,K)U(K,J) |    **
C**   | A(K+IA)=U(K,I),     A(K+JA)=D(K,K)*U(K,J)                |    **
C**   | A(I+JA)=AIJ=A(I,J) --> D(I,I)*U(I,J)                     |    **
C**   +----------------------------------------------------------+    **
      JB=0
   10 DO 80 J=NAT+1,NBT
      JA=LL(J)-LL(1)
      JM=J-JA+JB
      JB=JA
        DO 30 I=JM+1,J-1
        IA=LL(I)-LL(1)
        IM=MAX0(I-LL(I)+LL(I-1),JM)
          AIJ=A(I+JA)
          DO 20 K=IM,I-1
          AIJ=AIJ-A(K+IA)*A(K+JA)
   20     CONTINUE
          A(I+JA)=AIJ
   30   CONTINUE
C**   +----------------------------------------------------------+    **
C**   | D(J,J) =A(J,J) - \sum_{K=JM}^{J-1} U(K,J)*D(K,K)*U(K,J)  |    **
C**   | A(K+JA)=D(K,K)*U(K,J) --> U(K,J),  AKJ=U(K,J)            |    **
C**   | A(J+JA)=AJJ=A(J,J) --> D(J,J),     A(K+KA)=D(K,K)        |    **
C**   +----------------------------------------------------------+    **
        AJJ=A(J+JA)
        ADD=DABS(AJJ)
        DO 40 K=JM,J-1
        KA=LL(K)-LL(1)
          AKJ=A(K+JA)/A(K+KA)
          AJJ=AJJ-AKJ*A(K+JA)
          A(K+JA)=AKJ
          ADD=DMAX1(ADD,DABS(AJJ))
   40   CONTINUE
        A(J+JA)=AJJ
C**     +--------------------------------------------------------+    **
C**     | IF the pivoting element A(J,J) > ADD*EPS ?             |    **
C**     | Yes:80 : OK and Continue                               |    **
C**     | No     : Add pivoting by ADD and Expand one column     |    **
C**     +--------------------------------------------------------+    **
        IF(DABS(A(J+JA)).GT.ADD*EPS) GO TO 80
C**     +--------------------------------------------------------+    **
C**     | IF ADD.EQ.0 ?                                          |    **
C**     | Yes: No reference value available, get from diagonal   |    **
C**     +--------------------------------------------------------+    **
        IF(ADD.EQ.0.0) THEN
          DO 45 K=NAT+1,NBT
          KA=LL(K)-LL(1)
          ADD=DMAX1(ADD,DABS(A(K+KA)))
   45     CONTINUE
        ENDIF
        IF(ADD.EQ.0.0) ADD=1.0
C**     +--------------------------------------------------------+    **
C**     | If J.EQ.NNT.AND.NSD.EQ.0                               |    **
C**     | Then it is the Only & the Last ZERO_diagonal,          |    **
C**     | So set NSD=1 to prevent from any further expanding     |    **
C**     +--------------------------------------------------------+    **
        IF(J.EQ.NNT.AND.NSD.EQ.0) NSD=1
C**     +--------------------------------------------------------+    **
C**     | If NSD.GT.0 : NO MORE expanding is allowed             |    **
C**     +--------------------------------------------------------+    **
        IF(NSD.GT.0) THEN
          A(J+JA)=DMAX1(DABS(A(J+JA)),ADD*EPS*EPS)
C**     +--------------------------------------------------------+    **
C**     | If NSD.EQ.0 : Expand the matrix by                     |    **
C**     | Modify the diagonal : A(J,J)=A(J,J)+ADD                |    **
C**     | Add column : A(J,NNT)=-ADD, A(NNT,NNT)=ADD, A(*,NNT)=0 |    **
C**     +--------------------------------------------------------+    **
        ELSE
          A(J+JA)=A(J+JA)+ADD
          NNT=NNT+1
          IF(NNT.GT.MAXN) STOP 'VSDECP : NNT.GT.MAXN'
          LL(NNT)=LL(NNT-1)+NNT-J
          JA=LL(NNT)-LL(1)
          IF(NNT+JA.GT.MAXA) STOP 'VSDECP : NNT+JA.GT.MAXA'
          A(J+JA)=-ADD
          DO 60 I=J+1,NNT-1
          A(I+JA)=0.0
   60     CONTINUE
          A(NNT+JA)=ADD
        ENDIF
   80 CONTINUE
C**   +----------------------------------------------------------+    **
c**   | If NNT.GT.NBT Then decompose the newly expand matrix     |    **
C**   +----------------------------------------------------------+    **
      IF(NNT.GT.NBT) THEN
C**     +--------------------------------------------------------+    **
C**     | If newly expand numbers .EQ. previously expand numbers |    **
C**     | then it indicates that :                               |    **
C**     | A((NBT+1):NNT,(NBT+1):NNT)=A((NAT+1):NBT,(NAT+1):NBT)  |    **
C**     | So set NSD=NNT-NBT to prevent from ANY further expand  |    **
C**     +--------------------------------------------------------+    **
        IF(NNT-NBT.EQ.NBT-NAT.AND.NAT.NE.0) NSD=NNT-NBT
C**     +--------------------------------------------------------+    **
C**     | Decompose the newly expand matrix (NBT+1):NNT          |    **
C**     +--------------------------------------------------------+    **
        NAT=NBT
        NBT=NNT
        GO TO 10
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE VSSOLX(A,P,LL,NOT,NNT,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),P(NOT),LL(NNT),B(NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(NNT)),P(NOT),LL(NNT),B(NNT)                  **
C**   ------------------------------------------------------------    **
C**   Find {X} from [A]{X}={P} for given {P} & [A]=>[U]'[D][U].       **
C**   [D] & [U] are given & put in A(*)  (Expanded by sub. VSDECP)    **
C**   [U]'[D][U]{X}={B} ===> [U]'{Z}={B},  [D]{Y}={Z},  [U]{X}={Y}    **
C**   ------------------------------------------------------------    **
C*I   P(NOT) = The given constant vector                              **
C*O   P(NOT) = The solution vector                                    **
C*W   B(NNT) = The expanded vector (form by VSSOLX)                   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Expand P(NOT) to B(NNT)                                  |    **
C**   +----------------------------------------------------------+    **
      DO 40 I=1,NOT
      B(I)=P(I)
   40 CONTINUE
      DO 45 I=NOT+1,NNT
      B(I)=0.0
   45 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Z} from {B} by forward substitution of [U]'{Z}={B} |    **
C**   | Z(I) = BI = B(I) - \sum_{K=IM}^{I-1} U'(I,K)*Z(K)        |    **
C**   | A(K+IA)=U(K,I)=U'(I,K), B(K)=Z(K), B(I)=B(I) --> Z(I)    |    **
C**   +----------------------------------------------------------+    **
      DO 70 I=2,NNT
      IA=LL(I)-LL(1)
      IM=I-LL(I)+LL(I-1)
        BI=B(I)
        DO 60 K=IM,I-1
        BI=BI-A(K+IA)*B(K)
   60   CONTINUE
        B(I)=BI
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {Y} from {Z} from the equation [D]{Y}={Z}           |    **
C**   | Y(I) = Z(I) / D(I,I)                                     |    **
C**   | B(I) = Z(I) --> Y(I),     A(I+IA)=D(I,I)                 |    **
C**   +----------------------------------------------------------+    **
      DO 75 I=1,NNT
      IA=LL(I)-LL(1)
        B(I)=B(I)/A(I+IA)
   75 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=N:2  Compute  B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=NNT,2,-1
      IA=LL(I)-LL(1)
      IM=I-LL(I)+LL(I-1)
        BI=B(I)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
   90 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Move solution vector B(NOT) to P(NOT)                    |    **
C**   +----------------------------------------------------------+    **
      DO 95 I=1,NOT
      P(I)=B(I)
   95 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE VSBKSB(A,B,LL,NOT,NNT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(NNT),LL(NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(NNT)),B(NNT),LL(NNT)                         **
C**   ------------------------------------------------------------    **
C**   Find {X} from {Y} by backward substitution of [U]{X}={Y}        **
C**   ------------------------------------------------------------    **
C*I   B(NNT) = The given vector {Y}                                   **
C*O   B(NNT) = The solution vector {X}                                **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Find {X} from {Y} by backward substitution of [U]{X}={Y} |    **
C**   | For I=NNT:2 Compute B(K)=B(K)-U(K,I)*X(I) for K=IM:(I-1) |    **
C**   | A(K+IA)=U(K,I),     B(I)=BI=X(I),     B(K)=Y(K) --> X(K) |    **
C**   +----------------------------------------------------------+    **
      DO 90 I=NNT,2,-1
      IA=LL(I)-LL(1)
      IM=I-LL(I)+LL(I-1)
        BI=B(I)
        DO 80 K=IM,I-1
        B(K)=B(K)-A(K+IA)*BI
   80   CONTINUE
   90 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE VSSNGX(A,S,LL,NOT,NNT,NSD,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),S(1),LL(NNT),B(NNT)
C**   ============================================================    **
C**   DIMENSION A(NNT+LL(NNT)),S(NOT,NSD),LL(NNT),B(NNT)              **
C**   ------------------------------------------------------------    **
C**   Find the solution vectors [S] of the singular matrix [A]        **
C**   ------------------------------------------------------------    **
C*I   A(NNT+LL(NNT)) = Decomposed matrix D+U-I : U(NNT,NNT),D(NNT)    **
C*I   LL(NNT)        = Expanded storage index OF U(NNT,NNT)           **
C*O   S(NOT,NSD) = Solution vectors of singular matrix A(NOT,NOT)     **
C*I   NOT     = The order of the original matrix A(NOT,NOT)           **
C*I   NNT     = The order of the expanded matrix A(NNT,NNT)           **
C*I   NSD     = Number of singular degrees of the given matrix        **
C*W   B(NNT)  = The working array                                     **
C**   ============================================================    **
      DO 50 J=NNT-NSD+1,NNT
      JS=(J-NNT+NSD-1)*NOT
        DO 30 I=1,NNT
        B(I)=0.0
   30   CONTINUE
        B(J)=1.0
        CALL VSBKSB(A,B,LL,NOT,NNT)
        DO 45 I=1,NOT
        S(I+JS)=B(I)
   45   CONTINUE
   50 CONTINUE
      RETURN
      END
************************************************************************
