************************************************************************
      PROGRAM CUDMAN
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION AGEN(8,8),BG(8),APCD(4,9),BP(8),AFCD(3,10),BF(8)
      DATA MAG/8/,MAP/4/,MAF/3/,APCD/36*0.0/,AFCD/30*0.0/
C**   ============================================================    **
C**   Program for decomposition of an unsymmetric banded matrix       **
C**           and solve the equation by substitution                  **
C**   Using general, partial condensed and full condensed matrices    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Read :  Order N, Lower bandwidth MA, upper bandwidth MB, |    **
C**   |         Matrix & constant vector                         |    **
C**   | Compute NAP,NAF = # of columns reqd for condensed matrix |    **
C**   +----------------------------------------------------------+    **
      READ(*,'(3I4)') N,MA,MB
      NAP=N+(N-1)/MAP
      NAF=N+(N-1)/MAF
      DO 20 I=1,N
   20 READ(*,'(10F8.0)') (AGEN(I,J),J=1,N)
      READ(*,'(10F8.0)') (BG(J),J=1,N)
C**   +----------------------------------------------------------+    **
C**   | Setup Partial & Full condensed matrices                  |    **
C**   | Note that : the subscripts are the same                  |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,N
      DO 30 I=MAX0(J-MB,1),MIN0(J+MA,N)
      APCD(I,J)=AGEN(I,J)
   30 AFCD(I,J)=AGEN(I,J)
      BP(J)=BG(J)
   40 BF(J)=BG(J)
C**   +----------------------------------------------------------+    **
C**   | Write the given matrix & constant vector                 |    **
C**   +----------------------------------------------------------+    **
      CALL PRINTA(AGEN,N,N,MAG,'Given matrix in general form')
      CALL PRINTA(APCD,MAP,NAP,MAP,'In partial condensed form')
      CALL PRINTA(AFCD,MAF,NAF,MAF,'In full condensed form')
      WRITE(*,'(/'' Constant vector''//(5X,10F6.2))') (BG(J),J=1,N)
C**   +----------------------------------------------------------+    **
C**   | Perform Decomposition & substitutions                    |    **
C**   +----------------------------------------------------------+    **
      CALL CUDECP(AGEN,N,MA,MB,MAG)
      CALL CUSOLX(AGEN,BG,N,MA,MB,MAG)
C**   ------------------------------------------------------------    **
      CALL CUDECP(APCD,N,MA,MB,MAP)
      CALL CUSOLX(APCD,BP,N,MA,MB,MAP)
C**   ------------------------------------------------------------    **
      CALL CUDECP(AFCD,N,MA,MB,MAF)
      CALL CUSOLX(AFCD,BF,N,MA,MB,MAF)
C**   +----------------------------------------------------------+    **
C**   | Write the decomposed matrix & result vector              |    **
C**   +----------------------------------------------------------+    **
      CALL PRINTA(AGEN,N,N,MAG,'Decomposed matrix in general form')
      WRITE(*,'(/'' Result vector''//(5X,10F6.2))') (BG(J),J=1,N)
      CALL PRINTA(APCD,MAP,NAP,MAP,'In partial condensed form')
      WRITE(*,'(/'' Result vector''//(5X,10F6.2))') (BP(J),J=1,N)
      CALL PRINTA(AFCD,MAF,NAF,MAF,'In full condensed form')
      WRITE(*,'(/'' Result vector''//(5X,10F6.2))') (BF(J),J=1,N)
      STOP
      END
************************************************************************
      SUBROUTINE PRINTA(A,M,N,MM,NOTE)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(MM,N)
      CHARACTER*(*) NOTE
      DATA NN/10/
C**   ============================================================    **
C**   Print matrix A(M,N)                                             **
C**   ============================================================    **
      WRITE(*,'(/1X,A)') NOTE
      DO 20 JL=1,N,NN
      WRITE(*,'(/5X,10I6)') (J,J=JL,MIN0(JL+NN-1,N))
      DO 10 I=1,M
      WRITE(*,'(I5,10F6.2)') I,(A(I,J),J=JL,MIN0(JL+NN-1,N))
   10 CONTINUE
   20 CONTINUE
      RETURN
      END
************************************************************************
