************************************************************************
      SUBROUTINE INPUT(SM,PM,KK,NE)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SM(2,2,NE),PM(2,NE),KK(2,NE)
C**   ============================================================    **
C**   READ KK(2,NE)   = Unknown # of member end displacements         **
C**        SM(2,2,NE) = Member stiffness matrix                       **
C**        PM(2,NE)   = Member fixed end forces                       **
C**   ============================================================    **
      DO 40 K=1,NE
      READ(*,'(2I4)')   (KK(I,K),I=1,2)
   40 READ(*,'(6F8.0)') ((SM(I,J,K),J=1,2),I=1,2),(PM(I,K),I=1,2)
      RETURN
      END
************************************************************************
      SUBROUTINE FORMEQ(SM,PM,KK,S,P,L,NE,N,MM,NN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SM(2,2,NE),PM(2,NE),KK(2,NE)
      DIMENSION S(MM,NN),P(N),L(N)
C**   ============================================================    **
C**   Form S(N,N) = Structural stiffness matrix                       **
C**    and  P(N)  = Loading vector for S(N,N)*D(N) = P(N)             **
C**   ------------------------------------------------------------    **
C**   For constant banded : MM = Half-bandwidth; NN = N+(N-1)/MM      **
C**   For variable banded : MM = 1;              NN = L(N)+N-1        **
C**   ============================================================    **
      DO 10 JJ=1,NN
      DO 10 II=1,MM
   10 S(II,JJ)=0.0
      DO 15 JJ=1,N
   15 P(JJ)=0.0
C**   ------------------------------------------------------------    **
      DO 30 K=1,NE
      DO 30 J=1,2
      JJ=KK(J,K)
      IF(JJ.EQ.0.OR.JJ.GT.N) GO TO 30
      IF(MM.NE.1) THEN
C**     +--------------------------------------------------------+    **
C**     | For constant banded matrix (L(N) is dummy and unused)  |    **
C**     +--------------------------------------------------------+    **
        JS=JJ
      ELSE
C**     +--------------------------------------------------------+    **
C**     | For variable banded matrix (L(N) is location index)    |    **
C**     +--------------------------------------------------------+    **
        JS=L(JJ)
      ENDIF
      DO 20 I=1,2
      II=KK(I,K)
      IF(II.EQ.0.OR.II.GT.JJ) GO TO 20
      S(II,JS)=S(II,JS)+SM(I,J,K)
   20 CONTINUE
      P(JJ)=P(JJ)-PM(J,K)
   30 CONTINUE
C**   ------------------------------------------------------------    **
      WRITE(*,'(/'' Loading vector of strucutre''//(1X,1P6E11.4))') P
      RETURN
      END
************************************************************************
      SUBROUTINE PRINTV(S,L,N,MM,NN,NOTE)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION S(MM,NN),L(N)
      CHARACTER*(*) NOTE
C**   ============================================================    **
C**   Write S(MM,NN)                                                  **
C**   ============================================================    **
      WRITE(*,'(/1X,A/)') NOTE
      DO 40 I=1,MM
   40 WRITE(*,'(1X,1P8E9.2)') (S(I,J),J=1,NN)
      IF(MM.NE.1) THEN
C**     +--------------------------------------------------------+    **
C**     | For constant banded matrix (L(N) is dummy and unused)  |    **
C**     +--------------------------------------------------------+    **
        DO 50 J=1,N
        JM=MAX0(J-MM,1)
   50   WRITE(*,'(1X,1P8E9.2)') (0.0,I=1,JM-1),(S(I,J),I=JM,J)
      ELSE
C**     +--------------------------------------------------------+    **
C**     | For variable banded matrix (L(N) is location index)    |    **
C**     +--------------------------------------------------------+    **
        JM=1
        DO 60 J=1,N
        JS=L(J)
        IF(J.GT.1) JM=J-L(J)+L(J-1)
   60   WRITE(*,'(1X,1P8E9.2)') (0.0,I=1,JM-1),(S(I,JS),I=JM,J)
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE OUTPUT(SM,PM,KK,D,NE,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION SM(2,2,NE),PM(2,NE),KK(2,NE),D(N),DM(2),AM(2)
C**   ============================================================    **
C**   Write D(N)  = Nodal displacements of the structure              **
C**         AM(2) = Member end forces = SM(2,2)*DM(2) + PM(2)         **
C**   where DM(i) = D(KK(i,K)), if KK(i,K)>0   (else DM(i)=0)         **
C**   ============================================================    **
      WRITE(*,'(/'' Displacements of strucutre''//(1X,1P6E11.4))') D
      WRITE(*,'(/1X,A)')  'Member end forces'
      WRITE(*,'(/1X,A/)') 'Member   Left-moment   Right-moment'
      DO 50 K=1,NE
      DO 30 J=1,2
      JJ=KK(J,K)
      DM(J)=0.0
      IF(JJ.NE.0.AND.JJ.LE.N) DM(J)=D(JJ)
   30 CONTINUE
      DO 40 I=1,2
      AM(I)=PM(I,K)
      DO 40 J=1,2
   40 AM(I)=AM(I)+SM(I,J,K)*DM(J)
   50 WRITE(*,'(I6,1P2E15.5)') K,(AM(I),I=1,2)
      RETURN
      END
************************************************************************
