************************************************************************
      SUBROUTINE BVEXAV(A,B,V,N,X,IH,M,EPSI,NV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1),V(1),X(N),IH(N)
C**   ============================================================    **
C**   DIMENSION A,B(N*(N-1)/2),V(NV,N),X(N),IH(N) ! A,B(NA,N)         **
C**   ------------------------------------------------------------    **
C**   [B] {X} = d [A] {X}   --->   [B] [V] = [A] [V] [D]              **
C**   Normalize as : [V]'[A] [V] = [I],      [V]'[B] [V] = [D]        **
C**   ------------------------------------------------------------    **
C**   Input : A,B,V,N,M,EPSI,NV,NA       Output : A,V,X,M             **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [A] matrix required upper triangular part only         **
C*O          = [D] diagonal matrix                                    **
CIW   B(N,N) = [B] matrix required upper triangular part only         **
C*O   V(N,N) = Eigen vector matrix                                    **
C*I   N      = Order of matrix [A]                                    **
C*O   X(N)   = Eigenvalues (also in the diagonal of [A])              **
C*W   IH(N)  = Working array for row I of max.A(I,J) in col J         **
C*O   M      = Iteration count                                        **
C*I   EPSI   = Convergence if max.|A(I,J)| < min.A(K,K)*EPSI          **
C*I   NV,NA  = Row dimensions of array V(NV,N) and A(NA,N)            **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Solve [B] {Y} = e {Y}  for  [V] & [E]                    |    **
C**   +----------------------------------------------------------+    **
      M=0
      CALL JACOBI(B,V,N,X,IH,1,M,EPSI,NV)
C**   +----------------------------------------------------------+    **
C**   | Compute : [V] [E]^{-1/2} ---> [V]                        |    **
C**   +----------------------------------------------------------+    **
      DO 20 J=1,N
      JV=(J-1)*NV
      JA=(J-1)*J/2 ! JA=(J-1)*NA
      IF(B(J+JA).GT.0.0) THEN
        BB=1.0/DSQRT(B(J+JA))
        DO 10 I=1,N
        V(I+JV)=V(I+JV)*BB
   10   CONTINUE
      ENDIF
   20 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute : [V]'[A] [V] ---> [A]                           |    **
C**   +----------------------------------------------------------+    **
      CALL MAV(A,B,A,V,X,N,N,NV)
      CALL MAV(A,B,B,V,X,N,0,NV)
C**   +----------------------------------------------------------+    **
C**   | Compute : [G]'[A] [G] ---> [A];  [V] [G] ---> [V]        |    **
C**   +----------------------------------------------------------+    **
      CALL GAG(A,B,V,N,X,EPSI,NV)
C**   +----------------------------------------------------------+    **
C**   | Solve [A] {X} = 1/d {X}  for  [V] & [D]^{-1}             |    **
C**   +----------------------------------------------------------+    **
      CALL JACOBI(A,V,N,X,IH,-1,M,EPSI,NV)
C**   +----------------------------------------------------------+    **
C**   | Before norm. : [V]'[A] [V] = [D]^{-1}, [V]'[B] [V] = [I] |    **
C**   | Normalize as : [V]'[A] [V] = [I],      [V]'[B] [V] = [D] |    **
C**   +----------------------------------------------------------+    **
      DO 95 J=1,N
      JV=(J-1)*NV
      JA=(J-1)*J/2 ! JA=(J-1)*NA
      IF(A(J+JA).EQ.0.0) THEN
        X(J)=1.0D30
      ELSE
        X(J)=1.0/A(J+JA)
        BB=DSQRT(ABS(X(J)))
        DO 90 I=1,N
        V(I+JV)=V(I+JV)*BB
   90   CONTINUE
      ENDIF
   95 CONTINUE
      RETURN
      END
************************************************************************
