************************************************************************
      SUBROUTINE JACOBI(A,V,N,X,IH,JEVC,M,EPSI,NV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),V(1),X(N),IH(N)
C**   ============================================================    **
C**   DIMENSION A(N*(N-1)/2),V(NV,N),X(N),IH(N) ! A(NA,N)             **
C**   ------------------------------------------------------------    **
C**   [A] {X} = d {X} --> [A] [V] = [V] [D]                           **
C**   ------------------------------------------------------------    **
C**   Input : A,V,N,JEVC,M,EPSI,NV,NA      Output : A,V,X,M           **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [A] matrix required upper triangular part only         **
C*O          = [D] diagonal matrix                                    **
C*I   V(N,N) = Matrix to be post-multiplied by eigenvector matrix     **
C*O          = EiVec(JEVC=1); None(JEVC=0); input.V*EiVec(JEVC=-1)    **
C*I   N      = Order of matrix [A]                                    **
C*O   X(N)   = Eigenvalues (also in the diagonal of [A])              **
C*W   IH(N)  = Working array for row I of max.A(I,J) in col J         **
C*I   JEVC   = 1 : Find eigenvectors and return in V(N,N)             **
C*I          = 0 : Do not find eigenvectors and V(N,N) is dummy       **
C*I          =-1 : Find eigenvectors and post-multiply to V(N,N)      **
CIO   M      = Iteration count                                        **
C*I   EPSI   = Convergence if max.|A(I,J)| < min.A(K,K)*EPSI          **
C*I   NV,NA  = Row dimensions of array V(NV,N) and A(NA,N)            **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Initialize : Set M=0, [V]=[I] (JEVC=1), X(N) and IH(N)   |    **
C**   +----------------------------------------------------------+    **
C**   M=0
      IF(JEVC.GT.0) THEN
        DO 20 J=1,N
        JV=(J-1)*NV
          DO 10 I=1,N
          V(I+JV)=0.0
   10     CONTINUE
        V(J+JV)=1.0
   20   CONTINUE
      ENDIF
      IF(N.LE.1) RETURN
C**   +----------------------------------------------------------+    **
C**   | X(JX) = max.|A(IH(JX),JX)| in col JX  (IH(JX) < JX)      |    **
C**   | AMIN  = min.A(K,K) (K=1:N)                               |    **
C**   +----------------------------------------------------------+    **
      AMIN=DABS(A(1))
      DO 40 JX=2,N
      JA=(JX-1)*JX/2 ! JA=(JX-1)*NA
      AMIN=DMIN1(AMIN,DABS(A(JX+JA)))
      X(JX)=-1.0
      DO 40 II=1,JX-1
      IF(DABS(A(II+JA)).LE.X(JX)) GO TO 40
        X(JX)=DABS(A(II+JA))
        IH(JX)=II
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Begin loops : Return if converged                        |    **
C**   +----------------------------------------------------------+    **
C**   | Find IP,JP for max.|A(IP,JP)| from X(N) and IH(N)        |    **
C**   +----------------------------------------------------------+    **
   50 M=M+1
C**   CALL PRINTT(A,N,'Matrix A')
      AIJ=-1.0
      DO 60 JX=2,N
      IF(X(JX).LE.AIJ) GO TO 60
        AIJ=X(JX)
        IP=IH(JX)
        JP=JX
   60 CONTINUE
      IF(AIJ.LE.AMIN*EPSI) RETURN
C**   ------------------------------------------------------------    **
      IA=IP
      IB=JP
      JA=(IP-1)*IP/2 ! JA=(IP-1)*NA
      JB=(JP-1)*JP/2 ! JB=(JP-1)*NA
C**   +----------------------------------------------------------+    **
C**   | Compute the rotation angle                               |    **
C**   +----------------------------------------------------------+    **
      AIJ=2.*A(IA+JB)
      AII=A(IA+JA)
      AJJ=A(IB+JB)
      TANG=AIJ/(DABS(AJJ-AII)+DSQRT((AJJ-AII)**2+AIJ**2))
      AIJ=A(IA+JB)*TANG
      A(IA+JB)=0.0
      IF(AII.GE.AJJ) THEN
        A(IA+JA)=AII+AIJ
        A(IB+JB)=AJJ-AIJ
      ELSE
        A(IA+JA)=AJJ+AIJ
        A(IB+JB)=AII-AIJ
        TANG=1.0/TANG
      ENDIF
      COSS=1.0/(1.0+TANG*TANG)
      COSN=DSQRT(COSS)
      SINE=TANG*COSN
C**   AMIN=DMIN1(AMIN,DABS(A(IB+JB)))
C**   +----------------------------------------------------------+    **
C**   |                 IP                    JP                 |    **
C**   |    / .        *                     *                  \ |    **
C**   |    |   .   A(IA+JA)              A(IB+JB)              | |    **
C**   |    |     .    *                     *                  | |    **
C**   | IP |       A(IP,IP) * A(IA+JA) * A(IP,JP) * A(IA+JA) * | |    **
C**   |    |                .               *                  | |    **
C**   |    |                     .       A(IB+JB)              | |    **
C**   |    |                           .    *                  | |    **
C**   | JP |                             A(JP,JP) * A(IB+JB) * | |    **
C**   |    |                                      .            | |    **
C**   |    |                                           .       | |    **
C**   |    \                                                 . / |    **
C**   +----------------------------------------------------------+    **
C**   | Reset X(IP),IH(IP) and X(JP),IH(JP) for the changes      |    **
C**   +----------------------------------------------------------+    **
      JX=IP
      JY=JP
      X(JX)=0.0
      X(JY)=0.0
      DO 250 JJ=1,N
      IF(JJ.LT.IP) THEN
C**     JX=IP
C**     JY=JP
        IA=JJ
        IB=JJ
C**     JA=(IP-1)*IP/2 ! JA=(IP-1)*NA
C**     JB=(JP-1)*JP/2 ! JB=(JP-1)*NA
      ELSE IF(JJ.EQ.IP) THEN
        IA=IP
        GO TO 250
      ELSE
        JX=JJ
        JA=JA+JJ-1 ! JA=JA+NA
        IF(JJ.LT.JP) THEN
C**       JX=JJ
C**       JY=JP
C**       IA=IP
          IB=JJ
C**       JA=(JJ-1)*JJ/2 ! JA=(JJ-1)*NA
C**       JB=(JP-1)*JP/2 ! JB=(JP-1)*NA
        ELSE IF(JJ.EQ.JP) THEN
          IB=JP
          GO TO 250
        ELSE
C**       JX=JJ
          JY=JX
C**       IA=IP
C**       IB=JP
C**       JA=(JJ-1)*JJ/2 ! JA=(JJ-1)*NA
          JB=JA
        ENDIF
      ENDIF
      AIJ=A(IA+JA)
      A(IA+JA)= AIJ*COSN+A(IB+JB)*SINE
      A(IB+JB)=-AIJ*SINE+A(IB+JB)*COSN
C**   +----------------------------------------------------------+    **
C**   | Reset X(JX),IH(JX) if the max.X(JX) has been changed     |    **
C**   +----------------------------------------------------------+    **
      IF(IH(JX).EQ.IP.OR.IH(JY).EQ.JP) THEN
        X(JX)=-1.0
        DO 180 II=1,JJ-1
        IF(X(JX).GE.ABS(A(II+JA))) GO TO 180
          X(JX)=ABS(A(II+JA))
          IH(JX)=II
  180   CONTINUE
        IF(JJ.GE.JP) GO TO 250
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Update X(JX),IH(JX) and X(JY),IH(JY)                     |    **
C**   +----------------------------------------------------------+    **
      IF(DABS(A(IA+JA)).GT.X(JX)) THEN
        X(JX)=DABS(A(IA+JA))
        IH(JX)=IA
      ENDIF
      IF(DABS(A(IB+JB)).GT.X(JY)) THEN
        X(JY)=DABS(A(IB+JB))
        IH(JY)=IB
      ENDIF
  250 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | V(N,N) post-multiply by the rotation matrix              |    **
C**   +----------------------------------------------------------+    **
      IF(JEVC.NE.0) THEN
        JU=(IP-1)*NV
        JV=(JP-1)*NV
        DO 310 I=1,N
          AIJ=V(I+JU)
          V(I+JU)= AIJ*COSN+V(I+JV)*SINE
          V(I+JV)=-AIJ*SINE+V(I+JV)*COSN
  310   CONTINUE
      ENDIF
      GO TO 50
      END
************************************************************************
