************************************************************************
      SUBROUTINE SUBSPC
     * (LL,AKA,AMA,AV,W,N,MT,MW,VV,SKS,SMS,SV,Y,ID,NMAX,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION LL(N),AKA(1),AMA(1),AV(1),W(MT)
      DIMENSION VV(N),SKS(1),SMS(1),SV(1),Y(MT)
C**   ============================================================    **
C*I   LL(N)     = Input variable band matrix stored index             **
C*I   AKA(NN)   = Input stiffness matrix [K]                          **
C*I   AMA(NN)   = Input mass matrix [M] (if id=1)                     **
C*I   AMA(N)    = Input diagonal mass matrix [M] (if id=0)            **
C*O   AV(N,MT)  = Output eigenvectors [V]                             **
C*O   W(MT)     = Eigenvalues (omaga squares)                         **
C*W   VV(N)     = Working array                                       **
C*W   SKS(MT*(MT+1)/2) = Working arry to store [V]'[K] [V]            **
C*W   SMS(MT*(MT+1)/2) = Working arry to store [V]'[M] [V]            **
C*W   SV(MT,MT) = Working array                                       **
C*O   Y(MT)     = Working array                                       **
C*I   N         = Order of AKA and AMA                                **
C*I   MT        = Total no. of eigenvectors used in iteraton          **
C*I   MW        = Reqd. no. of eigenvectors                           **
C*I   ID        = 0, AMA is a diagonal matrix                         **
C*I             = 1, AMA is stored as AKA                             **
C*I   NMAX      = Allowed max. no. of subspace iterations             **
C*I   EPS       = Tolerence of eigenvales W(MW)                       **
C*I   NN        = LL(N)+N-1                                           **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Initialize                                               |    **
C**   +----------------------------------------------------------+    **
      NPT=1
      M=MT
      MMM=NMAX
      NNN=0
      KKK=0
C**   +----------------------------------------------------------+    **
C**   | Decompose [K]=[L]'[D] [L]; (1) AV=[Uo], {W}={0}          |    **
C**   +----------------------------------------------------------+    **
      CALL VBDECP(AKA,LL,N)
      CALL SETMVO(LL,AKA,AMA,AV,N,M,ID,VV)
      DO 20 J=1,M
   20 W(J)=0.0
C**   +----------------------------------------------------------+    **
C**   | (2) AV=[V1]=inv.[K][Uo]; (3) SKS=[V1]'[Uo]=[V1]'[K] [V1] |    **
C**   +----------------------------------------------------------+    **
   30 NNN=NNN+1
      DO 50 J=1,M
      Y(J)=W(J)
      JV=(J-1)*N
      DO 40 I=1,N
   40 VV(I)=AV(I+JV)
      CALL VBSOLX(AKA,AV(1+JV),LL,N)
   50 CALL CMPYAT(AV,VV,SKS((J-1)*J/2+1),J,N)
      IF(NNN/NPT*NPT.EQ.NNN.OR.NNN.EQ.MMM) THEN
        WRITE(*,'(/1X,A,I4)') 'Subspace iteration No.',NNN
        CALL PRINTT(SKS,M,'Matrix SKS')
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | (4) If NNN<MMM then AV=[U1]=[M][V1] else AV=[V1]         |    **
C**   | (5) SMS=[U1]'[V1]=[V1]'[M] [V1]                          |    **
C**   +----------------------------------------------------------+    **
      DO 80 J=1,M
      JV=(J-1)*N
      IF(NNN.LT.MMM) THEN
        DO 70 I=1,N
        VV(I)=AV(I+JV)
   70   CONTINUE
        CALL VBCMPY(AMA,VV,AV(1+JV),LL,N,ID)
      ELSE
        CALL VBCMPY(AMA,AV(1+JV),VV,LL,N,ID)
      ENDIF
      CALL CMPYAT(AV,VV,SMS((J-1)*J/2+1),J,N)
   80 CONTINUE
      IF(NNN/NPT*NPT.EQ.NNN.OR.NNN.EQ.MMM) THEN
        CALL PRINTT(SMS,M,'Matrix SMS')
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | (6) Solve [SKS] [SV] = [SMS] [SV] [W]                    |    **
C**   | (7) If NNN<MMM then AV=[U2]=[U1][SV] else AV=[V2]=[V1][SV]    **
C**   +----------------------------------------------------------+    **
   90 CALL BVEXAV(SMS,SKS,SV,M,W,VV,KKK,1.0D-5,M)
      CALL ORDERX(W,SV,VV,M)
      CALL AMPY(AV,SV,VV,N,M)
C**   +----------------------------------------------------------+    **
C**   | (8) Convergence test (If so, set MMM=0 for last iter.)   |    **
C**   +----------------------------------------------------------+    **
      IF(NNN.LT.MMM) THEN
        DO 120 J=1,MW
        IF(DABS(Y(J)-W(J)).GT.EPS*W(J)) GO TO 30
  120   CONTINUE
        MMM=0
        GO TO 30
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | (9) Output the eigenvalues and return                    |    **
C**   +----------------------------------------------------------+    **
      IF(NNN.EQ.MMM) WRITE(*,'(/A,1PE11.4)') ' Not converged within',EPS
      WRITE(*,'(/1X,A,I4,A)') 'Subspace performs',NNN,' iterations.'
      WRITE(*,'(1X,A,I4,A)')  'Jacobian performs',KKK,' rotations.'
      WRITE(*,'(1X,A)') 'The eigenvalues of the last two iterartions :'
      WRITE(*,'(/(1X,1P10E13.6))') (Y(J),J=1,M)
      WRITE(*,'(/(1X,1P10E13.6))') (W(J),J=1,M)
      RETURN
      END
************************************************************************
