************************************************************************
      SUBROUTINE AUEXBU(A,B,V,N,X,Y,IERR,EPSI,NV,NA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(1),B(1),V(1),X(N),Y(N)
C**   ============================================================    **
C**   DIMENSION A,B(NA,N),V(NV,N),X(N),Y(N)                           **
C**   ------------------------------------------------------------    **
C**   [A] {X} = d [B] {X}   --->   [A] [V] = [B] [V] [D]              **
C**   Normalize as : [V]'[A] [V] = [D],      [V]'[B] [V] = [I]        **
C**   ------------------------------------------------------------    **
C**   Input : A,B,V,N,EPSI,NV,NA         Output : A,V,X,IERR          **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [A] matrix required upper triangular part only         **
C*O          = [D] diagonal matrix                                    **
CIW   B(N,N) = [B] matrix required upper triangular part only         **
C*O   V(N,N) = Eigen vector matrix                                    **
C*I   N      = Order of matrix [A]                                    **
C*O   X(N)   = Eigenvalues (also in the diagonal of [A])              **
C*W   Y(N)   = Working array                                          **
C*O   IERR   = J: J-th eigenvalue no converge; = 0: normal return     **
C*I   EPSI   = Used by GAG                                            **
C*I   NV     = Dummy (for JACOBI only)                                **
C*I   NA     = Row dimensions of array V(NA,N) and A(NA,N)            **
C**   ============================================================    **
C     DO 20 J=1,N
C     JV=(J-1)*NA
C     DO 10 I=1,N
C  10 V(I+JV)=0.0
C  20 V(J+JV)=1.0
C**   +----------------------------------------------------------+    **
C**   | Solve [B] {Y} = e {Y}  for  [V] & [E]                    |    **
C**   +----------------------------------------------------------+    **
      CALL TRIDIS(NA,N,B,X,Y,V)
      CALL TRIQLI(NA,N,X,Y,V,IERR)
C**   +----------------------------------------------------------+    **
C**   | Compute : [V] [E]^{-1/2} ---> [V]                        |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,N
      JV=(J-1)*NA
      IF(X(J).GT.0.0) THEN
        BB=1.0/DSQRT(X(J))
        DO 30 I=1,N
        V(I+JV)=V(I+JV)*BB
   30   CONTINUE
      ENDIF
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute : [V]'[A] [V] ---> [A]                           |    **
C**   +----------------------------------------------------------+    **
      CALL MAV(A,B,A,V,X,N,N,NA,NA)
      CALL MAV(A,B,B,V,X,N,0,NA,NA)
C**   +----------------------------------------------------------+    **
C**   | Compute : [G]'[A] [G] ---> [A];  [V] [G] ---> [V]        |    **
C**   +----------------------------------------------------------+    **
      CALL GAG(A,B,V,N,X,EPSI,NA,NA)
C**   +----------------------------------------------------------+    **
C**   | Solve [A] {X} = d {X}  for  [V] & [D]                    |    **
C**   +----------------------------------------------------------+    **
      CALL TRIDIG(NA,N,A,X,Y,V)
      CALL TRIQLI(NA,N,X,Y,V,IERR)
      RETURN
      END
************************************************************************
