************************************************************************
      SUBROUTINE AZEXBZ(NA,N,A,B,D,E,F,Z,IERR)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),B(NA,N),D(N),E(N),F(N),Z(NA,N)
C**   ============================================================    **
C**   Find the eigenvalues and eigenvectors of [A]{Z} = e [B]{Z}      **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = Given real matrix [A]                                  **
C*O          = [Y] in upper triangular part (no need for user)        **
C*I   B(N,N) = Given real matrix [B]                                  **
C*O          = [U] in upper triangular part (no need for user)        **
C*O   D(J)   = Real part of J-th eigenvalue (if F(J).NE.0)            **
C*O   E(J)   = Imag part of J-th eigenvalue (if F(J).NE.0)            **
C*O   F(J)   = Absolute value of J-th eigenvalue of [B]{X}=e{X}       **
C**          = 0 & D(J)+E(J)i.NE.0 then J-th e-value is infinity      **
C**          = 0 & D(J)+E(J)i.EQ.0 then J-th e-value is arbitary      **
C*O   Z(N,J) = Z(1:N,J) = eigenvector of real eigenvalue D(J)         **
C**            Z(1:N,J)+Z(1:N,J+1)i = eigenvector of D(J)+E(J)i       **
C**            Z(1:N,J)-Z(1:N,J+1)i = eigenvector of D(J)-E(J)i       **
C*O   IERR   = J: J-th eigenvalue no converge; = 0: normal return     **
C**   ------------------------+-----------------------------------    **
C**           B               |               A  Z                    **
C**   ------------------------+-----------------------------------    **
C**      [V]'[B][V] = [T]     |          [V]'[A][V] = [C]             **
C**   [u][V]'[B][V][U] = [S]  |       [u][V]'[A][V][U]                **
C**    [k][d][S][d] = [J]   /   [k][d][u][V]'[A][V][U][d] = [G]       **
C**       [L][J][R] = [J] /  [L][k][d][u][V]'[A][V][U][d][R]          **
C**              [Qut] = [Z]'[L][k][d][u][V]'[A][V][U][d][R][Z]       **
C**           [Qdg] = [y][Z]'[L][k][d][u][V]'[A][V][U][d][R][Z][Y]    **
C**   ------------------------------------------------------------    **
C**     Note : [y],[k],[d],[u] are inverses of [Y],[K],[D],[U]        **
C**          : [V],[Z] are orthogonal matrices                        **
C**          : [U],[Y] are upper triangular matrices                  **
C**          : [D],[d] are diagonal matrices                          **
C**          : [K],[k] are quasi-diagonal orthogonal matrices         **
C**          : [L],[R] are static condensation matrices               **
C**          : [T],[Qut] are quasi-upper triangular matrices          **
C**          : [S],[Qdg] are quasi-diagonal matrices                  **
C**          : [J] is quasi-identity matrix : [V]'[J][V] .NE. [J]     **
C**          : [J] = [y][Z]'[J][Z][Y] only for restrict [Z] & [Y]     **
C**   ============================================================    **
      DO 20 J=1,N
      DO 15 I=1,N
   15 Z(I,J)=0.0
   20 Z(J,J)=1.0
      CALL ORTHEG(NA,N,1,N,B,D,Z)        !      [ ]'[B][ ]=[H]
      CALL HQRIDS(NA,N,1,N,B,D,E,Z,IERR) !      [V]'[B][V]=[T]
      IF(IERR.NE.0) RETURN               !
      CALL ZTAZ  (A,F,Z,N,NA)            !      [V]'[A][V]=[C]
      CALL QUTEVE(NA,N,1,N,B,D,E,Z)      !       [u][T][U]=[S]
      CALL QUTGEN(A,B,D,E,F,Z,N,NA)      ! [k][d][u][C][U][d]=[G]
      CALL STCOND(A,D,E,F,Z,N,NA)        !       [L][G][R]
      CALL ORTHEG(NA,N,1,N,A,D,Z)        !   [ ]'[L][G][R][ ]=[Hes]
      CALL HQRIDS(NA,N,1,N,A,D,E,Z,IERR) !   [Z]'[L][G][R][Z]=[Tri]
      CALL QUTEVE(NA,N,1,N,A,D,E,Z)      ! [V][U][d][R][Z][Y]=[Eve]
      RETURN   ! [y][Z]'[L][k][d][u][V]'[A][V][U][d][R][Z][Y]=[Eva]
      END
************************************************************************
      SUBROUTINE QUTGEN(A,U,WR,WI,F,Z,N,NA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),U(NA,N),WR(N),WI(N),F(N),Z(NA,N)
C**   ============================================================    **
C**   Compute  [G] = [K]^{-1}[D]^{-1}[U]^{-1}[C][U][D]^{-1}           **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [C] = [V]'[A][V]                                       **
C*O          = [G] = [k][d][u][V]'[A][V][U][d]                        **
C*I   U(N,N) = [U] : Trans. matrix for [U]^{-1}[V]'[B][V][U] = [S]    **
C*I   WR(N)  = Real part of eigenvalues                               **
C*I   WI(N)  = Imag part of eigenvalues                               **
C*O   F(N)   = Sqrt(WR(N)**2+WI(N)**2)                                **
C*I   Z(N,N) = [V][U] : Trans. matrix for [u][V]'[B][V][U] = [S]      **
C*O          = [V][U][d] : [d] = [D]^{-1} for [S] = [D][K][J][D]      **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [A][U] : [U] is an upper triangular matrix |    **
C**   +----------------------------------------------------------+    **
      DO 550 J=N,1,-1
      DO 550 I=1,N
      AIJ=0.0
      DO 530 K=1,J
  530 AIJ=AIJ+A(I,K)*U(K,J)
  550 A(I,J)=AIJ
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [U]^{-1}[A] : By backward substitutions    |    **
C**   +----------------------------------------------------------+    **
      DO 580 J=1,N
      DO 580 K=N,1,-1
      AKJ=A(K,J)/U(K,K)
      DO 560 I=1,K-1
  560 A(I,J)=A(I,J)-U(I,K)*AKJ
  580 A(K,J)=AKJ
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [D]^{-1}[A][D]^{-1} ; [Z] = [Z][D]^{-1}    |    **
C**   +----------------------------------------------------------+    **
      DO 650 J=1,N
      F(J)=CDABS(DCMPLX(WR(J),WI(J)))
      IF(F(J).EQ.0.0) GO TO 650
      DJ=1.0/DSQRT(F(J))
      DO 620 I=1,N
      Z(I,J)=Z(I,J)*DJ
      A(I,J)=A(I,J)*DJ
  620 A(J,I)=A(J,I)*DJ
  650 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [K]^{-1}[A]                                |    **
C**   +----------------------------------------------------------+    **
      DO 680 K=1,N
      IF(WI(K).NE.0.0) THEN
        IF(WI(K).LT.0.0) GO TO 680
        C=WR(K)/F(K)      ! Complex eigenvalue of [B]
        S=WI(K)/F(K)
        DO 660 J=1,N
        AKJ=A(K,J)
        A(K  ,J)=C*AKJ-S*A(K+1,J)
  660   A(K+1,J)=S*AKJ+C*A(K+1,J)
      ELSE IF(WR(K).LT.0.0) THEN
        DO 670 J=1,N      ! Negative real eigenvalue of [B]
  670   A(K,J)=-A(K,J)
      ENDIF
  680 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE STCOND(A,D,E,F,Z,N,NA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),D(N),E(N),F(N),Z(NA,N)
C**   ============================================================    **
C**   Perform static condensations  [L][G][R] ==> [A]                 **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [G] = [k][d][u][V]'[A][V][U][d] : matrix to be cond.   **
C*O          = [L][G][R] = [L][k][d][u][V]'[A][V][U][d][R]            **
C*W   D,E(N) = Working array                                          **
C*I   F(N)   = Absolute value of eigenvalue                           **
C*I   Z(N,N) = [V][U][d] : for [k][d][u][V]'[B][V][U][d] = [J]        **
C*O          = [V][U][d][R]                                           **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Compute  [A] = [L][G][R]                                 |    **
C**   |   ------[ L ]------     ------------[ R ]-------------   |    **
C**   |  | 1   -g13/g33    |   |    1                         |  |    **
C**   |  |   1 -g23/g33    |   |            1                 |  |    **
C**   |  |         1       |   |-g31/g33 -g32/g33  1 -g34/g33 |  |    **
C**   |  |     -g43/g33  1 |   |                        1     |  |    **
C**   +----------------------------------------------------------+    **
      DO 800 K=1,N
      IF(F(K).NE.0.0) GO TO 800
      IF(A(K,K).EQ.0.0) THEN
        S=0.0
        DO 705 I=1,N
        S=S+DABS(A(I,K)*A(K,I))
        A(I,K)=0.0
  705   A(K,I)=0.0
        A(K,K)=1.0
        IF(S.NE.0.0) WRITE(*,'('' MATRIX IS NOT WELLPOSED!'')')
        GO TO 800
      ENDIF
      DO 710 I=1,N
      E(I)=Z(I,K)
      D(I)=A(I,K)
  710 A(I,K)=0.0
      A(K,K)=D(K)
C**   +----------------------------------------------------------+    **
C**   | Compute  [A] = [L][G][R]  and  [Z] = [Z][R]              |    **
C**   +----------------------------------------------------------+    **
      DO 760 J=1,N
      IF(J.EQ.K) GO TO 760
      AKJ=A(K,J)/A(K,K)
      DO 720 I=1,N
      Z(I,J)=Z(I,J)-E(I)*AKJ
  720 A(I,J)=A(I,J)-D(I)*AKJ
      A(K,J)=0.0
  760 CONTINUE
  800 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE ZTAZ(A,W,Z,N,NA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),W(N),Z(NA,N)
C**   ============================================================    **
C**   Compute  [A] = [Z]'[A][Z]                                       **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = [A]                                                    **
C*O          = [Z]'[A][Z]                                             **
C*W   W(N)   = Working array                                          **
C*I   Z(N,N) = Orthogonal matrix                                      **
C**   ============================================================    **
      DO 30 I=1,N
      DO 10 K=1,N
   10 W(K)=A(I,K)
      DO 20 J=1,N
      S=0.0
      DO 15 K=1,N
   15 S=S+W(K)*Z(K,J)
   20 A(I,J)=S
   30 CONTINUE
      DO 80 J=1,N
      DO 60 K=1,N
   60 W(K)=A(K,J)
      DO 70 I=1,N
      S=0.0
      DO 65 K=1,N
   65 S=S+Z(K,I)*W(K)
   70 A(I,J)=S
   80 CONTINUE
      RETURN
      END
************************************************************************
