************************************************************************
      SUBROUTINE ORTHEG(NA,N,ILW,IGH,A,D,Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),D(N),Z(NA,N)
C**   ============================================================    **
C**   Simularity transform real matrix to upper-Hessenberg matrix     **
C**   By Householder QR method                                        **
C**   ------------------------------------------------------------    **
C*I   ILW,IGH = Part of matrix considered is A(ILW:IGH,ILW:IGH)       **
C*I   A(N,N)  = [B] = Real matrix                                     **
C*O           = [H] = Upper Hessenberg matrix = [P]'[B][P]            **
C*O           + Information about the orthogonal transformations      **
C*O   D(N)    = Further informations about the transformations        **
C*I   Z(N,N)  = [I] or [X] = Matrix to be post-multiplied by [P]      **
C*O           = [P] or [X][P]                                         **
C**   ============================================================    **
      DO 80 K=ILW,IGH-2
      TK=0.0
      DO 20 I=K+1,IGH
   20 TK=TK+DABS(A(I,K))
      IF(TK.EQ.0.0) GO TO 80
C**   +----------------------------------------------------------+    **
C**   | Scale u-vector to avoid overflow, {u} = {w}sqrt(2) = {D} |    **
C**   +----------------------------------------------------------+    **
      S=0.0
      DO 25 I=K+1,IGH
      D(I)=A(I,K)/TK
   25 S=S+D(I)*D(I)
      S=DSIGN(DSQRT(S),D(K+1))
      D(K+1)=D(K+1)+S
      T=DSQRT(S*D(K+1))
      DO 30 I=K+1,IGH
   30 D(I)=D(I)/T
C**   +----------------------------------------------------------+    **
C**   | Compute  [A] = [[I] - 2{w}<w>][A] = [A] - {u}<u>[A]      |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=K+1,N
      P=0.0
      DO 35 I=K+1,IGH
   35 P=P+D(I)*A(I,J)
      DO 40 I=K+1,IGH
   40 A(I,J)=A(I,J)-D(I)*P
C**   +----------------------------------------------------------+    **
C**   | Compute  [A] = [A][[I] - 2{w}<w>] = [A] - [A]{u}<u>      |    **
C**   +----------------------------------------------------------+    **
      DO 50 I=1,IGH
      P=0.0
      DO 45 J=K+1,IGH
   45 P=P+A(I,J)*D(J)
      DO 50 J=K+1,IGH
   50 A(I,J)=A(I,J)-P*D(J)
      A(K+1,K)=-S*TK
C**   +----------------------------------------------------------+    **
C**   | Compute  [Z] = [Z][[I] - 2{w}<w>] = [Z] - [Z]{u}<u>      |    **
C**   +----------------------------------------------------------+    **
      DO 60 I=1,IGH
      Q=0.0
      DO 55 J=K+1,IGH
   55 Q=Q+Z(I,J)*D(J)
      DO 60 J=K+1,IGH
   60 Z(I,J)=Z(I,J)-Q*D(J)
      D(K+1)=D(K+1)*TK*T
   80 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE HQRIDS(NA,N,ILW,IGH,H,WR,WI,Z,IERR)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(NA,N),WR(N),WI(N),Z(NA,N)
C**   ============================================================    **
C**   Upper-Hessenberg ==> Quasi-Upper-Triangular Transformed by      **
C**   QR algorithm with Implicit Double Shifts                        **
C**   Also update the transformation matrix Z(N,N)                    **
C**   ------------------------------------------------------------    **
C*I   H(N,N) = Upper Hessenberg matrix       : [H] = [P]'[B][P]       **
C*O          = Quasi upper triangular matrix : [T] = [V]'[B][V]       **
C*O   WR(N)  = Real part of eigenvalues                               **
C*O   WI(N)  = Imag part of eigenvalues                               **
C*I   Z(N,N) = Transformation matrix : [P] for [P]'[B][P]=[H]         **
C*O          = Transformation matrix : [V] for [V]'[B][V]=[T]         **
C*O   IERR   = J: J-th eigenvalue no converge; = 0: normal return     **
C**   ============================================================    **
      IERR=0
C**   +----------------------------------------------------------+    **
C**   | Get eigenvalues outside (ILW:IGH)                        |    **
C**   +----------------------------------------------------------+    **
      DO 20 J=1,N
      IF(J.LT.ILW.OR.J.GT.IGH) THEN
        WR(J)=H(J,J)
        WI(J)=0.0
      ENDIF
   20 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find H(K-1,K)=0 for isolating H(K:L,K:L)                 |    **
C**   +----------------------------------------------------------+    **
C**   | D  H  H  H  H  H  H  H | 1                               |    **
C**   | H  D  H  H  H  H  H  H |    This program transforms      |    **
C**   |    0  D  H  H  H  H  H | K                               |    **
C**   |       H  D  H  H  H  H |    Upper-Hessenber-matrix       |    **
C**   |          H  D  H  H  H |                                 |    **
C**   |             H  D  H  H | L  ----< to >-----------------  |    **
C**   |                0  D  H |                                 |    **
C**   |                   X  D | N  Quasi-upper-triangular-matrix|    **
C**   +----------------------------------------------------------+    **
C**   DO 280 L=IGH,ILW,-1/-2
      L=IGH
  100 DO 270 ITER=1,30
      DO 120 K=L,ILW+1,-1
      DD=DABS(H(K-1,K-1))+DABS(H(K,K))
      IF(DD+DABS(H(K,K-1)).EQ.DD) GO TO 130
  120 CONTINUE
      K=ILW
  130 IF(K.EQ.L) THEN
        WR(K)=H(K,K)                   ! One real root
        WI(K)=0.0
        L=L-1
        GO TO 280
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | | H(L-1,L-1)-X H(L-1,L) | = X*X-2*SM*X+SP = (X-SM)**2-V  |    **
C**   | |  H(L,L-1)    H(L,L)-X |   2*SM = X1+X2  ;  SP = X1*X2  |    **
C**   |                           V =-(X1-SM)*(X2-SM) = SM*SM-SP |    **
C**   +----------------------------------------------------------+    **
      W =(H(L-1,L-1)-H(L,L))*0.5              ! W =(D-G)/2
      SM=W+H(L,L)                             ! SM=(D+G)/2
      SP=H(L-1,L-1)*H(L,L)-H(L,L-1)*H(L-1,L)  ! SP=D*G-E*F
      V=W*W+H(L,L-1)*H(L-1,L)                 ! V=W*W+E*F=SM*SM-SP
      IF(K.EQ.L-1) THEN
        IF(V.LT.0.0) THEN
          WR(K)=SM                    ! Two complex roots
          WR(L)=SM
          WI(K)=DSQRT(-V)
          WI(L)=-WI(K)
          L=L-2
          GO TO 280
        ELSE
          WR(K)=SM+DSIGN(DSQRT(V),W)  ! Two real roots
          WR(L)=SP/WR(K)
          WI(K)=0.0
          WI(L)=0.0
        ENDIF
C**     +----------------------------------------------------------+    **
C**     | Perform Jacobi transformation for H(L,K) = E ==> 0       |    **
C**     |    | C  S || D  F || C -S | ==> | DD  FF |               |    **
C**     |    |-S  C || E  G || S  C |     |  0  GG |               |    **
C**     |       E*C*C - (D-G)*C*S - F*S*S = 0                      |    **
C**     |    ==>  C/S = ((D-G)+SQRT((D-G)**2+4*E*F)/(2*E)          |    **
C**     +----------------------------------------------------------+    **
        C=W+DSIGN(DSQRT(V),W)         ! Transform for two real roots
        S=H(L,K)
        T=DABS(C)+DABS(S)
        C=C/T
        S=S/T
        T=DSQRT(C*C+S*S)
        C=C/T
        S=S/T
        DO 160 J=K,N
        T=H(K,J)
        H(K,J)= C*T+S*H(L,J)
        H(L,J)=-S*T+C*H(L,J)
  160   CONTINUE
        DO 170 J=1,L
        T=H(J,K)
        H(J,K)= T*C+H(J,L)*S
        H(J,L)=-T*S+H(J,L)*C
  170   CONTINUE
        DO 175 J=ILW,IGH
        T=Z(J,K)
        Z(J,K)= T*C+Z(J,L)*S
        Z(J,L)=-T*S+Z(J,L)*C
  175   CONTINUE
        L=L-2
        GO TO 280
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Double shift QR will add 2 more sub-diagonals            |    **
C**   +----------------------------------------------------------+    **
      DO 190 J=K,L-2
      DO 190 I=J+2,MIN0(J+3,L)
  190 H(I,J)=0.0
C**   +----------------------------------------------------------+    **
C**   | {P,Q,R}*H(K+1,K) = 1st col of [H]**2-2*SM*[H]+SP*[I]     |    **
C**   +----------------------------------------------------------+    **
      P=((H(K,K)-SM)**2-V)/H(K+1,K)+H(K,K+1)  ! V=SM*SM-SP
      Q=(H(K,K)-SM)+(H(K+1,K+1)-SM)           ! SM=(X1+X2)/2
      R=H(K+2,K+1)                            ! SP=X1*X2
C**   ------------------------------------------------------------    **
      DO 260 I=K,L-1
      T=DABS(P)+DABS(Q)+DABS(R)
      IF(T.NE.0.0) THEN
        P=P/T
        Q=Q/T
        R=R/T
C**     +--------------------------------------------------------+    **
C**     | Store new sub-diagonal element in H(I,I-1)             |    **
C**     +--------------------------------------------------------+    **
        S=DSIGN(DSQRT(P*P+Q*Q+R*R),P)
        IF(I.GT.K) H(I,I-1)=-S*T
C**     +--------------------------------------------------------+    **
C**     | Comput  {u}={P+S,Q,R}/S  ;  <v>=<P+S,Q,R>/(P+S)        |    **
C**     +--------------------------------------------------------+    **
        P=P+S
        PS=P/S
        QS=Q/S
        RS=R/S
        QP=Q/P
        RP=R/P
        IF(I.LT.L-1) THEN
C**       +------------------------------------------------------+    **
C**       | Compute  [H] = [[I] - 2{w}<w>][H] = [H] - {u}<v>[A]  |    **
C**       +------------------------------------------------------+    **
          DO 210 J=I,N
          T=H(I,J)+QP*H(I+1,J)+RP*H(I+2,J)
          H(I  ,J)=H(I  ,J)-PS*T
          H(I+1,J)=H(I+1,J)-QS*T
          H(I+2,J)=H(I+2,J)-RS*T
  210     CONTINUE
C**       +------------------------------------------------------+    **
C**       | Compute  [H] = [H][[I] - 2{w}<w>] = [H] - [H]{u}<v>  |    **
C**       +------------------------------------------------------+    **
          DO 220 J=1,MIN0(I+3,L)
          T=H(J,I)*PS+H(J,I+1)*QS+H(J,I+2)*RS
          H(J,I  )=H(J,I  )-T
          H(J,I+1)=H(J,I+1)-T*QP
          H(J,I+2)=H(J,I+2)-T*RP
  220     CONTINUE
C**       +------------------------------------------------------+    **
C**       | Compute  [Z] = [Z][[I] - 2{w}<w>] = [Z] - [Z]{u}<v>  |    **
C**       +------------------------------------------------------+    **
          DO 225 J=ILW,IGH
          T=Z(J,I)*PS+Z(J,I+1)*QS+Z(J,I+2)*RS
          Z(J,I  )=Z(J,I  )-T
          Z(J,I+1)=Z(J,I+1)-T*QP
          Z(J,I+2)=Z(J,I+2)-T*RP
  225     CONTINUE
        ELSE
          DO 230 J=I,N
          T=H(I,J)+QP*H(I+1,J)
          H(I  ,J)=H(I  ,J)-PS*T
          H(I+1,J)=H(I+1,J)-QS*T
  230     CONTINUE
          DO 240 J=1,MIN0(I+3,L)
          T=H(J,I)*PS+H(J,I+1)*QS
          H(J,I  )=H(J,I  )-T
          H(J,I+1)=H(J,I+1)-T*QP
  240     CONTINUE
          DO 245 J=ILW,IGH
          T=Z(J,I)*PS+Z(J,I+1)*QS
          Z(J,I  )=Z(J,I  )-T
          Z(J,I+1)=Z(J,I+1)-T*QP
  245     CONTINUE
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Get {P,Q,R} for next transformation                      |    **
C**   +----------------------------------------------------------+    **
C**   IF(I.EQ.L-1) GO TO 260
      P=H(I+1,I)
      Q=H(I+2,I)
      R=0.0
      IF(I+3.LE.L) R=H(I+3,I)
  260 CONTINUE
  270 CONTINUE
      WRITE(*,'('' NOT CONVERGE IN 30 ITERATIONS !'')')
      IERR=L
      RETURN
  280 IF(L.GE.ILW) GO TO 100
      RETURN
      END
************************************************************************
      SUBROUTINE QUTEVE(NA,N,ILW,IGH,H,WR,WI,Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(NA,N),WR(N),WI(N),Z(NA,N)
      DATA EPSMAH/1.0D-15/
C**   ============================================================    **
C**   Find Eigenvectors of Quasi-Upper-Triangular matrix by           **
C**   Solving the singular matrix system equations                    **
C**   ------------------------------------------------------------    **
C*I   H(N,N) = Quasi upper triangular matrix : [T] = [V]'[B][V]       **
C*O          = Transformation matrix : [U] for [U]^{-1}[T][U]=[S]     **
C*I   WR(N)  = Real part of eigenvalues                               **
C*I   WI(N)  = Imag part of eigenvalues                               **
C*I   Z(N,N) = [V] : Transformation matrix for     [V]'[B][V]=[T]     **
C*O          = [Z] = [V][U] : Tran. matrix for [Z]^{-1}[B][Z]=[S]     **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Compute norm HNORM                                       |    **
C**   +----------------------------------------------------------+    **
      HNORM=0.0
      DO 20 J=1,N
      DO 20 I=1,MIN0(J+1,N)
   20 HNORM=HNORM+DABS(H(I,J))
C**   +----------------------------------------------------------+    **
C**   | Get  WR(N),WI(N) : Eigen values                          |    **
C**   +----------------------------------------------------------+    **
      DO 500 L=N,1,-1
      XR=WR(L)
      XI=WI(L)
      IF(XI.EQ.0.0) THEN
C**     +--------------------------------------------------------+    **
C**     | Eigenvector of real eigenvalue                         |    **
C**     +--------------------------------------------------------+    **
        H(L,L)=1.0
        DO 300 I=L-1,1,-1
        W=H(I,I)-XR
        R=0.0
        I1=I+1
        IF(WI(I).GT.0.0) I1=I+2
        DO 250 J=I1,L
        R=R-H(I,J)*H(J,L)
  250   CONTINUE
        IF(WI(I).EQ.0.0) THEN
          IF(W.EQ.0.0) W=HNORM/EPSMAH
          H(I,L)=R/W
        ELSE IF(WI(I).LT.0.0) THEN
          W1=W
          R1=R
        ELSE
C**       +------------------------------------------------------+    **
C**       | |    W     H(I,I+1) || H(I  ,L) | = | R  |           |    **
C**       | | H(I+1,I)    W1    || H(I+1,L) |   | R1 |           |    **
C**       +------------------------------------------------------+    **
          Q=(WR(I)-XR)**2+WI(I)**2      ! Q=W*W1-H(I,I+1)*H(I+1,I)
          H(I  ,L)=(W1*R-H(I,I+1)*R1)/Q
          H(I+1,L)=(W*R1-H(I+1,I)*R)/Q
        ENDIF
  300   CONTINUE
      ELSE IF(XI.LT.0.0) THEN
C**     +--------------------------------------------------------+    **
C**     | Eigenvectors of complex eigenvalues                    |    **
C**     +--------------------------------------------------------+    **
C**     | | HKK-XR+XI*i     HKL      || UKK+UKL*i | = | 0 |      |    **
C**     | |    HLK       HLL-XR+XI*i || ULK+ULL*i |   | 0 |      |    **
C**     |--------------------------------------------------------|    **
C**     | | 1  0 || UKK+UKL*i | = | -((HLL-XR)+XI*i)*i/HLK |     |    **
C**     | | 0  1 || ULK+ULL*i |   |              0+1*i     |     |    **
C**     |--------------------------------------------------------|    **
C**     | | 1  0 || UKK+UKL*i | = | -HKL*i/((HKK-XR)+XI*i) |     |    **
C**     | | 0  1 || ULK+ULL*i |   |  0+1*i                 |     |    **
C**     +--------------------------------------------------------+    **
        K=L-1
        IF(DABS(H(L,K)).GT.DABS(H(K,L))) THEN
          H(K,K)=XI/H(L,K)
          H(K,L)=-(H(L,L)-XR)/H(L,K)
        ELSE
          CALL CDIV(0.0D0,-H(K,L),H(K,K)-XR,XI,H(K,K),H(K,L))
        ENDIF
        H(L,K)=0.0
        H(L,L)=1.0
C**     ----------------------------------------------------------    **
        DO 400 I=L-2,1,-1
        W=H(I,I)-XR
        S=0.0
        R=0.0
        I1=I+1
        IF(WI(I).GT.0.0) I1=I+2
        DO 350 J=I1,L
        S=S-H(I,J)*H(J,K)
        R=R-H(I,J)*H(J,L)
  350   CONTINUE
        IF(WI(I).EQ.0.0) THEN
          CALL CDIV(S,R,W,XI,H(I,K),H(I,L))
        ELSE IF(WI(I).LT.0.0) THEN
          W1=W
          S1=S
          R1=R
        ELSE
C**       +------------------------------------------------------+    **
C**       | W+XI*i  H(I,I+1) || H(I  ,K)+H(I  ,L)i | = | S +R *i |    **
C**       | H(I+1,I) W1+XI*i || H(I+1,K)+H(I+1,L)i |   | S1+R1*i |    **
C**       +------------------------------------------------------+    **
          QR=(WR(I)-XR)**2+WI(I)**2-XI**2 ! W*W1-H(I,I+1)*H(I+1,I)-XI*XI
          QI=(WR(I)-XR)*XI*2.0            ! (W+W1)*XI
          CALL CDIV(W1*S-H(I,I+1)*S1-XI*R,W1*R-H(I,I+1)*R1+XI*S
     *                                  ,QR,QI,H(I,K),H(I,L))
          CALL CDIV(W*S1-H(I+1,I)*S-XI*R1,W*R1-H(I+1,I)*R+XI*S1
     *                                  ,QR,QI,H(I+1,K),H(I+1,L))
        ENDIF
  400   CONTINUE
      ENDIF
  500 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute [Z] = [V][U] = V(ILW:IGH,ILW:IGH)*U(ILW:IGH,I:N) |    **
C**   +----------------------------------------------------------+    **
      DO 520 I=N,1,-1
      IF(I.GE.ILW.AND.I.LE.IGH) GO TO 520
      DO 510 J=I,N
  510 Z(I,J)=H(I,J)
  520 CONTINUE
      DO 550 J=N,1,-1
      DO 550 I=ILW,IGH
      S=0.0
      DO 530 K=ILW,MIN0(J,IGH)
  530 S=S+Z(I,K)*H(K,J)
      Z(I,J)=S
  550 CONTINUE
      RETURN
      END
************************************************************************
