************************************************************************
      SUBROUTINE TRIDIG(NA,N,A,D,E,Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),D(N),E(N),Z(NA,N)
C**   ============================================================    **
C**   Simularity transform symmetric matrix to tridiagonal matrix     **
C**   By Householder method :    [P]'[A][P] ==> [T]                   **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = Symmetric matrix given upper triangular part           **
C*O          = [T] in upper triangular part                           **
C*O   D(K)   = T(K,K)   = Diagonal elements of tridiagonal matrix     **
C*O   E(K)   = T(K-1,K) = T(K,K-1) = Sub-diagonal elements of [T]     **
C*I   Z(N,N)  = [I] or [X] = Matrix to be post-multiplied by [P]      **
C*O           = [P] or [X][P]                                         **
C**   ============================================================    **
      DO 80 K=N,1,-1
      AMX=0.0
      DO 20 I=1,K-1
   20 AMX=DMAX1(AMX,DABS(A(I,K)))
C**   +----------------------------------------------------------+    **
C**   | In case of A(1:K-1,K)=0, set D(K) & E(K) (also for K=1)  |    **
C**   +----------------------------------------------------------+    **
      E(K)=0.0
      D(K)=A(K,K)
      IF(AMX.EQ.0.0) GO TO 80
C**   +----------------------------------------------------------+    **
C**   | Scale u-vector to avoid overflow, {u} = {w}sqrt(2) = {D} |    **
C**   +----------------------------------------------------------+    **
      S=0.0
      DO 25 I=1,K-1
      D(I)=A(I,K)/AMX
   25 S=S+D(I)*D(I)
      S=DSIGN(DSQRT(S),D(K-1))
      E(K)=-S*AMX
      D(K-1)=D(K-1)+S
      T=DSQRT(S*D(K-1))
      DO 30 I=1,K-1
   30 D(I)=D(I)/T
C**   +----------------------------------------------------------+    **
C**   | Compute {p} = [A]{u} = [A]{w}sqrt(2) = {E}               |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,K-1
      EJ=A(J,J)*D(J)
      DO 35 I=1,J-1
      E(I)=E(I)+A(I,J)*D(J)
   35 EJ  =EJ  +A(I,J)*D(I)
   40 E(J)=EJ
C**   +----------------------------------------------------------+    **
C**   | Compute {q} = {p}-k{u} ; k = <u>{p}/2 = <w>[A]{w} = WAW  |    **
C**   +----------------------------------------------------------+    **
      UAU=0.0
      DO 50 I=1,K-1
   50 UAU=UAU+E(I)*D(I)
      WAW=UAU*0.5
      DO 60 I=1,K-1
   60 E(I)=E(I)-WAW*D(I)
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [A] - {u}<q> - {q}<u>                      |    **
C**   +----------------------------------------------------------+    **
      DO 70 J=1,K-1
      DO 65 I=1,J
   65 A(I,J)=A(I,J)-D(I)*E(J)-E(I)*D(J)
   70 A(J,K)=0.0
      A(K-1,K)=E(K)
C**   +----------------------------------------------------------+    **
C**   | Compute  S(1:N) = [Z]{u}  from  D(1:K-1) = <u>           |    **
C**   +----------------------------------------------------------+    **
      DO 75 I=1,N
      S=0.0
      DO 72 J=1,K-1
   72 S=S+Z(I,J)*D(J)
C**   +----------------------------------------------------------+    **
C**   | Compute  [Z] = [Z][[I] - 2{w}<w>] = [Z] - [Z]{u}<u>      |    **
C**   +----------------------------------------------------------+    **
      DO 75 J=1,K-1
   75 Z(I,J)=Z(I,J)-S*D(J)
   80 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE TRIQLI(NA,N,D,E,Z,IERR)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION D(N),E(N),Z(NA,N)
      EQUIVALENCE (C0,C1),(S0,S1),(G0,G1),(P0,P1)
C**   ============================================================    **
C**   Simularity transform tridiagonal matrix to diagonal matrix      **
C**   By QL Implicit shift algorithm : [Q]'[T][Q] ==> [D]             **
C**   ------------------------------------------------------------    **
C*I   D(K)   = T(K,K) = Diagonal element of tridiagonal matrix [T]    **
C*O          = D(K,K) = Eigenvalues of [T]                            **
CWI   E(K)   = T(K-1,K) = T(K,K-1) =  Sub-diagonal elements of [T]    **
C*I   Z(N,N)  = [P] or [X] = Matrix to be post-multiplied by [Q]      **
C*O           = [P][Q] or [X][Q]                                      **
C*O   IERR   = J: J-th eigenvalue no converge; = 0: normal return     **
C**   ============================================================    **
      IERR=0
      IF(N.EQ.1) RETURN
      DO 10 I=1,N-1
   10 E(I)=E(I+1)
      E(N)=0.0
C**   ------------------------------------------------------------    **
      DO 80 K=1,N
      DO 70 ITER=1,30
      DO 20 L=K,N-1
      DD=DABS(D(L))+DABS(D(L+1))
      IF(DD+DABS(E(L)).EQ.DD) GO TO 30
   20 CONTINUE
      L=N
   30 IF(L.EQ.K) GO TO 80
C**   +----------------------------------------------------------+    **
C**   | | D-X  E  | = | -Y    E   |   Y=X-D ; Z=Y/E ; P=(F-D)/2E |    **
C**   | |  E  F-X |   |  E  F-D-Y |   Y*Y - (F-D)*Y - E*E = 0    |    **
C**   | -------------------------------------------------------- |    **
C**   | Z*Z - 2PZ - 1 = 0 ; (Y/E)**2 - 2((F-D)/2E)*(Y/E) - 1 = 0 |    **
C**   | Large.Z = P+sqrt(P*P+1) ; Small.Z = -1/Large.Z           |    **
C**   +----------------------------------------------------------+    **
      P=DABS(D(K+1)-D(K))/(E(K)+E(K))
      SHIFT=DMIN1(D(K),D(K+1))-E(K)/(P+DSIGN(DSQRT(P*P+1),P))
C**    ------------------------   -------------------------------     **
C**   |D(I-1) E(I-1)           | | D(I-1)  E(I-1)*C0 *S0         |    **
C**   |E(I-1)  D(I)  E0    F1  | |C0*E(I-1) D(I)-P0   G0         |    **
C**   |        E0    D1    G1  | |S0*E(I-1)    G0   F(I+1) G(I+1)|    **
C**   |        F1    G1  F(I+2)| |                  G(I+1) F(I+2)|    **
C**    -------< before >-------   ----------< after >------------     **
C**     C1,S1,P1,G1,D(I),E(I)  ==>   C0,S0,P0,G0,F(I+1),G(I+1)        **
C**     --------------------------------------------------------      **
C**   | F1=E(I)*S1;    R1=SQRT(F1**2+G1**2);    G(I+1)=R1        |    **
C**   | E0=E(I)*C1;    C0=G1/R1;   S0=F1/R1;                     |    **
C**   | P0=(D0-D1)*S0**2+E0*2*C0*S0      =S0*T0                  |    **
C**   | G0=(D0-D1)*C0*S0+E0*(C0**2-S0**2)=C0*T0-E0               |    **
C**   | D0=D(I);   D1=D(I+1)-P1;   F(I+1)=D(I+1)-P1+P0=D1+P0     |    **
C**     --------------------------------------------------------      **
C**   |    | C0 -S0 || D0  E0 || C0  S0 | ==> | D0-P0   G0  |    |    **
C**   |    | S0  C0 || E0  D1 ||-S0  C0 |     |  G0   D1+P0 |    |    **
C**   +----------------------------------------------------------+    **
      G1=D(L)-SHIFT ! D(L-2) E(L-2)               !
      C1=1.0        ! E(L-2) D(L-1) E(L-1) E(L-1) ! => Force to get
      S1=1.0        !        E(L-1)  D(L)    G1   ! S0:C0=E(L-1):G1
      P1=0.0        !        E(L-1)   G1     XX   !
C**   ------------------------------------------------------------    **
      DO 60 I=L-1,K,-1
      E0=E(I)*C1
      F1=E(I)*S1
      R1=DSQRT(F1*F1+G1*G1)
      E(I+1)=R1
      IF(R1.EQ.0.0) THEN
        D(I+1)=D(I+1)-P1
        E(L)=0.0
        GO TO 70
      ENDIF
      C0=G1/R1
      S0=F1/R1
      D1=D(I+1)-P1
      T0=(D(I)-D1)*S0+2*C0*E0
      P0=S0*T0
      G0=C0*T0-E0
      D(I+1)=D1+P0
C**   +----------------------------------------------------------+    **
C**   | Post multiply the transformation (rotation) matrix       |    **
C**   +----------------------------------------------------------+    **
      DO 50 J=1,N
      Z0=Z(J,I)
      Z(J,I  )=Z0*C0-Z(J,I+1)*S0
   50 Z(J,I+1)=Z0*S0+Z(J,I+1)*C0
   60 CONTINUE
      D(K)=D(K)-P1
      E(K)=G0
      E(L)=0.0
      IF(T0.EQ.0.0) GO TO 80
   70 CONTINUE
      WRITE(*,'('' NO CONVERGE IN 30 ITERATIONS !'')')
      IERR=K
      RETURN
   80 CONTINUE
      RETURN
      END
************************************************************************
