************************************************************************
      SUBROUTINE TRIDIS(NA,N,A,D,E,Z)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NA,N),D(N),E(N),Z(NA,N)
C**   ============================================================    **
C**   Simularity transform symmetric matrix to tridiagonal matrix     **
C**   By Householder method :    [P]'[A][P] ==> [T]                   **
C**   ------------------------------------------------------------    **
C*I   A(N,N) = Symmetric matrix given upper triangular part           **
C*O   D(K)   = T(K,K)   = Diagonal elements of tridiagonal matrix     **
C*O   E(K)   = T(K-1,K) = T(K,K-1) = Sub-diagonal elements of [T]     **
C*O   Z(N,N) = Transformation matrix : [P] for [P]'[A][P] = [T]       **
C**   ============================================================    **
      DO 10 J=1,N
      DO 10 I=1,J
   10 Z(I,J)=A(I,J) ! change to A(J,I) if given lower triangular part
C**   ------------------------------------------------------------    **
      DO 80 K=N,1,-1
      AMX=0.0
      DO 20 I=1,K-1
   20 AMX=DMAX1(AMX,DABS(Z(I,K)))
C**   +----------------------------------------------------------+    **
C**   | In case of A(1:K-1,K)=0, set D(K) & E(K) (also for K=1)  |    **
C**   +----------------------------------------------------------+    **
      E(K)=0.0
      D(K)=Z(K,K)
      Z(K,K)=1.0
      IF(AMX.EQ.0.0) GO TO 80
C**   +----------------------------------------------------------+    **
C**   | Scale u-vector to avoid overflow, {u} = {w}sqrt(2) = {D} |    **
C**   +----------------------------------------------------------+    **
      S=0.0
      DO 25 I=1,K-1
      D(I)=Z(I,K)/AMX
   25 S=S+D(I)*D(I)
      S=DSIGN(DSQRT(S),D(K-1))
      E(K)=-S*AMX
      D(K-1)=D(K-1)+S
      T=DSQRT(S*D(K-1))
      DO 30 I=1,K-1
   30 D(I)=D(I)/T
C**   +----------------------------------------------------------+    **
C**   | Compute {p} = [A]{u} = [A]{w}sqrt(2) = {E}               |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,K-1
      EJ=Z(J,J)*D(J)
      DO 35 I=1,J-1
      E(I)=E(I)+Z(I,J)*D(J)
   35 EJ  =EJ  +Z(I,J)*D(I)
   40 E(J)=EJ
C**   +----------------------------------------------------------+    **
C**   | Compute {q} = {p}-k{u} ; k = <u>{p}/2 = <w>[A]{w} = WAW  |    **
C**   +----------------------------------------------------------+    **
      UAU=0.0
      DO 50 I=1,K-1
   50 UAU=UAU+E(I)*D(I)
      WAW=UAU*0.5
      DO 60 I=1,K-1
   60 E(I)=E(I)-WAW*D(I)
C**   +----------------------------------------------------------+    **
C**   | Compute [A] = [A] - {u}<q> - {q}<u> ; set Z(*,K) = {u}   |    **
C**   +----------------------------------------------------------+    **
      DO 70 J=1,K-1
      DO 65 I=1,J
   65 Z(I,J)=Z(I,J)-D(I)*E(J)-E(I)*D(J)
   70 Z(J,K)=D(J)
   80 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute transformation matrix [Z] = [Zn]...[Z3][Z2][I]   |    **
C**   +----------------------------------------------------------+    **
      DO 90 K=2,N
C**   +----------------------------------------------------------+    **
C**   | Compute  S(1:K-1) = <u>[Z]  from  Z(*,K) = {u}           |    **
C**   +----------------------------------------------------------+    **
      DO 86 J=1,K-1
      S=0.0
      DO 82 I=1,K-1
   82 S=S+Z(I,K)*Z(I,J)
C**   +----------------------------------------------------------+    **
C**   | Compute  [Z] = [[I] - 2{w}<w>][Z] = [Z] - {u}<u>[Z]      |    **
C**   +----------------------------------------------------------+    **
      DO 85 I=1,K-1
   85 Z(I,J)=Z(I,J)-Z(I,K)*S
   86 Z(K,J)=0.0
      DO 88 I=1,K-1
   88 Z(I,K)=0.0
   90 CONTINUE
      RETURN
      END
************************************************************************
