************************************************************************
      SUBROUTINE AITINT(X,Y,N,DX,AY,L,XX,YY,NN,YE,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),DX(N),AY(N),L(N)
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Aitken's iteration formula                               |    **
C**   |                                                          |    **
C**   |                      1     | y(x|x3,2) x2-x |            |    **
C**   |    y(x|x4,3,2) = --------- |                |            |    **
C**   |                   x4 - x2  | y(x|x4,3) x4-x |            |    **
C**   |                                                          |    **
C**   |                  (x2-x)y(x|x4,3) - (x4-x)y(x|x3,2)       |    **
C**   |                = ---------------------------------       |    **
C**   |                               x2 - x4                    |    **
C**   | x1 y(x|x1)                                               |    **
C**   | x2 y(x|x2)    y(x|x2,1)                                  |    **
C**   | x3 y(x|x3)(3) y(x|x3,2)(2) y(x|x3,2,1)(1)                |    **
C**   | xM y(x|x4)(4) y(x|x4,3)(3) y(x|x4,3,2)(2) y(x|x4,3,2,1)(1)    **
C**   | x3      AY(3)   \    AY(I)   \      AY(1)   \            |    **
C**   | xM      AY(M)   ->   AY(I+1) ->     AY(I)   ->       AY(1)    **
C**   |                                                          |    **
C**   | 20 AY(I)=(AY(I+1)*DX(I)-AY(I)*DX(M))/(DX(I)-DX(M))       |    **
C**   | For extrapolation from |DX(1)| > |DX(2)| > ... to 0      |    **
C**   | 20 AY(I)=AY(I+1)+(AY(I+1)-AY(I))*DX(M)/(DX(I)-DX(M))     |    **
C**   | For interpolation from |DX(1)| < |DX(2)| < |DX(3)| < ... |    **
C**   | 20 AY(I)=AY(I)+(AY(I+1)-AY(I))*DX(I)/(DX(I)-DX(M))       |    **
C**   +----------------------------------------------------------+    **
      DO 10 I=1,N
   10 DX(I)=DABS(X(I)-XX)
      CALL SORTAL(DX,L,N)
C**   ------------------------------------------------------------    **
      YY=0.0
      DO 50 M=1,N
      NN=M
      I=L(M)
      AY(M)=Y(I)
      DX(M)=X(I)-XX
      DO 20 I=M-1,1,-1
   20 AY(I)=AY(I)+(AY(I+1)-AY(I))*DX(I)/(DX(I)-DX(M))
C     WRITE(*,'(3X,1P6E13.5)') DX(M),(AY(I),I=M,1,-1)
      YE=AY(1)-YY
      YY=AY(1)
      IF(DABS(YE).LT.EPS*DABS(YY)) RETURN
   50 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE SORTAL(A,L,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),L(N)
C**   ============================================================    **
      DO 10 I=1,N
   10 L(I)=I
      M=N
   20 NE=M-1
      IF(NE) 60,60,30
   30 DO 50 I=1,NE
      J=L(I)
      K=L(I+1)
      IF(A(J)-A(K)) 50,50,40
   40 L(I)=K
      L(I+1)=J
      M=I
   50 CONTINUE
      IF(M-NE) 20,20,60
   60 RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR AITKEN'S INTERPOLATION                         **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20),DX(20),DY(20),L(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      WRITE(*,'(9X,''XX'',11X,''YY'',11X,''YE'',8X,''NN'')')
   30 READ(*,'(2F10.0)') XX,EPS
      IF(EPS.EQ.0.0) GO TO 10
      CALL AITINT(X,Y,N,DX,DY,L,XX,YY,NN,YE,EPS)
      WRITE(*,'(3X,1P3E13.5,I5)') XX,YY,YE,NN
      GO TO 30
      END
************************************************************************
