************************************************************************
      SUBROUTINE DIFINT(X,Y,N,DX,DY,L,XX,YY,NN,YE,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),DX(N),DY(N),L(N)
C**   ============================================================    **
C**   Newton's interpolation formula                                  **
C**                                                                   **
C**   F(x) = f(x1) + (x-x1)f[x1,2] + (x-x1)(x-x2)f[x1,2,3] + ...      **
C**   or (for example, N=4) :                                         **
C**   F(x) = f(x4) + (x-x4)f[x4,3] + (x-x4)(x-x3)f[x4,3,2] + ...      **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   |           f(4)-f(3)              f[x4,3]-f[x3,2]         |    **
C**   | f[x4,3] = ---------; f[x4,3,2] = ---------------         |    **
C**   |            x4 - x3                   x4 - x2             |    **
C**   |                                                          |    **
C**   |                    f[x4,3,2]-f[x3,2,1]                   |    **
C**   |      p[x4,3,2,1] = -------------------                   |    **
C**   |                          x4 - x1                         |    **
C**   |                                                          |    **
C**   | x1 f(x1)                                                 |    **
C**   | x2 f(x2)    f[x2,1]                                      |    **
C**   | x3 f(x3)(3) f[x3,2](2) f[x3,2,1](1)                      |    **
C**   | xM f(x4)(4) f[x4,3](3) f[x4,3,2](2) f[x4,3,2,1](1)       |    **
C**   | x3    DY(3)   \  DY(I)   \    DY(1)   \                  |    **
C**   | xM    DY(M)   -> DY(I+1) ->   DY(I)   ->     DY(1)       |    **
C**   +----------------------------------------------------------+    **
      DO 10 I=1,N
   10 DX(I)=DABS(XX-X(I))
      CALL SORTAL(DX,L,N)
C**   ------------------------------------------------------------    **
      YY=0.0
      PP=1.0
      DO 50 M=1,N
      NN=M
      I=L(M)
      DY(M)=Y(I)
      DX(M)=XX-X(I)
      DO 20 I=M-1,1,-1
      DY(I)=(DY(I+1)-DY(I))/(DX(I)-DX(M))
   20 CONTINUE
C     WRITE(*,'(3X,1P6E13.5)') DX(M),(DY(I),I=M,1,-1)
C**   +----------------------------------------------------------+    **
C**   | YY = f(x1) + (x-x1)f[x1,2] + (x-x1)(x-x2)f[x1,2,3] + ... |    **
C**   | PP = (x-x1)(x-x2)(x-x3)...                               |    **
C**   +----------------------------------------------------------+    **
      YE=PP*DY(1)
      YY=YY+YE
      PP=PP*DX(M)
      IF(DABS(YE).LT.EPS*DABS(YY)) RETURN
   50 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE SORTAL(A,L,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),L(N)
C**   ============================================================    **
      DO 10 I=1,N
   10 L(I)=I
      M=N
   20 NE=M-1
      IF(NE) 60,60,30
   30 DO 50 I=1,NE
      J=L(I)
      K=L(I+1)
      IF(A(J)-A(K)) 50,50,40
   40 L(I)=K
      L(I+1)=J
      M=I
   50 CONTINUE
      IF(M-NE) 20,20,60
   60 RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR NEWTON'S INTERPOLATION                         **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20),DX(20),DY(20),L(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      WRITE(*,'(9X,''XX'',11X,''YY'',11X,''YE'',8X,''NN'')')
   30 READ(*,'(2F10.0)') XX,EPS
      IF(EPS.EQ.0.0) GO TO 10
      CALL DIFINT(X,Y,N,DX,DY,L,XX,YY,NN,YE,EPS)
      WRITE(*,'(3X,1P3E13.5,I5)') XX,YY,YE,NN
      GO TO 30
      END
************************************************************************
