************************************************************************
      SUBROUTINE LAGRAG(X,Y,N,XX,YY,M)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N)
C**   ============================================================    **
C**   Lagrange's interpolation formula
C**
C**   F(x) = \sum_{k=IA}^IB \prod_{i=IA}^IB (X-X(i))/(X(k)-X(i))
C**                                i!=k
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | X(I) must be in ascending or descending order            |    **
C**   | If M >= N or M <= 1 then use all data : IA=1, IB=N       |    **
C**   +----------------------------------------------------------+    **
      MM=M/2
      IA=1
      IB=N
      IF(M.GE.N.OR.M.LE.1) GO TO 30
C**   +----------------------------------------------------------+    **
C**   | Find IA and IB such that: 1 <= IA and IB=IA+M-1 <= N and |    **
C**   | #{X(IA:IB) < XX} - #{X(IA:IB) > XX} as small as possible |    **
C**   +----------------------------------------------------------+    **
      PP=DSIGN(1.0D0,X(N)-X(1))
      DO 10 I=1,N
      IF(XX*PP.LE.X(I)*PP) GO TO 20
   10 CONTINUE
      I=N
   20 IA=MAX0(I-MM,1)
      IB=MIN0(IA+M-1,N)
      IA=IB-M+1
C**   ------------------------------------------------------------    **
   30 YY=0.0
      DO 60 K=IA,IB
      PP=Y(K)
      DO 50 I=IA,IB
      IF(I.NE.K) PP=PP*(XX-X(I))/(X(K)-X(I))
   50 CONTINUE
   60 YY=YY+PP
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR LAGRANGE'S INTERPOLATION                       **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      WRITE(*,'(9X,''XX'',11X,''YY'',9X,''M'')')
   30 READ(*,'(F10.0,I5)') XX,M
      IF(M.EQ.0.0) GO TO 10
      CALL LAGRAG(X,Y,N,XX,YY,M)
      WRITE(*,'(3X,1P2E13.5,I5)') XX,YY,M
      GO TO 30
      END
************************************************************************
