************************************************************************
      SUBROUTINE NEWTON(X,Y,N,C,P,F)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),C(N),P(N),F(N)
C**   ============================================================    **
C**   Newton's interpolation formula                                  **
C**                                                                   **
C**   F(X(i)) = Y(i), for i=1,2,...,N                                 **
C**                                                                   **
C**   F(X) = C(1)*X**(N-1) + C(2)*X**(N-2) + ... + C(N-1)*X + C(N)    **
C**                                                                   **
C**   F(X) = Y(1)+(X-X(1))F[X1,X2]+(X-X(1))(X-X(2))F[X1,X2,X3]+...    **
C**   or:                                                             **
C**   F(X) = Y(4)+(X-X(4))F[X4,X3]+(X-X(4))(X-X(3))F[X4,X3,X2]+...    **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   |           f(4)-f(3)              f[x4,3]-f[x3,2]         |    **
C**   | f[x4,3] = ---------; f[x4,3,2] = ---------------         |    **
C**   |            x4 - x3                   x4 - x2             |    **
C**   |                                                          |    **
C**   |                    f[x4,3,2]-f[x3,2,1]                   |    **
C**   |      p[x4,3,2,1] = -------------------                   |    **
C**   |                          x4 - x1                         |    **
C**   |                                                          |    **
C**   | x1 f(x1)                                                 |    **
C**   | x2 f(x2)    f[x2,1]                                      |    **
C**   | x3 f(x3)(3) f[x3,2](2) f[x3,2,1](1)                      |    **
C**   | xM f(x4)(4) f[x4,3](3) f[x4,3,2](2) f[x4,3,2,1](1)       |    **
C**   | x3     F(3)   \   F(I)   \     F(1)   \                  |    **
C**   | xM     F(M)   ->  F(I+1) ->    F(I)   ->      F(1)       |    **
C**   +----------------------------------------------------------+    **
      PM=1.0
      DO 50 M=1,N
      F(M)=Y(M)
      DO 20 I=M-1,1,-1
      F(I)=(F(I+1)-F(I))/(X(M)-X(I))
   20 CONTINUE
C     WRITE(*,'(3X,1P6E13.5)') X(M),(F(I),I=M,1,-1)
C**   +----------------------------------------------------------+    **
C**   | F = C(1)*X**(N-1) + C(2)*X**(N-2) + ... + C(N-1)*X + C(N)|    **
C**   |   = Y(1)+(X-X(1))F[X1,X2]+(X-X(1))(X-X(2))F[X1,X2,X3]+...|    **
C**   |                                                          |    **
C**   | P = P(1)*X**(M-1) + P(2)*X**(M-2) + ... + P(M-1)*X + P(M)|    **
C**   |   = (X-X(1))(X-X(2))...(X-X(M-1))                        |    **
C**   +----------------------------------------------------------+    **
      P(M)=PM
      DO 40 I=M-1,1,-1
      C(I+1)=C(I)+P(I+1)*F(1)
      P(I+1)=P(I+1)-P(I)*X(M)
   40 CONTINUE
      C(1)=F(1)
      PM=-PM*X(M)
   50 CONTINUE
      RETURN
      END
************************************************************************
      FUNCTION POLFUN(C,N,X)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION C(N)
C**   ============================================================    **
C**   F(X) = C(1)*X**(N-1) + C(2)*X**(N-2) + ... + C(N-1)*X + C(N)    **
C**   ============================================================    **
      POLFUN=0.0
      DO 20 I=1,N
      POLFUN=POLFUN*X+C(I)
   20 CONTINUE
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR NEWTON'S INTERPOLATION FORMULA                 **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20),C(20),P(20),F(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      CALL NEWTON(X,Y,N,C,P,F)
      WRITE(*,'(10X,''C'',12X,''P'',12X,''X'',12X,''Y'',12X,''F'')')
      WRITE(*,'(I3,1P5E13.5)') (I,C(I),P(I),X(I),Y(I),F(I),I=1,N)
      DO 50 I=1,N
      YY=POLFUN(C,N,X(I))
      WRITE(*,'(I3,1P2E13.5)') I,X(I),YY
   50 CONTINUE
      GO TO 10
      END
************************************************************************
