************************************************************************
      SUBROUTINE SPLINE(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),X(N),Y(N),Z(N),A(1),L(N)
C**   ============================================================    **
CIO   H(I) = Parameter/Length between nodes I & J=I+1 or I=N & J=1    **
C*I   X(I),Y(I),Z(I) = Coordinates of the I-th node                   **
C*I   N    = Number of nodes                                          **
C*I   KB   = Index for boundary conditions at both ends               **
C**        = 0 : left_y'' given, right_y'' given                      **
C**        = 1 : left_y'' given, right_y'  given                      **
C**        = 2 : left_y'  given, right_y'' given                      **
C**        = 3 : left_y'  given, right_y'  given                      **
C**        = 4 : left_y' = right_y' for:                              **
C**              2-D and 3-D closed curve                             **
C**              1-D periodic curve : use KH = 1 or -1                **
C*I   KH   = Index for H(I) values:                                   **
C**        = 0 : set H(I) = Length between nodes I & J                **
C**        = 1 : set H(I) = 1                                         **
C**        = otherwise : H(I) are given                               **
C**          In any case, this subroutine set H(N)=0 for KB<4         **
C*I   M    = Space dimension of the curve                             **
C**        = 1 : For 1-D curve y(x)                                   **
C**        = 2 : For 2-D curve (x(t),y(t))                            **
C**        = 3 : For 3-D curve (x(t),y(t),z(t))                       **
C*O   A(*) = A(2*N-2) for KB<4,  A(3*N-4) for KB=4.                   **
C*O   L(N) = Location index for variable banded matrix A              **
C**          The coefficient matrix of the spline equation is         **
C**          (Data in [] are for KB=4 only)                           **
C**       / (Hn+H1)/3  H1/6                               [Hn/6] \    **
C**       |       (H1+H2)/3  H2/6                         [  0 ] |    **
C**       |             (H2+H3)/3  H3/6                   [  0 ] |    **
C**   A = |                        ....                   [  0 ] |    **
C**       |                            ....    Hn-2/6     [  0 ] |    **
C**       |                                (Hn-2+Hn-1)/3  Hn-1/6 |    **
C**       \                                          (Hn-1+Hn)/3 /    **
C**   ============================================================    **
      IF(KH.EQ.0) THEN
        DO 10 I=1,N
        J=I+1
        IF(I.EQ.N) J=1
        IF(M.EQ.1) H(I)=(X(J)-X(I))**2
        IF(M.EQ.2) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2
        IF(M.EQ.3) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2+(Z(J)-Z(I))**2
        H(I)=DSQRT(H(I))
   10   CONTINUE
      ELSE IF(KH.EQ.1) THEN
        DO 20 I=1,N
        H(I)=1.0
   20   CONTINUE
      ENDIF
      IF(KB.LT.4) H(N)=0.0
C**   ------------------------------------------------------------    **
      L(1)=0
      A(1)=(H(N)+H(1))/3.0
      DO 40 I=1,N-1
      A(2*I  )=H(I)/6.0
      A(2*I+1)=(H(I)+H(I+1))/3.0
      L(I+1)=L(I)+1
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | For y'' given at  left end (KB=00xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.1) A(1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For y'' given at right end (KB=0x0B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.2) A(2*N-1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) THEN
        L(N)=L(N)+N-2
        A(3*N-3)=A(2*N-1)
        A(3*N-4)=A(2*N-2)
        A(2*N-2)=H(N)/6.0
        DO 55 I=2*N-1,3*N-5
   55   A(I)=0.0
      ENDIF
      CALL VBDECP(A,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINF(H,Y,YPP,N,KB,YPP1,YPPN,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),Y(N),YPP(N),A(1),L(N)
C**   ============================================================    **
C*I   H,Y,N,KB,A,L = Should be the same as SPLINE                     **
C*O   YPP(I) = y'' at the I-th node                                   **
C*I   YPP1   = y''  y''  y'   y'   -  at the  left end                **
C*I   YPPN   = y''  y'   y''  y'   -  at the right end                **
C**   for KB = 0    1    2    3    4                                  **
C**                                                                   **
C**                 / (y2-y1)/H1 - (y1-yn)/Hn     \                   **
C**                 | (y3-y2)/H2 - (y2-y1)/H1     |                   **
C**             -1  | (y4-y3)/H3 - (y3-y2)/H2     |                   **
C**   {YPP} = [A] * |          .....              |                   **
C**                 |          .....              |                   **
C**                 |          .....              |                   **
C**                 \ (y1-yn)/Hn - (yn-yn-1)/Hn-1 /                   **
C**   ============================================================    **
      DO 60 I=2,N-1
      YPP(I)=(Y(I+1)-Y(I))/H(I)-(Y(I)-Y(I-1))/H(I-1)
   60 CONTINUE
      YPP(1)=(Y(2)-Y(1))/H(1)
      YPP(N)=-(Y(N)-Y(N-1))/H(N-1)
C**   +----------------------------------------------------------+    **
C**   | For y''= YPP1 at  left end (KB=00xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.1) YPP(1)=YPP1*A(1)
C**   +----------------------------------------------------------+    **
C**   | For y''= YPPN at right end (KB=0x0B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.2) YPP(N)=YPPN*A(2*N-1)
C**   +----------------------------------------------------------+    **
C**   | For y' = YPP1 at  left end (KB=01xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.2.OR.KB.EQ.3) YPP(1)=YPP(1)-YPP1
C**   +----------------------------------------------------------+    **
C**   | For y' = YPPN at right end (KB=0x1B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.1.OR.KB.EQ.3) YPP(N)=YPP(N)+YPPN
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) YPP(1)=YPP(1)-(Y(1)-Y(N))/H(N)
      IF(KB.GE.4) YPP(N)=YPP(N)+(Y(1)-Y(N))/H(N)
      CALL VBSOLX(A,YPP,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLING(H,Y,YPP,N,I,T,YT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),Y(N),YPP(N)
C**   ============================================================    **
C**   Find YT for given H,Y,YPP,N,I,T                                 **
C**   ============================================================    **
      J=I+1
      IF(I.EQ.N) J=1
      S=1-T
      YT=S*Y(I)+T*Y(J)+((S*S*S-S)*YPP(I)+(T*T*T-T)*YPP(J))*H(I)*H(I)/6
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR SPLINE CURVE FITTING                           **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(20),X(20),Y(20),Z(20),XPP(20),YPP(20),ZPP(20)
      DIMENSION A(60),L(20)
C**   ============================================================    **
C**   N  = Number of given nodes                                      **
C**   M  = 1:3 = Dimensions of the curve                              **
C**   KB = 0:4 see SUBROUTINE SPLINE                                  **
C**   KH = 1 : set H(I) = 1 by SUBROUTINE SPLINE                      **
C**      = 0 : set H(I) = Length between nodes I & J by SUB.SPLINE    **
C**   NS = Number of line segments for a curve between two nodes      **
C**   XPP1,XPPN,...  = See SUBROUTINE SPLINF                          **
C**   X(N),Y(N),Z(N) = Coordinates of nodes                           **
C**   ============================================================    **
   10 READ(*,'(5I5,6F5.0)') N,M,KB,KH,NS,XPP1,XPPN,YPP1,YPPN,ZPP1,ZPPN
      IF(M.LT.1.OR.M.GT.3) M=1
      IF(NS.EQ.0) NS=16
      KH=MAX0(KH,0)
      READ(*,'(3F10.0)') (X(I),Y(I),Z(I),I=1,N)
      WRITE(*,'(5I5,6F5.2)') N,M,KB,KH,NS,XPP1,XPPN,YPP1,YPPN,ZPP1,ZPPN
C**   ------------------------------------------------------------    **
      CALL SPLINE(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ------------------------------------------------------------    **
      IF(M.GE.1) CALL SPLINF(H,Y,YPP,N,KB,YPP1,YPPN,A,L)
      IF(M.GE.2) CALL SPLINF(H,X,XPP,N,KB,XPP1,XPPN,A,L)
      IF(M.GE.3) CALL SPLINF(H,Z,ZPP,N,KB,ZPP1,ZPPN,A,L)
      IF(M.GE.1) WRITE(*,'('' YP'',1P6E13.5)') (YPP(I),I=1,N)
      IF(M.GE.2) WRITE(*,'('' XP'',1P6E13.5)') (XPP(I),I=1,N)
      IF(M.GE.3) WRITE(*,'('' ZP'',1P6E13.5)') (ZPP(I),I=1,N)
C**   ------------------------------------------------------------    **
      WRITE(*,'(4X,''I    K  Sum.H(t)     X(t)      Y(t)      Z(t)'')')
      HO=0.0
      DO 50 I=1,N
      IF(H(I).EQ.0.0) GO TO 50
      DO 40 K=0,NS
      IF(K.EQ.0.AND.I.GT.1) GO TO 40
      T=FLOAT(K)/NS
      HT=HO+H(I)*T
      IF(M.GE.1) CALL SPLING(H,Y,YPP,N,I,T,YT)
      IF(M.GE.2) CALL SPLING(H,X,XPP,N,I,T,XT)
      IF(M.GE.3) CALL SPLING(H,Z,ZPP,N,I,T,ZT)
      IF(M.EQ.1) WRITE(*,'(2I5,4F10.5)') I,K,HT,HT,YT
      IF(M.EQ.2) WRITE(*,'(2I5,4F10.5)') I,K,HT,XT,YT
      IF(M.EQ.3) WRITE(*,'(2I5,4F10.5)') I,K,HT,XT,YT,ZT
      IF(K.EQ.0) THEN
        WRITE(1,'('' SP1;'')')
        WRITE(1,'('' LT;'')')
        WRITE(1,'('' PU'',2I6,'';'')') INT(XT*1000),INT(YT*1000)
      ELSE
        WRITE(1,'('' PD'',2I6,'';'')') INT(XT*1000),INT(YT*1000)
      ENDIF
   40 CONTINUE
      HO=HO+H(I)
   50 CONTINUE
      GO TO 10
      END
************************************************************************
