************************************************************************
      SUBROUTINE SPLINE(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),X(N),Y(N),Z(N),A(1),L(N)
C**   ============================================================    **
CIO   H(I) = Parameter/Length between nodes I & J=I+1 or I=N & J=1    **
C*I   X(I),Y(I),Z(I) = Coordinates of the I-th node                   **
C*I   N    = Number of nodes                                          **
C*I   KB   = Index for boundary conditions at both ends               **
C**        = 0 : left_y'' given, right_y'' given                      **
C**        = 1 : left_y'' given, right_y'  given                      **
C**        = 2 : left_y'  given, right_y'' given                      **
C**        = 3 : left_y'  given, right_y'  given                      **
C**        = 4 : left_y' = right_y' for:                              **
C**              2-D and 3-D closed curve                             **
C**              1-D periodic curve : use KH = 1 or -1                **
C*I   KH   = Index for H(I) values:                                   **
C**        = 0 : set H(I) = Length between nodes I & J                **
C**        = 1 : set H(I) = 1                                         **
C**        = otherwise : H(I) are given                               **
C**          In any case, this subroutine set H(N)=0 for KB<4         **
C*I   M    = Space dimension of the curve                             **
C**        = 1 : For 1-D curve y(x)                                   **
C**        = 2 : For 2-D curve (x(t),y(t))                            **
C**        = 3 : For 3-D curve (x(t),y(t),z(t))                       **
C*O   A(*) = A(2*N-2) for KB<4,  A(3*N-4) for KB=4.                   **
C*O   L(N) = Location index for variable banded matrix A              **
C**          The coefficient matrix of the spline equation is         **
C**          (Data in [] are for KB=4 only)                           **
C**       / (Hn+H1)/3  H1/6                               [Hn/6] \    **
C**       |       (H1+H2)/3  H2/6                         [  0 ] |    **
C**       |             (H2+H3)/3  H3/6                   [  0 ] |    **
C**   A = |                        ....                   [  0 ] |    **
C**       |                            ....    Hn-2/6     [  0 ] |    **
C**       |                                (Hn-2+Hn-1)/3  Hn-1/6 |    **
C**       \                                          (Hn-1+Hn)/3 /    **
C**   ============================================================    **
      IF(KH.EQ.0) THEN
        DO 10 I=1,N
        J=I+1
        IF(I.EQ.N) J=1
        IF(M.EQ.1) H(I)=(X(J)-X(I))**2
        IF(M.EQ.2) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2
        IF(M.EQ.3) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2+(Z(J)-Z(I))**2
        H(I)=DSQRT(H(I))
   10   CONTINUE
      ELSE IF(KH.EQ.1) THEN
        DO 20 I=1,N
        H(I)=1.0
   20   CONTINUE
      ENDIF
      IF(KB.LT.4) H(N)=0.0
C**   ------------------------------------------------------------    **
      L(1)=0
      A(1)=(H(N)+H(1))/3.0
      DO 40 I=1,N-1
      A(2*I  )=H(I)/6.0
      A(2*I+1)=(H(I)+H(I+1))/3.0
      L(I+1)=L(I)+1
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | For y'' given at  left end (KB=00xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.1) A(1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For y'' given at right end (KB=0x0B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.2) A(2*N-1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) THEN
        L(N)=L(N)+N-2
        A(3*N-3)=A(2*N-1)
        A(3*N-4)=A(2*N-2)
        A(2*N-2)=H(N)/6.0
        DO 55 I=2*N-1,3*N-5
   55   A(I)=0.0
      ENDIF
      CALL VBDECP(A,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINO(X,Y,DYX,N,KB,DYX1,DYXN,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),DYX(N),A(1),L(N)
C**   ============================================================    **
C*I   X,Y,N,KB,A,L = Should be the same as SPLINE                     **
C*O   DYX(I) = k at the I-th point (k=curvature, a=rotation angle)    **
C*I   DYX1   = k    k    a    a    -  at the  left end                **
C*I   DYXN   = k    a    k    a    -  at the right end                **
C**   for KB = 0    1    2    3    4                                  **
C**                                                                   **
C**                 / atan(y2-y1)/(x2-x1) - atan(y1-yn)/(x1-xn)  \    **
C**                 | atan(y3-y2)/(x3-x2) - atan(y2-y1)/(x2-x1)  |    **
C**             -1  | atan(y4-y3)/(x4-x3) - atan(y3-y2)/(x3-x2)  |    **
C**   {DYX} = [A] * |                   .....                    |    **
C**                 |                   .....                    |    **
C**                 |                   .....                    |    **
C**                 \ atan(y1-yn)/(x1-xn)-atan(yn-yn-1)/(xn-xn-1)/    **
C**   ============================================================    **
      DO 60 I=2,N-1
      DYX(I)=DATAN2(Y(I+1)-Y(I),X(I+1)-X(I))
     *      -DATAN2(Y(I)-Y(I-1),X(I)-X(I-1))
      IF(DABS(DYX(I)).GT.3.14159) DYX(I)=DYX(I)-DSIGN(6.28318D0,DYX(I))
   60 CONTINUE
      DYX(1)=DATAN2(Y(2)-Y(1),X(2)-X(1))
      DYX(N)=-DATAN2(Y(N)-Y(N-1),X(N)-X(N-1))
C**   +----------------------------------------------------------+    **
C**   | For k  = DYX1 at  left end (KB=00xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.1) DYX(1)=DYX1*A(1)
C**   +----------------------------------------------------------+    **
C**   | For k  = DYXN at right end (KB=0x0B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.2) DYX(N)=DYXN*A(2*N-1)
C**   +----------------------------------------------------------+    **
C**   | For a  = DYX1 at  left end (KB=01xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.2.OR.KB.EQ.3) DYX(1)=DYX(1)-DYX1
C**   +----------------------------------------------------------+    **
C**   | For a  = DYXN at right end (KB=0x1B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.1.OR.KB.EQ.3) DYX(N)=DYX(N)+DYXN
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) THEN
      DYX(1)=DYX(1)-DATAN2(Y(1)-Y(N),X(1)-X(N))
      DYX(N)=DYX(N)+DATAN2(Y(1)-Y(N),X(1)-X(N))
      IF(DABS(DYX(1)).GT.3.14159) DYX(1)=DYX(1)-DSIGN(6.28318D0,DYX(1))
      IF(DABS(DYX(N)).GT.3.14159) DYX(N)=DYX(N)-DSIGN(6.28318D0,DYX(N))
      ENDIF
      CALL VBSOLX(A,DYX,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLIN2(H,X,Y,DYX,N,I,NS,XT,YT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),X(N),Y(N),DYX(N)
      DIMENSION XT(1),YT(1),SA(129),SB(129)
C**   ============================================================    **
C**   Find XT(NS+1),YT(NS+1) for given H,X,Y,DYX,N,I,NS=2**K          **
C**   ============================================================    **
      J=I+1
      IF(I.EQ.N) J=1
      XT(   1)=X(I)
      YT(   1)=Y(I)
      XT(NS+1)=X(J)
      YT(NS+1)=Y(J)
      SA(   1)=-(2*DYX(I)+DYX(J))*H(I)/6
      SB(NS+1)=-(DYX(I)+2*DYX(J))*H(I)/6
      IF(NS.GT.128) WRITE(*,'(''NS > 128'')')
      IX=NS
   20 DO 50 IA=1,NS,IX
      IB=IA+IX
      IC=(IA+IB)/2
      CALL SPLINP(XT(IA),YT(IA),XT(IB),YT(IB),XT(IC),YT(IC)
     *           ,SA(IA),SB(IB),SA(IC),SB(IC))
   50 CONTINUE
      IX=IX/2
      IF(IX.GT.1) GO TO 20
      RETURN
      END
************************************************************************
      SUBROUTINE SPLIN3(H,X,Y,Z,DZY,DXZ,DYX,N,I,NS,XT,YT,ZT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),X(N),Y(N),Z(N),DZY(N),DXZ(N),DYX(N)
      DIMENSION XT(1),YT(1),ZT(1),CX(129),CY(129),CZ(129)
C**   ============================================================    **
C**   Find XT,YT,ZT(NS+1) for given H,X,Y,Z,DZY,DXZ,DYX,N,I,NS=2**K   **
C**   ============================================================    **
      J=I+1
      IF(I.EQ.N) J=1
      XT(   1)=X(I)
      YT(   1)=Y(I)
      ZT(   1)=Z(I)
      XT(NS+1)=X(J)
      YT(NS+1)=Y(J)
      ZT(NS+1)=Z(J)
      CX(   1)=-(2*DZY(I)+DZY(J))*H(I)/6
      CY(   1)=-(2*DXZ(I)+DXZ(J))*H(I)/6
      CZ(   1)=-(2*DYX(I)+DYX(J))*H(I)/6
      CX(NS+1)=-(DZY(I)+2*DZY(J))*H(I)/6
      CY(NS+1)=-(DXZ(I)+2*DXZ(J))*H(I)/6
      CZ(NS+1)=-(DYX(I)+2*DYX(J))*H(I)/6
      CALL ABCXYZ(XT(1),YT(1),ZT(1),XT(NS+1),YT(NS+1),ZT(NS+1)
     *           ,CX(1),CY(1),CZ(1),CX(NS+1),CY(NS+1),CZ(NS+1))
      IF(NS.GT.128) WRITE(*,'(''NS > 128'')')
      IX=NS
   20 DO 50 IA=1,NS,IX
      IB=IA+IX
      IC=(IA+IB)/2
      CALL SPLINQ
     *(XT(IA),YT(IA),ZT(IA),XT(IB),YT(IB),ZT(IB),XT(IC),YT(IC),ZT(IC)
     *,CX(IA),CY(IA),CZ(IA),CX(IB),CY(IB),CZ(IB),CX(IC),CY(IC),CZ(IC))
   50 CONTINUE
      IX=IX/2
      IF(IX.GT.1) GO TO 20
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINP(XA,YA,XB,YB,XC,YC,XYA,YXB,XYC,YXC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C**   Find XC,YC,XYA,YXB,XYC,YXC from XA,YA,XB,YB,XYA,YXB             **
C**   ------------------------------------------------------------    **
C*I   XA,YA = Coordinates of left  end point A                        **
C*I   XB,YB = Coordinates of right end point B                        **
C*O   XC,YC = Coordinates of middle point C                           **
C*I   XYA   = Angle from line AB to tangent of curve AB at A          **
C*I   YXB   = Angle from line BA to tangent of curve BA at B          **
C*O   XYA   = Angle from line AC to tangent of curve AC at A          **
C*O   YXC   = Angle from line CA to tangent of curve CA at C          **
C*O   XYC   = Angle from line CB to tangent of curve CB at C          **
C*O   YXB   = Angle from line BC to tangent of curve BC at B          **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | CXY = Angle from line AB to line AC or from BA to BC     |    **
C**   +----------------------------------------------------------+    **
      CXY=(YXB+XYA)*0.25
      SYX=(YXB-XYA)*0.25
      YXC=CXY-SYX
      XYC=CXY+SYX
      XYA=YXC-SYX
      YXB=XYC+SYX
      DXY=DSIN(CXY)/DCOS(CXY)
      XC=0.5*((XB+XA)-(YB-YA)*DXY)
      YC=0.5*((YB+YA)+(XB-XA)*DXY)
      END
************************************************************************
      SUBROUTINE SPLINQ(XA,YA,ZA,XB,YB,ZB,XC,YC,ZC
     *                 ,TXA,TYA,TZA,TXB,TYB,TZB,TXC,TYC,TZC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C**   Find (XC,YC,ZC),(TXC,TYC,TZC)                                   **
C**   From (XA,YA,ZA),(XB,YB,ZB),(TXA,TYA,TZA),(TXB,TYB,TZB)          **
C**   ------------------------------------------------------------    **
C*I   XA,YA,ZA    = Coordinates of left  end point A                  **
C*I   XB,YB,ZB    = Coordinates of right end point B                  **
C*O   XC,YC,ZC    = Coordinates of middle point C                     **
C*I   TXA,TYA,TZA = Unit vector of tangent of curve ACB at A          **
C*I   TXB,TYB,TZB = Unit vector of tangent of curve ACB at C          **
C*O   TXC,TYC,TZC = Unit vector of tangent of curve ACB at B          **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | TX,TY,TZ = Unit vector of line AB                        |    **
C**   +----------------------------------------------------------+    **
      TX=XB-XA
      TY=YB-YA
      TZ=ZB-ZA
      H=DSQRT(TX*TX+TY*TY+TZ*TZ)
      TX=TX/H
      TY=TY/H
      TZ=TZ/H
C**   ------------------------------------------------------------    **
      CALL VCROSS(TX,TY,TZ,TXA,TYA,TZA,AX,AY,AZ,AL)
      CALL VCROSS(TXB,TYB,TZB,TX,TY,TZ,BX,BY,BZ,BL)
      CALL VCROSS(BX,BY,BZ,AX,AY,AZ,VX,VY,VZ,VL)
      CA=TX*TXA+TY*TYA+TZ*TZA
      CB=TX*TXB+TY*TYB+TZ*TZB
      SA=DSQRT(1-CA*CA)
      SB=DSQRT(1-CB*CB)
      A=DATAN2(SA,CA)
      B=DATAN2(SB,CB)
C**   +----------------------------------------------------------+    **
C**   | CX,CY,CZ = Unit vector normal to plane ABC               |    **
C**   | DX,DY,DZ = Unit vector normal to line AB and vector C    |    **
C**   | CL = Angle from line AB to line AC                       |    **
C**   | DL = Distance from center of line AB to point C          |    **
C**   +----------------------------------------------------------+    **
      CX=AX*A+BX*B
      CY=AY*A+BY*B
      CZ=AZ*A+BZ*B
      CL=DSQRT(CX*CX+CY*CY+CZ*CZ)
      IF(CL.NE.0) THEN
      CX=CX/CL
      CY=CY/CL
      CZ=CZ/CL
      ENDIF
      CALL VCROSS(CX,CY,CZ,TX,TY,TZ,DX,DY,DZ,DL)
      DL=0.5*H*DTAN(0.25*CL)
      XC=0.5*(XA+XB)+DX*DL
      YC=0.5*(YA+YB)+DY*DL
      ZC=0.5*(ZA+ZB)+DZ*DL
C**   +----------------------------------------------------------+    **
C**   | PX,PY,PZ = Vector difference of vector A and vector B    |    **
C**   | PC/4 = -Rotation angle about (CX,CY,CZ) at point C       |    **
C**   | PD/4 =  Rotation angle about (DX,DY,DZ) at point C       |    **
C**   +----------------------------------------------------------+    **
      PX=AX*A-BX*B
      PY=AY*A-BY*B
      PZ=AZ*A-BZ*B
      PC=PX*CX+PY*CY+PZ*CZ
      PD=DSIGN(DSQRT(PX*PX+PY*PY+PZ*PZ-PC*PC),TX*VX+TY*VY+TZ*VZ)
      SC=DSIN(0.25*PC)
      CC=DSQRT(1-SC*SC)
      SD=DSIN(0.25*PD)
      CD=DSQRT(1-SD*SD)
      TXC=(TX*CD-CX*SD)*CC-DX*SC
      TYC=(TY*CD-CY*SD)*CC-DY*SC
      TZC=(TZ*CD-CZ*SD)*CC-DZ*SC
      RETURN
      END
************************************************************************
      SUBROUTINE VCROSS(AX,AY,AZ,BX,BY,BZ,CX,CY,CZ,CL)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C**   Cross product of two vectors:                                   **
C**   / Cx \     / Ax \   / Bx \     / 0  -Az  Ay \ / Bx \            **
C**   | Cy |  =  | Ay | x | By |  =  | Az  0  -Ax | | By |            **
C**   \ Cz /     \ Az /   \ Bz /     \-Ay  Ax  0  / \ Bz /            **
C**   And normalize to unit vector                                    **
C**   ============================================================    **
      CX=      -AZ*BY+AY*BZ
      CY= AZ*BX      -AX*BZ
      CZ=-AY*BX+AX*BY
      CL=DSQRT(CX*CX+CY*CY+CZ*CZ)
      IF(CL.EQ.0) RETURN
      CX=CX/CL
      CY=CY/CL
      CZ=CZ/CL
      RETURN
      END
************************************************************************
      SUBROUTINE ABCXYZ(XA,YA,ZA,XB,YB,ZB,AX,AY,AZ,BX,BY,BZ)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C*I   XA,YA,ZA = Coordinates of node A                                **
C*I   XB,YB,ZB = Coordinates of node B                                **
C*I   AX,AY,AZ = Angles from vector AB to tangent of curve AB at A    **
C*I   BX,BY,BZ = Angles from vector BA to tangent of curve BA at B    **
C*O   AX,AY,AZ = Unit vector of tangent of curve AB at A              **
C*O   BX,BY,BZ = Unit vector of tangent of curve AB at B              **
C**   ============================================================    **
      RX=DATAN2(ZB-ZA,YB-YA)
      RY=DATAN2(XB-XA,ZB-ZA)
      RZ=DATAN2(YB-YA,XB-XA)
C**   ------------------------------------------------------------    **
      RAX=RX+AX
      RAY=RY+AY
      RAZ=RZ+AZ
      IF(DABS(RAX).GT.3.14159) RAX=RAX-DSIGN(6.28318D0,RAX)
      IF(DABS(RAY).GT.3.14159) RAY=RAY-DSIGN(6.28318D0,RAY)
      IF(DABS(RAZ).GT.3.14159) RAZ=RAZ-DSIGN(6.28318D0,RAZ)
      PX=DTAN(RAX)
      PY=DTAN(RAY)
      PZ=DTAN(RAZ)
      PP=DABS(PX*PY*PZ)**(1.0D0/3)
      PX=(PX/PP)**2
      PY=(PY/PP)**2
      PZ=(PZ/PP)**2
      AX=DSIGN(1.0D0,RAY)/DSQRT(1+PZ+PZ*PX)
      AY=DSIGN(1.0D0,RAZ)/DSQRT(1+PX+PX*PY)
      AZ=DSIGN(1.0D0,RAX)/DSQRT(1+PY+PY*PZ)
C**   ------------------------------------------------------------    **
      RBX=RX-BX
      RBY=RY-BY
      RBZ=RZ-BZ
      IF(DABS(RBX).GT.3.14159) RBX=RBX-DSIGN(6.28318D0,RBX)
      IF(DABS(RBY).GT.3.14159) RBY=RBY-DSIGN(6.28318D0,RBY)
      IF(DABS(RBZ).GT.3.14159) RBZ=RBZ-DSIGN(6.28318D0,RBZ)
      QX=DTAN(RBX)
      QY=DTAN(RBY)
      QZ=DTAN(RBZ)
      QQ=DABS(QX*QY*QZ)**(1.0D0/3)
      QX=(QX/QQ)**2
      QY=(QY/QQ)**2
      QZ=(QZ/QQ)**2
      BX=DSIGN(1.0D0,RBY)/DSQRT(1+QZ+QZ*QX)
      BY=DSIGN(1.0D0,RBZ)/DSQRT(1+QX+QX*QY)
      BZ=DSIGN(1.0D0,RBX)/DSQRT(1+QY+QY*QZ)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINL(H,XT,YT,ZT,NS,M)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XT(1),YT(1),ZT(1)
C**   ============================================================    **
C**   H = Curve length of XT(NS+1),YT(NS+1),ZT(NS+1)                  **
C**   ============================================================    **
      H=0.0
      IF(M.NE.3) THEN
        DO 20 K=1,NS
        H=H+DSQRT((XT(K+1)-XT(K))**2+(YT(K+1)-YT(K))**2)
   20   CONTINUE
      ELSE
        DO 30 K=1,NS
        H=H
     *  +DSQRT((XT(K+1)-XT(K))**2+(YT(K+1)-YT(K))**2+(ZT(K+1)-ZT(K))**2)
   30   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR SPLINE CURVE FITTING                           **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(20),X(20),Y(20),Z(20),DZY(20),DXZ(20),DYX(20)
      DIMENSION A(60),L(20),XT(129),YT(129),ZT(129)
C**   ============================================================    **
C**   N  = Number of given nodes                                      **
C**   M  = 2 or 3 = Dimensions of the curve                           **
C**   KB = 0:4 see SUBROUTINE SPLINE                                  **
C**   KL = 1 : set H(I) = 1 by SUBROUTINE SPLINE                      **
C**      = 0 : set H(I) = Length between nodes I & J by SUB.SPLINE    **
C**      < 0 : use H(I) = Length of the curve IJ get by SUB.SPLINL    **
C**            -KL = Number of iterations to get H(I)   (2 is OK)     **
C**   NL = Number of line segments for a curve between two nodes      **
C**   DZY1,DZYN,...  = See SUBROUTINE SPLINO                          **
C**   X(N),Y(N),Z(N) = Coordinates of nodes                           **
C**   ============================================================    **
   10 READ(*,'(5I5,6F5.0)') N,M,KB,KL,NL,DZY1,DZYN,DXZ1,DXZN,DYX1,DYXN
      IF(M.NE.3) M=2
      KH=MAX0(KL,0)
      NS=8
      DO 15 NN=1,4
   15 IF(NS.LT.NL) NS=2*NS
      READ(*,'(3F10.0)') (X(I),Y(I),Z(I),I=1,N)
      WRITE(*,'(5I5,6F5.2)') N,M,KB,KL,NS,DZY1,DZYN,DXZ1,DXZN,DYX1,DYXN
C**   ------------------------------------------------------------    **
   20 CALL SPLINE(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ------------------------------------------------------------    **
      CALL SPLINO(X,Y,DYX,N,KB,DYX1,DYXN,A,L)
      WRITE(*,'('' XY'',1P6E13.5)') (DYX(I),I=1,N)
      IF(M.EQ.3) THEN
      CALL SPLINO(Y,Z,DZY,N,KB,DZY1,DZYN,A,L)
      CALL SPLINO(Z,X,DXZ,N,KB,DXZ1,DXZN,A,L)
      WRITE(*,'('' YZ'',1P6E13.5)') (DZY(I),I=1,N)
      WRITE(*,'('' ZX'',1P6E13.5)') (DXZ(I),I=1,N)
      ENDIF
C**   ------------------------------------------------------------    **
      WRITE(*,'(4X,''I    K      X(t)      Y(t)      Z(t)'')')
      DO 50 I=1,N
      IF(H(I).EQ.0.0) GO TO 50
      N1=MIN0(I,2)
      IF(M.NE.3) THEN
        CALL SPLIN2(H,X,Y,DYX,N,I,NS,XT,YT)
        IF(KH.GT.KL) THEN
          CALL SPLINL(H(I),XT,YT,ZT,NS,M)
        ELSE
          WRITE(*,'(2I5,2F10.5)') (I,K-1,XT(K),YT(K),K=N1,NS+1)
        ENDIF
      ELSE
        CALL SPLIN3(H,X,Y,Z,DZY,DXZ,DYX,N,I,NS,XT,YT,ZT)
        IF(KH.GT.KL) THEN
          CALL SPLINL(H(I),XT,YT,ZT,NS,M)
        ELSE
          WRITE(*,'(2I5,3F10.5)') (I,K-1,XT(K),YT(K),ZT(K),K=N1,NS+1)
        ENDIF
      ENDIF
   50 CONTINUE
      KH=KH-1
      IF(KH.GE.KL) GO TO 20
      GO TO 10
      END
************************************************************************
