************************************************************************
      SUBROUTINE SPLINS(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),X(N),Y(N),Z(N),A(1),L(N)
C**   ============================================================    **
CIO   H(I) = Parameter/Length between nodes I & J=I+1 or I=N & J=1    **
C*I   X(I),Y(I),Z(I) = Coordinates of the I-th node                   **
C*I   N    = Number of nodes                                          **
C*I   KB   = Index for boundary conditions at both ends               **
C**        = 0 : left_y'' given, right_y'' given                      **
C**        = 1 : left_y'' given, right_y'  given                      **
C**        = 2 : left_y'  given, right_y'' given                      **
C**        = 3 : left_y'  given, right_y'  given                      **
C**        = 4 : left_y' = right_y' for:                              **
C**              2-D and 3-D closed curve                             **
C**              1-D periodic curve : use KH = 1 or -1                **
C*I   KH   = Index for H(I) values:                                   **
C**        = 0 : set H(I) = Length between nodes I & J                **
C**        = 1 : set H(I) = 1                                         **
C**        = otherwise : H(I) are given                               **
C**          In any case, this subroutine set H(N)=1D30 for KB<4      **
C*I   M    = Space dimension of the curve                             **
C**        = 1 : For 1-D curve y(x)                                   **
C**        = 2 : For 2-D curve (x(t),y(t))                            **
C**        = 3 : For 3-D curve (x(t),y(t),z(t))                       **
C*O   A(*) = A(2*N-2) for KB<4,  A(3*N-4) for KB=4.                   **
C*O   L(N) = Location index for variable banded matrix A              **
C**          The coefficient matrix of the spline equation is         **
C**          (Data in [] are for KB=4 only)                           **
C**       / 4/Hn+4/H1  2/H1                               [2/Hn] \    **
C**       |        4/H1+4/H2  2/H2                        [  0 ] |    **
C**       |               4/H2+4/H3 2/H3                  [  0 ] |    **
C**   A = |                         ....                  [  0 ] |    **
C**       |                            ....   2/Hn-2      [  0 ] |    **
C**       |                                4/Hn-2+4/Hn-1  2/Hn-1 |    **
C**       \                                          4/Hn-1+4/Hn /    **
C**   ============================================================    **
      IF(KH.EQ.0) THEN
        DO 10 I=1,N
        J=I+1
        IF(I.EQ.N) J=1
        IF(M.EQ.1) H(I)=(X(J)-X(I))**2
        IF(M.EQ.2) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2
        IF(M.EQ.3) H(I)=(X(J)-X(I))**2+(Y(J)-Y(I))**2+(Z(J)-Z(I))**2
        H(I)=DSQRT(H(I))
   10   CONTINUE
      ELSE IF(KH.EQ.1) THEN
        DO 20 I=1,N
        H(I)=1.0
   20   CONTINUE
      ENDIF
      IF(KB.LT.4) H(N)=1.0D30
C**   ------------------------------------------------------------    **
      L(1)=0
      A(1)=4.0/H(1)+4.0/H(N)
      DO 40 I=1,N-1
      A(2*I  )=2.0/H(I)
      A(2*I+1)=4.0/H(I)+4.0/H(I+1)
      L(I+1)=L(I)+1
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | For y'  given at  left end (KB=01xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.2.OR.KB.EQ.3) A(1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For y'  given at right end (KB=0x1B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.1.OR.KB.EQ.3) A(2*N-1)=1.0D30
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) THEN
        L(N)=L(N)+N-2
        A(3*N-3)=A(2*N-1)
        A(3*N-4)=A(2*N-2)
        A(2*N-2)=2.0/H(N)
        DO 55 I=2*N-1,3*N-5
   55   A(I)=0.0
      ENDIF
      CALL VBDECP(A,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINR(H,Y,YP,N,KB,YP1,YPN,A,L)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),Y(N),YP(N),A(1),L(N)
C**   ============================================================    **
C*I   H,Y,N,KB,A,L = Should be the same as SPLINS                     **
C*O   YP(I)  = y'  at the I-th node                                   **
C*I   YP1    = y''  y''  y'   y'   -  at the  left end                **
C*I   YPN    = y''  y'   y''  y'   -  at the right end                **
C**   for KB = 0    1    2    3    4                                  **
C**                                                                   **
C**                 / 6(y2-y1)/H1**2 + 6(y1-yn)/Hn**2     \           **
C**                 | 6(y3-y2)/H2**2 + 6(y2-y1)/H1**2     |           **
C**             -1  | 6(y4-y3)/H3**2 + 6(y3-y2)/H2**2     |           **
C**   {YP}  = [A] * |              .....                  |           **
C**                 |              .....                  |           **
C**                 |              .....                  |           **
C**                 \ 6(y1-yn)/Hn**2 + 6(yn-yn-1)/Hn-1**2 /           **
C**   ============================================================    **
      DO 60 I=2,N-1
      YP(I)=6.0*(Y(I+1)-Y(I))/H(I)**2+6.0*(Y(I)-Y(I-1))/H(I-1)**2
   60 CONTINUE
      YP(1)=6.0*(Y(2)-Y(1))/H(1)**2
      YP(N)=6.0*(Y(N)-Y(N-1))/H(N-1)**2
C**   +----------------------------------------------------------+    **
C**   | For y' = YP1  at  left end (KB=01xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.2.OR.KB.EQ.3) YP(1)=YP1*A(1)
C**   +----------------------------------------------------------+    **
C**   | For y' = YPN  at right end (KB=0x1B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.1.OR.KB.EQ.3) YP(N)=YPN*A(2*N-1)
C**   +----------------------------------------------------------+    **
C**   | For y''= YP1  at  left end (KB=00xB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.1) YP(1)=YP(1)-YP1
C**   +----------------------------------------------------------+    **
C**   | For y''= YPN  at right end (KB=0x0B)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.EQ.0.OR.KB.EQ.2) YP(N)=YP(N)+YPN
C**   +----------------------------------------------------------+    **
C**   | For closed curve           (KB=1xxB)                     |    **
C**   +----------------------------------------------------------+    **
      IF(KB.GE.4) YP(1)=YP(1)+6.0*(Y(1)-Y(N))/H(N)**2
      IF(KB.GE.4) YP(N)=YP(N)+6.0*(Y(1)-Y(N))/H(N)**2
      CALL VBSOLX(A,YP,L,N)
      RETURN
      END
************************************************************************
      SUBROUTINE SPLINT(H,Y,YP,N,I,T,YT)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(N),Y(N),YP(N)
C**   ============================================================    **
C**   Find YT for given H,Y,YP,N,I,T                                  **
C**   ============================================================    **
      J=I+1
      IF(I.EQ.N) J=1
      S=1-T
      YT=S*S*((3-2*S)*Y(I)+T*YP(I)*H(I))+T*T*((3-2*T)*Y(J)-S*YP(J)*H(I))
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR SPLINE CURVE FITTING                           **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION H(20),X(20),Y(20),Z(20),XP(20),YP(20),ZP(20)
      DIMENSION A(60),L(20)
C**   ============================================================    **
C**   N  = Number of given nodes                                      **
C**   M  = 1:3 = Dimensions of the curve                              **
C**   KB = 0:4 see SUBROUTINE SPLINS                                  **
C**   KH = 1 : set H(I) = 1 by SUBROUTINE SPLINS                      **
C**      = 0 : set H(I) = Length between nodes I & J by SUB.SPLINS    **
C**   NS = Number of line segments for a curve between two nodes      **
C**   XP1, XPN, ...  = See SUBROUTINE SPLINR                          **
C**   X(N),Y(N),Z(N) = Coordinates of nodes                           **
C**   ============================================================    **
   10 READ(*,'(5I5,6F5.0)') N,M,KB,KH,NS,XP1,XPN,YP1,YPN,ZP1,ZPN
      IF(M.LT.1.OR.M.GT.3) M=1
      IF(NS.EQ.0) NS=16
      KH=MAX0(KH,0)
      READ(*,'(3F10.0)') (X(I),Y(I),Z(I),I=1,N)
      WRITE(*,'(5I5,6F5.2)') N,M,KB,KH,NS,XP1,XPN,YP1,YPN,ZP1,ZPN
C**   ------------------------------------------------------------    **
      CALL SPLINS(H,X,Y,Z,N,M,KB,KH,A,L)
C**   ------------------------------------------------------------    **
      IF(M.GE.1) CALL SPLINR(H,Y,YP,N,KB,YP1,YPN,A,L)
      IF(M.GE.2) CALL SPLINR(H,X,XP,N,KB,XP1,XPN,A,L)
      IF(M.GE.3) CALL SPLINR(H,Z,ZP,N,KB,ZP1,ZPN,A,L)
      IF(M.GE.1) WRITE(*,'('' YP'',1P6E13.5)') (YP(I),I=1,N)
      IF(M.GE.2) WRITE(*,'('' XP'',1P6E13.5)') (XP(I),I=1,N)
      IF(M.GE.3) WRITE(*,'('' ZP'',1P6E13.5)') (ZP(I),I=1,N)
C**   ------------------------------------------------------------    **
      WRITE(*,'(4X,''I    K  Sum.H(t)     X(t)      Y(t)      Z(t)'')')
      HO=0.0
      DO 50 I=1,N
      IF(H(I).GE.0.9D30) GO TO 50
      DO 40 K=0,NS
      IF(K.EQ.0.AND.I.GT.1) GO TO 40
      T=FLOAT(K)/NS
      HT=HO+H(I)*T
      IF(M.GE.1) CALL SPLINT(H,Y,YP,N,I,T,YT)
      IF(M.GE.2) CALL SPLINT(H,X,XP,N,I,T,XT)
      IF(M.GE.3) CALL SPLINT(H,Z,ZP,N,I,T,ZT)
      IF(M.EQ.1) WRITE(*,'(2I5,4F10.5)') I,K,HT,HT,YT
      IF(M.EQ.2) WRITE(*,'(2I5,4F10.5)') I,K,HT,XT,YT
      IF(M.EQ.3) WRITE(*,'(2I5,4F10.5)') I,K,HT,XT,YT,ZT
   40 CONTINUE
      HO=HO+H(I)
   50 CONTINUE
      GO TO 10
      END
************************************************************************
