************************************************************************
      SUBROUTINE THIELE(X,Y,N,C,P)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),C(N),P(N)
      DATA EPS/1.0D-15/
C**   ============================================================    **
C**   Thiele's interpolation formula                                  **
C**                                                                   **
C**   F(X(i)) = Y(i), for i=1,2,...,N;  N=1,3,...,2*M+1;              **
C**                                                                   **
C**          C(1)*X**M + C(3)*X**(M-1) + ... + C(N-2)*X + C(2*M+1)    **
C**   F(X) = -----------------------------------------------------    **
C**               X**M + C(2)*X**(M-1) + ... + C(N-3)*X + C(2*M  )    **
C**                                                                   **
C**   F(X(i)) = Y(i), for i=1,2,...,N;  N=2,4,...,2*M;                **
C**                                                                   **
C**               X**M + C(2)*X**(M-1) + ... + C(N-2)*X + C(2*M  )    **
C**   F(X) = -----------------------------------------------------    **
C**                      C(1)*X**(M-1) + ... + C(N-3)*X + C(2*M-1)    **
C**                                                                   **
C**   F(X) = Y(X4)+(X-X4)   (X-X3)           (X-X2)                   **
C**                -------- ---------------- --------------------     **
C**                P(X4,3)+ P(X4,3,2)-Y(X4)+ P(X4,3,2,1)-P(X4,3)+     **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   |            x4 - x3                   x4 - x2             |    **
C**   | p(x4,3) = ---------; p(x4,3,2) = --------------- + f(x3) |    **
C**   |           f(4)-f(3)              p(x4,3)-p(x3,2)         |    **
C**   |                                                          |    **
C**   |                          x4 - x1                         |    **
C**   |      p(x4,3,2,1) = ------------------- + p(x3,2)         |    **
C**   |                    p(x4,3,2)-p(x3,2,1)                   |    **
C**   |                                                          |    **
C**   | x1 f(x1)                                                 |    **
C**   | x2 f(x2)    p(x2,1)                                      |    **
C**   | x3 f(x3)(3) p(x3,2)(2) p(x3,2,1)(1)                      |    **
C**   | xM f(x4)(4) p(x4,3)(3) p(x4,3,2)(2) p(x4,3,2,1)(1)       |    **
C**   | x3     P(3)   \   P(I)   \     P(1)   \                  |    **
C**   | xM     P(M)   ->  P(I+1) ->    P(I)   ->      P(1)       |    **
C**   +----------------------------------------------------------+    **
      DO 30 M=1,N
      P(M)=Y(M)
      PI1=0.0
      DO 20 I=M-1,1,-1
      PI=P(I)
      DP=P(I+1)-P(I)
      IF(DP.EQ.0.0) DP=EPS*DMAX1(DABS(P(I)),EPS)
      P(I)=(X(M)-X(I))/DP+PI1
   20 PI1=PI
C     WRITE(*,'(3X,1P6E13.5)') X(M),(P(I),I=M,1,-1)
   30 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | P(4),P(3); P(2)=p(x4,3,2)-f(x4);P(1)=p(x4,3,2,1)-p(x4,3) |    **
C**   +----------------------------------------------------------+    **
      DO 40 I=1,N-2
   40 P(I)=P(I)-P(I+2)
C**   +----------------------------------------------------------+    **
C**   | F(x)=f(x4)+(x-x4)   (x-x3)           (x-x2)              |    **
C**   |            -------- ---------------- --------------------|    **
C**   |            p(x4,3)+ p(x4,3,2)-f(x4)+ p(x4,3,2,1)-p(x4,3)+|    **
C**   +----------------------------------------------------------+    **
      DO 60 M=1,N
      C(M)=0.0
      DO 50 I=M,3,-2
   50 C(I)=C(I)-C(I-2)*X(M)+C(I-1)*P(M)
      IF(MOD(M,2).EQ.0) C(2)=C(2)-X(M)+C(1)*P(M)
   60 IF(MOD(M,2).NE.0) C(1)=C(1)+P(M)
      RETURN
      END
************************************************************************
      FUNCTION RATFUN(C,N,X)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION C(N),D(2)
      DATA EPS/1.0D-15/
C**   ============================================================    **
C**   N = 1,3,...,2*M+1                                               **
C**                                                                   **
C**          C(1)*X**M + C(3)*X**(M-1) + ... + C(2*M+1)    D(2)       **
C**   F(X) = ------------------------------------------ = ------      **
C**               X**M + C(2)*X**(M-1) + ... + C(2*M  )    D(1)       **
C**                                                                   **
C**   N = 2,4,...,2*M                                                 **
C**                                                                   **
C**               X**M + C(2)*X**(M-1) + ... + C(2*M  )    D(1)       **
C**   F(X) = ------------------------------------------ = ------      **
C**                      C(1)*X**(M-1) + ... + C(2*M-1)    D(2)       **
C**   ============================================================    **
      D(1)=1.0
      D(2)=C(1)
      K=1
      DO 20 I=2,N
      D(K)=D(K)*X+C(I)
   20 K=3-K
      DK=D(K)
      IF(DK.EQ.0.0) DK=EPS
      RATFUN=D(3-K)/DK
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR THIELE'S INTERPOLATION FORMULA                 **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20),C(20),P(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      CALL THIELE(X,Y,N,C,P)
      WRITE(*,'(10X,''C'',12X,''P'',12X,''X'',12X,''Y'')')
      WRITE(*,'(I3,1P4E13.5)') (I,C(I),P(I),X(I),Y(I),I=1,N)
      DO 50 I=1,N
      YY=RATFUN(C,N,X(I))
   50 WRITE(*,'(I3,1P2E13.5)') I,X(I),YY
      GO TO 10
      END
************************************************************************
