************************************************************************
      SUBROUTINE THIINT(X,Y,N,DX,PY,L,XX,YY,NN,YE,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),DX(N),PY(N),L(N)
C**   ============================================================    **
C**   Thiele's interpolation formula                                  **
C**                                                                   **
C**   F(x) = f(x4)+(x-x4)   (x-x3)           (x-x2)                   **
C**                -------- ---------------- --------------------     **
C**                p(x4,3)+ p(x4,3,2)-f(x4)+ p(x4,3,2,1)-p(x4,3)+     **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   |            x4 - x3                   x4 - x2             |    **
C**   | p(x4,3) = ---------; p(x4,3,2) = --------------- + f(x3) |    **
C**   |           f(4)-f(3)              p(x4,3)-p(x3,2)         |    **
C**   |                                                          |    **
C**   |                          x4 - x1                         |    **
C**   |      p(x4,3,2,1) = ------------------- + p(x3,2)         |    **
C**   |                    p(x4,3,2)-p(x3,2,1)                   |    **
C**   |                                                          |    **
C**   | x1 f(x1)                                                 |    **
C**   | x2 f(x2)    p(x2,1)                                      |    **
C**   | x3 f(x3)(3) p(x3,2)(2) p(x3,2,1)(1)                      |    **
C**   | xM f(x4)(4) p(x4,3)(3) p(x4,3,2)(2) p(x4,3,2,1)(1)       |    **
C**   | x3    PY(3)   \  PY(I)   \    PY(1)   \                  |    **
C**   | xM    PY(M)   -> PY(I+1) ->   PY(I)   ->     PY(1)       |    **
C**   +----------------------------------------------------------+    **
      DO 10 I=1,N
   10 DX(I)=DABS(XX-X(I))
      CALL SORTAL(DX,L,N)
C**   ------------------------------------------------------------    **
      YY=0.0
      DO 50 M=1,N
      YO=YY
      NN=M
      I=L(M)
      PY(M)=Y(I)
      DX(M)=XX-X(I)
      PYIP1=0.0
      DO 20 I=M-1,1,-1
      PYI=PY(I)
      DIFPY=PY(I+1)-PY(I)
      IF(DIFPY.EQ.0.0) DIFPY=EPS*DMAX1(DABS(PY(I)),EPS)
      PY(I)=(DX(I)-DX(M))/DIFPY+PYIP1
   20 PYIP1=PYI
C     WRITE(*,'(3X,1P6E13.5)') DX(M),(PY(I),I=M,1,-1)
C**   +----------------------------------------------------------+    **
C**   | YY = f(x4)+(x-x4)   (x-x3)           (x-x2)              |    **
C**   |            -------- ---------------- --------------------|    **
C**   |            p(x4,3)+ p(x4,3,2)-f(x4)+ p(x4,3,2,1)-p(x4,3)+|    **
C**   +----------------------------------------------------------+    **
      DD=0.0
      DO 40 I=1,M-2
   40 DD=DX(I+1)/(PY(I)-PY(I+2)+DD)
      IF(M.GT.1) DD=DX(M)/(PY(M-1)+DD)
      YY=PY(M)+DD
      YE=YY-YO
      IF(DABS(YE).LT.EPS*DABS(YY)) RETURN
   50 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE SORTAL(A,L,N)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),L(N)
C**   ============================================================    **
      DO 10 I=1,N
   10 L(I)=I
      M=N
   20 NE=M-1
      DO 50 I=1,NE
      J=L(I)
      K=L(I+1)
      IF(A(J)-A(K)) 50,50,40
   40 L(I)=K
      L(I+1)=J
      M=I
   50 CONTINUE
      IF(M.LE.NE) GO TO 20
      RETURN
      END
************************************************************************
C**   TEST PROGRAM FOR THIELE'S INTERPOLATION                         **
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(20),Y(20),DX(20),DY(20),L(20)
C**   ============================================================    **
   10 READ(*,'(I5)') N
      READ(*,'(2F10.0)') (X(I),Y(I),I=1,N)
      WRITE(*,'(9X,''XX'',11X,''YY'',11X,''YE'',8X,''NN'')')
   30 READ(*,'(2F10.0)') XX,EPS
      IF(EPS.EQ.0.0) GO TO 10
      CALL THIINT(X,Y,N,DX,DY,L,XX,YY,NN,YE,EPS)
      WRITE(*,'(3X,1P3E13.5,I5)') XX,YY,YE,NN
      GO TO 30
      END
************************************************************************
