************************************************************************
      SUBROUTINE FFTR(A,B,N,INV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:N-1),B(0:N-1)
C**   ============================================================    **
C**   FFT of real data for any even N                                 **
C**   ------------------------------------------------------------    **
C**   f(0:1,0:N-1) ===> F(0:1,0:N-1) : f(1,I)=0, F(1,0)=F(1,N/2)=0    **
C**                F(0,N-I)=F(0,I), F(1,N-I)=-F(1,I),  for I=1,N/2    **
C**   ============================================================    **
C**   For INV=+-1 : DIMENSION A(0:2*N-1)                              **
C**   Input  : A(2*I)=f(0,I), for I=0,N-1  (others unused)            **
C**   Output : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=0,N-1            **
C**   ------------------------------------------------------------    **
C**   For INV=+-2 : DIMENSION A(0:N+1)                                **
C**   Input  : A(I) = f(0,I), for I=0,N-1                             **
C**   Output : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=0,N/2            **
C**   ------------------------------------------------------------    **
C**   For INV=+-3 : DIMENSION A(0:N-1)                                **
C**   Input  : A(I) = f(0,I), for I=0,N-1                             **
C**   Output : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=1,N/2-1          **
C**            A(0) = F(0,0), A(1)  =  F(0,N/2)                       **
C**   ============================================================    **
C**   Working : B(0:N-1) required for N .NE. 2**M only                **
C**   ============================================================    **
      IF(IABS(INV).NE.1) GOTO 40
      DO 20 I=0,N-1
   20 A(I)=A(2*I)
C**   ------------------------------------------------------------    **
   40 CALL FFTA(A,B,N/2,ISIGN(1,INV))
C**   ------------------------------------------------------------    **
      AN0R=A(0)-A(1)
      A(0)=A(0)+A(1)
      A(1)=0.0
C**   ------------------------------------------------------------    **
      ANG=-3.1415926535897932D0*ISIGN(2,INV)/N
      W0=DCOS(ANG)
      W1=DSIN(ANG)
      U0=W0
      U1=W1
      J=N-2
      DO 50 I=2,N/2,2
      H0=(A(I  )+A(J  ))*0.5
      G1=(A(J  )-A(I  ))*0.5
      G0=(A(I+1)+A(J+1))*0.5
      H1=(A(I+1)-A(J+1))*0.5
      P0=G0*U1+G1*U0
      P1=G1*U1-G0*U0
      A(I  )=H0-P1
      A(J  )=H0+P1
      A(I+1)=P0+H1
      A(J+1)=P0-H1
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   50 J=J-2
C**   ------------------------------------------------------------    **
      GOTO (60,90,55),IABS(INV)
   55 A(1)=AN0R
      RETURN
   60 J=2*N-2
      DO 80 I=2,N-1,2
      A(J)=A(I)
      A(J+1)=-A(I+1)
   80 J=J-2
   90 A(N)=AN0R
      A(N+1)=0.0
      RETURN
      END
************************************************************************
      SUBROUTINE FFTC(A,B,N,INV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:N-1),B(0:N-1)
C**   ============================================================    **
C**   FFT of conjugate data for any even N                            **
C**   ------------------------------------------------------------    **
C**   f(0:1,0:N-1) <=== F(0:1,0:N-1) : f(1,I)=0, F(1,0)=F(1,N/2)=0    **
C**                F(0,N-I)=F(0,I), F(1,N-I)=-F(1,I),  for I=1,N/2    **
C**   ============================================================    **
C**   For INV=+-1 : DIMENSION A(0:2*N-1)                              **
C**   Input  : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=0,N/2            **
C**            note that  :   A(1)=0,  A(N+1)=0,  others unused       **
C**   Output : A(2*I)=f(0,I), A(2*I+1)=f(1,I)=0, for I=0,N-1          **
C**   ------------------------------------------------------------    **
C**   For INV=+-2 : DIMENSION A(0:N+1)                                **
C**   Input  : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=0,N/2            **
C**            note that  :   A(1)=0,  A(N+1)=0                       **
C**   Output : A(I) = f(0,I), for I=0,N-1  (f(1,I)=0 not given)       **
C**   ------------------------------------------------------------    **
C**   For INV=+-3 : DIMENSION A(0:N-1)                                **
C**   Input  : A(2*I)=F(0,I), A(2*I+1)=F(1,I), for I=1,N/2-1          **
C**            A(0) = F(0,0), A(1)  =  F(0,N/2)                       **
C**   Output : A(I) = f(0,I), for I=0,N-1  (f(1,I)=0 not given)       **
C**   ============================================================    **
C**   Working : B(0:N-1) required for N .NE. 2**M only                **
C**   ============================================================    **
      AN0R=A(1)
      IF(IABS(INV).LT.3) AN0R=A(N)
      A(1)=A(0)-AN0R
      A(0)=A(0)+AN0R
C**   ------------------------------------------------------------    **
      ANG=3.1415926535897932D0*ISIGN(2,INV)/N
      W0=DCOS(ANG)
      W1=DSIN(ANG)
      U0=W0
      U1=W1
      J=N-2
      DO 50 I=2,N/2,2
      H0=A(I  )+A(J  )   !   Reverse from:    H0=(A(I  )+A(J  ))*0.5
      P1=A(J  )-A(I  )   !                    G1=(A(J  )-A(I  ))*0.5
      P0=A(I+1)+A(J+1)   !  1 <           /   G0=(A(I+1)+A(J+1))*0.5
      H1=A(I+1)-A(J+1)   !      \       /     H1=(A(I+1)-A(J+1))*0.5
      G0=P0*U1-P1*U0     !        \   /       P0=G0*U1+G1*U0
      G1=P1*U1+P0*U0     !  2 <-----X------   P1=G1*U1-G0*U0
      A(I  )=H0-G1       !        /   \       A(I  )=H0-P1
      A(J  )=H0+G1       !      /       \     A(J  )=H0+P1
      A(I+1)=G0+H1       !  3 <           \   A(I+1)=P0+H1
      A(J+1)=G0-H1       !                    A(J+1)=P0-H1
      T0=U0              !
      U0=T0*W0-U1*W1     !   Except *0.5 to be done outside FFTC
      U1=T0*W1+U1*W0
   50 J=J-2
C**   ------------------------------------------------------------    **
      CALL FFTA(A,B,N/2,ISIGN(1,INV))
C**   ------------------------------------------------------------    **
      IF(IABS(INV).NE.1) RETURN
      DO 80 I=N-1,0,-1
      A(2*I+1)=0.0
   80 A(2*I)=A(I)
      RETURN
      END
************************************************************************
      SUBROUTINE FFTSC(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:N-1),B(0:N-1)
C**   ============================================================    **
C**   Half-SIN or Half-COS transform of real data for even N          **
C**   ------------------------------------------------------------    **
C**   Input  : A(0:N-1)/A(0:N)/A(0:N-1) = AI() = real dat             **
C**   Output : A(0:N-1)/A(0:N)/A(0:N-1) = Asin()/Acos()/Ac()          **
C**   ------------------------------------------------------------    **
C**   ICS=+-1: Asin(K) = \sum_{J=0}^{N-1} AI(J)*SIN(K*J*PI/N)         **
C**   ICS=+-2: Acos(K) = \sum_{J=1}^{N-1} AI(J)*COS(K*J*PI/N)         **
C**                              + (AI(0)+AI(N)*COS(K*PI))*0.5        **
C**   ICS=+3 : Ac(K)   = \sum_{J=0}^{N-1} AI(J)*COS(K*(J+.5)*PI/N)    **
C**   ICS=-3 : Ac(K)   = \sum_{J=1}^{N-1} AI(J)*COS((K+.5)*J*PI/N)    **
C**                                 + 0.5*AI(0)*COS((K+.5)*0*PI/N)    **
C**   ------------------------------------------------------------    **
C**   Working : B(0:N-1) required for N .NE. 2**M only                **
C**   ============================================================    **
      IF(IABS(ICS).GE.3) THEN
         CALL FFTCH(A,B,N,ICS)
         RETURN
       ENDIF
C**   ------------------------------------------------------------    **
      ANG=3.1415926535897932D0/N
      W0=DCOS(ANG)
      W1=DSIN(ANG)
C**   ------------------------------------------------------------    **
      U0=W0
      U1=W1
      IF(IABS(ICS).EQ.1) THEN
        A(0)=0.0
      ELSE
        A1  =(A(0)-A(N))*0.5
        A(0)=(A(0)+A(N))*0.5
      ENDIF
      DO 50 I=1,N/2
      IF(IABS(ICS).EQ.1) THEN
        R0=(A(I)-A(N-I))*0.5
        R1=(A(I)+A(N-I))*U1
        A(I)  =R1+R0
        A(N-I)=R1-R0
      ELSE
        R0=(A(I)+A(N-I))*0.5
        R1=(A(I)-A(N-I))*U1
        A1=(A(I)-A(N-I))*U0+A1
        A(I)  =R0-R1
        A(N-I)=R0+R1
      ENDIF
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   50 CONTINUE
C**   ------------------------------------------------------------    **
      CALL FFTR(A,B,N,-3)
      IF(IABS(ICS).NE.1) A(N)=A(1)
      A(1)=0.0
C**   ------------------------------------------------------------    **
      IF(IABS(ICS).EQ.1) THEN
        A1=-0.5*A(0)
        DO 60 I=0,N-2,2
        A1=A1+A(I)
        A(I)=A(I+1)
        A(I+1)=A1
   60   CONTINUE
      ELSE
        DO 70 I=1,N-1,2
        A1=A1+A(I)
        A(I)=A1
   70   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE FFTCH(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:N-1),B(0:N-1)
C**   ============================================================    **
C**   Half-SIN or Half-COS transform of real data for even N          **
C**   ------------------------------------------------------------    **
C**   Input  : A(0:N-1) = AI(0:N-1) = real data                       **
C**   Output : A(0:N-1) = AC(0:N-1)                                   **
C**   ------------------------------------------------------------    **
C**   ICS=+3 : AC(K)   = \sum_{J=0}^{N-1} AI(J)*COS(K*(J+.5)*PI/N)    **
C**   ICS=-3 : AC(K)   = \sum_{J=1}^{N-1} AI(J)*COS((K+.5)*J*PI/N)    **
C**                                 + 0.5*AI(0)*COS((K+.5)*0*PI/N)    **
C**   ------------------------------------------------------------    **
C**   Working : B(0:N-1) required for N .NE. 2**M only                **
C**   ============================================================    **
      ANG=3.1415926535897932D0/N
      W0=DCOS(ANG)
      W1=DSIN(ANG)
C**   ------------------------------------------------------------    **
      IF(ICS.GT.0) THEN
      U0=DCOS(0.5*ANG)
      U1=DSIN(0.5*ANG)
      DO 20 I=0,N/2-1
      R0=(A(I)+A(N-1-I))*0.5
      R1=(A(I)-A(N-1-I))*U1
      A(I)    =R0-R1
      A(N-1-I)=R0+R1
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   20 CONTINUE
C**   ------------------------------------------------------------    **
      CALL FFTR(A,B,N,-3)
      AN=A(1)
      A(1)=0.0
C**   ------------------------------------------------------------    **
      AJ=-0.5*AN
      U0=W0
      U1=W1
      DO 30 I=N-2,0,-2
      DJ    =A(I)*U0+A(I+1)*U1
      A(I)  =A(I)*U1-A(I+1)*U0
      A(I+1)=AJ
      AJ=AJ-DJ
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   30 CONTINUE
      RETURN
C**   ------------------------------------------------------------    **
      ELSE
      AN=A(N-1)
      U0=W0
      U1=W1
      DO 70 I=N-2,2,-2
      DJ    =A(I+1)-A(I-1)
      A(I+1)=DJ*U1-A(I)*U0
      A(I)  =DJ*U0+A(I)*U1
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   70 CONTINUE
C**   ------------------------------------------------------------    **
      A(1)=-2.0*AN
      CALL FFTC(A,B,N,3)
C**   ------------------------------------------------------------    **
      U0=DCOS(0.5*ANG)
      U1=DSIN(0.5*ANG)
      DO 80 I=0,N/2-1
      R0=(A(I)+A(N-1-I))*0.25
      R1=(A(I)-A(N-1-I))*0.125/U1
      A(I)    =R0-R1
      A(N-1-I)=R0+R1
      T0=U0
      U0=T0*W0-U1*W1
      U1=T0*W1+U1*W0
   80 CONTINUE
      RETURN
      ENDIF
      END
************************************************************************
      SUBROUTINE FFTCS(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:N-1),B(0:N-1)
C**   ============================================================    **
C**   SIN or COS transform of real data for even N                    **
C**   ------------------------------------------------------------    **
C**   Input  : A(0:N-1) = AI(0:N-1) = real data                       **
C**   Output : A(0:N-1) = Asin(0:N-1) or Acos(0:N-1)                  **
C**   ------------------------------------------------------------    **
C**   ICS=+-1: Asin(K) = \sum_{J=0}^{N-1} AI(J)*SIN(K*J*2PI/N)        **
C**   ICS=+-2: Acos(K) = \sum_{J=0}^{N-1} AI(J)*COS(K*J*2PI/N)        **
C**   ------------------------------------------------------------    **
C**   Working : B(0:N/2-1) required for N .NE. 2**M only              **
C**   ============================================================    **
      IF(IABS(ICS).EQ.1) THEN
        A(0)=0.0
        DO 20 I=1,N/2-1
          A(I)=A(I)-A(N-I)
   20   CONTINUE
      ELSE
        A(0)=A(0)+A(0)
        DO 30 I=1,N/2-1
          A(I)=A(I)+A(N-I)
   30   CONTINUE
        A(N/2)=A(N/2)+A(N/2)
      ENDIF
C**   ------------------------------------------------------------    **
      CALL FFTSC(A,B,N/2,ICS)
C**   ------------------------------------------------------------    **
      IF(IABS(ICS).EQ.1) THEN
        A(N/2)=0.0
        DO 40 I=1,N/2-1
          A(N-I)=-A(I)
   40   CONTINUE
      ELSE
        DO 60 I=1,N/2-1
          A(N-I)=A(I)
   60   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
