************************************************************************
      PROGRAM POLY
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION A(5100)
      DATA EPSX,EPSF/1.0D-15,1.0D-5/
      DATA ITM,ITS,IP,NF/15,4,21,0/,NDM/5100/
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Input coefficients in A(1:M) --> A(I) * X**(M-I)         |    **
C**   | M     = Number of real coefficients = Poly. order + 1    |    **
C**   |       = 2 * Number of complex coefficients               |    **
C**   | ITM   = Maximum numbers of iterations allowed            |    **
C**   | ITS   = Max. # of iterations for no additional # of RR=1 |    **
C**   | IP    = Debug output control (1+2+4+8+16+32+64+128)      |    **
C**   | NF    > 0 : By FFT method (NF=data points used in FFT)   |    **
C**   |       = 0 : By root shift (Itersections of circles)      |    **
C**   |       < 0 : For complex coefficients                     |    **
C**   +----------------------------------------------------------+    **
   10 CALL INPEQU(A,M,NF,EPSX,EPSF,ITM,ITS,IP)
      NO=1
      NX=NO+M                ! A(NO) for AO(M)
      NY=NX+M                ! A(NX) for AX(M)
      NR=NY+M                ! A(NY) for AY(M)
C**   +----------------------------------------------------------+    **
C**   | By FFT method                                            |    **
C**   +----------------------------------------------------------+    **
      IF(NF.LE.0) GOTO 30
      NM=NR+M*4+NF+2         ! A(NR) for Working array AR(4*M+NF+2)
      IF(NM.GT.NDM) STOP
      CALL POLYFT(A(NO),A(NX),A(NY),A(NR),M,EPSX,EPSF,ITM,ITS,IP,NF)
      GOTO 10
C**   +----------------------------------------------------------+    **
C**   | By root shift (intersections of circles)                 |    **
C**   +----------------------------------------------------------+    **
   30 IF(NF.LT.0) GOTO 50
      NM=NR+M*13             ! A(NR) for Working array AR(13*M)
      IF(NM.GT.NDM) STOP
      CALL POLYRT(A(NO),A(NX),A(NY),A(NR),M,EPSX,EPSF,ITM,ITS,IP)
      GOTO 10
C**   +----------------------------------------------------------+    **
C**   | For complex coefficients                                 |    **
C**   +----------------------------------------------------------+    **
   50 NM=NR+M*14             ! A(NR) for Working array AR(14*M)
      IF(NM.GT.NDM) STOP
      CALL POLYCT(A(NO),A(NX),A(NY),A(NR),M,EPSX,EPSF,ITM,ITS,IP)
      GOTO 10
      END
************************************************************************
      SUBROUTINE INPEQU(AO,M,NF,EPSX,EPSF,ITM,ITS,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      LOGICAL ANSWER
      DIMENSION AO(1)
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Input data from terminal                                 |    **
C**   +----------------------------------------------------------+    **
   10 CALL TERMD(' EPSX  :',EPSX)
      CALL TERMD(' EPSF  :',EPSF)
      CALL TERMI(' ITM   :',ITM)
      CALL TERMI(' ITS   :',ITS)
      CALL TERMI(' IDEBUG:',IP)
      CALL TERMI(' NF-FFT:',NF)
      CALL TERMI(' M     :',M)
C**   +----------------------------------------------------------+    **
C**   | Input coefficients of the polynomial                     |    **
C**   +----------------------------------------------------------+    **
      I=0
   20 I=I+1
      CALL TERMI(' I     :',I)
      IF(I.GT.M) GOTO 30
      CALL TERMD(' AO(I) :',AO(I))
      GOTO 20
C**   ------------------------------------------------------------    **
   30 IF(ANSWER(' WANT TO RETYPE (Y/N) ? ','?')) GOTO 10
      IF(M.LE.0) STOP
      RETURN
      END
************************************************************************
      SUBROUTINE GRAEFF(AA,KA,BB,KB,RR,KR,AO,MO,EPS,ITM,ITS,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AA(MO),KA(MO),BB(MO),KB(MO),RR(MO),KR(MO),AO(MO)
      DATA BASE/1024.0D0/,KPOS,KNEG/7,-8/,KMAX/1073741822/
C**   ============================================================    **
C**   BASE**( KPOS-1) .GT. 2**BitR8      (10*(7-1).GT.56)             **
C**   BASE**(-KNEG-2) .GT. 2**BitR8      (10*(8-2).GT.56)             **
C**   KMAX*2+3 .LE. 2**MIN0(BitI4,2*BitI4)-1    (2**30-2)             **
C**   ------------------------------------------------------------    **
C**   Input   : AO(MO),MO,EPS,ITM,ITS,IP                              **
C**   Output  : RR(MO),KR(MO)                                         **
C**   Working : AA(MO),KA(MO),BB(MO),KB(MO)                           **
C**   ------------------------------------------------------------    **
C*I   AO(I) = Coefficient of X**(M-I)                                 **
C*I   MO    = Number of coefficients = Polynormial order + 1          **
C*I   EPS   = Error tolerance for RR=1 as |RR-1| <= EPS               **
C*I   ITM   = Maximum numbers of iterations allowed                   **
C*I   ITS   = Max. # of iterations for no additional # of RR=1        **
C*I   IP    = Debug output control (1+2+4+8+16+32+64+128)             **
C**   ------------------------------------------------------------    **
C*O   If KR(I).GT.0 then RR(I) is the magnitude of the roots          **
C**                      KR(I) is the multiplicity                    **
C**   ------------------------------------------------------------    **
C**   By Graeffe's root squaring method                               **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Adjust M such that A(M).NE.0 and AO(M+1:MO)=0            |    **
C**   +----------------------------------------------------------+    **
      M=MO
   10 IF(M.LE.1) GOTO 90
      IF(DABS(AO(M)).LT.1000.0D0*EPS) THEN
        M=M-1
        GOTO 10
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Normalize : AO(1)=|AO(M)|=1, AO(II)=AA(II)*BASE**KA(II)  |    **
C**   +----------------------------------------------------------+    **
   15 AI=AO(1)
      AM=DEXP(DLOG(DABS(AO(M)/AI))/(M-1))
      DO 20 II=1,M
      AA(II)=AO(II)/AI
      KA(II)=0
      CALL NORMAL(AA(II),KA(II))
      AK=KA(II)
      IF(IP/4*2.NE.IP/2)
     *WRITE(2,'(I8,1PE16.8,0PF20.0,1PE24.16)') II,AA(II),AK,AO(II)
   20 AI=AI*AM
      AA(M)=AA(M)/DABS(AA(M))
      KA(M)=0
      IF(IP/4*2.NE.IP/2) WRITE(2,'(2I4)') ITM,ITS
C**   +----------------------------------------------------------+    **
C**   | Root squaring                                            |    **
C**   +----------------------------------------------------------+    **
      KM=0
      I1O=0
      DO 50 IT=1,ITM+ITS*2
      DO 30 II=1,M
      IL=II
      IR=II
      S=2.0D0
      BB(II)=AA(II)*AA(II)
      KB(II)=KA(II)+KA(II)
      CALL NORMAL(BB(II),KB(II))
   25 IF(IL.LE.1.OR.IR.GE.M) GO TO 30
      IL=IL-1
      IR=IR+1
      S=-S
      SA=S*AA(IL)*AA(IR)
      IF(SA.NE.0.0D0) THEN
        KI=KA(IL)+KA(IR)-KB(II)
        IF(KI.GE.KPOS.OR.BB(II).EQ.0.0D0) THEN
          BB(II)=SA
          KB(II)=KA(IL)+KA(IR)
          CALL NORMAL(BB(II),KB(II))
        ELSE IF(KI.GT.KNEG) THEN
          BB(II)=BB(II)+SA*BASE**KI
          CALL NORMAL(BB(II),KB(II))
        ENDIF
      ENDIF
      GOTO 25
   30 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Move BB(M),KB(M) --> AA(M),KA(M)                         |    **
C**   | Count I1N = Number of II with AA(II)**2/BB(II)=1         |    **
C**   +----------------------------------------------------------+    **
      I1N=0
      DO 40 II=1,M
      IF(BB(II).NE.0.0D0.AND.AA(II).NE.0.0D0) THEN
        RR(II)=AA(II)*AA(II)/BB(II)*BASE**(KA(II)+KA(II)-KB(II))
      ELSE
        RR(II)=0.0D0
      ENDIF
      AA(II)=BB(II)
      KA(II)=KB(II)
      IF(DABS(RR(II)-1.0D0).LE.EPS) I1N=I1N+1
      IF(KA(II).GT.KM) KM=KA(II)
      IF(IP/4*2.NE.IP/2)
     *WRITE(2,'(I8,1PE16.8,I8,E24.16)') II,AA(II),KA(II),RR(II)
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Set  MS  = Number of squaring has been performed         |    **
C**   | I1N/I1O  = Number of RR(*)=1 at this/last cycle          |    **
C**   | IT-ITO+1 = The number of cycles which have the same I1N  |    **
C**   +----------------------------------------------------------+    **
      IF(I1N.NE.I1O) ITO=IT
      I1O=I1N
      MS=IT
      IF(IP/4*2.NE.IP/2) WRITE(2,'(4I4)') IT,I1N,I1O,ITO
C**   +----------------------------------------------------------+    **
C**   | Convergence test                                         |    **
C**   +----------------------------------------------------------+    **
      IF(KM.GT.KMAX) GOTO 60
      IF(IT.GE.ITM.AND.IT-ITO.GE.ITS) GOTO 60
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find MM = II-IL = Multiplicity of root S, where RR(II)=1 |    **
C**   +----------------------------------------------------------+    **
   60 IL=1
      DO 80 II=2,M
      IF(DABS(RR(II)-1.0D0).GT.EPS) GOTO 80
      S=AA(II)/AA(IL)
      KK=KA(II)-KA(IL)
      CALL RSQRT(S,KK,MS)
      MM=II-IL
      MQ=KK/MM
      MR=KK-MQ*MM
      IF(MM.GT.1) S=DEXP(DLOG(S*BASE**MR)/MM)
      S=S*BASE**MQ*AM
C**   +----------------------------------------------------------+    **
C**   | Set RR(IL:IL+MM-1)=S, KR(IL)=MM, KR(IL+1:IL+MM-1)=-MM    |    **
C**   +----------------------------------------------------------+    **
      MX=MM
      DO 70 IR=1,MM
      RR(IL)=S
      KR(IL)=MX
      IF(IP/2*2.NE.IP)
     *WRITE(2,'(I8,1PE24.16,I8,E24.16)') IL,RR(IL),KR(IL),RR(IL+1)
      MX=-MM
      IL=IL+1
   70 CONTINUE
   80 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Return RR(M),KR(M) (RR(M+1:MO)=0, KR(M+1)=MO-M, if M<MO) |    **
C**   +----------------------------------------------------------+    **
   90 KR(MO)=-MO
      IF(M.EQ.MO) RETURN
      DO 95 II=M,MO-1
      RR(II)=0.0D0
   95 KR(II)=M-MO
      KR(M)=MO-M
      RETURN
      END
************************************************************************
      SUBROUTINE NORMAL(A,K)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DATA BASE/1024.0D0/
C**   ============================================================    **
C**   Normalize : A*BASE**K such that BASE > |A| >= 1                 **
C**   ============================================================    **
      X=DABS(A)
      IF(X.LE.0.0D0) THEN
        K=0
        RETURN
      ENDIF
   10 IF(X.GE.BASE) THEN
        X=X/BASE
        K=K+1
        GO TO 10
      ENDIF
   20 IF(X.LT.1.0D0) THEN
        X=X*BASE
        K=K-1
        GOTO 20
      ENDIF
      A=DSIGN(X,A)
      RETURN
      END
************************************************************************
      SUBROUTINE RSQRT(A,K,MM)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DATA BASE/1024.0D0/
C**   ============================================================    **
C**   Square root of A*BASE**K  MM times  (A is positive)             **
C**   ============================================================    **
      DO 10 I=1,MM
      IF(K/2*2.NE.K) THEN
        A=DSQRT(A*BASE)
        K=(K-1)/2
      ELSE
        A=DSQRT(A)
        K=K/2
      ENDIF
   10 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE POLYRT(AO,RR,KR,A,M,EPSX,EPSF,ITM,ITS,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AO(M),RR(M),KR(M),A(M,13)
C**   ============================================================    **
      CALL POLYNM(AO,RR,KR,A(1,1),A(1,2),A(1,3),A(1,4),A(1,5),A(1,6)
     *           ,A(1,7),A(1,8),A(1,9),A(1,10),A(1,11),A(1,12),A(1,13)
     *           ,M,EPSX,EPSF,ITM,ITS,IP)
      RETURN
      END
************************************************************************
      SUBROUTINE POLYNM(AO,RR,KR,AA,KA,BB,KB,AR,AL,AC,PR,PL,PC,NR,NL,NC
     *                 ,M,EPSX,EPSF,ITM,ITS,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AO(M),RR(M),KR(M),AA(M),KA(M),BB(M),KB(M),BUF(14)
      DIMENSION AR(M),AL(M),AC(M),PR(M),PL(M),PC(M),NR(M),NL(M),NC(M)
      DATA BASE/1024.0D0/,ZERO/0.0D0/,IM/2/
C**   ============================================================    **
C**   Input   : AO(M),M,EPSX,EPSF,ITM,ITS,IP                          **
C**   Output  : RR(M),KR(M)                                           **
C**   Working : AA(M),KA(M),BB(M),KB(M)                               **
C**             AR(M),AL(M),AC(M),PR(M),PL(M),PC(M),NR(M),NL(M),NC(M) **
C**   ------------------------------------------------------------    **
C**   AO(I) = Coefficient of X**(M-I)                                 **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).LT.0 then                             **
C**        RR(I) = Real root                                          **
C**        KR(I) = Its multiplicity                                   **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).EQ.0 then                             **
C**        (RR(I),+-RR(I+1)) = Complex roots                          **
C**        KR(I) = Its multiplicity (always even)                     **
C**   ------------------------------------------------------------    **
C**   By Graeffe's method and root shift and Bairstow's method        **
C**   ============================================================    **
      CALL GRAEFF(AA,KA,BB,KB,RR,KR,AO,M,EPSX,ITM,ITS,IP)
C**   ------------------------------------------------------------    **
      II=1
      DO 500 IL=1,M-1
      IF(KR(IL).LE.0.OR.IL.LT.II) GO TO 500
      IR=IL+KR(IL)-1
C**   ------------------------------------------------------------    **
      II=IL
      X=RR(II)
      IF(X.EQ.0.0D0) GOTO 500
C**   +----------------------------------------------------------+    **
C**   | Scale the II-th root to unit magnitude                   |    **
C**   +----------------------------------------------------------+    **
      KFM=0
      XX=1.0D0
      KX=0
      DO 20 I=M,1,-1
      AA(I)=AO(I)*XX
      KA(I)=KX
      CALL NORMAL(AA(I),KA(I))
      IF(KA(I).GT.KFM.AND.AA(I).NE.0.0D0) KFM=KA(I)
      XX=XX*X
   20 CALL NORMAL(XX,KX)
C**   +----------------------------------------------------------+    **
C**   | FFM = Sum of the abs of the coefficients                 |    **
C**   +----------------------------------------------------------+    **
      FFM=0.0D0
      DO 30 I=1,M
      KI=KA(I)-KFM
      IF(KI.NE.0.AND.AA(I).NE.0.0D0) AA(I)=AA(I)*BASE**KI
   30 FFM=FFM+DABS(AA(I))
      EFM=EPSF*FFM
      EPS=EPSX*100.0D0
C**   +----------------------------------------------------------+    **
C**   | Square the roots for pure imag roots                     |    **
C**   +----------------------------------------------------------+    **
      DO 210 I=1,M
      AL(I)=AA(I)
      AR(I)=AA(I)
      IX=I
      IY=I
      S=2.0D0
      SA=0.0D0
  205 IF(IX.GT.1.AND.IY.LT.M) THEN
      IX=IX-1
      IY=IY+1
      S=-S
      SA=AA(IX)*AA(IY)-SA
      GOTO 205
      ENDIF
  210 AC(I)=AA(I)*AA(I)+S*SA
C**   +----------------------------------------------------------+    **
C**   | Shift the roots                                          |    **
C**   +----------------------------------------------------------+    **
      DO 220 K=M,2,-1
      DO 220 I=2,K
      AR(I)=AR(I)+AR(I-1)
      AL(I)=AL(I)-AL(I-1)
  220 AC(I)=AC(I)+AC(I-1)
C**   +----------------------------------------------------------+    **
C**   | Find  M-MR = # of real roots (+X,0)                      |    **
C**   | Find  M-ML = # of real roots (-X,0)                      |    **
C**   | Find  M-MC = # of imag roots (0,+X) or (0,-X)            |    **
C**   +----------------------------------------------------------+    **
      DO 230 K=M-KR(IL),M
      IF(DABS(AR(K)).GT.EFM) MR=K
      IF(DABS(AL(K)).GT.EFM) ML=K
      IF(DABS(AC(K)).GT.EFM) MC=K
  230 CONTINUE
C**   ------------------------------------------------------------    **
      CALL PUTX( X,ZERO,M-MR,RR,KR,II,EPS)
      IF(II.GT.IR) GOTO 500
      CALL PUTX(-X,ZERO,M-ML,RR,KR,II,EPS)
      IF(II.GT.IR) GOTO 500
      CALL PUTX( ZERO,X,M-MC,RR,KR,II,EPS)
      IF(II.GT.IR) GOTO 500
C**   +----------------------------------------------------------+    **
C**   | Check complex root for Err < EFM (EFM increase with KKK) |    **
C**   +----------------------------------------------------------+    **
      DO 400 KKK=1,3
C**   +----------------------------------------------------------+    **
C**   | Find complex roots : real parts between +0.5X and +X     |    **
C**   +----------------------------------------------------------+    **
      IF(KKK.EQ.1)
     *CALL GRAEFF(AR,KA,BB,KB,PR,NR,AR,MR,EPSX,ITM,ITS/2,IP)
      DO 310 IV=MR-1,1,-1
      IF(NR(IV).LE.0.OR.PR(IV).GT.2.005D0) GOTO 310
      P=2.0D0-PR(IV)**2
      Q=1.0D0
      DO 300 ISTEP=1,20
      CALL SYNTH2(AA,BB,M,P,Q,IM)
      CALL PQERR(BB,M,P,Q,IM,ERS,FX)
      PO=P
      P=XMIN(P,FX,0.001D0,ISTEP,IVY,BUF)
      IF(DABS(P-PO).LE.EPSX) GOTO 305
  300 CONTINUE
  305 CALL BAIRST(AA,BB,M,P,Q,IM,EPSX,EFM,IP)
      IF(DABS(Q-1.0).GT.0.125.OR.DABS(P).GT.2.25) GO TO 310
      CALL CHECK2(AA,BB,M,MB,P,Q,IM,EFM,IP)
      IF(M.EQ.MB) GOTO 310
      CALL PUTX(0.5D0*P*X,DSQRT(Q-0.25D0*P*P)*X,M-MB,RR,KR,II,EPS)
      NR(IV)=0
      IF(II.GT.IR) GOTO 500
  310 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find complex roots : real parts between -X and -0.5X     |    **
C**   +----------------------------------------------------------+    **
      IF(KKK.EQ.1)
     *CALL GRAEFF(AL,KA,BB,KB,PL,NL,AL,ML,EPSX,ITM,ITS/2,IP)
      DO 330 IV=ML-1,1,-1
      IF(NL(IV).LE.0.OR.PL(IV).GE.2.005D0) GOTO 330
      P=PL(IV)**2-2.0D0
      Q=1.0D0
      DO 320 ISTEP=1,20
      CALL SYNTH2(AA,BB,M,P,Q,IM)
      CALL PQERR(BB,M,P,Q,IM,ERS,FX)
      PO=P
      P=XMIN(P,FX,0.001D0,ISTEP,IVY,BUF)
      IF(DABS(P-PO).LE.EPSX) GOTO 325
  320 CONTINUE
  325 CALL BAIRST(AA,BB,M,P,Q,IM,EPSX,EFM,IP)
      IF(DABS(Q-1.0).GT.0.125.OR.DABS(P).GT.2.25) GO TO 330
      CALL CHECK2(AA,BB,M,MB,P,Q,IM,EFM,IP)
      IF(M.EQ.MB) GOTO 330
      CALL PUTX(0.5D0*P*X,DSQRT(Q-0.25D0*P*P)*X,M-MB,RR,KR,II,EPS)
      NL(IV)=0
      IF(II.GT.IR) GOTO 500
  330 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find complex roots : real parts between -0.5X and +0.5X  |    **
C**   +----------------------------------------------------------+    **
      EFM=EFM*4.0D0
      IF(KKK.EQ.1)
     *CALL GRAEFF(AC,KA,BB,KB,PC,NC,AC,MC,EPSX,ITM,ITS/2,IP)
      DO 350 IV=MC-1,1,-1
      IF(NC(IV).LE.0.OR.PC(IV).GT.2.010D0) GOTO 350
      P=PC(IV)
      Q=1.0D0
      DO 333 ISTEP=1,20
      CALL SYNTH2(AA,BB,M,P,Q,IM)
      CALL PQERR(BB,M,P,Q,IM,ERS,FX)
      PO=P
      P=XMIN(P,FX,0.001D0,ISTEP,IVY,BUF)
      IF(DABS(P-PO).LE.EPSX) GOTO 335
  333 CONTINUE
  335 DO 340 KK=1,2
      CALL BAIRST(AA,BB,M,P,Q,IM,EPSX,EFM,IP)
      IF(DABS(Q-1.0).GT.0.125.OR.DABS(P).GT.2.25) GO TO 340
      CALL CHECK2(AA,BB,M,MB,P,Q,IM,EFM,IP)
      IF(M.EQ.MB) GOTO 340
      CALL PUTX(0.5D0*P*X,DSQRT(Q-0.25D0*P*P)*X,M-MB,RR,KR,II,EPS)
      NC(IV)=0
      IF(II.GT.IR) GOTO 500
  340 P=-P
  350 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Increase the error tolerance EFM                         |    **
C**   +----------------------------------------------------------+    **
      EFM=EFM*2.5D0
      IF(II.GT.IR) GOTO 500
  400 CONTINUE
      WRITE(2,'(1X,A,1PE16.8)') 'NO ENOUGH ACCURATE ROOTS FOUND FOR',X
  500 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Output the answers                                       |    **
C**   +----------------------------------------------------------+    **
  700 WRITE(2,'(/A8,2A24,A8/)') 'I','Real.X(I)','Imag.X(I)','No'
      DO 710 II=1,M-1
      IF(KR(II).LE.0) GOTO 710
      IF(KR(II+1).NE.0) THEN
        WRITE(2,'(I8,1PE24.16,24X,I8)') II,RR(II),KR(II)
      ELSE
        WRITE(2,'(I8,1P2E24.16,I8)') II,RR(II),RR(II+1),KR(II)
      ENDIF
  710 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE BAIRST(AA,BB,M,P,Q,IM,EPS,EFM,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AA(M),BB(M)
C**   ============================================================    **
C**   Input   : AA(M),M,P,Q,IM,EPS,EFM,IP                             **
C**   Output  : P,Q IN THE FACTOR (X*X-P*X+Q)                         **
C**   Working : BB(M)                                                 **
C**   ------------------------------------------------------------    **
C**   By Bairstow's method                                            **
C**   ============================================================    **
      IF(M.LE.3) RETURN
      DO 40 I=1,20
C**   +----------------------------------------------------------+    **
C**   | [X*X-P*X+Q][B(1)*X**(M-3)+..+B(M-2)]+B(M-1)*(X-P)+B(M)   |    **
C**   +----------------------------------------------------------+    **
      CALL SYNTH2(AA,BB,M,P,Q,2)
C**   ------------------------------------------------------------    **
      B0=BB(M)
      B1=BB(M-1)
      EE=DSQRT(DABS(B0*B0-P*B0*B1+Q*B1*B1))
C**   +----------------------------------------------------------+    **
C**   | [X*X-P*X+Q][C(1)*X**(M-4)+..+C(M-3)]+C(M-2)*(X-P)+C(M-1) |    **
C**   +----------------------------------------------------------+    **
      CALL SYNTH2(BB,BB,M-1,P,Q,2)
C**   +----------------------------------------------------------+    **
C**   | / C(M-1)  C(M-2) \ / -DP \   / B( M ) \                  |    **
C**   | |                | |     | = |        |                  |    **
C**   | \ C(M-2)  C(M-3) / \ +DQ /   \ B(M-1) /                  |    **
C**   +----------------------------------------------------------+    **
      D=BB(M-1)*BB(M-3)-BB(M-2)**2
      IF(DABS(D).LT.EPS) WRITE(2,'(1X,A)') 'BAIRST : |D| < EPS'
      DP=(BB(M-2)*B1-BB(M-3)*B0)/D
      DQ=(BB(M-1)*B1-BB(M-2)*B0)/D
      P=P+DP
      Q=Q+DQ
      IF(IP/32*2.NE.IP/16)
     *WRITE(2,'('' BAIRST '',1PE24.16,3E14.5)') P,Q,DP,DQ,D,EE,B1,B0
      IF(DABS(DP)+DABS(DQ).LE.EPS) RETURN
C**   +----------------------------------------------------------+    **
C**   | For general use following statement should be deleted    |    **
C**   +----------------------------------------------------------+    **
      IF(DABS(Q-1.0).GT.0.125.OR.DABS(P).GT.2.25) RETURN
   40 CONTINUE
      WRITE(2,'(1X,A)') 'BAIRST : NOT CONVERGE IN 20 ITERATION'
      RETURN
      END
************************************************************************
      SUBROUTINE PUTX(XR,XI,N,RR,KR,II,EPS)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION RR(1),KR(1)
C**   ============================================================    **
C**   Test if (XR,XI) already exist in  RR(1:II-1)                    **
C**   ============================================================    **
      IF(N.LE.0) RETURN
      DO 30 I=1,II-1
      IF(KR(I).LE.0) GOTO 30
      IF(KR(I+1).NE.0) THEN
         IF(DABS(XR-RR(I))+DABS(XI-RR(I+1)).LE.EPS) RETURN
      ELSE
         IF(DABS(XR-RR(I)).LE.EPS) RETURN
      ENDIF
   30 CONTINUE
C**   ------------------------------------------------------------    **
      RR(II)=XR
      KR(II)=N
      IF(XI.NE.0.0D0) THEN
        RR(II+1)=XI
        KR(II+1)=0
      ENDIF
      II=II+N
      WRITE(2,'('' PUTX '',I8,1P2E24.16)') II,XR,XI
      RETURN
      END
************************************************************************
      SUBROUTINE CHECK2(AA,BB,MA,MB,P,Q,IM,EFM,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AA(MA),BB(MA)
C**   ============================================================    **
C**   Check # of the factor (X*X - P*X + Q)   --->  (MB-MA)/2         **
C**           or the factor (X - P)           --->  (MB-MA)           **
C**   ------------------------------------------------------------    **
C**   Input   : AA(MA),MA,P,Q,IM,EFM,IP                               **
C**   Output  : MB                                                    **
C**   Working : BB(MA)                                                **
C**   ============================================================    **
      MB=MA
      DO 10 M=MA,IM+1,-IM
      IF(M.EQ.MA) THEN
        CALL SYNTH2(AA,BB,MB,P,Q,IM)
      ELSE
        CALL SYNTH2(BB,BB,MB,P,Q,IM)
      ENDIF
      CALL PQERR(BB,MB,P,Q,IM,ERS,ERR)
      IF(IP/64*2.NE.IP/32)
     *WRITE(2,'(I8,1P2E16.8)') (I,AA(I),BB(I),I=1,MB)
      IF(IP/32*2.NE.IP/16)
     *WRITE(2,'('' CHECK '',I5,1PE24.16,3E12.4)') MB,P,Q,ERR,EFM
      IF(ERR.GT.EFM) RETURN
      MB=MB-IM
   10 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE SYNTH2(AA,BB,M,P,Q,IM)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AA(M),BB(M)
C**   ============================================================    **
C*I   A(1)*X**(M-1)+..+A(M)                                           **
C*O   =  (X-P)  *  (B(1)*X**(M-2)+..+B(M-1)) + B(M)                   **
C*O   =(X*X-P*X+Q)*(B(1)*X**(M-3)+..+B(M-2)) + B(M-1)*(X-P) + B(M)    **
C**   ------------------------------------------------------------    **
C**   Input   : AA(M),M,P,Q,IM  (IM=1 or 2)                           **
C**   Output  : BB(M)                                                 **
C**   ============================================================    **
      BB(1)=AA(1)
      IF(IM.EQ.1) THEN
        DO 20 I=2,M
   20   BB(I)=AA(I)+BB(I-1)*P
      ELSE
        BB(2)=AA(2)+P*BB(1)
        DO 80 I=3,M
   80   BB(I)=AA(I)+BB(I-1)*P-BB(I-2)*Q
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE PQERR(BB,M,P,Q,IM,ES,ER)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION BB(M)
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | |(X-P)*B1+B0)|**2=((X-P)*B1+B0)*((conj.X-P)*B1+B0)       |    **
C**   | =(B0-P*B1)**2+(X+conj.X)*(B0-P*B1)*B1+|X|**2*B1**2       |    **
C**   | =(B0-P*B1)**2+    P     *(B0-P*B1)*B1+ Q    *B1**2       |    **
C**   | =B0**2-P*B0*B1+Q*B1**2                                   |    **
C**   +----------------------------------------------------------+    **
      IF(IM.EQ.1) THEN
        ES=BB(M)
      ELSE
        ES=BB(M)**2-P*BB(M)*BB(M-1)+Q*BB(M-1)**2
      ENDIF
      ER=DABS(ES)
      RETURN
      END
************************************************************************
      SUBROUTINE POLYFT(AO,RR,KR,A,M,EPSX,EPSF,ITM,ITS,IP,NF)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AO(M),RR(M),KR(M),A(M,10)
C**   ============================================================    **
      K=NF/2+2
      CALL FFTANG(AO,RR,KR,A(1,1),A(1,2),A(1,3),A(1,4)
     *           ,A(1,5),A(K,5),M,EPSX,EPSF,ITM,ITS,IP,NF)
      RETURN
      END
************************************************************************
      SUBROUTINE FFTANG(AO,RR,KR,AA,KA,BB,KB,FF,LL
     *                 ,M,EPSX,EPSF,ITM,ITS,IP,NF)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AO(M),RR(M),KR(M),AA(M),KA(M),BB(M),KB(M)
      DIMENSION FF(NF),LL(NF),BUF(14)
      DATA BASE/1024.0D0/,INV/-2/
C**   ============================================================    **
C**   Input   : AO(M),M,EPSX,EPSF,ITM,ITS,IP,NF                       **
C**   Output  : RR(M),KR(M)                                           **
C**   Working : AA(M),KA(M),BB(M),KB(M),FF(NF+2),LL(NF/2+1)           **
C**   ------------------------------------------------------------    **
C**   AO(I) = Coefficient of X**(M-I)                                 **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).NE.0 then                             **
C**        RR(I) = Real root                                          **
C**        KR(I) = Its multiplicity                                   **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).EQ.0 then                             **
C**        (RR(I),+-RR(I+1)) = complex roots                          **
C**        KR(I) = Its multiplicity (always even)                     **
C**   ------------------------------------------------------------    **
C**   By Graeffe's method and FFT and Bairstow's method               **
C**   ============================================================    **
      CALL GRAEFF(AA,KA,BB,KB,RR,KR,AO,M,EPSX,ITM,ITS,IP)
C**   ------------------------------------------------------------    **
      II=1
      DO 85 IL=1,M-1
      IF(KR(IL).LE.0.OR.IL.LT.II) GO TO 85
      IR=IL+KR(IL)-1
C**   ------------------------------------------------------------    **
      II=IL
      X=RR(II)
      IF(X.EQ.0.0D0) GOTO 85
C**   ------------------------------------------------------------    **
      DO 15 I=1,NF+2
   15 FF(I)=0.0D0
C**   +----------------------------------------------------------+    **
C**   | Scale the II-th root to unit magnitude                   |    **
C**   +----------------------------------------------------------+    **
      KFM=0
      XX=1.0D0
      KX=0
      DO 20 I=M,1,-1
      AA(I)=AO(I)*XX
      KA(I)=KX
      CALL NORMAL(AA(I),KA(I))
      IF(KA(I).GT.KFM.AND.AA(I).NE.0.0D0) KFM=KA(I)
      XX=XX*X
   20 CALL NORMAL(XX,KX)
C**   +----------------------------------------------------------+    **
C**   | FFM = Sum of the abs of the coefficients                 |    **
C**   +----------------------------------------------------------+    **
      FFM=0.0D0
      DO 30 I=1,M
      KI=KA(I)-KFM
      IF(KI.NE.0.AND.AA(I).NE.0.0D0) AA(I)=AA(I)*BASE**KI
      FFM=FFM+DABS(AA(I))
   30 FF(I)=AA(I)
      EFM=EPSF*FFM
      EPS=EPSX*100.0D0
C**   +----------------------------------------------------------+    **
C**   | Find real roots +X or -X                                 |    **
C**   +----------------------------------------------------------+    **
      IM=1
      P=1.0D0
      Q=0.0D0
      DO 40 K=1,2
      CALL CHECK2(AA,BB,M,MB,P,Q,IM,EFM,IP)
      CALL PUTX(P*X,Q,M-MB,RR,KR,II,EPS)
      IF(II.GT.IR) GOTO 85
   40 P=-P
C**   +----------------------------------------------------------+    **
C**   | Compute function values along the unit circle by FFT     |    **
C**   +----------------------------------------------------------+    **
      CALL FFTR(FF,FF,NF,INV)
      DO 50 I=1,NF/2+1
      AM=DSQRT(FF(2*I-1)**2+FF(2*I)**2)
      IF(IP/256*2.NE.IP/128)
     *WRITE(2,'(I8,1P3E16.8)') I,AM,FF(2*I-1),FF(2*I)
   50 FF(I)=AM
C**   +----------------------------------------------------------+    **
C**   | Find locations with local minimum errors                 |    **
C**   +----------------------------------------------------------+    **
      IN=0
      DL=FF(2)-FF(1)
      DO 60 I=2,NF/2
      DR=FF(I+1)-FF(I)
      IF(DL.GT.0.0D0.OR.DR.LT.0.0D0) GOTO 60
      IN=IN+1
      LL(IN)=I
      IF(IP/128*2.NE.IP/64)
     *WRITE(2,'(I8,1P3E16.9)') I,FF(I-1),FF(I),FF(I+1)
   60 DL=DR
C**   +----------------------------------------------------------+    **
C**   | Index sort according to errors                           |    **
C**   +----------------------------------------------------------+    **
      IS=IN
   65 IE=IS-1
      DO 70 I=1,IE
      IU=LL(I)
      IV=LL(I+1)
      IF(FF(IU).LE.FF(IV)) GOTO 70
      LL(I)=IV
      LL(I+1)=IU
      IS=I
   70 CONTINUE
      IF(IS.LE.IE) GOTO 65
C**   +----------------------------------------------------------+    **
C**   | Refine the complex roots by minimizing the errors        |    **
C**   | Then by Bairstow's method                                |    **
C**   +----------------------------------------------------------+    **
      IM=2
      DX=2.0D0/NF
      DO 80 IV=1,IN
      XX=DX*(LL(IV)-1)
      DO 77 ISTEP=1,12
      P=2.0D0*DCOS(XX*3.1415926535897932D0)
      Q=1.0D0
      CALL SYNTH2(AA,BB,M,P,Q,IM)
      CALL PQERR(BB,M,P,Q,IM,ERS,FX)
      XO=XX
      XX=XMIN(XX,FX,0.5*DX,ISTEP,IVY,BUF)
      IF(DABS(XX-XO).LE.EPSX) GOTO 78
   77 CONTINUE
C**   ------------------------------------------------------------    **
   78 CALL BAIRST(AA,BB,M,P,Q,IM,EPSX,EFM,IP)
      IF(DABS(Q-1.0).GT.0.125.OR.DABS(P).GT.2.25) GO TO 80
      CALL CHECK2(AA,BB,M,MB,P,Q,IM,EFM,IP)
      IF(M.EQ.MB) GOTO 80
      CALL PUTX(0.5D0*P*X,DSQRT(Q-0.25D0*P*P)*X,M-MB,RR,KR,II,EPS)
      IF(II.GT.IR) GOTO 85
   80 CONTINUE
   85 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Output the answers                                       |    **
C**   +----------------------------------------------------------+    **
   90 WRITE(2,'(/A8,2A24,A8/)') 'I','Real.X(I)','Imag.X(I)','No'
      DO 100 II=1,M-1
      IF(KR(II).LE.0) GOTO 100
      IF(KR(II+1).NE.0) THEN
        WRITE(2,'(I8,1PE24.16,24X,I8)') II,RR(II),KR(II)
      ELSE
        WRITE(2,'(I8,1P2E24.16,I8)') II,RR(II),RR(II+1),KR(II)
      ENDIF
  100 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE POLYCT(AO,RR,KR,A,M2,EPSX,EPSF,ITM,ITS,IP)
C**   ============================================================    **
      IMPLICIT REAL*8(A-H,O-Z),INTEGER*4(I-N)
      DIMENSION AO(M2),RR(M2),KR(M2),A(1)
C**   ============================================================    **
C**   Input   : AO(M2),M2,EPSX,EPSF,ITM,ITS,IP                        **
C**   Output  : RR(M2),KR(M2)                                         **
C**   Working : A(MM*14)         (MM=M2-1, M=M2/2, M2 is even)        **
C**   ------------------------------------------------------------    **
C**   (AO(2*I-1),AO(2*I)) = Complex coefficient of X**(M-I)           **
C**   ------------------------------------------------------------    **
C**   A(I) = Coefficient of Y**(MM-I)                                 **
C**        Half # of roots Y = Roots X                                **
C**        Half # of roots Y = Conjugate of roots X                   **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).LT.0 then                             **
C**        RR(I) = Real root                                          **
C**        KR(I) = Its multiplicity                                   **
C**   ------------------------------------------------------------    **
C**   If KR(I).GT.0 and KR(I+1).EQ.0 then                             **
C**        (RR(I),+-RR(I+1)) = complex roots                          **
C**        KR(I) = Its multiplicity (always even)                     **
C**   ------------------------------------------------------------    **
C**   Double the degree of the polynomial such that                   **
C**   The roots include the original roots and their conjugates       **
C**   So the coefficients are all real                                **
C**   ============================================================    **
      M=M2/2
      MM=M2-1
      DO 150 I=1,MM,2
      IX=I
      IY=I
      SR=0.0D0
      SI=0.0D0
  135 IF(IX.GT.0.AND.IY.LT.MM) THEN
        IY=IY+2
        SI=SI+AO(IX)*AO(IY)+AO(IX+1)*AO(IY+1)
        IX=IX-2
        IF(IX.GT.0) THEN
        SR=SR+AO(IX)*AO(IY)+AO(IX+1)*AO(IY+1)
        ENDIF
        GO TO 135
      ENDIF
      A(I  )=2.0D0*SR+AO(I)*AO(I)+AO(I+1)*AO(I+1)
      A(I+1)=2.0D0*SI
  150 CONTINUE
C**   ------------------------------------------------------------    **
      CALL POLYRT(A,RR,KR,A(MM+1),MM,EPSX,EPSF,ITM,ITS,IP)
C**   ------------------------------------------------------------    **
      WRITE(2,'(/A8,2A24,A8/)') 'I','Real.X(I)','Imag.X(I)','No'
      DO 300 II=1,MM-1
      IF(KR(II).LE.0) GOTO 300
      XR=RR(II)
      IF(KR(II+1).EQ.0) THEN
        XI=RR(II+1)
        IX=2
      ELSE
        XI=0.0D0
        IX=1
      ENDIF
C**   ------------------------------------------------------------    **
      DO 260 IR=1,IX
      DO 210 I=1,M2
  210 A(I)=AO(I)
      DO 230 K=MM,3,-2
      DO 220 I=3,K,2
      A(I  )=A(I  )+A(I-2)*XR-A(I-1)*XI
  220 A(I+1)=A(I+1)+A(I-2)*XI+A(I-1)*XR
      IF(DSQRT(A(K)**2+A(K+1)**2).GT.EPSF) GOTO 250
  230 CONTINUE
C**   ------------------------------------------------------------    **
  250 N=(MM-K)/2
      WRITE(2,'(I8,1P2E24.16,I8)') II,XR,XI,N
      XI=-XI
  260 CONTINUE
  300 CONTINUE
      RETURN
      END
************************************************************************
