************************************************************************
      FUNCTION XMin(X,F,dX,Istep,IVY,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(14),B(14)
      EQUIVALENCE (A(13),ST),(A(14),VY)
     *  ,(A(1),X1),(A(2),X2),(A(3),X3),(A(4),F1),(A(5),F2),(A(6),F3)
     *  ,(A(7),XL),(A(8),XM),(A(9),XR),(A(10),FL),(A(11),FM),(A(12),FR)
      DATA Pmax/1.0D30/
C**   ============================================================    **
C**   FIND  XMin                                                      **
C**   ------------------------------------------------------------    **
C**   This program combines 3 methods to get the best guess of X :    **
C**   (1) from quadratic eq. of newest_3_points (if concave_upward)   **
C**       but the max_step_size is set to 3.75*previous_step_size     **
C**   (2) from quadratic eq. of valley_set_of_3_points (if any)       **
C**       when (1)_is_NA or X_from_(1) outside the bounds_of_valley   **
C**   (3) step on the downward_side twice the previous_step_size      **
C**       when both (1)&(2)_are_NA                                    **
C**   Method(1) makes the program so fast as quadratic convergence    **
C**   Method(2) makes the program so stable as global convergence     **
C**   Method(3) makes the program so insensitive to init_step_size    **
C**   ------------------------------------------------------------    **
C**   XMin  = new guess of X for minimizing F(X)                      **
C**   X     = argument of function                                    **
C**   F     = F(X) = function value of X                              **
C**   dX    = return XMin=X+dX if no better formula to find XMin      **
C**   Istep = number of call to Function XMin                         **
C**   IVY   = 1/0 if valley set is found/not found                    **
C**   B(14) = buffer must NOT be changed between each calling Istep   **
C**   ------------------------------------------------------------    **
C**   for Istep = 1  :  (X1=X,F1=F) (ST=1) (VY=0)                     **
C**       XMin  = X1 + dX   if dX.NE.0                                **
C**       XMin  = X1 + 0.1  if dX.EQ.0                                **
C**   for Istep = 2  :  (X2=X,F2=F) (ST=2) (VY=0)                     **
C**       XMin  = X1 + (X1-X2)*2  if F1<F2                            **
C**       XMin  = X2 + (X2-X1)*2  if F2<F1                            **
C**   for Istep > 2  :  (X3=X,F3=F) (ST>2) (VY=1 if found valley)     **
C**       XMin  = Y1  from quadratic eq. of newest point (X1,X2,X3)   **
C**       XMin  = Y2  from quadratic eq. of valley set (XL,XM,XR)     **
C**                   if VY=1 and when Y1 is NA or Outside (XL,XR)    **
C**   ============================================================    **
      IF(Istep.EQ.1) THEN
        DO 10 I=1,14
        A(I)=Pmax
  10    CONTINUE
        ST=0
        VY=0
      ELSE
        DO 20 I=1,14
        A(I)=B(I)
  20    CONTINUE
      ENDIF
      ST=ST+1
C**   +----------------------------------------------------------+    **
C**   | SAVE  the newest 3 points to (X1,X2,X3)                  |    **
C**   +----------------------------------------------------------+    **
      X1=X2
      X2=X3
      X3=X
      F1=F2
      F2=F3
      F3=F
C**   +----------------------------------------------------------+    **
C**   | FIND  new points (XL,XM,XR) that bracket the root        |    **
C**   +----------------------------------------------------------+    **
      IF(X.GT.XM) THEN
        IF(F.GT.FM.OR.(F.EQ.FM.AND.FR.GT.FL)) THEN
          XR=X
          FR=F
        ELSE
          XL=XM
          XM=X
          FL=FM
          FM=F
        ENDIF
      ELSE IF(X.LT.XM) THEN
        IF(F.GT.FM.OR.(F.EQ.FM.AND.FL.GT.FR)) THEN
          XL=X
          FL=F
        ELSE
          XR=XM
          FR=FM
          XM=X
          FM=F
        ENDIF
      ENDIF
      IF(FL.LT.Pmax.AND.FR.LT.Pmax) VY=1
      Iget=0
      XX=0.0
C**   +----------------------------------------------------------+    **
C**   | FIND a new point from qradratic equation on (X1,X2,X3)   |    **
C**   | if it is concave upward  d2F/dX2 = 1/ddF > 0             |    **
C**   | and make sure that ABS(XX-(X3+X1)/2) < 3*ABS(X3-X1)      |    **
C**   |                                                          |    **
C**   | X1  X2      X3    <---3.75*(X3-X2)--->    XXmax          |    **
C**   |  *---*---+---*---+---x---+---x---+---x---+-*             |    **
C**   |        *---+---+---x---+---+---x---+---+---*             |    **
C**   |        0        <---3*(X3-X1)--->        -ddX            |    **
C**   +----------------------------------------------------------+    **
      IF(ST.GT.2) THEN
        FF=(F3-F2)*(X1-X2)-(F1-F2)*(X3-X2)
        IF(FF.NE.0) THEN
          ddF=(X3-X1)*(X3-X2)*(X1-X2)/FF
          IF(ddF.GT.0) THEN
            ddX=0.5*(F3-F1)/(X3-X1)*ddF
            Rate=DABS(ddX/(X3-X1))/3.0
            IF(Rate.LE.1) Rate=1.0
            XX=0.5*(X3+X1)-ddX/Rate
            Iget=4
          ENDIF
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | if minimum is bracketed by (XL,XM,XR) and if no XX get   |    **
C**   | or XX goes outside (XL,XR) then use quadratic equation   |    **
C**   +----------------------------------------------------------+    **
      IF(VY.NE.0) THEN
        IF(Iget.EQ.0.OR.(XX-XL)*(XX-XR).GT.0) THEN
          FF=(FR-FM)*(XL-XM)-(FL-FM)*(XR-XM)
          XX=0.5*(XR+XL)
          IF(FF.NE.0) XX=XX-0.5*(FR-FL)*(XR-XM)*(XL-XM)/FF
          Iget=3
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | if none XX get so far, then:                             |    **
C**   | if known at least 2 points, go the downward side         |    **
C**   | if known only one point, set XX=X+dX or XX=X+0.1         |    **
C**   +----------------------------------------------------------+    **
      IF(Iget.EQ.0) THEN
        IF(ST.GE.2) THEN
          IF(F3.GT.F2) THEN
            XX=X2
            X2=X3
            X3=XX
            XX=F2
            F2=F3
            F3=XX
          ENDIF
          XX=X3+(X3-X2)*2.0
          Iget=2
        ELSE
          IF(dX.NE.0) THEN
            XX=X+dX
          ELSE
            XX=X+0.1D0
          ENDIF
          Iget=1
        ENDIF
      ENDIF
C**   ------------------------------------------------------------    **
      DO 30 I=1,14
      B(I)=A(I)
  30  CONTINUE
      IVY=VY
      XMin=XX
      RETURN
      END
************************************************************************
