************************************************************************
      SUBROUTINE ADAMSC(N,FF,DT,T,YI,YO,X,Mmax,Muse,EPSreq,EPSold,ID)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION YI(N),YO(N),X(N,16),ID(3)
      EXTERNAL FF
      DATA MAXC/0/
C**   ============================================================    **
C**   Input T,YI(N) at T=TI;          Output T,YO(N) at T=TO=TI+DT    **
C**   Input ID(3)=-3 for the first step, then DON'T CHANGE ID(1:3)    **
C**   YI(N) & YO(N) can be the same location : Loc.YO(N)=Loc.YI(N)    **
C**   ------------------------------------------------------------    **
C*I   N      = Number of dependent variables                          **
C*S   FF     = Subroutine to compute F(N) for given T & Y(N)          **
C*I   DT     = Time_step = Final_time - Initial_time                  **
CIO   T      = Initial time TI & Final time TO=TI+DT                  **
C*I   YI(N)  = y_i(TI)                                                **
C*O   YO(N)  = y_i(TO)                                                **
C*I   X(N,1:3) = F3(Y3,T3),F2(Y2,T2),F1(Y1,T1) : Tk=TI-k*DT           **
C*O   X(N,1:3) = F2(Y2,T2),F1(Y1,T1),FI(YI,TI) : Yk=y_i(Tk)           **
C*W   X(N,16)  = Buffers X(9*N),X(12*N),X(16*N) for Mmax=(0,1,2)      **
C*W            : X(N,6:8) for SUBROUTINE ADAMS(...,FP,YC,YP)          **
C*W            : X(N,9:9) for YI(N) in case of Loc.YO(N)=Loc.YI(N)    **
C**   -----------------------------------------------------------     **
C**   X1            X2           X3             X4             X5     **
C**     ID() = 1:         X10    X3     X11     X4     X12     X5     **
C**     ID() = 2:                   X13 X11 X14 X4 X15 X12 X16 X5     **
C**             :     ------------- -------------- --------------     **
C**   ID(3)  = 0:2  ===>  ID(1)         ID(2)          ID(3)          **
C**   -----------------------------------------------------------     **
C**   MAXC   = Additional corrector cycles allowed = 0                **
C**   MUSC   = Actual corrector cycles used                           **
C*I   Mmax   = Allowed Max_steps = 2**Mmax                            **
C*O   Muse   = Actual Used_steps = 2**Muse                            **
C*I   EPSreq = Req. epsilon such that Max.|dYC/YC(i)| < EPSreq        **
C*O   EPSold = Max. epsilon = Max_{i=1}^n |1-YP(i)/YC(i)|*19/270      **
C**   To adjust DT : DT_new = (EPSreq/EPSold)**(1/4) * DT / 2**Muse   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | For the initial 3-steps by RUNGEC, also set ID(1:3)=0    |    **
C**   +----------------------------------------------------------+    **
      IF(ID(3).GE.0) GO TO 100
      ID(3)=ID(3)+1
      ISTEP=ID(3)+3
      IF(ISTEP.LE.0) STOP 'ADAMSC: ID(3) < -3'
      ID(ISTEP)=0
      CALL RUNGEC(N,FF,DT,T,YI,YO,X(1,ISTEP),Mmax,Muse,EPSreq,EPSold)
      RETURN
C**   +----------------------------------------------------------+    **
C**   | Find FI(N) = X(N,4)                                      |    **
C**   | Save YI(N) in X(N,9) in case of Loc.YO(N)=Loc.YI(N)      |    **
C**   +----------------------------------------------------------+    **
  100 CALL FF(T,YI,X(1,4),N)
      DO 110 I=1,N
      X(I,9)=YI(I)
  110 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Perform 1_step at DT_step_size                           |    **
C**   +----------------------------------------------------------+    **
      IF(ID(3).GT.0) GO TO 200
      T1=T
      CALL ADAMS(N,FF,DT,T1,YI,YO,X(1,1),X(1,2),X(1,3),X(1,4),X(1,5)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      IF(EPSold.LE.EPSreq.OR.Mmax.LE.0) GO TO 500
      ID(3)=1
      DO 120 I=1,N
      YI(I)=X(I,9)
  120 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Perform 2_steps at DT/2_step_size                        |    **
C**   +----------------------------------------------------------+    **
  200 IF(ID(3).GT.1) GO TO 400
      IF(ID(1).LT.1) THEN
        ID(1)=1
        DO 210 I=1,N
        X(I,10)=(-X(I,1)+9D0*(X(I,2)+X(I,3))-X(I,4))/16D0
  210   CONTINUE
      ENDIF
      IF(ID(2).LT.1) THEN
        ID(2)=1
        DO 220 I=1,N
        X(I,11)=(X(I,2)-4D0*X(I,10)+6D0*X(I,3)+X(I,4))/4D0
  220   CONTINUE
      ENDIF
      T2=T
      HT=DT*0.5D0
      CALL ADAMS(N,FF,HT,T2,YI,YO,X(1,10),X(1,3),X(1,11),X(1,4),X(1,12)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      CALL FF(T2,YO,X(1,12),N)
      CALL ADAMS(N,FF,HT,T2,YO,YO,X(1,3),X(1,11),X(1,4),X(1,12),X(1,5)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      IF(EPSold.LE.EPSreq.OR.Mmax.LE.1) GO TO 500
      ID(3)=2
      DO 240 I=1,N
      YI(I)=X(I,9)
  240 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Perform 4_steps at DT/4_step_size                        |    **
C**   +----------------------------------------------------------+    **
  400 IF(ID(3).GT.2) GO TO 500
      IF(ID(2).LT.1) STOP 'ADAMSC: INTERNAL ERROR !'
      IF(ID(2).LT.2) THEN
        ID(2)=2
        DO 430 I=1,N
        X(I,13)=(-X(I,10)+9D0*(X(I,3)+X(I,11))-X(I,4))/16D0
        X(I,14)=(X(I,10)-5D0*(X(I,3)-X(I,4))+15D0*X(I,11))/16D0
  430   CONTINUE
      ENDIF
      T4=T
      HT=DT*0.25D0
      CALL ADAMS(N,FF,HT,T4,YI,YO,X(1,13),X(1,11),X(1,14),X(1,4),X(1,15)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      CALL FF(T4,YO,X(1,15),N)
      CALL ADAMS(N,FF,HT,T4,YO,YO,X(1,11),X(1,14),X(1,4),X(1,15),X(1,12)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      CALL FF(T4,YO,X(1,12),N)
      CALL ADAMS(N,FF,HT,T4,YO,YO,X(1,14),X(1,4),X(1,15),X(1,12),X(1,16)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      CALL FF(T4,YO,X(1,16),N)
      CALL ADAMS(N,FF,HT,T4,YO,YO,X(1,4),X(1,15),X(1,12),X(1,16),X(1,5)
     *             ,X(1,6),X(1,7),X(1,8),MAXC,MUSC,EPSreq,EPSold)
      IF(EPSold.LE.EPSreq) GO TO 500
      ID(3)=3
C**   +----------------------------------------------------------+    **
C**   | Shift one step                                           |    **
C**   +----------------------------------------------------------+    **
  500 Muse=MIN0(ID(3),2)
      ID(1)=ID(2)
      ID(2)=ID(3)
      ID(3)=MAX0(ID(3)-1,0)
      T=T+DT
C**   +----------------------------------------------------------+    **
C**   | Shift F_values for DT_step_size                          |    **
C**   +----------------------------------------------------------+    **
      DO 510 I=1,N
        X(I,1)=X(I,2)
        X(I,2)=X(I,3)
        X(I,3)=X(I,4)
  510 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Shift F_values for DT/2_step_size & DT/4_step_size       |    **
C**   +----------------------------------------------------------+    **
      IF(ID(1).GE.1) THEN
        DO 520 I=1,N
        X(I,10)=X(I,11)
  520   CONTINUE
      ENDIF
      IF(ID(2).EQ.1) THEN
        DO 530 I=1,N
        X(I,11)=X(I,12)
  530   CONTINUE
      ELSEIF(ID(2).GE.2) THEN
        DO 540 I=1,N
        X(I,11)=X(I,12)
        X(I,13)=X(I,15)
        X(I,14)=X(I,16)
  540   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE ADAMS(N,FF,DT,T,YI,YO,F3,F2,F1,FI,FC,FP,YC,YP
     *                          ,MAXC,MUSC,EPSreq,EPSold)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION YI(N),YO(N),F3(N),F2(N),F1(N)
      DIMENSION FI(N),FC(N),FP(N),YC(N),YP(N)
      EXTERNAL FF
C**   ------------------------------------------------------------    **
C**   Input  T,YI(N),FI(N) at T=TI & F3(N),F2(N),F1(N) at T-3DT,...   **
C**   Output T,YO(N)       at T=TO=TI+DT                              **
C**   YI(N) & YO(N) can be the same location : Loc.YO(N)=Loc.YI(N)    **
C**   ------------------------------------------------------------    **
C*I   N      = Number of dependent variables                          **
C*S   FF     = Subroutine to compute F(N) for given T & Y(N)          **
C*I   DT     = Time_step = Final_time - Initial_time                  **
CIO   T      = Initial time TI & Final time TO=TI+DT                  **
C*I   YI(N)  = y_i(TI)                                                **
C*O   YO(N)  = y_i(TO)                                                **
C*I   F3(N),F2(N),F1(N),FI(N) = f_i(Y3,T3),..,f_i(Y1,T1),f_i(YI,TI)   **
C*W   FC(N),YC(N) = f_i(YC,TO),y_i(TO) Corrector                      **
C*W   FP(N),YP(N) = f_i(YP,TO),y_i(TO) Predictor                      **
C*I   MAXC   = Additional corrector cycles allowed                    **
C*O   MUSC   = Actual corrector cycles used                           **
C*I   EPSreq = Req. epsilon such that Max.|dYC/YC(i)| < EPSreq        **
C*O   EPSold = Max. epsilon = Max_{i=1}^n |1-YP(i)/YC(i)|*19/270      **
C**   ============================================================    **
      DT24=DT/24D0
      DT924=9D0*DT24
      F19270=19D0/270D0
      T=T+DT
C**   +----------------------------------------------------------+    **
C**   | Find the Predictor YP(N) & FP(N)                         |    **
C**   +----------------------------------------------------------+    **
      DO 10 I=1,N
      YP(I)=YI(I)+(55D0*FI(I)-59D0*F1(I)+37D0*F2(I)-9D0*F3(I))*DT24
   10 CONTINUE
      CALL FF(T,YP,FP,N)
C**   +----------------------------------------------------------+    **
C**   | Find the Corrector YC(N)                                 |    **
C**   +----------------------------------------------------------+    **
      DO 20 I=1,N
      YC(I)=YI(I)+(9D0*FP(I)+19D0*FI(I)-5D0*F1(I)+F2(I))*DT24
   20 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find the congergent Corrector                            |    **
C**   +----------------------------------------------------------+    **
      DO 50 MUSC=1,MAXC
      CALL FF(T,YC,FC,N)
      EPSold=0.0
      DO 30 I=1,N
      DY=(FC(I)-FP(I))*DT924
      FP(I)=FC(I)
      YC(I)=YC(I)+DY
      IF(YC(I).NE.0.0) EPSold=DMAX1(EPSold,DABS(DY/YC(I)))
   30 CONTINUE
      IF(EPSold.LE.EPSreq.OR.MUSC.GE.MAXC) GO TO 60
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find accepted YO from YC and YP                          |    **
C**   +----------------------------------------------------------+    **
   60 EPSold=0.0
      DO 70 I=1,N
      DY=(YC(I)-YP(I))*F19270
      YO(I)=YC(I)-DY
      IF(YO(I).NE.0.0) EPSold=DMAX1(EPSold,DABS(DY/YO(I)))
   70 CONTINUE
C**   CALL FF(T,YO,FC,N)
      RETURN
      END
************************************************************************
