************************************************************************
      SUBROUTINE RUNGEC(N,FF,DT,T,YI,YO,X,Mmax,Muse,EPSreq,EPSold)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION YI(N),YO(N),X(N,7)
      EXTERNAL FF
C**   ============================================================    **
C**   Input T,YI(N) at T=TI;          Output T,YO(N) at T=TO=TI+DT    **
C**   Also Output FI(N)=f_i(YI,TI) in X(N,1)                          **
C**   YI(N) & YO(N) can be the same location : Loc.YO(N)=Loc.YI(N)    **
C**   ------------------------------------------------------------    **
C*I   N      = Number of dependent variables                          **
C*S   FF     = Subroutine to compute F(N) for given T & Y(N)          **
C*I   DT     = Time_step = Final_time - Initial_time                  **
CIO   T      = Initial time TI & Final time TO=TI+DT                  **
C*I   YI(N)  = y_i(TI)                                                **
C*O   YO(N)  = y_i(TO)                                                **
C*W   X(N,7) = Buffers X(4*N) if Mmax=0; Buffers X(7*N) if Mmax>0     **
C*I   Mmax   = Index for accuracy control                             **
C**          = 0: No control on accuracy (EPSreq,EPSold,Muse:NA)      **
C**          > 0: Control accuracy with allowed Max_steps = 2**Mmax   **
C*O   Muse   = Actual Used_steps = 2**Muse                            **
C*I   EPSreq = Req. epsilon such that EPSold < EPSreq                 **
C*O   EPSold = Max. epsilon = Max_{i=1}^n |1-Y2(i)/Y1(i)|/15          **
C**   To adjust DT : DT_new = (EPSreq/EPSold)**(1/4) * DT / 2**Muse   **
C**   ============================================================    **
      CALL FF(T,YI,X(1,1),N)
C**   ------------------------------------------------------------    **
      IF(Mmax.LE.0) THEN
        CALL RUNGE(N,FF,DT,T,YI,YO,X(1,1),X(1,2),X(1,3),X(1,4))
C**     CALL FF(T,YO,X(1,2),N)
        RETURN
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Compute Y2(N) = X(N,5) by Large_step_size = 2*h          |    **
C**   +----------------------------------------------------------+    **
      T2=T
      CALL RUNGE(N,FF,DT,T2,YI,X(1,5),X(1,1),X(1,2),X(1,3),X(1,4))
C**   +----------------------------------------------------------+    **
C**   | Compute Y1(N) = X(N,6) by Small_step_size = h = DT/Nstep |    **
C**   +----------------------------------------------------------+    **
      NSTEP=1
      DO 90 Muse=1,Mmax
      NSTEP=NSTEP*2
      HT=DT/NSTEP
C**   +----------------------------------------------------------+    **
C**   | Restart Nstep of HT = h = DT/Nstep                       |    **
C**   +----------------------------------------------------------+    **
      T1=T
      CALL RUNGE(N,FF,HT,T1,YI,X(1,6),X(1,1),X(1,2),X(1,3),X(1,4))
      DO 50 ISTEP=2,NSTEP
      CALL FF(T1,X(1,6),X(1,7),N)
      CALL RUNGE(N,FF,HT,T1,X(1,6),X(1,6),X(1,7),X(1,2),X(1,3),X(1,4))
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find the Max. reletive error = |1 - Y2(I))/Y1(I)|/15     |    **
C**   +----------------------------------------------------------+    **
      EPSold=0.0
      DO 60 I=1,N
      IF(X(I,6).NE.0.) EPSold=DMAX1(EPSold,DABS((1.0-X(I,5)/X(I,6))))
   60 CONTINUE
      EPSold=EPSold/15.0
C**   +----------------------------------------------------------+    **
C**   | If EPSold >  EPSreq : Double the number of steps         |    **
C**   | If EPSold <= EPSreq : Return YO(I)=Y1(I)+(Y1(I)-Y2(I))/15|    **
C**   +----------------------------------------------------------+    **
      IF(EPSold.GT.EPSreq.AND.Muse.LT.Mmax) THEN
        DO 70 I=1,N
        X(I,5)=X(I,6)
   70   CONTINUE
      ELSE
        T=T+DT
        DO 80 I=1,N
        YO(I)=X(I,6)+(X(I,6)-X(I,5))/15D0
   80   CONTINUE
C**     CALL FF(T,YO,X(1,2),N)
        RETURN
      ENDIF
   90 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE RUNGE(N,FF,DT,T,YI,YO,FI,FO,FC,YC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION YI(N),YO(N),FI(N),FO(N),FC(N),YC(N)
      EXTERNAL FF
C**   ------------------------------------------------------------    **
C**   Input  T,YI(N),FI(N) at T=TI                                    **
C**   Output T,YO(N)       at T=TO=TI+DT                              **
C**   YI(N) & YO(N) can be the same location : Loc.YO(N)=Loc.YI(N)    **
C**   ------------------------------------------------------------    **
C*I   N      = Number of dependent variables                          **
C*S   FF     = Subroutine to compute F(N) for given T & Y(N)          **
C*I   DT     = Time_step = Final_time - Initial_time                  **
CIO   T      = Initial time TI & Final time TO=TI+DT                  **
C*I   YI(N)  = y_i(TI)                                                **
C*O   YO(N)  = y_i(TO)                                                **
C*I   FI(N)  = f_i(YI,TI)                                             **
C*W   FO(N)  = Working array for Fa(N),Fa(N)+Fb(N)                    **
C*W   FC(N)  = Working array for       Fb(N),Fc(N)                    **
C*W   YC(N)  = Working array for Ya(N),Yb(N),Yc(N)                    **
C**   ============================================================    **
      HT=DT*0.5
      T=T+HT
      DO 10 I=1,N
      YC(I)=YI(I)+FI(I)*HT
   10 CONTINUE
      CALL FF(T,YC,FO,N)
C**   ------------------------------------------------------------    **
      DO 20 I=1,N
      YC(I)=YI(I)+FO(I)*HT
   20 CONTINUE
      CALL FF(T,YC,FC,N)
C**   ------------------------------------------------------------    **
      T=T+HT
      DO 30 I=1,N
      YC(I)=YI(I)+FC(I)*DT
      FO(I)=FO(I)+FC(I)
   30 CONTINUE
      CALL FF(T,YC,FC,N)
C**   ------------------------------------------------------------    **
      HT=DT/6.0
      DO 40 I=1,N
      YO(I)=YI(I)+(FI(I)+FO(I)+FO(I)+FC(I))*HT
   40 CONTINUE
C**   CALL FF(T,YO,FO,N)
      RETURN
      END
************************************************************************
