************************************************************************
      PROGRAM ELPTST
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION U(51,51),P(51,51),IP(51,51),BL(51),BD(51),BR(51),BU(51)
      DATA NDIM/51/
C**   ============================================================    **
C**   This program setup the data for calling ELLIP* to solve         **
C**   Poisson equation : d^2u/dx^2 + d^2u/dy^2 = p, x^2+y^2 < r^2     **
C**   with boundary condition :     u(x,y) = b  on  x^2+y^2 = r^2     **
C**   ============================================================    **
      READ(*,'(I4,5F8.0,I4)') N,R,P0,B,W,EPS,NITER
      IF(N/2*2.EQ.N.OR.N.GT.NDIM) STOP 'N IS EVEN OR N.GT.NDIM'
      H=2*R/(N-1)
      PHH=P0*H*H
C**   +----------------------------------------------------------+    **
C**   | Initialize                                               |    **
C**   +----------------------------------------------------------+    **
      DO 20 J=1,N
      DO 10 I=1,N
      U(I,J)=0
      IP(I,J)=-3
   10 CONTINUE
   20 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Boundary pt on horizontal line, set theta, U()=B, IP()=-2|    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,N
      Y=(J-N/2-1)*H
      X=DSQRT(R**2-Y**2)
      MI=X/H
      THETA=DABS(X-MI*H)/H
      IF(THETA.LT.0.333333) THEN
        THETA=1+THETA
        MI=MI-1
      ENDIF
      BL(J)=THETA
      BR(J)=THETA
      U(-MI-1+N/2+1,J)=B   ! U(-MI-1+N/2+1,J)=b(-X,Y)
      U( MI+1+N/2+1,J)=B   ! U( MI+1+N/2+1,J)=b(+X,Y)
      IP(-MI-1+N/2+1,J)=-2
      IP( MI+1+N/2+1,J)=-2
C**   +----------------------------------------------------------+    **
C**   | For interior points, set  P()=p(x,y)*h*h and IP()=0      |    **
C**   +----------------------------------------------------------+    **
      DO 30 I=-MI,MI
      P(I+N/2+1,J)=PHH     ! P(I+N/2+1,J)=p(X,Y)*H*H ; X=I*H
      IP(I+N/2+1,J)=0
   30 CONTINUE
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Boundary pt on vertical line, set theta, U()=B, IP()=-2  |    **
C**   +----------------------------------------------------------+    **
      DO 50 I=1,N
      X=(I-N/2-1)*H
      Y=DSQRT(R**2-X**2)
      MJ=Y/H
      THETA=DABS(Y-MJ*H)/H
      IF(THETA.LT.0.333333) THEN
        THETA=1+THETA
        MJ=MJ-1
      ENDIF
      BD(I)=THETA
      BU(I)=THETA
      U(I,-MJ-1+N/2+1)=B   ! U(I,-MJ-1+N/2+1)=b(X,-Y)
      U(I, MJ+1+N/2+1)=B   ! U(I, MJ+1+N/2+1)=b(X,+Y)
      IP(I,-MJ-1+N/2+1)=-2
      IP(I, MJ+1+N/2+1)=-2
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Calculate acceleration factor W for subroutine ELLIPT    |    **
C**   +----------------------------------------------------------+    **
      IF(W.EQ.0.0) W=2/(1+DSQRT(1-DCOS(3.14159D0/(N-1))**2))
C**   +----------------------------------------------------------+    **
C**   | Compute U(I,J) and Output results                        |    **
C**   +----------------------------------------------------------+    **
      IF(NITER.NE.0) THEN
        CALL ELLIPT(U,P,IP,BL,BD,BR,BU,W,EPS,NITER,N,N,NDIM)
      ELSE
        CALL ELLIPS(U,P,IP,BL,BD,BR,BU,N,N,NDIM)
      ENDIF
      DO 80 J=N,1,-1
   80 WRITE(*,'(1X,25I3)') (IP(I,J),I=1,N)
      DO 90 J=N,1,-1
   90 WRITE(*,'(1X,10F8.4)') (U(I,J),I=1,N)
      STOP
      END
************************************************************************
      SUBROUTINE ELLIPT(U,P,IP,BL,BD,BR,BU,W,EPS,NITER,N,M,NDIM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION U(NDIM,M),P(NDIM,M),IP(NDIM,M),BL(M),BD(M),BR(N),BU(N)
C**   ============================================================    **
C**   Use Gauss-Seidel iterations to solve Poisson or Laplace eq.     **
C**   ------------------------------------------------------------    **
C**            Output : IP(i,j)        -3 : Outside point             **
C**   M  -3 -3 -2 -2 -2 -2 -2 -3 -3    -2 : Boundary point            **
C**  (U) -3 -2  9  8  8  8 10 -2 -3     0 : Inside point              **
C**      -2  9  0  0  0  0  0 10 -2     1 : Boundary on Left  side    **
C**      -2  1  0  0  0  0  0  2 -2     2 : Boundary on Right side    **
C**   j  -2  1  0  0  0  0  0  2 -2     4 : Boundary on Down  side    **
C**      -2  1  0  0  0  0  0  2 -2     8 : Boundary on Up    side    **
C**      -2  5  0  0  0  0  0  6 -2     5 = 1+4 : Bou. on L+D side    **
C**  (D) -3 -2  5  4  4  4  6 -2 -3     6 = 2+4 : Bou. on R+D side    **
C**   1  -3 -3 -2 -2 -2 -2 -2 -3 -3     9 = 1+8 : Bou. on L+U side    **
C**       1(L)        i        (R)N    10 = 2+8 : Bou. on R+U side    **
C**   ------------------------------------------------------------    **
C*I   U(I,J) = Assumed initial function values for IP=0,-2            **
C*O          = Computed function values for IP=0                      **
C*I   P(I,J) = p(x,y)*h*h (h=grid spacing) for IP=0                   **
C*I   BL(J)  = Distance_to_boundary_point / h for IP=-2               **
C*I   W      = Acceleration factor (1 <= W <= 2)                      **
C*I   EPS    = Convergence if |Unew-Uold| < |Uold|*EPS                **
C*I   NITER  = Maximum number of iterations allowed                   **
C*I   N,M    = Grid points in X/Y-dirctions                           **
C*I   NDIM   = DIMENSION of U(NDIM,M),P(NDIM,M),IP(NDIM,M)            **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | For points near boundary : set IP()=(1+2+4+8)            |    **
C**   +----------------------------------------------------------+    **
      DO 20 J=1,M
      DO 10 I=1,N
      IF(IP(I,J).NE.0) GO TO 10
      IF(IP(I-1,J).EQ.-2) IP(I,J)=IP(I,J)+1
      IF(IP(I,J-1).EQ.-2) IP(I,J)=IP(I,J)+4
      IF(IP(I+1,J).EQ.-2) IP(I,J)=IP(I,J)+2
      IF(IP(I,J+1).EQ.-2) IP(I,J)=IP(I,J)+8
   10 CONTINUE
   20 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Use Gauss-Seidel iterations to solve U(I,J)              |    **
C**   +----------------------------------------------------------+    **
      DO 60 ITER=1,NITER
      NERR=0
      DO 50 J=1,M
      DO 40 I=1,N
      UOLD=U(I,J)
      K=IP(I,J)
      IF(K.LE.-2) GO TO 40
      IF(K.EQ.0) THEN
        UNEW=(U(I-1,J)+U(I,J-1)+U(I+1,J)+U(I,J+1)-P(I,J))*0.25
      ELSE IF(K.GT.0) THEN
        FL=1
        FD=1
        FR=1
        FU=1
        IF(MOD(K  ,2).EQ.1) FL=BL(J)
        IF(MOD(K/4,2).EQ.1) FD=BD(I)
        IF(MOD(K/2,2).EQ.1) FR=BR(J)
        IF(MOD(K/8,2).EQ.1) FU=BU(I)
        UNEW=(U(I-1,J)/(FL*(FL+FR))+U(I,J-1)/(FD*(FD+FU))
     *       +U(I+1,J)/(FR*(FL+FR))+U(I,J+1)/(FU*(FD+FU))
     *       -P(I,J)*0.5)*FL*FR*FD*FU/(FL*FR+FD*FU)
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Convergence control                                      |    **
C**   +----------------------------------------------------------+    **
      U(I,J)=(1-W)*UOLD+W*UNEW
      IF(DABS(U(I,J)-UOLD).GT.DABS(UOLD)*EPS) NERR=NERR+1
   40 CONTINUE
   50 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Debug output and convergence test                        |    **
C**   +----------------------------------------------------------+    **
      IF(MOD(ITER,10).EQ.1.OR.NITER-ITER.LE.10.OR.NERR.EQ.0) THEN
        WRITE(*,'('' ITER :'',I4)') ITER
        DO 55 J=N,1,-1
   55   WRITE(*,'(1X,10F8.4)') (U(I,J),I=1,N)
      ENDIF
      IF(NERR.EQ.0) RETURN
   60 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE ELLIPS(U,P,IP,BL,BD,BR,BU,N,M,NDIM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION U(NDIM,M),P(NDIM,M),IP(NDIM,M),BL(M),BD(M),BR(N),BU(N)
      DIMENSION S(180000),T(3000),L(2,3000)
      DATA MAXA,MAXN/180000,3000/
C**   ============================================================    **
C**   Use direct method to solve Poisson or Laplace equation          **
C**   ------------------------------------------------------------    **
C**             Input : IP(i,j)               Output : IP(i,j)        **
C**   M  -3 -3 -2 -2 -2 -2 -2 -3 -3     -3 -3 -2 -2 -2 -2 -2 -3 -3    **
C**  (U) -3 -2  0  0  0  0  0 -2 -3     -3 -2 41 42 43 44 45 -2 -3    **
C**      -2  0  0  0  0  0  0  0 -2     -2 34 35 36 37 38 39 40 -2    **
C**      -2  0  0  0  0  0  0  0 -2     -2 27 28 29 30 31 32 33 -2    **
C**   j  -2  0  0  0  0  0  0  0 -2     -2 20 21 22 23 24 25 26 -2    **
C**      -2  0  0  0  0  0  0  0 -2     -2 13 14 15 16 17 18 19 -2    **
C**      -2  0  0  0  0  0  0  0 -2     -2  6  7  8  9 10 11 12 -2    **
C**  (D) -3 -2  0  0  0  0  0 -2 -3     -3 -2  1  2  3  4  5 -2 -3    **
C**   1  -3 -3 -2 -2 -2 -2 -2 -3 -3     -3 -3 -2 -2 -2 -2 -2 -3 -3    **
C**       1(L)        i        (R)N      1(L)        i        (R)N    **
C**   ------------------------------------------------------------    **
C*I   U(I,J) = Assumed initial function values for IP=0,-2            **
C*O          = Computed function values for IP=0                      **
C*I   P(I,J) = p(x,y)*h*h (h=grid spacing) for IP=0                   **
C*I   BL(J)  = Distance_to_boundary_point / h for IP=-2               **
C*I   N,M    = Grid points in X/Y-dirctions                           **
C*I   NDIM   = DIMENSION of U(NDIM,M),P(NDIM,M),IP(NDIM,M)            **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | For interior points : set IP() = NEQ = Equation Number   |    **
C**   +----------------------------------------------------------+    **
      NEQ=0
      DO 20 J=1,M
      DO 10 I=1,N
      IF(IP(I,J).LE.-2) GO TO 10
      NEQ=NEQ+1
      IP(I,J)=NEQ
   10 CONTINUE
   20 CONTINUE
      IF(NEQ.GT.MAXN) WRITE(*,'(I8,'' > MAXN'')') NEQ
      IF(NEQ.GT.MAXN) STOP 'DIMENSION TOO SMALL'
C**   +----------------------------------------------------------+    **
C**   | Find L(1,K) = Top  non-zero row_number                   |    **
C**   |  and L(2,K) = Left non-zero col_number                   |    **
C**   +----------------------------------------------------------+    **
      DO 30 K=1,NEQ
   30 L(1,K)=K
      DO 40 J=1,M
      DO 35 I=1,N
      K=IP(I,J)
      IF(K.LT.0) GO TO 35
      KMIN=K
      IF(IP(I-1,J).GT.0) KMIN=MIN0(KMIN,IP(I-1,J))
      IF(IP(I,J-1).GT.0) KMIN=MIN0(KMIN,IP(I,J-1))
      IF(IP(I+1,J).GT.K) L(1,IP(I+1,J))=MIN0(K,L(1,IP(I+1,J)))
      IF(IP(I,J+1).GT.K) L(1,IP(I,J+1))=MIN0(K,L(1,IP(I,J+1)))
      L(2,K)=KMIN
   35 CONTINUE
   40 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find L(1,K)/L(2,K) = K_col/K_row location index          |    **
C**   +----------------------------------------------------------+    **
      L(1,1)=0
      L(2,1)=0
      DO 50 K=2,NEQ
      L(1,K)=L(2,K-1)+(K-L(1,K))
   50 L(2,K)=L(1,K)+(K-L(2,K))
      IF(L(2,NEQ)+NEQ.GT.MAXA) WRITE(*,'(I8,'' > MAXA'')') L(2,NEQ)+NEQ
      IF(L(2,NEQ)+NEQ.GT.MAXA) STOP 'DIMENSION TOO SMALL'
C**   +----------------------------------------------------------+    **
C**   | Initialize and Form S(NEQ,NEQ) & T(NEQ)                  |    **
C**   +----------------------------------------------------------+    **
      DO 60 K=1,L(2,NEQ)+NEQ
   60 S(K)=0.0
      DO 70 J=1,M
      DO 65 I=1,N
      K=IP(I,J)
      IF(K.LT.0) GO TO 65
      IF(IP(I-1,J).NE.-2.AND.IP(I,J-1).NE.-2.AND.
     *   IP(I+1,J).NE.-2.AND.IP(I,J+1).NE.-2) THEN
C**     +--------------------------------------------------------+    **
C**     | All 5-points inside the boundary                       |    **
C**     +--------------------------------------------------------+    **
        T(K)=-P(I,J)
        S(L(2,K)+K)=4.0
        S(L(2,K)+IP(I-1,J))=-1
        S(L(2,K)+IP(I,J-1))=-1
        S(L(1,IP(I+1,J))+K)=-1
        S(L(1,IP(I,J+1))+K)=-1
      ELSE
C**     +--------------------------------------------------------+    **
C**     | Some given points on the boundary                      |    **
C**     +--------------------------------------------------------+    **
        FL=1
        FD=1
        FR=1
        FU=1
        IF(IP(I-1,J).EQ.-2) FL=BL(J)
        IF(IP(I,J-1).EQ.-2) FD=BD(I)
        IF(IP(I+1,J).EQ.-2) FR=BR(J)
        IF(IP(I,J+1).EQ.-2) FU=BU(I)
        T(K)=-P(I,J)
        S(L(2,K)+K)=2/(FL*FR)+2/(FD*FU)
        IF(IP(I-1,J).EQ.-2) T(K)=T(K)+U(I-1,J)*2.0/(FL*(FL+FR))
        IF(IP(I-1,J).NE.-2) S(L(2,K)+IP(I-1,J))=-2/(FL*(FL+FR))
        IF(IP(I,J-1).EQ.-2) T(K)=T(K)+U(I,J-1)*2.0/(FD*(FD+FU))
        IF(IP(I,J-1).NE.-2) S(L(2,K)+IP(I,J-1))=-2/(FD*(FD+FU))
        IF(IP(I+1,J).EQ.-2) T(K)=T(K)+U(I+1,J)*2.0/(FR*(FL+FR))
        IF(IP(I+1,J).NE.-2) S(L(1,IP(I+1,J))+K)=-2/(FR*(FL+FR))
        IF(IP(I,J+1).EQ.-2) T(K)=T(K)+U(I,J+1)*2.0/(FU*(FD+FU))
        IF(IP(I,J+1).NE.-2) S(L(1,IP(I,J+1))+K)=-2/(FU*(FD+FU))
      ENDIF
   65 CONTINUE
   70 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Solve linear equation : S(NEQ,NEQ) * U(NEQ) = T(NEQ)     |    **
C**   +----------------------------------------------------------+    **
      CALL USDECP(S,L,NEQ,NNT,NSD,MAXA,MAXN)
      CALL USSOLX(S,T,L,NEQ,NNT,T)
C**   +----------------------------------------------------------+    **
C**   | Get function values U(I,J) from T(IP(I,J))               |    **
C**   +----------------------------------------------------------+    **
      DO 80 J=M,1,-1
      DO 75 I=1,N
      IF(IP(I,J).GT.0) U(I,J)=T(IP(I,J))
   75 CONTINUE
      WRITE(*,'(1X,10F8.4)') (U(I,J),I=1,N)
   80 CONTINUE
      RETURN
      END
************************************************************************
