************************************************************************
      SUBROUTINE LINEAR
     *(A,LI,LJ,D,X,II,JJ,NX,ICASE,KKK,NIA,NJA,T,MI,MJ,IS,IC)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NIA,NJA),LI(NIA),LJ(NJA),D(NX),X(NX)
      DIMENSION T(NIA,NJA),MI(NIA),MJ(NJA)
      DATA EPS,BIG/1.0D-6,1.0D10/
C**   ============================================================    **
C**   Linear programming with upper bound by simplex method           **
C**   ------------------------------------------------------------    **
C**              1                        JJ-1   JJ                   **
C**             +-----------------------------+------+                **
C**          1  |                             |      |                **
C**             |                             >=     | L(I)<0         **
C**             |              A              =   B  | L(I)=0         **
C**             |                             <=     | L(I)>0         **
C**        II-1 |                             |      |                **
C**             +-----------------------------+------+                **
C**         II  |              C              |  -Z  |                **
C**             +-----------------------------+------+                **
C**        II+1 |              C'             |  -Z' |  *  M          **
C**             +-----------------------------+------+                **
C**   ------------------------------------------------------------    **
C**   Minimize: Z = \sum C(J)*X(LJ(J))                                **
C**   Subject to: \sum A(I,J)*X(LJ(J)) [-|+] X([-|+]LI(I)) = B(I)     **
C**       and   :     D(K) >= X(K) >= 0                               **
C**     where:  I = 1 to II-1;    J = 1 to JJ-1;    K = 1 to NX       **
C**   ------------------------------------------------------------    **
C**   Input  : A(II  ,JJ),LI(II-1),LJ(JJ-1),D(NX),II,JJ,NX,NIA,NJA,IS,IC
C**   Output : A(II+1,JJ),LI(II-1),LJ(JJ-1),D(NX),X(NX),ICASE,KKK     **
C**   Working: T(II  ,JJ),MI(II-1),MJ(JJ-1)                           **
C**            NIA >= II+1;    NJA >= JJ                              **
C**   ------------------------------------------------------------    **
C**   If IS.GT.0 then T unused & IS=file # to save A                  **
C**   If IS.EQ.0 then T,MI,MJ unused                                  **
C**   If IS.LT.0 then T is a working array to save A                  **
C**   ============================================================    **
C01   +----------------------------------------------------------+    **
C**   | Call CHKOUT is for print out only, it can be deleted     |    **
C**   +----------------------------------------------------------+    **
      CALL CHKOUT(A,LI,LJ,D,LI,LJ,II,JJ,NX,NIA,NJA,IC,0,1,1)
C**   +----------------------------------------------------------+    **
C**   | KKKK,KKKO = Re-perform control variables                 |    **
C**   | KKK       = Number of simplex pivot                      |    **
C**   +----------------------------------------------------------+    **
      KKKK=0
      KKKO=0
      KKK=0
C02   +----------------------------------------------------------+    **
C**   | Re-set L(I) from the input values of (-s,0,+s)           |    **
C**   | (1) Change all constraints to <= B(I), such that:        |    **
C**   |     L(I) > 0 : for slack variable X(I),      I<=NX       |    **
C**   |     L(I) < 0 : for artificial variable X(I), I> NX       |    **
C**   | (2) Make sure that D(I) >= 0 for non-artificial variable |    **
C**   +----------------------------------------------------------+    **
      NY=NX
      DO 130 I=1,II-1
      IF(LI(I).LT.0) THEN
        DO 125 J=1,JJ
        A(I,J)=-A(I,J)
  125   CONTINUE
        LI(I)=-LI(I)
      ELSE IF(LI(I).EQ.0) THEN
        NY=NY+1
        LI(I)=-NY
      ENDIF
  130 CONTINUE
C**   ------------------------------------------------------------    **
      DO 150 J=1,NX
      IF(D(J).LT.0.0) STOP 1111
  150 CONTINUE
C03   +----------------------------------------------------------+    **
C**   | Save A(II,JJ) on (1) file# IS>0 or (2) T(II,JJ) if IS<0  |    **
C**   | For IS=0, A(II,JJ) will not be saved and no re-perform   |    **
C**   +----------------------------------------------------------+    **
      IF(IS.GT.0) THEN
        REWIND IS
        WRITE (IS) ((A(I,J),I=1,II),J=1,JJ)
      ELSE IF(IS.LT.0) THEN
        DO 170 J=1,JJ
        DO 165 I=1,II
        T(I,J)=A(I,J)
  165   CONTINUE
  170   CONTINUE
      ENDIF
C04   +----------------------------------------------------------+    **
C**   | Save |LI(I)| & |LJ(J)| on MI(I) & MJ(J) for re-perform   |    **
C**   +----------------------------------------------------------+    **
      IF(IS.NE.0) THEN
        DO 180 I=1,II-1
        MI(I)=IABS(LI(I))
  180   CONTINUE
        DO 190 J=1,JJ-1
        MJ(J)=IABS(LJ(J))
  190   CONTINUE
      ENDIF
C05   +----------------------------------------------------------+    **
C**   | Set up simplex tableau :                                 |    **
C**   | (1) Reset A(II+1,*)=0 : Objective fun. for artifical var.|    **
C**   | (2) Set LJ(J)<0 : To disqualify J as basic variable, if  |    **
C**   |     (2a) Artificial variables  (J>NX)                    |    **
C**   |     (2b) Non-artificial variables (J<=NX) but D(J) = 0   |    **
C**   | KK=II+1 : Phase 1 objective function at row II+1         |    **
C**   | KK=II   : Phase 2 objective function at row II           |    **
C**   +----------------------------------------------------------+    **
  200 KK=II+1
      A(KK,JJ)=0.0
      DO 210 J=1,JJ-1
        A(KK,J)=0.0
        LJJ=IABS(LJ(J))
        IF(LJJ.GT.NX) THEN
          LJJ=-LJJ
        ELSE IF(D(LJJ).EQ.0.0) THEN
          LJJ=-LJJ
        ENDIF
        LJ(J)=LJJ
  210 CONTINUE
C06   +----------------------------------------------------------+    **
C**   | (3) Change B(I)=A(I,JJ) to >= 0     (for I=1 to II-1)    |    **
C**   +----------------------------------------------------------+    **
      DO 270 I=1,II-1
      IF(A(I,JJ).LT.0.0) THEN
        DO 230 J=1,JJ
        A(I,J)=-A(I,J)
  230   CONTINUE
        LI(I)=-LI(I)
      ENDIF
C07   +----------------------------------------------------------+    **
C**   | (4) For artificial var (I>NX) : LI(I) must be < 0        |    **
C**   +----------------------------------------------------------+    **
      IF(LI(I).GT.NX) LI(I)=-LI(I)
C08   +----------------------------------------------------------+    **
C**   | (5) Basic var X(L(I))=A(I,JJ) must be <= D(LI(I))        |    **
C**   |     otherwise change basic var S to D-S': i.e. change :  |    **
C**   |     (5a) A(I,JJ)  to A(I,JJ)-D(LI(I))                    |    **
C**   |     (5b) D(LI(I)) to -D(LI(I)) to indicate bounded var   |    **
C**   |     (5c) LI(I)    to -LI(I)  to indicate non-basic var   |    **
C**   +----------------------------------------------------------+    **
      LII=LI(I)
      IF(LII.GT.0) THEN
        IF(A(I,JJ).GT.D(LII)) THEN
          A(I,JJ)=A(I,JJ)-D(LII)
          D(LII)=-D(LII)
          LI(I)=-LII
        ENDIF
      ENDIF
C09   +----------------------------------------------------------+    **
C**   | (6) For any constraint not satisfied (LI(I) <= 0)        |    **
C**   |     substract from A(II+1,J) by A(I,J)                   |    **
C**   +----------------------------------------------------------+    **
      IF(LI(I).LE.0) THEN
        DO 260 J=1,JJ
        A(KK,J)=A(KK,J)-A(I,J)
  260   CONTINUE
      ENDIF
  270 CONTINUE
C10   +----------------------------------------------------------+    **
C**   | Feasibility check                                        |    **
C**   +----------------------------------------------------------+    **
      ICASE=0
      DO 280 J=1,JJ-1
C**   +----------------------------------------------------------+    **
C**   | LJ(J)<0 indicate X(J) must be 0 & can not be a basic var |    **
C**   | So, test only for J<JJ and LJ(J)>0                       |    **
C**   | (0) Stay at phase 1 (go to 300) :   if any A(II+1,J)<0   |    **
C**   | (1) Feasible (ICASE=0, phase 2) :   if all A(II+1,J)=0   |    **
C**   | (2) May be infeasible : if not (0) and any A(II+1,J)>0   |    **
C**   +----------------------------------------------------------+    **
      IF(LJ(J).LE.0) GO TO 280
      IF(ABS(A(KK,J)).LE.EPS) GO TO 280
      IF(A(KK,J).LT.0.0) GO TO 300
      ICASE=-1
  280 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | (2a) A(II+1,JJ)< 0 : infeasible (ICASE=-1 & go to 710)   |    **
C**   | (2b) A(II+1,JJ)>=0 : feasible, set LJ(J)<0 if A(II+1,J)>0|    **
C**   +----------------------------------------------------------+    **
      IF(ICASE.LT.0) THEN
        IF(A(KK,JJ).LT.-EPS) GO TO 710
        DO 285 J=1,JJ-1
        IF(A(KK,J).GT.0.0) LJ(J)=-IABS(LJ(J))
  285   CONTINUE
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Feasible case : get into phase 2 to find the optimal sol |    **
C**   +----------------------------------------------------------+    **
      ICASE=0
      KK=II
C11   +----------------------------------------------------------+    **
C**   | Begin simplex pivot  (KK=II+1:phase 1) (KK=II:phase 2)   |    **
C**   +----------------------------------------------------------+    **
C**   | Find XMIN=A(KK,KJ)=min.A(KK,1:JJ-1) -> XMIN<0 & LJ(KJ)>0 |    **
C**   | Case 1:  P(LJ(KJ)) = +1   (P(LJ(KJ))=SIGN(1,D(LJ(KJ))))  |    **
C**   | Case 2:  P(LJ(KJ)) = -1                                  |    **
C**   | If all A(KK,1:JJ-1) >= 0 : End of phase 1/2 -> go to 700 |    **
C**   +----------------------------------------------------------+    **
  300 IF(KKK.EQ.0)CALL CHKOUT(A,LI,LJ,D,LI,LJ,II,JJ,NX,NIA,NJA,IC,0,1,1)
      XMIN=-EPS
      KJ=0
      DO 310 J=1,JJ-1
      IF(A(KK,J).GE.XMIN.OR.LJ(J).LE.0) GO TO 310
        XMIN=A(KK,J)
        KJ=J
  310 CONTINUE
      IF(KJ.LE.0) GO TO 700
C12   +----------------------------------------------------------+    **
C**   | Find XMIN=min.XI      (for KI=1 to II-1)                 |    **
C**   | Case A: A(KI,KJ)>0 -> XI=A(KI,JJ)/A(KI,KJ)               |    **
C**   | Case B: A(KI,KJ)<0 -> XI=(A(KI,JJ)-D(LI(I)))/A(KI,KJ)    |    **
C**   | Case C: (KI=0)     -> XI=D(LJ(KJ))                       |    **
C**   +----------------------------------------------------------+    **
      LJK=LJ(KJ)
      XMIN=ABS(D(LJK))
      KI=0
      DO 450 I=1,II-1
      IF(ABS(A(I,KJ)).LE.EPS) GO TO 450
        IF(A(I,KJ).GT.0.0) THEN
          XI=A(I,JJ)/A(I,KJ)
        ELSE
C**       +------------------------------------------------------+    **
C**       | Case B:  for slack var LI(I)>0 and bounded var only  |    **
C**       +------------------------------------------------------+    **
          LII=LI(I)
          IF(LII.LE.0) GO TO 450
          IF(D(LII).GE.BIG) GO TO 450
          XI=(A(I,JJ)-D(LII))/A(I,KJ)
        ENDIF
C**     ----------------------------------------------------------    **
        IF(XI.GT.XMIN) GO TO 450
        IF(XI.EQ.XMIN.AND.KI.NE.0) THEN
          IF(LI(I).GT.LI(KI)) GO TO 450
        ENDIF
          XMIN=XI
          KI=I
  450 CONTINUE
C13   +----------------------------------------------------------+    **
C**   | If XMIN >= BIG : Un-bounded for new basic var (go to 740)|    **
C**   +----------------------------------------------------------+    **
      IF(XMIN.GE.BIG) GO TO 740
C14   +----------------------------------------------------------+    **
C**   | Cases 1A,2A,1B,2B (KI>0)                                 |    **
C**   +----------------------------------------------------------+    **
      IF(KI.GT.0) THEN
        LKI=LI(KI)
        LIK=IABS(LKI)
        BKK=ISIGN(1,LKI)
C**     +--------------------------------------------------------+    **
C**     | Sweep LI(KI),LJ(KJ) and Set LJ(KJ) to <0 for artif. var|    **
C**     +--------------------------------------------------------+    **
        LI(KI)=LJK
        LJ(KJ)=LIK
        IF(LIK.GT.NX) LJ(KJ)=-LIK
C15     +--------------------------------------------------------+    **
C**     | Cases 1B,2B : Change basic var to non-bas var X -> D-X'|    **
C**     | (A(KI,KJ)<0)  X(LIK)'= D(LIK)-X(LIK) = 0, P(LIK) = -1  |    **
C**     +--------------------------------------------------------+    **
        IF(A(KI,KJ).LT.0.0) THEN
          A(KI,JJ)=A(KI,JJ)-D(LIK)
          D(LIK)=-D(LIK)
          BKK=-1.0
        ENDIF
      ENDIF
C16   +----------------------------------------------------------+    **
C**   | Case 2A,2B (KI>0) : X(LJK) < D(LJK), P(LJK) = -1 --> +1  |    **
C**   | Case 1C    (KI=0) : X(LJK)'= 0,      P(LJK) = +1 --> -1  |    **
C**   | Case 2C    (KI=0) : X(LJK) = 0,      P(LJK) = -1 --> +1  |    **
C**   +----------------------------------------------------------+    **
      IF(D(LJK).LT.0.0.OR.KI.EQ.0) THEN
        DJ=ABS(D(LJK))
        DO 540 I=1,KK
          A(I,JJ)=A(I,JJ)-A(I,KJ)*DJ
          A(I,KJ)=-A(I,KJ)
  540   CONTINUE
        D(LJK)=-D(LJK)
      ENDIF
C17   +----------------------------------------------------------+    **
C**   | Cases 1A,2A,1B,2B (KI>0) : Perform Gauss elimination     |    **
C**   +----------------------------------------------------------+    **
      IF(KI.GT.0) CALL GAUSS(A,II,JJ,KI,KJ,BKK,KK,LKI,NIA,NJA)
C18   +----------------------------------------------------------+    **
C**   | End of simplex loop : go to Begin of loop (300)          |    **
C**   +----------------------------------------------------------+    **
      KKK=KKK+1
      CALL CHKOUT(A,LI,LJ,D,MI,MJ,II,JJ,NX,NIA,NJA,IC,KKK,KI,KJ)
      GO TO 300
C19   +----------------------------------------------------------+    **
C**   | Re-perform phase 1 simplex 5-times maximum               |    **
C**   +----------------------------------------------------------+    **
  700 IF(KK.EQ.II) GO TO 710
      IF(KKKK.GE.5.OR.KKKO.EQ.KKK) GO TO 750
      KKKK=KKKK+1
      KKKO=KKK
      IF(IS.NE.0) THEN
      CALL RECOMP(A,LI,LJ,D,II,JJ,NX,NIA,NJA,T,MI,MJ,IS)
      CALL CHKOUT(A,LI,LJ,D,MI,MJ,II,JJ,NX,NIA,NJA,IC,0,1,1)
      ENDIF
      GO TO 200
C20   +----------------------------------------------------------+    **
C**   | Feasible solution in X(*) (also for Unbounded/Infeasible)|    **
C**   +----------------------------------------------------------+    **
  710 DO 720 J=1,NX
      X(J)=MAX(0.0D0,-D(J))
  720 CONTINUE
      DO 730 I=1,II-1
      IF(LI(I).GT.0) X(LI(I))=A(I,JJ)
  730 CONTINUE
      DO 735 J=1,NX
      X(J)=MIN(X(J),ABS(D(J)))
  735 CONTINUE
      RETURN
C21   +----------------------------------------------------------+    **
C**   | Unbounded solution                                       |    **
C**   +----------------------------------------------------------+    **
  740 ICASE=1
      GO TO 710
C22   +----------------------------------------------------------+    **
C**   | Infeasible solution                                      |    **
C**   +----------------------------------------------------------+    **
  750 ICASE=-2
      GO TO 710
      END
************************************************************************
      SUBROUTINE GAUSS(A,II,JJ,KI,KJ,BKK,KK,LKI,NIA,NJA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NIA,NJA)
C**   ============================================================    **
C**   Perform Gauss elimination pivot at A(KI,KJ)                     **
C**   A(*,KJ) will be used by new non-basic var   (BKK = -1 or +1)    **
C**   LKI=LI(KI) < 0 : New non-bas var is art var (has 1*M in obj)    **
C**   ============================================================    **
      AKK=A(KI,KJ)
      A(KI,KJ)=BKK
      DO 630 J=1,JJ
      A(KI,J)=A(KI,J)/AKK
  630 CONTINUE
      DO 640 I=1,KK
      IF(I.EQ.KI) GO TO 640
        AIK=A(I,KJ)
        A(I,KJ)=0.0
        IF(I.EQ.II+1.AND.LKI.LT.0) A(I,KJ)=1.0
        DO 635 J=1,JJ
        A(I,J)=A(I,J)-AIK*A(KI,J)
  635   CONTINUE
  640 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE RECOMP(A,LI,LJ,D,II,JJ,NX,NIA,NJA,T,MI,MJ,IS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NIA,NJA),LI(NIA),LJ(NJA),D(NX)
      DIMENSION T(NIA,NJA),MI(NIA),MJ(NJA)
      DATA EPS/1.0D-6/
C**   ============================================================    **
C**   Perform Gauss elimination from the original matrix              **
C**   For phase 1 only : To reduce roundoff error                     **
C**   ============================================================    **
  800 IF(IS.GT.0) THEN
        REWIND IS
        READ (IS) ((A(I,J),I=1,II),J=1,JJ)
      ELSE IF(IS.LT.0) THEN
        DO 820 J=1,JJ
        DO 815 I=1,II
        A(I,J)=T(I,J)
  815   CONTINUE
  820   CONTINUE
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | LI(I) = Basic variables;   LJ(J) = Non-basic variables   |    **
C**   +----------------------------------------------------------+    **
      DO 830 I=1,II-1
      LI(I)=IABS(LI(I))
  830 CONTINUE
      DO 840 J=1,JJ-1
      LJ(J)=IABS(LJ(J))
  840 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Set MI(I) = -MI(I) < 0  if MI(I) not in LI(1:II-1)       |    **
C**   +----------------------------------------------------------+    **
      NI=0
      DO 850 I=1,II-1
        DO 845 KI=1,II-1
        IF(MI(I).EQ.LI(KI)) GO TO 850
  845   CONTINUE
        MI(I)=-MI(I)
        NI=NI+1
  850 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Set MJ(J) = -MJ(J) < 0  if MJ(J) not in LJ(1:JJ-1)       |    **
C**   +----------------------------------------------------------+    **
      NJ=0
      DO 860 J=1,JJ-1
        DO 855 KJ=1,JJ-1
        IF(MJ(J).EQ.LJ(KJ)) GO TO 860
  855   CONTINUE
        MJ(J)=-MJ(J)
        NJ=NJ+1
  860 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Move signed MI(I) & MJ(J) to LI(I) & LJ(J)               |    **
C**   | Reset MI(I) & MJ(J) to original positive values          |    **
C**   +----------------------------------------------------------+    **
      DO 865 I=1,II-1
      LI(I)=MI(I)
  865 MI(I)=IABS(MI(I))
      DO 870 J=1,JJ-1
      LJ(J)=MJ(J)
  870 MJ(J)=IABS(MJ(J))
C**   +----------------------------------------------------------+    **
C**   | Find pivot element A(KI,KJ) : LI(KI) < 0 and LJ(KJ) < 0  |    **
C**   +----------------------------------------------------------+    **
      IF(NI.NE.NJ) STOP 2222
      IF(NI.EQ.0) GO TO 960
      DO 950 K=1,NI
        AKK=0.0
        DO 920 J=1,JJ-1
        IF(LJ(J).GE.0) GO TO 920
          DO 910 I=1,II-1
          IF(LI(I).GE.0) GO TO 910
          IF(AKK.GE.ABS(A(I,J))) GO TO 910
            AKK=ABS(A(I,J))
            KI=I
            KJ=J
  910     CONTINUE
  920   CONTINUE
        IF(AKK.LE.EPS) GO TO 960
C**     +--------------------------------------------------------+    **
C**     | Elimination by pivot row : Sweep -LI(KI) <--> -LJ(KJ)  |    **
C**     +--------------------------------------------------------+    **
        LIK=-LI(KI)
        LI(KI)=-LJ(KJ)
        LJ(KJ)=LIK
C**     ----------------------------------------------------------    **
C**     CALL GAUSS(A,II,JJ,KI,KJ,1.0D0,II,1,NIA,NJA)   ! Also OK      **
C**     ----------------------------------------------------------    **
        AKK=A(KI,KJ)
        A(KI,KJ)=1.0
        DO 930 J=1,JJ
        A(KI,J)=A(KI,J)/AKK
  930   CONTINUE
        DO 940 I=1,II
        IF(I.EQ.KI) GO TO 940
          AIK=A(I,KJ)
          A(I,KJ)=0.0
          DO 935 J=1,JJ
          A(I,J)=A(I,J)-AIK*A(KI,J)
  935     CONTINUE
  940   CONTINUE
  950 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Reform simplex tableau                                   |    **
C**   | (1) Reset P(LI(I)) = +1                                  |    **
C**   | (2) Change var X --> D-X'  if P(LJ(J)) = -1              |    **
C**   +----------------------------------------------------------+    **
  960 DO 970 I=1,II-1
      LII=LI(I)
      IF(LII.GT.NX) GO TO 970
        D(LII)=ABS(D(LII))
  970 CONTINUE
      DO 990 J=1,JJ-1
      LJJ=LJ(J)
      IF(LJJ.GT.NX) GO TO 990
      IF(D(LJJ).GE.0.0) GO TO 990
        DJ=-D(LJJ)
        DO 980 I=1,II
        A(I,JJ)=A(I,JJ)-A(I,J)*DJ
        A(I,J)=-A(I,J)
  980   CONTINUE
  990 CONTINUE
      RETURN
      END
************************************************************************
