C   IMSL ROUTINE NAME   - FFTCC                                          FFTCC       2
C                                                                        FFTCC       3
C----------------------------------------------------------------------- FFTCC       4
C                                                                        FFTCC       5
C   COMPUTER            - DEC11/SINGLE                                   FFTCC       6
C                                                                        FFTCC       7
C   LATEST REVISION     - JANUARY 1, 1978                                FFTCC       8
C                                                                        FFTCC       9
C   PURPOSE             - COMPUTE THE FAST FOURIER TRANSFORM OF A        FFTCC      10
C                           COMPLEX VALUED SEQUENCE                      FFTCC      11
C                                                                        FFTCC      12
C   USAGE               - CALL FFTCC (A,N,IWK,WK)                        FFTCC      13
C                                                                        FFTCC      14
C   ARGUMENTS    A      - COMPLEX VECTOR OF LENGTH N. ON INPUT A         FFTCC      15
C                           CONTAINS THE COMPLEX VALUED SEQUENCE TO BE   FFTCC      16
C                           TRANSFORMED. ON OUTPUT A IS REPLACED BY THE  FFTCC      17
C                           FOURIER TRANSFORM.                           FFTCC      18
C                N      - INPUT NUMBER OF DATA POINTS TO BE              FFTCC      19
C                           TRANSFORMED. N MAY BE ANY POSITIVE           FFTCC      20
C                           INTEGER.                                     FFTCC      21
C                IWK    - INTEGER WORK VECTOR OF LENGTH 6*N+150.         FFTCC      22
C                           (SEE PROGRAMMING NOTES FOR FURTHER DETAILS)  FFTCC      23
C                WK     - REAL WORK VECTOR OF LENGTH 6*N+150.            FFTCC      24
C                           (SEE PROGRAMMING NOTES FOR FURTHER DETAILS)  FFTCC      25
C                                                                        FFTCC      26
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                          FFTCC      27
C                       - SINGLE/H36,H48,H60                             FFTCC      28
C                                                                        FFTCC      29
C   REQD. IMSL ROUTINES - NONE REQUIRED                                  FFTCC      30
C                                                                        FFTCC      31
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND            FFTCC      32
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL       FFTCC      33
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP   FFTCC      34
C                                                                        FFTCC      35
C   REMARKS  1.  FFTCC COMPUTES THE FOURIER TRANSFORM, X, ACCORDING      FFTCC      36
C                TO THE FOLLOWING FORMULA;                               FFTCC      37
C                                                                        FFTCC      38
C                  X(K+1) = SUM FROM J = 0 TO N-1 OF                     FFTCC      39
C                           A(J+1)*CEXP((0.0,(2.0*PI*J*K)/N))            FFTCC      40
C                  FOR K=0,1,...,N-1 AND PI=3.1415...                    FFTCC      41
C                                                                        FFTCC      42
C                NOTE THAT X OVERWRITES A ON OUTPUT.                     FFTCC      43
C            2.  FFTCC CAN BE USED TO COMPUTE                            FFTCC      44
C                                                                        FFTCC      45
C                  X(K+1) = (1/N)*SUM FROM J = 0 TO N-1 OF               FFTCC      46
C                           A(J+1)*CEXP((0.0,(-2.0*PI*J*K)/N))           FFTCC      47
C                  FOR K=0,1,...,N-1 AND PI=3.1415...                    FFTCC      48
C                                                                        FFTCC      49
C                BY PERFORMING THE FOLLOWING STEPS;                      FFTCC      50
C                                                                        FFTCC      51
C                     DO 10 I=1,N                                        FFTCC      52
C                        A(I) = CONJG(A(I))                              FFTCC      53
C                  10 CONTINUE                                           FFTCC      54
C                     CALL FFTCC (A,N,IWK,WK)                            FFTCC      55
C                     DO 20 I=1,N                                        FFTCC      56
C                        A(I) = CONJG(A(I))/N                            FFTCC      57
C                  20 CONTINUE                                           FFTCC      58
C                                                                        FFTCC      59
C   COPYRIGHT           - 1978 BY IMSL, INC. ALL RIGHTS RESERVED.        FFTCC      60
C                                                                        FFTCC      61
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN  FFTCC      62
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,     FFTCC      63
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.         FFTCC      64
C                                                                        FFTCC      65
C----------------------------------------------------------------------- FFTCC      66
C                                                                        FFTCC      67
      SUBROUTINE FFTCC (A,N,IWK,WK)                                      FFTCC      68
C                                  SPECIFICATIONS FOR ARGUMENTS          FFTCC      69
      INTEGER            N,IWK(1)                                        FFTCC      70
      REAL*8             WK(1)                                           FFTCC      71
      COMPLEX*16         A(N)                                            FFTCC      72
C                                  SPECIFICATIONS FOR LOCAL VARIABLES    FFTCC      73
      INTEGER            I,IAM,IAP,IBM,IBP,IC,ICC,ICF,ICK,ID,IDM1,II,    FFTCC      74
     1                   IJA,IKB,IKT,ILL,IM,IRD,ISF,ISK,ISP,ISS,ITA,ITB, FFTCC      75
     2                   J,JA,JF,JJ,JK,K,K0,K1,K2,K3,KA,KB,KD2,KF,KH,KN, FFTCC      76
     3                   KT,KTP,L,L1,M,MM,MM1,MP                         FFTCC      77
      REAL*8             CM,SM,C1,C2,C3,S1,S2,S3,C30,RAD,A0,A1,A4,B4,    FFTCC      78
     1                   A2,A3,B0,B1,B2,B3,ZERO,HALF,ONE,TWO,Z0(2),      FFTCC      79
     2                   Z1(2),Z2(2),Z3(2),Z4(2)                         FFTCC      80
      COMPLEX*16         ZA0,ZA1,ZA2,ZA3,ZA4,AK2                         FFTCC      81
      EQUIVALENCE        (ZA0,Z0(1)),(ZA1,Z1(1)),(ZA2,Z2(1)),            FFTCC      82
     1                   (ZA3,Z3(1)),(A0,Z0(1)),(B0,Z0(2)),(A1,Z1(1)),   FFTCC      83
     2                   (B1,Z1(2)),(A2,Z2(1)),(B2,Z2(2)),(A3,Z3(1)),    FFTCC      84
     3                   (B3,Z3(2)),(ZA4,Z4(1)),(Z4(1),A4),(Z4(2),B4)    FFTCC      85
      DATA               RAD/6.2831853071795864D0/,                      FFTCC      86
     1                   C30/.86602540378443860D0/                       FFTCC      87
      DATA               ZERO,HALF,ONE,TWO/0.0D0,0.5D0,1.0D0,2.0D0/      FFTCC      88
C                                  FIRST EXECUTABLE STATEMENT            FFTCC      89
      IF (N .EQ. 1) GO TO 9005                                           FFTCC      90
      K = N                                                              FFTCC      91
      M = 0                                                              FFTCC      92
      J = 2                                                              FFTCC      93
      JJ = 4                                                             FFTCC      94
      JF = 0                                                             FFTCC      95
C                                  DETERMINE THE SQUARE FACTORS OF N     FFTCC      96
      IWK(1) = 1                                                         FFTCC      97
    5 I = K/JJ                                                           FFTCC      98
      IF (I*JJ .NE. K) GO TO 10                                          FFTCC      99
      M = M+1                                                            FFTCC     100
      IWK(M+1) = J                                                       FFTCC     101
      K = I                                                              FFTCC     102
      GO TO 5                                                            FFTCC     103
   10 J = J + 2                                                          FFTCC     104
      IF (J .EQ. 4) J = 3                                                FFTCC     105
      JJ = J * J                                                         FFTCC     106
      IF (JJ .LE. K) GO TO 5                                             FFTCC     107
      KT = M                                                             FFTCC     108
C                                  DETERMINE THE REMAINING FACTORS OF N  FFTCC     109
      J = 2                                                              FFTCC     110
   15 I = K / J                                                          FFTCC     111
      IF (I*J .NE. K) GO TO 20                                           FFTCC     112
      M = M + 1                                                          FFTCC     113
      IWK(M+1) = J                                                       FFTCC     114
      K = I                                                              FFTCC     115
      GO TO 15                                                           FFTCC     116
   20 J = J + 1                                                          FFTCC     117
      IF (J .EQ. 3) GO TO 15                                             FFTCC     118
      J = J + 1                                                          FFTCC     119
      IF(J.LE.K) GO TO 15                                                FFTCC     120
      K = IWK(M+1)                                                       FFTCC     121
      IF (IWK(KT+1) .GT. IWK(M+1)) K = IWK(KT+1)                         FFTCC     122
      IF(KT.LE.0) GO TO 30                                               FFTCC     123
      KTP = KT + 2                                                       FFTCC     124
      DO 25  I = 1,KT                                                    FFTCC     125
         J = KTP - I                                                     FFTCC     126
         M = M+1                                                         FFTCC     127
         IWK(M+1) = IWK(J)                                               FFTCC     128
   25 CONTINUE                                                           FFTCC     129
   30 MP = M+1                                                           FFTCC     130
      IC = MP+1                                                          FFTCC     131
      ID = IC+MP                                                         FFTCC     132
      ILL = ID+MP                                                        FFTCC     133
      IRD = ILL+MP+1                                                     FFTCC     134
      ICC = IRD+MP                                                       FFTCC     135
      ISS = ICC+MP                                                       FFTCC     136
      ICK = ISS+MP                                                       FFTCC     137
      ISK = ICK+K                                                        FFTCC     138
      ICF = ISK+K                                                        FFTCC     139
      ISF = ICF+K                                                        FFTCC     140
      IAP = ISF+K                                                        FFTCC     141
      KD2 = (K-1) / 2 + 1                                                FFTCC     142
      IBP = IAP + KD2                                                    FFTCC     143
      IAM = IBP + KD2                                                    FFTCC     144
      IBM = IAM + KD2                                                    FFTCC     145
      MM1 = M-1                                                          FFTCC     146
      I = 1                                                              FFTCC     147
   35 L = MP - I                                                         FFTCC     148
      J = IC - I                                                         FFTCC     149
      IWK(ILL+L) = 0                                                     FFTCC     150
      IF ((IWK(J-1) + IWK(J)) .EQ. 4) IWK(ILL+L) = 1                     FFTCC     151
      IF (IWK(ILL+L) .EQ. 0) GO TO 40                                    FFTCC     152
      I = I + 1                                                          FFTCC     153
      L = L - 1                                                          FFTCC     154
      IWK(ILL+L) = 0                                                     FFTCC     155
   40 I = I + 1                                                          FFTCC     156
      IF(I.LE.MM1) GO TO 35                                              FFTCC     157
      IWK(ILL+1) = 0                                                     FFTCC     158
      IWK(ILL+MP) = 0                                                    FFTCC     159
      IWK(IC) = 1                                                        FFTCC     160
      IWK(ID) = N                                                        FFTCC     161
      DO 45  J = 1,M                                                     FFTCC     162
         K = IWK(J+1)                                                    FFTCC     163
         IWK(IC+J) = IWK(IC+J-1) * K                                     FFTCC     164
         IWK(ID+J) = IWK(ID+J-1) / K                                     FFTCC     165
         WK(IRD+J) = RAD/IWK(IC+J)                                       FFTCC     166
         C1 = RAD/K                                                      FFTCC     167
         IF (K .LE. 2) GO TO 45                                          FFTCC     168
         WK(ICC+J) = COS(C1)                                             FFTCC     169
         WK(ISS+J) = SIN(C1)                                             FFTCC     170
   45 CONTINUE                                                           FFTCC     171
      MM = M                                                             FFTCC     172
      IF (IWK(ILL+M) .EQ. 1) MM = M - 1                                  FFTCC     173
      IF (MM .LE. 1) GO TO 50                                            FFTCC     174
      SM = IWK(IC+MM-2) * WK(IRD+M)                                      FFTCC     175
      CM = COS(SM)                                                       FFTCC     176
      SM = SIN(SM)                                                       FFTCC     177
   50 KB = 0                                                             FFTCC     178
      KN = N                                                             FFTCC     179
      JJ = 0                                                             FFTCC     180
      I = 1                                                              FFTCC     181
      C1 = ONE                                                           FFTCC     182
      S1 = ZERO                                                          FFTCC     183
      L1 = 1                                                             FFTCC     184
   55 IF (IWK(ILL+I+1) .EQ. 1) GO TO 60                                  FFTCC     185
      KF = IWK(I+1)                                                      FFTCC     186
      GO TO 65                                                           FFTCC     187
   60 KF = 4                                                             FFTCC     188
      I = I+1                                                            FFTCC     189
   65 ISP = IWK(ID+I)                                                    FFTCC     190
      IF (L1 .EQ. 1) GO TO 70                                            FFTCC     191
      S1 = JJ * WK(IRD+I)                                                FFTCC     192
      C1 = COS(S1)                                                       FFTCC     193
      S1 = SIN(S1)                                                       FFTCC     194
C                                  FACTORS OF 2, 3, AND 4 ARE            FFTCC     195
C                                  HANDLED SEPARATELY.                   FFTCC     196
   70 IF (KF .GT. 4) GO TO 140                                           FFTCC     197
      GO TO (75,75,90,115), KF                                           FFTCC     198
   75 K0 = KB + ISP                                                      FFTCC     199
      K2 = K0 + ISP                                                      FFTCC     200
      IF (L1 .EQ. 1) GO TO 85                                            FFTCC     201
   80 K0 = K0 - 1                                                        FFTCC     202
      IF (K0 .LT. KB) GO TO 190                                          FFTCC     203
      K2 = K2 - 1                                                        FFTCC     204
      ZA4 = A(K2+1)                                                      FFTCC     205
      A0 = A4*C1-B4*S1                                                   FFTCC     206
      B0 = A4*S1+B4*C1                                                   FFTCC     207
      A(K2+1) = A(K0+1)-ZA0                                              FFTCC     208
      A(K0+1) = A(K0+1)+ZA0                                              FFTCC     209
      GO TO 80                                                           FFTCC     210
   85 K0 = K0 - 1                                                        FFTCC     211
      IF (K0 .LT. KB) GO TO 190                                          FFTCC     212
      K2 = K2 - 1                                                        FFTCC     213
      AK2 = A(K2+1)                                                      FFTCC     214
      A(K2+1) = A(K0+1)-AK2                                              FFTCC     215
      A(K0+1) = A(K0+1)+AK2                                              FFTCC     216
      GO TO 85                                                           FFTCC     217
   90 IF (L1 .EQ. 1) GO TO 95                                            FFTCC     218
      C2 = C1 * C1 - S1 * S1                                             FFTCC     219
      S2 = TWO * C1 * S1                                                 FFTCC     220
   95 JA = KB + ISP - 1                                                  FFTCC     221
      KA = JA + KB                                                       FFTCC     222
      IKB = KB+1                                                         FFTCC     223
      IJA = JA+1                                                         FFTCC     224
      DO 110 II = IKB,IJA                                                FFTCC     225
         K0 = KA - II + 1                                                FFTCC     226
         K1 = K0 + ISP                                                   FFTCC     227
         K2 = K1 + ISP                                                   FFTCC     228
         ZA0 = A(K0+1)                                                   FFTCC     229
         IF (L1 .EQ. 1) GO TO 100                                        FFTCC     230
         ZA4 = A(K1+1)                                                   FFTCC     231
         A1 = A4*C1-B4*S1                                                FFTCC     232
         B1 = A4*S1+B4*C1                                                FFTCC     233
         ZA4 = A(K2+1)                                                   FFTCC     234
         A2 = A4*C2-B4*S2                                                FFTCC     235
         B2 = A4*S2+B4*C2                                                FFTCC     236
         GO TO 105                                                       FFTCC     237
  100    ZA1 = A(K1+1)                                                   FFTCC     238
         ZA2 = A(K2+1)                                                   FFTCC     239
  105    A(K0+1) = CMPLX(A0+A1+A2,B0+B1+B2)                              FFTCC     240
         A0 = -HALF * (A1+A2) + A0                                       FFTCC     241
         A1 = (A1-A2) * C30                                              FFTCC     242
         B0 = -HALF * (B1+B2) + B0                                       FFTCC     243
         B1 = (B1-B2) * C30                                              FFTCC     244
         A(K1+1) = CMPLX(A0-B1,B0+A1)                                    FFTCC     245
         A(K2+1) = CMPLX(A0+B1,B0-A1)                                    FFTCC     246
  110 CONTINUE                                                           FFTCC     247
      GO TO 190                                                          FFTCC     248
  115 IF (L1 .EQ. 1) GO TO 120                                           FFTCC     249
      C2 = C1 * C1 - S1 * S1                                             FFTCC     250
      S2 = TWO * C1 * S1                                                 FFTCC     251
      C3 = C1 * C2 - S1 * S2                                             FFTCC     252
      S3 = S1 * C2 + C1 * S2                                             FFTCC     253
  120 JA = KB + ISP - 1                                                  FFTCC     254
      KA = JA + KB                                                       FFTCC     255
      IKB = KB+1                                                         FFTCC     256
      IJA = JA+1                                                         FFTCC     257
      DO 135 II = IKB,IJA                                                FFTCC     258
         K0 = KA - II + 1                                                FFTCC     259
         K1 = K0 + ISP                                                   FFTCC     260
         K2 = K1 + ISP                                                   FFTCC     261
         K3 = K2 + ISP                                                   FFTCC     262
         ZA0 = A(K0+1)                                                   FFTCC     263
         IF (L1 .EQ. 1) GO TO 125                                        FFTCC     264
         ZA4 = A(K1+1)                                                   FFTCC     265
         A1 = A4*C1-B4*S1                                                FFTCC     266
         B1 = A4*S1+B4*C1                                                FFTCC     267
         ZA4 = A(K2+1)                                                   FFTCC     268
         A2 = A4*C2-B4*S2                                                FFTCC     269
         B2 = A4*S2+B4*C2                                                FFTCC     270
         ZA4 = A(K3+1)                                                   FFTCC     271
         A3 = A4*C3-B4*S3                                                FFTCC     272
         B3 = A4*S3+B4*C3                                                FFTCC     273
         GO TO 130                                                       FFTCC     274
  125    ZA1 = A(K1+1)                                                   FFTCC     275
         ZA2 = A(K2+1)                                                   FFTCC     276
         ZA3 = A(K3+1)                                                   FFTCC     277
  130    A(K0+1) = CMPLX(A0+A2+A1+A3,B0+B2+B1+B3)                        FFTCC     278
         A(K1+1) = CMPLX(A0+A2-A1-A3,B0+B2-B1-B3)                        FFTCC     279
         A(K2+1) = CMPLX(A0-A2-B1+B3,B0-B2+A1-A3)                        FFTCC     280
         A(K3+1) = CMPLX(A0-A2+B1-B3,B0-B2-A1+A3)                        FFTCC     281
  135 CONTINUE                                                           FFTCC     282
      GO TO 190                                                          FFTCC     283
  140 JK = KF - 1                                                        FFTCC     284
      KH = JK/2                                                          FFTCC     285
      K3 = IWK(ID+I-1)                                                   FFTCC     286
      K0 = KB + ISP                                                      FFTCC     287
      IF (L1 .EQ. 1) GO TO 150                                           FFTCC     288
      K = JK - 1                                                         FFTCC     289
      WK(ICF+1) = C1                                                     FFTCC     290
      WK(ISF+1) = S1                                                     FFTCC     291
      DO 145 J = 1,K                                                     FFTCC     292
         WK(ICF+J+1) = WK(ICF+J) * C1 - WK(ISF+J) * S1                   FFTCC     293
         WK(ISF+J+1) = WK(ICF+J) * S1 + WK(ISF+J) * C1                   FFTCC     294
  145 CONTINUE                                                           FFTCC     295
  150 IF (KF .EQ. JF) GO TO 160                                          FFTCC     296
      C2 = WK(ICC+I)                                                     FFTCC     297
      WK(ICK+1) = C2                                                     FFTCC     298
      WK(ICK+JK) = C2                                                    FFTCC     299
      S2 = WK(ISS+I)                                                     FFTCC     300
      WK(ISK+1) = S2                                                     FFTCC     301
      WK(ISK+JK) = -S2                                                   FFTCC     302
      DO 155 J = 1,KH                                                    FFTCC     303
         K = JK - J                                                      FFTCC     304
         WK(ICK+K) = WK(ICK+J) * C2 - WK(ISK+J) * S2                     FFTCC     305
         WK(ICK+J+1) = WK(ICK+K)                                         FFTCC     306
         WK(ISK+J+1) = WK(ICK+J) * S2 + WK(ISK+J) * C2                   FFTCC     307
         WK(ISK+K) = -WK(ISK+J+1)                                        FFTCC     308
  155 CONTINUE                                                           FFTCC     309
  160 K0 = K0 - 1                                                        FFTCC     310
      K1 = K0                                                            FFTCC     311
      K2 = K0 + K3                                                       FFTCC     312
      ZA0 = A(K0+1)                                                      FFTCC     313
      A3 = A0                                                            FFTCC     314
      B3 = B0                                                            FFTCC     315
      DO 175 J = 1,KH                                                    FFTCC     316
         K1 = K1 + ISP                                                   FFTCC     317
         K2 = K2 - ISP                                                   FFTCC     318
         IF (L1 .EQ. 1) GO TO 165                                        FFTCC     319
         K = KF - J                                                      FFTCC     320
         ZA4 = A(K1+1)                                                   FFTCC     321
         A1 = A4*WK(ICF+J)-B4*WK(ISF+J)                                  FFTCC     322
         B1 = A4*WK(ISF+J)+B4*WK(ICF+J)                                  FFTCC     323
         ZA4 = A(K2+1)                                                   FFTCC     324
         A2 = A4*WK(ICF+K)-B4*WK(ISF+K)                                  FFTCC     325
         B2 = A4*WK(ISF+K)+B4*WK(ICF+K)                                  FFTCC     326
         GO TO 170                                                       FFTCC     327
  165    ZA1 = A(K1+1)                                                   FFTCC     328
         ZA2 = A(K2+1)                                                   FFTCC     329
  170    WK(IAP+J) = A1 + A2                                             FFTCC     330
         WK(IAM+J) = A1 - A2                                             FFTCC     331
         WK(IBP+J) = B1 + B2                                             FFTCC     332
         WK(IBM+J) = B1 - B2                                             FFTCC     333
         A3 = A1 + A2 + A3                                               FFTCC     334
         B3 = B1 + B2 + B3                                               FFTCC     335
  175 CONTINUE                                                           FFTCC     336
      A(K0+1) = CMPLX(A3,B3)                                             FFTCC     337
      K1 = K0                                                            FFTCC     338
      K2 = K0 + K3                                                       FFTCC     339
      DO 185 J = 1,KH                                                    FFTCC     340
         K1 = K1 + ISP                                                   FFTCC     341
         K2 = K2 - ISP                                                   FFTCC     342
         JK = J                                                          FFTCC     343
         A1 = A0                                                         FFTCC     344
         B1 = B0                                                         FFTCC     345
         A2 = ZERO                                                       FFTCC     346
         B2 = ZERO                                                       FFTCC     347
         DO 180  K = 1,KH                                                FFTCC     348
            A1 = A1 + WK(IAP+K) * WK(ICK+JK)                             FFTCC     349
            A2 = A2 + WK(IAM+K) * WK(ISK+JK)                             FFTCC     350
            B1 = B1 + WK(IBP+K) * WK(ICK+JK)                             FFTCC     351
            B2 = B2 + WK(IBM+K) * WK(ISK+JK)                             FFTCC     352
            JK = JK + J                                                  FFTCC     353
            IF (JK .GE. KF) JK = JK - KF                                 FFTCC     354
  180    CONTINUE                                                        FFTCC     355
         A(K1+1) = CMPLX(A1-B2,B1+A2)                                    FFTCC     356
         A(K2+1) = CMPLX(A1+B2,B1-A2)                                    FFTCC     357
  185 CONTINUE                                                           FFTCC     358
      IF (K0 .GT. KB) GO TO 160                                          FFTCC     359
      JF = KF                                                            FFTCC     360
  190 IF ( I .GE. MM ) GO TO 195                                         FFTCC     361
      I = I + 1                                                          FFTCC     362
      GO TO 55                                                           FFTCC     363
  195 I = MM                                                             FFTCC     364
      L1 = 0                                                             FFTCC     365
      KB = IWK(ID+I-1) + KB                                              FFTCC     366
      IF (KB .GE. KN) GO TO 215                                          FFTCC     367
  200 JJ = IWK(IC+I-2) + JJ                                              FFTCC     368
      IF (JJ .LT. IWK(IC+I-1)) GO TO 205                                 FFTCC     369
      I = I - 1                                                          FFTCC     370
      JJ = JJ - IWK(IC+I)                                                FFTCC     371
      GO TO 200                                                          FFTCC     372
  205 IF (I .NE. MM) GO TO 210                                           FFTCC     373
      C2 = C1                                                            FFTCC     374
      C1 = CM * C1 - SM * S1                                             FFTCC     375
      S1 = SM * C2 + CM * S1                                             FFTCC     376
      GO TO 70                                                           FFTCC     377
  210 IF (IWK(ILL+I) .EQ. 1) I = I + 1                                   FFTCC     378
      GO TO 55                                                           FFTCC     379
  215 I = 1                                                              FFTCC     380
      JA = KT - 1                                                        FFTCC     381
      KA = JA + 1                                                        FFTCC     382
      IF(JA.LT.1) GO TO 225                                              FFTCC     383
      DO 220  II = 1,JA                                                  FFTCC     384
         J = KA - II                                                     FFTCC     385
         IWK(J+1) = IWK(J+1) - 1                                         FFTCC     386
         I = IWK(J+1) + I                                                FFTCC     387
  220 CONTINUE                                                           FFTCC     388
C                                  THE RESULT IS NOW PERMUTED TO         FFTCC     389
C                                  NORMAL ORDER.                         FFTCC     390
  225 IF (KT .LE. 0) GO TO 270                                           FFTCC     391
      J = 1                                                              FFTCC     392
      I = 0                                                              FFTCC     393
      KB = 0                                                             FFTCC     394
  230 K2 = IWK(ID+J) + KB                                                FFTCC     395
      K3 = K2                                                            FFTCC     396
      JJ = IWK(IC+J-1)                                                   FFTCC     397
      JK = JJ                                                            FFTCC     398
      K0 = KB + JJ                                                       FFTCC     399
      ISP = IWK(IC+J) - JJ                                               FFTCC     400
  235 K = K0 + JJ                                                        FFTCC     401
  240 ZA4 = A(K0+1)                                                      FFTCC     402
      A(K0+1) = A(K2+1)                                                  FFTCC     403
      A(K2+1) = ZA4                                                      FFTCC     404
      K0 = K0 + 1                                                        FFTCC     405
      K2 = K2 + 1                                                        FFTCC     406
      IF (K0 .LT. K) GO TO 240                                           FFTCC     407
      K0 = K0 + ISP                                                      FFTCC     408
      K2 = K2 + ISP                                                      FFTCC     409
      IF (K0 .LT. K3) GO TO 235                                          FFTCC     410
      IF (K0 .GE. K3 + ISP) GO TO 245                                    FFTCC     411
      K0 = K0 - IWK(ID+J) + JJ                                           FFTCC     412
      GO TO 235                                                          FFTCC     413
  245 K3 = IWK(ID+J) + K3                                                FFTCC     414
      IF (K3 - KB .GE. IWK(ID+J-1)) GO TO 250                            FFTCC     415
      K2 = K3 + JK                                                       FFTCC     416
      JK = JK + JJ                                                       FFTCC     417
      K0 = K3 - IWK(ID+J) + JK                                           FFTCC     418
      GO TO 235                                                          FFTCC     419
  250 IF (J .GE. KT) GO TO 260                                           FFTCC     420
      K = IWK(J+1) + I                                                   FFTCC     421
      J = J + 1                                                          FFTCC     422
  255 I = I + 1                                                          FFTCC     423
      IWK(ILL+I) = J                                                     FFTCC     424
      IF (I .LT. K) GO TO 255                                            FFTCC     425
      GO TO 230                                                          FFTCC     426
  260 KB = K3                                                            FFTCC     427
      IF (I .LE. 0) GO TO 265                                            FFTCC     428
      J = IWK(ILL+I)                                                     FFTCC     429
      I = I - 1                                                          FFTCC     430
      GO TO 230                                                          FFTCC     431
  265 IF (KB .GE. N) GO TO 270                                           FFTCC     432
      J = 1                                                              FFTCC     433
      GO TO 230                                                          FFTCC     434
  270 JK = IWK(IC+KT)                                                    FFTCC     435
      ISP = IWK(ID+KT)                                                   FFTCC     436
      M = M - KT                                                         FFTCC     437
      KB = ISP/JK-2                                                      FFTCC     438
      IF (KT .GE. M-1 ) GO TO 9005                                       FFTCC     439
      ITA = ILL+KB+1                                                     FFTCC     440
      ITB = ITA+JK                                                       FFTCC     441
      IDM1 = ID-1                                                        FFTCC     442
      IKT = KT+1                                                         FFTCC     443
      IM = M+1                                                           FFTCC     444
      DO 275 J = IKT,IM                                                  FFTCC     445
         IWK(IDM1+J) = IWK(IDM1+J)/JK                                    FFTCC     446
  275 CONTINUE                                                           FFTCC     447
      JJ = 0                                                             FFTCC     448
      DO 290 J = 1,KB                                                    FFTCC     449
         K = KT                                                          FFTCC     450
  280    JJ = IWK(ID+K+1) + JJ                                           FFTCC     451
         IF (JJ .LT. IWK(ID+K)) GO TO 285                                FFTCC     452
         JJ = JJ - IWK(ID+K)                                             FFTCC     453
         K = K + 1                                                       FFTCC     454
         GO TO 280                                                       FFTCC     455
  285    IWK(ILL+J) = JJ                                                 FFTCC     456
         IF (JJ .EQ. J) IWK(ILL+J) = -J                                  FFTCC     457
  290 CONTINUE                                                           FFTCC     458
C                                  DETERMINE THE PERMUTATION CYCLES      FFTCC     459
C                                  OF LENGTH GREATER THAN OR EQUAL       FFTCC     460
C                                  TO TWO.                               FFTCC     461
      DO 300  J = 1,KB                                                   FFTCC     462
         IF (IWK(ILL+J) .LE. 0) GO TO 300                                FFTCC     463
         K2 = J                                                          FFTCC     464
  295    K2 = IABS(IWK(ILL+K2))                                          FFTCC     465
         IF (K2 .EQ. J) GO TO 300                                        FFTCC     466
         IWK(ILL+K2) = -IWK(ILL+K2)                                      FFTCC     467
         GO TO 295                                                       FFTCC     468
  300 CONTINUE                                                           FFTCC     469
C                                  REORDER A FOLLOWING THE               FFTCC     470
C                                  PERMUTATION CYCLES                    FFTCC     471
      I = 0                                                              FFTCC     472
      J = 0                                                              FFTCC     473
      KB = 0                                                             FFTCC     474
      KN = N                                                             FFTCC     475
  305 J = J + 1                                                          FFTCC     476
      IF (IWK(ILL+J) .LT. 0) GO TO 305                                   FFTCC     477
      K = IWK(ILL+J)                                                     FFTCC     478
      K0 = JK * K + KB                                                   FFTCC     479
  310 ZA4 = A(K0+I+1)                                                    FFTCC     480
      WK(ITA+I) = A4                                                     FFTCC     481
      WK(ITB+I) = B4                                                     FFTCC     482
      I = I + 1                                                          FFTCC     483
      IF (I .LT. JK) GO TO 310                                           FFTCC     484
      I = 0                                                              FFTCC     485
  315 K = -IWK(ILL+K)                                                    FFTCC     486
      JJ = K0                                                            FFTCC     487
      K0 = JK * K + KB                                                   FFTCC     488
  320 A(JJ+I+1) = A(K0+I+1)                                              FFTCC     489
      I = I + 1                                                          FFTCC     490
      IF (I .LT. JK) GO TO 320                                           FFTCC     491
      I = 0                                                              FFTCC     492
      IF (K .NE. J) GO TO 315                                            FFTCC     493
  325 A(K0+I+1) = CMPLX(WK(ITA+I),WK(ITB+I))                             FFTCC     494
      I = I + 1                                                          FFTCC     495
      IF (I .LT. JK) GO TO 325                                           FFTCC     496
      I = 0                                                              FFTCC     497
      IF (J .LT. K2) GO TO 305                                           FFTCC     498
      J = 0                                                              FFTCC     499
      KB = KB + ISP                                                      FFTCC     500
      IF (KB .LT. KN) GO TO 305                                          FFTCC     501
 9005 RETURN                                                             FFTCC     502
      END                                                                FFTCC     503
C   IMSL ROUTINE NAME   - FFT3D                                          FFT3D       2
C                                                                        FFT3D       3
C----------------------------------------------------------------------- FFT3D       4
C                                                                        FFT3D       5
C   COMPUTER            - DEC11/SINGLE                                   FFT3D       6
C                                                                        FFT3D       7
C   LATEST REVISION     - JUNE 1, 1980                                   FFT3D       8
C                                                                        FFT3D       9
C   PURPOSE             - COMPUTE THE FAST FOURIER TRANSFORM OF          FFT3D      10
C                           A COMPLEX VALUED 1,2 OR 3 DIMENSIONAL        FFT3D      11
C                           ARRAY                                        FFT3D      12
C                                                                        FFT3D      13
C   USAGE               - CALL FFT3D (A,IA1,IA2,N1,N2,N3,IJOB,IWK,RWK,   FFT3D      14
C                           CWK)                                         FFT3D      15
C                                                                        FFT3D      16
C   ARGUMENTS    A      - COMPLEX ARRAY. A MAY BE A THREE                FFT3D      17
C                           DIMENSIONAL ARRAY OF DIMENSION N1 BY N2      FFT3D      18
C                           BY N3, A TWO DIMENSIONAL ARRAY OF            FFT3D      19
C                           DIMENSION N1 BY N2, OR A VECTOR OF           FFT3D      20
C                           LENGTH N1. ON INPUT A CONTAINS THE           FFT3D      21
C                           ARRAY TO BE TRANSFORMED. ON OUTPUT           FFT3D      22
C                           A IS REPLACED BY THE FOURIER OR              FFT3D      23
C                           INVERSE FOURIER TRANSFORM (DEPENDING ON      FFT3D      24
C                           THE VALUE OF INPUT PARAMETER IJOB).          FFT3D      25
C                IA1    - FIRST DIMENSION OF THE ARRAY A EXACTLY         FFT3D      26
C                           AS SPECIFIED IN THE DIMENSION STATEMENT      FFT3D      27
C                           IN THE CALLING PROGRAM. (INPUT)              FFT3D      28
C                IA2    - SECOND DIMENSION OF THE ARRAY A EXACTLY        FFT3D      29
C                           AS SPECIFIED IN THE DIMENSION STATEMENT      FFT3D      30
C                           IN THE CALLING PROGRAM. (INPUT)              FFT3D      31
C                N1     - LIMITS ON THE FIRST, SECOND, AND THIRD         FFT3D      32
C                N2         SUBSCRIPTS OF THE ARRAY A, RESPECTIVELY.     FFT3D      33
C                N3         (INPUT)                                      FFT3D      34
C                IJOB   - INPUT OPTION PARAMETER.                        FFT3D      35
C                           IF IJOB IS POSITIVE, THE FAST FOURIER        FFT3D      36
C                             TRANSFORM OF A IS TO BE CALCULATED.        FFT3D      37
C                           IF IJOB IS NEGATIVE, THE INVERSE             FFT3D      38
C                             FAST FOURIER TRANSFORM OF A IS TO BE       FFT3D      39
C                             CALCULATED.                                FFT3D      40
C                IWK    - INTEGER WORK VECTOR OF LENGTH                  FFT3D      41
C                           6*MAX(N1,N2,N3)+150.                         FFT3D      42
C                RWK    - REAL WORK VECTOR OF LENGTH                     FFT3D      43
C                           6*MAX(N1,N2,N3)+150.                         FFT3D      44
C                CWK    - COMPLEX WORK VECTOR OF LENGTH                  FFT3D      45
C                           MAX(N2,N3).                                  FFT3D      46
C                                                                        FFT3D      47
C   PRECISION/HARDWARE  - SINGLE AND DOUBLE/H32                          FFT3D      48
C                       - SINGLE/H32,H48,H60                             FFT3D      49
C                                                                        FFT3D      50
C   REQD. IMSL ROUTINES - FFTCC                                          FFT3D      51
C                                                                        FFT3D      52
C   NOTATION            - INFORMATION ON SPECIAL NOTATION AND            FFT3D      53
C                           CONVENTIONS IS AVAILABLE IN THE MANUAL       FFT3D      54
C                           INTRODUCTION OR THROUGH IMSL ROUTINE UHELP   FFT3D      55
C                                                                        FFT3D      56
C   REMARKS  1.  IF IJOB IS POSITIVE, FFT3D CALCULATES THE FOURIER       FFT3D      57
C                TRANSFORM, X, ACCORDING TO THE FOLLOWING FORMULA        FFT3D      58
C                                                                        FFT3D      59
C                  X(I+1,J+1,K+1)=TRIPLE SUM OF A(L+1,M+1,N+1)*          FFT3D      60
C                  EXP(2*PI*SQRT(-1)*(I*L/N1+J*M/N2+K*N/N3))             FFT3D      61
C                  WITH L=0...N1-1, M=0...N2-1, N=0...N3-1               FFT3D      62
C                  AND PI=3.1415...                                      FFT3D      63
C                                                                        FFT3D      64
C                IF IJOB IS NEGATIVE, FFT3D CALCULATES THE INVERSE       FFT3D      65
C                FOURIER TRANSFORM, X, ACCORDING TO THE FOLLOWING        FFT3D      66
C                FORMULA                                                 FFT3D      67
C                                                                        FFT3D      68
C                  X(I+1,J+1,K+1)=1/(N1*N2*N3)*TRIPLE SUM OF             FFT3D      69
C                  A(L+1,M+1,N+1)*                                       FFT3D      70
C                  EXP(-2*PI*SQRT(-1)*(I*L/N1+J*M/N2+K*N/N3))            FFT3D      71
C                  WITH L=0...N1-1, M=0...N2-1, N=0...N3-1               FFT3D      72
C                  AND PI=3.1415...                                      FFT3D      73
C                                                                        FFT3D      74
C                NOTE THAT X OVERWRITES A ON OUTPUT.                     FFT3D      75
C            2.  IF A IS A TWO DIMENSIONAL ARRAY, SET N3 = 1.            FFT3D      76
C                IF A IS A ONE DIMENSIONAL ARRAY (VECTOR),               FFT3D      77
C                SET IA2 = N2 = N3 = 1.                                  FFT3D      78
C                                                                        FFT3D      79
C   COPYRIGHT           - 1980 BY IMSL, INC. ALL RIGHTS RESERVED.        FFT3D      80
C                                                                        FFT3D      81
C   WARRANTY            - IMSL WARRANTS ONLY THAT IMSL TESTING HAS BEEN  FFT3D      82
C                           APPLIED TO THIS CODE. NO OTHER WARRANTY,     FFT3D      83
C                           EXPRESSED OR IMPLIED, IS APPLICABLE.         FFT3D      84
C                                                                        FFT3D      85
C----------------------------------------------------------------------- FFT3D      86
C                                                                        FFT3D      87
      SUBROUTINE FFT3D  (A,IA1,IA2,N1,N2,N3,IJOB,IWK,RWK,CWK)            FFT3D      88
C                                  SPECIFICATIONS FOR ARGUMENTS          FFT3D      89
      INTEGER            IA1,IA2,N1,N2,N3,IJOB,IWK(1)                    FFT3D      90
      REAL*8             RWK(1)                                          FFT3D      91
      COMPLEX*16         A(IA1,IA2,N3),CWK(1)                            FFT3D      92
C                                  SPECIFICATIONS FOR LOCAL VARIABLES    FFT3D      93
      INTEGER            I,J,K,L,M,N                                     FFT3D      94
      REAL*8             R123                                            FFT3D      95
      COMPLEX*16         C123                                            FFT3D      96
C                                  FIRST EXECUTABLE STATEMENT            FFT3D      97
      IF (IJOB.GT.0) GO TO 10                                            FFT3D      98
C                                  INVERSE TRANSFORM                     FFT3D      99
      DO 5 I=1,N1                                                        FFT3D     100
      DO 5 J=1,N2                                                        FFT3D     101
      DO 5 K=1,N3                                                        FFT3D     102
         A(I,J,K) = CONJG(A(I,J,K))                                      FFT3D     103
    5 CONTINUE                                                           FFT3D     104
C                                  TRANSFORM THIRD SUBSCRIPT             FFT3D     105
   10 DO 25 L=1,N1                                                       FFT3D     106
      DO 25 M=1,N2                                                       FFT3D     107
         DO 15 N=1,N3                                                    FFT3D     108
            CWK(N) = A(L,M,N)                                            FFT3D     109
   15    CONTINUE                                                        FFT3D     110
         CALL FFTCC (CWK,N3,IWK,RWK)                                     FFT3D     111
         DO 20 K=1,N3                                                    FFT3D     112
            A(L,M,K) = CWK(K)                                            FFT3D     113
   20    CONTINUE                                                        FFT3D     114
   25 CONTINUE                                                           FFT3D     115
C                                  TRANSFORM SECOND SUBSCRIPT            FFT3D     116
      DO 40 L=1,N1                                                       FFT3D     117
      DO 40 K=1,N3                                                       FFT3D     118
         DO 30 M=1,N2                                                    FFT3D     119
            CWK(M) = A(L,M,K)                                            FFT3D     120
   30    CONTINUE                                                        FFT3D     121
         CALL FFTCC (CWK,N2,IWK,RWK)                                     FFT3D     122
         DO 35 J=1,N2                                                    FFT3D     123
            A(L,J,K) = CWK(J)                                            FFT3D     124
   35    CONTINUE                                                        FFT3D     125
   40 CONTINUE                                                           FFT3D     126
C                                  TRANSFORM FIRST SUBSCRIPT             FFT3D     127
      DO 45 J=1,N2                                                       FFT3D     128
      DO 45 K=1,N3                                                       FFT3D     129
         CALL FFTCC (A(1,J,K),N1,IWK,RWK)                                FFT3D     130
   45 CONTINUE                                                           FFT3D     131
      IF (IJOB.GT.0) GO TO 55                                            FFT3D     132
      R123 = N1*N2*N3                                                    FFT3D     133
      C123 = CMPLX(R123,0.0)                                             FFT3D     134
      DO 50 I=1,N1                                                       FFT3D     135
      DO 50 J=1,N2                                                       FFT3D     136
      DO 50 K=1,N3                                                       FFT3D     137
         A(I,J,K) = CONJG(A(I,J,K))/C123                                 FFT3D     138
   50 CONTINUE                                                           FFT3D     139
   55 RETURN                                                             FFT3D     140
      END                                                                FFT3D     141
