************************************************************************
      SUBROUTINE FFT2D(A,N1,N2,INV1,INV2,W,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMPLEX*16 A(NRA,N2),W(1)
C**   ============================================================    **
C**   INV1,INV2 = +1 : Fourior transform                              **
C**   INV1,INV2 = -1 : Inverse transform                              **
C**   ------------------------------------------------------------    **
C**   Input & output : A(N1,N2) = complex data                        **
C**   ------------------------------------------------------------    **
C**   W(N) : Working array                                            **
C**          N .GE. MAX(N1,N2+N2) for any N1  and any N2              **
C**          N .GE. MAX(N1,   N2) for any N1  and N2=2**K             **
C**          N .GE.        N2+N2  for N1=2**M and any N2              **
C**          N .GE.           N2  for N1=2**M and N2=2**K             **
C**   ------------------------------------------------------------    **
      IF(N2.GT.1) THEN
        DO 40 I=1,N1
          DO 20 J=1,N2
            W(J)=A(I,J)
   20     CONTINUE
          CALL FFTA(W,W(N2+1),N2,INV2)
          DO 30 J=1,N2
            A(I,J)=W(J)
   30     CONTINUE
   40   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      IF(N1.GT.1) THEN
        DO 50 J=1,N2
          CALL FFTA(A(1,J),W,N1,INV1)
   50   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      RETURN
      END
************************************************************************
      SUBROUTINE FFT2SC(A,N1,N2,ICS1,ICS2,W,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NRA,N2),W(1)
C**   ============================================================    **
C**   2-D Half-SIN/COS transform                                      **
C**   ------------------------------------------------------------    **
C**   ICS1=+-1  Half-SIN transform for any N1 (NR=N1  )               **
C**   ICS1=+-2  Half-COS transform for any N1 (NR=N1+1)               **
C**   ICS1=+-3  Half-COS trans/inv for any N1 (NR=N1  )               **
C**   ICS2=+-1  Half-SIN transform for any N2 (NC=N2  )               **
C**   ICS2=+-2  Half-COS transform for any N2 (NC=N2+1)               **
C**   ICS2=+-3  Half-COS trans/inv for any N2 (NC=N2  )               **
C**   ------------------------------------------------------------    **
C**   Input & output : A(NR,NC) = real data                           **
C**   ------------------------------------------------------------    **
C**   W(N) : Working array                                            **
C**          N .GE. MAX(N1,N2+NC) for any N1  and any N2              **
C**          N .GE. MAX(N1,   NC) for any N1  and N2=2**J             **
C**          N .GE.        N2+NC  for N1=2**I and any N2              **
C**          N .GE.           NC  for N1=2**I and N2=2**J             **
C**   ============================================================    **
      NR=N1
      NC=N2
      IF(IABS(ICS1).EQ.2.AND.N1.GT.1) NR=N1+1
      IF(IABS(ICS2).EQ.2.AND.N2.GT.1) NC=N2+1
C**   ------------------------------------------------------------    **
      IF(N2.GT.1) THEN
        DO 40 I=1,NR
          DO 20 J=1,NC
            W(J)=A(I,J)
   20     CONTINUE
          CALL FFTSC(W,W(NC+1),N2,ICS2)
          DO 30 J=1,NC
            A(I,J)=W(J)
   30     CONTINUE
   40   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      IF(N1.GT.1) THEN
        DO 50 J=1,NC
          CALL FFTSC(A(1,J),W,N1,ICS1)
   50   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      RETURN
      END
************************************************************************
      SUBROUTINE FFT2CS(A,N1,N2,ICS1,ICS2,W,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NRA,N2),W(1)
C**   ============================================================    **
C**   ICS1,ICS2 = +-1 : SIN transform                                 **
C**   ICS1,ICS2 = +-2 : COS transform                                 **
C**   ------------------------------------------------------------    **
C**   Input & output : A(N1,N2) = real data                           **
C**   ------------------------------------------------------------    **
C**   W(N) : Working array                                            **
C**          N .GE. MAX(N1/2,N2/2+N2) for even N1 and even N2         **
C**          N .GE. MAX(N1/2,     N2) for even N1 and N2=2**J         **
C**          N .GE.          N2/2+N2  for N1=2**I and even N2         **
C**          N .GE.               N2  for N1=2**I and N2=2**J         **
C**   ============================================================    **
      IF(N2.GT.1) THEN
        DO 40 I=1,N1
          DO 20 J=1,N2
            W(J)=A(I,J)
   20     CONTINUE
          CALL FFTCS(W,W(N2+1),N2,ICS2)
          DO 30 J=1,N2
            A(I,J)=W(J)
   30     CONTINUE
   40   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      IF(N1.GT.1) THEN
        DO 50 J=1,N2
          CALL FFTCS(A(1,J),W,N1,ICS1)
   50   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      RETURN
      END
************************************************************************
      SUBROUTINE FFT2H(A,N1,N2,ICS1,ICS2,W,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(NRA,N2),W(1)
C**   ============================================================    **
C**   2-D Half-SIN/COS transform                                      **
C**   ------------------------------------------------------------    **
C**   ICS1=+-1  Half-SIN transform for any N1 (NR=N1  )               **
C**   ICS1=+-2  Half-COS transform for any N1 (NR=N1+1)               **
C**   ICS1=+-3  Half-COS trans/inv for any N1 (NR=N1  )               **
C**   ICS2=+-1  Half-SIN transform for any N2 (NC=N2  )               **
C**   ICS2=+-2  Half-COS transform for any N2 (NC=N2+1)               **
C**   ICS2=+-3  Half-COS trans/inv for any N2 (NC=N2  )               **
C**   ------------------------------------------------------------    **
C**   Input & output : A(NR,NC) = real data                           **
C**   ------------------------------------------------------------    **
C**   Working array  : W(N)                                           **
C**   N .GE. MAX(N1,N2)*4  for any N1  and any N2                     **
C**   N .GE. MAX(N1,N2)*2  for N1=2**M and N2=2**K                    **
C**   ============================================================    **
      NR=N1
      NC=N2
      IF(IABS(ICS1).EQ.2.AND.N1.GT.1) NR=N1+1
      IF(IABS(ICS2).EQ.2.AND.N2.GT.1) NC=N2+1
C**   ------------------------------------------------------------    **
      IF(N2.GT.1) THEN
        DO 40 I=1,NR
          DO 20 J=1,N2
            W(N2+J)=0.0
            W(J)=A(I,J)
   20     CONTINUE
          IF(IABS(ICS2).EQ.2) THEN
            W(1) =0.5*W(1)
            W(NC)=0.5*A(I,NC)
          ELSEIF(IABS(ICS2).GE.3) THEN
            CALL FFTSC(W,W(N2+1),N2,ICS2)
            GO TO 25
          ENDIF
          CALL FFTCS(W,W(2*N2+1),2*N2,ICS2)
   25     DO 30 J=1,NC
            A(I,J)=W(J)
   30     CONTINUE
   40   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      IF(N1.GT.1) THEN
        DO 80 J=1,NC
          IF(IABS(ICS1).GE.3) THEN
            CALL FFTSC(A(1,J),W,N1,ICS1)
            GO TO 80
          ENDIF
          DO 50 I=1,N1
            W(N1+I)=0.0
            W(I)=A(I,J)
   50     CONTINUE
          IF(IABS(ICS1).EQ.2) THEN
            W(1) =0.5*W(1)
            W(NR)=0.5*A(NR,J)
          ENDIF
          CALL FFTCS(W,W(2*N1+1),2*N1,ICS1)
          DO 60 I=1,NR
            A(I,J)=W(I)
   60     CONTINUE
   80   CONTINUE
      ENDIF
C**   ------------------------------------------------------------    **
      RETURN
      END
************************************************************************
