************************************************************************
      SUBROUTINE FFTR(A,B,N,INV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),B(N)
C**   ============================================================    **
C**   FFT of real data for any even N                                 **
C**   ------------------------------------------------------------    **
C**   f(2,N) ===> F(2,N)  :         f(2,I)=0,  F(2,1)=F(2,N/2+1)=0    **
C**        F(1,N+1-I)=F(1,I+1), F(2,N+1-I)=-F(2,I+1),  for I=1,N/2    **
C**   ============================================================    **
C**   For INV=+-1  :  DIMENSION A(2*N)                                **
C**   Input  : A(2*I-1)=f(1,I), for I=1,N  (others unused)            **
C**   Output : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=1,N              **
C**   ------------------------------------------------------------    **
C**   For INV=+-2  :  DIMENSION A(N+2)                                **
C**   Input  : A(I)=f(1,I), for I=1,N                                 **
C**   Output : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=1,N/2+1          **
C**   ------------------------------------------------------------    **
C**   For INV=+-3  :  DIMENSION A(N)                                  **
C**   Input  : A(I)=f(1,I), for I=1,N                                 **
C**   Output : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=2,N/2            **
C**            A(1)  =  F(1,1), A(2) = F(1,N/2+1)                     **
C**   ============================================================    **
C**   Working : B(N) required for N .NE. 2**M only                    **
C**   ============================================================    **
      IF(IABS(INV).NE.1) GOTO 40
      DO 20 I=1,N
   20 A(I)=A(2*I-1)
C**   ------------------------------------------------------------    **
   40 CALL FFTA(A,B,N/2,ISIGN(1,INV))
C**   ------------------------------------------------------------    **
      AN1R=A(1)-A(2)
      A(1)=A(1)+A(2)
      A(2)=0.0
C**   ------------------------------------------------------------    **
      ANG=-3.1415926535897932D0*ISIGN(2,INV)/N
      W1=DCOS(ANG)
      W2=DSIN(ANG)
      U1=W1
      U2=W2
      J=N-1
      DO 50 I=3,N/2+1,2
      H1=(A(I  )+A(J  ))*0.5
      G2=(A(J  )-A(I  ))*0.5
      G1=(A(I+1)+A(J+1))*0.5
      H2=(A(I+1)-A(J+1))*0.5
      P1=G1*U2+G2*U1
      P2=G2*U2-G1*U1
      A(I  )=H1-P2
      A(J  )=H1+P2
      A(I+1)=P1+H2
      A(J+1)=P1-H2
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   50 J=J-2
C**   ------------------------------------------------------------    **
      GOTO (60,90,55),IABS(INV)
   55 A(2)=AN1R
      RETURN
   60 J=2*N-1
      DO 80 I=3,N,2
      A(J)=A(I)
      A(J+1)=-A(I+1)
   80 J=J-2
   90 A(N+1)=AN1R
      A(N+2)=0.0
      RETURN
      END
************************************************************************
      SUBROUTINE FFTC(A,B,N,INV)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),B(N)
C**   ============================================================    **
C**   FFT of conjugate data for any even N                            **
C**   ------------------------------------------------------------    **
C**   f(2,N) <=== F(2,N)  :         f(2,I)=0,  F(2,1)=F(2,N/2+1)=0    **
C**        F(1,N+1-I)=F(1,I+1), F(2,N+1-I)=-F(2,I+1),  for I=1,N/2    **
C**   ============================================================    **
C**   For INV=+-1  :  DIMENSION A(2*N)                                **
C**   Input  : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=1,N/2+1          **
C**            note that  :     A(2)=0,  A(N+2)=0,  others unused     **
C**   Output : A(2*I-1)=f(1,I), A(2*I)=f(2,I)=0, for I=1,N            **
C**   ------------------------------------------------------------    **
C**   For INV=+-2  :  DIMENSION A(N+2)                                **
C**   Input  : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=1,N/2+1          **
C**            note that  :     A(2)=0,  A(N+2)=0                     **
C**   Output : A(I)=f(1,I), for I=1,N  (f(2,I)=0 not given)           **
C**   ------------------------------------------------------------    **
C**   For INV=+-3  :  DIMENSION A(N)                                  **
C**   Input  : A(2*I-1)=F(1,I), A(2*I)=F(2,I), for I=2,N/2            **
C**            A(1)  =  F(1,1), A(2) = F(1,N/2+1)                     **
C**   Output : A(I)=f(1,I), for I=1,N  (f(2,I)=0 not given)           **
C**   ============================================================    **
C**   Working : B(N) required for N .NE. 2**M only                    **
C**   ============================================================    **
      AN1R=A(2)
      IF(IABS(INV).LT.3) AN1R=A(N+1)
      A(2)=A(1)-AN1R
      A(1)=A(1)+AN1R
C**   ------------------------------------------------------------    **
      ANG=3.1415926535897932D0*ISIGN(2,INV)/N
      W1=DCOS(ANG)
      W2=DSIN(ANG)
      U1=W1
      U2=W2
      J=N-1
      DO 50 I=3,N/2+1,2
      H1=A(I  )+A(J  )   !   Reverse from:    H1=(A(I  )+A(J  ))*0.5
      P2=A(J  )-A(I  )   !                    G2=(A(J  )-A(I  ))*0.5
      P1=A(I+1)+A(J+1)   !  1 <           /   G1=(A(I+1)+A(J+1))*0.5
      H2=A(I+1)-A(J+1)   !      \       /     H2=(A(I+1)-A(J+1))*0.5
      G1=P1*U2-P2*U1     !        \   /       P1=G1*U2+G2*U1
      G2=P2*U2+P1*U1     !  2 <-----X------   P2=G2*U2-G1*U1
      A(I  )=H1-G2       !        /   \       A(I  )=H1-P2
      A(J  )=H1+G2       !      /       \     A(J  )=H1+P2
      A(I+1)=G1+H2       !  3 <           \   A(I+1)=P1+H2
      A(J+1)=G1-H2       !                    A(J+1)=P1-H2
      T1=U1              !
      U1=T1*W1-U2*W2     !   Except *0.5 to be done outside FFTC
      U2=T1*W2+U2*W1
   50 J=J-2
C**   ------------------------------------------------------------    **
      CALL FFTA(A,B,N/2,ISIGN(1,INV))
C**   ------------------------------------------------------------    **
      IF(IABS(INV).NE.1) RETURN
      DO 80 I=N,1,-1
      A(2*I)=0.0
   80 A(2*I-1)=A(I)
      RETURN
      END
************************************************************************
      SUBROUTINE FFTSC(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),B(N)
C**   ============================================================    **
C**   Half-SIN or Half-COS transform of real data for even N          **
C**   ------------------------------------------------------------    **
C**   Input  : A(N)/A(N+1)/A(N) = AI() = real data                    **
C**   Output : A(N)/A(N+1)/A(N) = Asin(N)/Acos(N+1)/Ac(N)             **
C**   ------------------------------------------------------------    **
C**   ICS=+-1: Asin(K+1) = \sum_{J=0}^{N-1} AI(J+1)*SIN(K*J*PI/N)     **
C**   ICS=+-2: Acos(K+1) = \sum_{J=1}^{N-1} AI(J+1)*COS(K*J*PI/N)     **
C**                                + (AI(1)+AI(N+1)*COS(K*PI))*0.5    **
C**   ICS=+3 : Ac(K+1)=\sum_{J=0}^{N-1} AI(J+1)*COS(K*(J+.5)*PI/N)    **
C**   ICS=-3 : Ac(K+1)=\sum_{J=1}^{N-1} AI(J+1)*COS((K+.5)*J*PI/N)    **
C**                                 + 0.5*AI(1)*COS((K+.5)*0*PI/N)    **
C**   ------------------------------------------------------------    **
C**   Working : B(N) required for N .NE. 2**M                         **
C**   ============================================================    **
      IF(IABS(ICS).GE.3) THEN
         CALL FFTCH(A,B,N,ICS)
         RETURN
       ENDIF
C**   ------------------------------------------------------------    **
      ANG=3.1415926535897932D0/N
      W1=DCOS(ANG)
      W2=DSIN(ANG)
C**   ------------------------------------------------------------    **
      U1=W1
      U2=W2
      IF(IABS(ICS).EQ.1) THEN
        A(1)=0.0
      ELSE
        A2  =(A(1)-A(N+1))*0.5
        A(1)=(A(1)+A(N+1))*0.5
      ENDIF
      DO 50 I=2,N/2+1
      IF(IABS(ICS).EQ.1) THEN
        R1=(A(I)-A(N+2-I))*0.5
        R2=(A(I)+A(N+2-I))*U2
        A(I)    =R2+R1
        A(N+2-I)=R2-R1
      ELSE
        R1=(A(I)+A(N+2-I))*0.5
        R2=(A(I)-A(N+2-I))*U2
        A2=(A(I)-A(N+2-I))*U1+A2
        A(I)    =R1-R2
        A(N+2-I)=R1+R2
      ENDIF
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   50 CONTINUE
C**   ------------------------------------------------------------    **
      CALL FFTR(A,B,N,-3)
      IF(IABS(ICS).NE.1) A(N+1)=A(2)
      A(2)=0.0
C**   ------------------------------------------------------------    **
      IF(IABS(ICS).EQ.1) THEN
        A2=-0.5*A(1)
        DO 60 I=1,N-1,2
        A2=A2+A(I)
        A(I)=A(I+1)
        A(I+1)=A2
   60   CONTINUE
      ELSE
        DO 70 I=2,N,2
        A2=A2+A(I)
        A(I)=A2
   70   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE FFTCH(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),B(N)
C**   ============================================================    **
C**   Half-SIN or Half-COS transform of real data for even N          **
C**   ------------------------------------------------------------    **
C**   Input  : A(N) = AI(N) = real data                               **
C**   Output : A(N) = AC(N)                                           **
C**   ------------------------------------------------------------    **
C**   ICS=+3 : AC(K+1)=\sum_{J=0}^{N-1} AI(J+1)*COS(K*(J+.5)*PI/N)    **
C**   ICS=-3 : AC(K+1)=\sum_{J=1}^{N-1} AI(J+1)*COS((K+.5)*J*PI/N)    **
C**                                 + 0.5*AI(1)*COS((K+.5)*0*PI/N)    **
C**   ------------------------------------------------------------    **
C**   Working : B(N) required for N .NE. 2**M                         **
C**   ============================================================    **
      ANG=3.1415926535897932D0/N
      W1=DCOS(ANG)
      W2=DSIN(ANG)
C**   ------------------------------------------------------------    **
      IF(ICS.GT.0) THEN
      U1=DCOS(0.5*ANG)
      U2=DSIN(0.5*ANG)
      DO 20 I=1,N/2
      R1=(A(I)+A(N+1-I))*0.5
      R2=(A(I)-A(N+1-I))*U2
      A(I)    =R1-R2
      A(N+1-I)=R1+R2
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   20 CONTINUE
C**   ------------------------------------------------------------    **
      CALL FFTR(A,B,N,-3)
      AN=A(2)
      A(2)=0.0
C**   ------------------------------------------------------------    **
      AJ=-0.5*AN
      U1=W1
      U2=W2
      DO 30 I=N-1,1,-2
      DJ    =A(I)*U1+A(I+1)*U2
      A(I)  =A(I)*U2-A(I+1)*U1
      A(I+1)=AJ
      AJ=AJ-DJ
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   30 CONTINUE
      RETURN
C**   ------------------------------------------------------------    **
      ELSE
      AN=A(N)
      U1=W1
      U2=W2
      DO 70 I=N-1,3,-2
      DJ    =A(I+1)-A(I-1)
      A(I+1)=DJ*U2-A(I)*U1
      A(I)  =DJ*U1+A(I)*U2
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   70 CONTINUE
C**   ------------------------------------------------------------    **
      A(2)=-2.0*AN
      CALL FFTC(A,B,N,3)
C**   ------------------------------------------------------------    **
      U1=DCOS(0.5*ANG)
      U2=DSIN(0.5*ANG)
      DO 80 I=1,N/2
      R1=(A(I)+A(N+1-I))*0.25
      R2=(A(I)-A(N+1-I))*0.125/U2
      A(I)    =R1-R2
      A(N+1-I)=R1+R2
      T1=U1
      U1=T1*W1-U2*W2
      U2=T1*W2+U2*W1
   80 CONTINUE
      RETURN
      ENDIF
      END
************************************************************************
      SUBROUTINE FFTCS(A,B,N,ICS)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(N),B(N)
C**   ============================================================    **
C**   SIN or COS transform of real data for even N                    **
C**   ------------------------------------------------------------    **
C**   Input  : A(N) = AI(N) = real data                               **
C**   Output : A(N) = Asin(N) or Acos(N)                              **
C**   ------------------------------------------------------------    **
C**   ICS=+-1: Asin(K+1) = \sum_{J=0}^{N-1} AI(J+1)*SIN(K*J*2PI/N)    **
C**   ICS=+-2: Acos(K+1) = \sum_{J=0}^{N-1} AI(J+1)*COS(K*J*2PI/N)    **
C**   ------------------------------------------------------------    **
C**   Working : B(N/2) required for N .NE. 2**M                       **
C**   ============================================================    **
      IF(IABS(ICS).EQ.1) THEN
        A(1)=0.0
        DO 20 I=2,N/2
          A(I)=A(I)-A(N+2-I)
   20   CONTINUE
      ELSE
        A(1)=A(1)+A(1)
        DO 30 I=2,N/2
          A(I)=A(I)+A(N+2-I)
   30   CONTINUE
        A(N/2+1)=A(N/2+1)+A(N/2+1)
      ENDIF
C**   ------------------------------------------------------------    **
      CALL FFTSC(A,B,N/2,ICS)
C**   ------------------------------------------------------------    **
      IF(IABS(ICS).EQ.1) THEN
        A(N/2+1)=0.0
        DO 40 I=2,N/2
          A(N+2-I)=-A(I)
   40   CONTINUE
      ELSE
        DO 60 I=2,N/2
          A(N+2-I)=A(I)
   60   CONTINUE
      ENDIF
      RETURN
      END
************************************************************************
