************************************************************************
      SUBROUTINE FFTTRR(W,E,YST,Y,DT,N,U,U0,V0)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Y(N),U(N+2)
C**   ============================================================    **
C**   This subroutine computes transient response of the equation     **
C**   Rel.A(I) + 2*E*W*V(I) + W*W*U(I) = W*W*YST*Y(I)                 **
C**   with initial conditions :   U(1)=U0,   V(1)=V0                  **
C**   and N=2**M by Fast Fourier Transform.  (V=dU/dT, A=dV/dT)       **
C**   ------------------------------------------------------------    **
C*I   W    = Natural frequence of the system                          **
C*I   E    = Damping ratio of the system                              **
C*I   YST  = Scale constant                                           **
C**        > 0 : for transient response                               **
C**        < 0 : for steady-state response                            **
C*I   Y(I) = Input force divided by W*W*YST at T=(I-1)*DT             **
C*I   DT   = Time interval                                            **
C*I   N    = Number of data points                                    **
C*O   U(I) = Output response at T=(I-1)*DT                            **
C*I   U0   = Initial displacement at T=0                              **
C*I   V0   = Initial velocity at T=0                                  **
C**   ------------------------------------------------------------    **
C**   Array U(N+2) may be equivalence to Y(N).                        **
C**   If N .NE. 2**Integer then array U() must has length 2*N+2       **
C**   ============================================================    **
      DO 10 I=1,N
   10 U(I)=Y(I)*YST*W*W
C**   +----------------------------------------------------------+    **
C**   | Perform Fourier Transform by FFT(Real)                   |    **
C**   +----------------------------------------------------------+    **
      CALL FFTR(U,U(N+3),N,2)
C**   +----------------------------------------------------------+    **
C**   | Multiple by transfer function H(I*DW) and divided by N   |    **
C**   +----------------------------------------------------------+    **
      DW=6.2831853071795864D0/(DT*N)
      DO 30 I=1,N/2+1
      H1= W**2-((I-1)*DW)**2
      H2=-2.*E*W*DW*(I-1)
      HH=(H1*H1+H2*H2)*N
      U1=U(2*I-1)
      U(2*I-1)=(U1*H1-U(2*I)*H2)/HH
   30 U(2*I  )=(U1*H2+U(2*I)*H1)/HH
      U(N+2)=0.0
C**   +----------------------------------------------------------+    **
C**   | For YST<0 : Compute steady-state response by Inverse FFT |    **
C**   +----------------------------------------------------------+    **
      IF(YST.LT.0) THEN
        CALL FFTC(U,U(N+3),N,-2)
        RETURN
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | For YST>0 : Get initial value of steady-state response   |    **
C**   +----------------------------------------------------------+    **
      UO=0.0
      VO=0.0
      DO 40 I=2,N/2
      UO=UO+U(2*I-1)
   40 VO=VO-U(2*I)*DW*(I-1)
      UO=UO+UO+U(1)+U(N+1)
      VO=VO+VO
C**   +----------------------------------------------------------+    **
C**   | Then : Compute the steady-state response by Inverse FFT  |    **
C**   +----------------------------------------------------------+    **
      CALL FFTC(U,U(N+3),N,-2)
C**   +----------------------------------------------------------+    **
C**   | And : Adjust I.C. to U0 and V0 for transient response    |    **
C**   +----------------------------------------------------------+    **
      UU=U0-UO
      VV=(V0-VO)/W
      D=DSQRT(1.0-E*E)
      O=DATAN(E/D)
      EDT=1.0/D
      WDDT=W*D*DT
      EWDT=DEXP(-E*W*DT)
C**   ------------------------------------------------------------    **
      DO 50 I=1,N
      WDT=WDDT*(I-1)
      U(I)=U(I)+EDT*(UU*DCOS(WDT-O)+VV*DSIN(WDT))
   50 EDT=EDT*EWDT
      RETURN
      END
************************************************************************
