************************************************************************
      PROGRAM PLATEM
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:8704)
      DATA NDM/8705/
C**   +----------------------------------------------------------+    **
C**   | N1 x N2 = sub-sections in 2-directions                   |    **
C**   +----------------------------------------------------------+    **
   50 READ (*,'(2I5)') N1,N2
      NA=0
      NW=NA+N1*N2
      NT=NW+MAX0(N1,N2+N2)
      IF(NT.GT.NDM) STOP 1111
      CALL PLATS(A(NA),A(NW),N1,N2,N1)
      GOTO 50
      END
************************************************************************
      SUBROUTINE PLATS(A,W,N1,N2,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:NRA-1,0:N2-1),W(0:N2-1)
      DATA PI/3.1415926535897932D0/
C**   ============================================================    **
C**   Analysis of simply supported rectangular plate : XL * YL        **
C**   ------------------------------------------------------------    **
C**   XL,YL    = X/Y-Length of the rectangular plate                  **
C**   D        = Rigidity of the plate                                **
C**   F0,F1,NP = Factors for distributed loading                      **
C**   A(I,J)   = Loading per unit area at point (I,J)                 **
C**            = (F0 + F1*((I-N1/2)*DX+(J-N2/2)*DY))*PI**NP           **
C**   IP       = 0:15 = Index for intermediate output                 **
C**   ============================================================    **
C**   WRITE(*,'('' XL,YL,D,F0,F1,NP,IP ?'')')
      READ (*,'(5F10.0,2I5)') XL,YL,D,F0,F1,NP,IP
      IF(XL.EQ.0.0) XL=1.0
      IF(YL.EQ.0.0) YL=XL
      IF(D.EQ.0.0) D=1.0
      IF(F0.EQ.0.0.AND.F1.EQ.0.0) F0=1.0
C**   ------------------------------------------------------------    **
      DX=XL/N1
      DY=YL/N2
C**   +----------------------------------------------------------+    **
C**   | Set A(I,J) = Loading per unit area                       |    **
C**   | at point (I,J) for I=1:N1-1, J=1:N2-1                    |    **
C**   +----------------------------------------------------------+    **
      DO 120 J=1,N2-1
      DO 110 I=1,N1-1
      A(I,J)=(F0+F1*((I-N1/2)*DX+(J-N2/2)*DY))*PI**NP
  110 CONTINUE
  120 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Call PLATE to comput A(I,J) = deflection at (I,J)        |    **
C**   +----------------------------------------------------------+    **
      CALL PLATE(A,N1,N2,D,XL,YL,W,IP,NRA)
      DMAX=A(0,0)
      DMIN=A(0,0)
      DO 200 J=0,N2-1
      DO 200 I=0,N1-1
      DMAX=DMAX1(DMAX,A(I,J))
      DMIN=DMIN1(DMIN,A(I,J))
  200 CONTINUE
      WRITE(*,'('' Max. & Min. deflection :'',1P2E15.7)') DMAX,DMIN
      WRITE(*,'('' XL,YL,D,F0,F1,NP,N1,N2 :'',5F7.1,3I5)')
     *             XL,YL,D,F0,F1,NP,N1,N2
      RETURN
      END
************************************************************************
      SUBROUTINE PLATE(A,N1,N2,D,XL,YL,W,IP,NRA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(0:NRA-1,0:N2-1),W(0:N2-1)
      DATA PI/3.1415926535897932D0/
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | B.C. for simply supported rectangular plate XL * YL      |    **
C**   +----------------------------------------------------------+    **
      DO 20 I=0,N1-1
   20 A(I,0)=0.0
      DO 30 J=0,N2-1
   30 A(0,J)=0.0
      IF(IP/2*2.NE.IP) THEN
        DO 35 J=0,N2-1
        WRITE(*,'('' Loading : A1(I,'',I4,'') :'')') J
   35   WRITE(*,'('' I ='',I4,1PE15.6)') (I,A(I,J),I=0,N1-1)
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Amplitute of the sine-loading                            |    **
C**   +----------------------------------------------------------+    **
      CALL FFT2SC(A,N1,N2,1,1,W,NRA)
      IF(IP/4*2.NE.IP/2) THEN
        DO 45 J=0,N2-1
        WRITE(*,'('' Sin-transform of A1 : A2(I,'',I4,'') :'')') J
   45   WRITE(*,'('' I ='',I4,1PE15.6)') (I,A(I,J),I=0,N1-1)
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Amplitute of the sine-deflection due to the sine-loading |    **
C**   +----------------------------------------------------------+    **
      PPAA=N1*N2*D*PI**4/4
      DO 60 J=1,N2-1
      DO 60 I=1,N1-1
      A(I,J)=A(I,J)/(PPAA*((I/XL)**2+(J/YL)**2)**2)
   60 CONTINUE
      IF(IP/8*2.NE.IP/4) THEN
        DO 65 J=0,N2-1
        WRITE(*,'('' A2*H : A3(I,'',I4,'') :'')') J
   65   WRITE(*,'('' I ='',I4,1PE15.6)') (I,A(I,J),I=0,N1-1)
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Deflection at point (I,J)                                |    **
C**   +----------------------------------------------------------+    **
      CALL FFT2SC(A,N1,N2,-1,-1,W,NRA)
      IF(IP/16*2.NE.IP/8) THEN
        DO 75 J=0,N2-1
        WRITE(*,'('' Deflection : A4(I,'',I4,'') :'')') J
   75   WRITE(*,'('' I ='',I4,1PE15.6)') (I,A(I,J),I=0,N1-1)
      ENDIF
      RETURN
      END
************************************************************************
