************************************************************************
      SUBROUTINE FEMRCM(JOEP,JOES,LVSN,EOJP,EOJS,NELM,NODE)
C**   ============================================================    **
      INTEGER JOEP(1),JOES(1),LVSN(1),EOJP(1),EOJS(1)
C**   ============================================================    **
C**   Perform the RCM ordering to get LVSN(NODE)                      **
C**   ------------------------------------------------------------    **
C**   Input  : JOEP(NELM+1),JOES(JOEP(NELM+1)-1),NELM,NODE            **
C**   Output : EOJP(NODE+1),EOJS(EOJP(NODE+1)-1),LVSN(NODE)           **
C**   ------------------------------------------------------------    **
C*I   JOES(JOEP(I)),...,JOES(JOEP(I+1)-1) = Nodes Of Element I        **
C*O   EOJS(EOJP(J)),...,EOJS(EOJP(J+1)-1) = Elements of Node J        **
C*O   LVSN(I) = The original node number of the new I-th node         **
C*I   NELM    = Number of Elements                                    **
C*I   NODE    = Number of Nodes                                       **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Set LVSN(J) temporarily as the degree of node J          |    **
C**   +----------------------------------------------------------+    **
      DO 100 NOD=1,NODE
  100 LVSN(NOD)=0
      DO 180 MEM=1,NELM
      JSTRT=JOEP(MEM)
      JSTOP=JOEP(MEM+1)-1
      IF(JSTRT.GT.JSTOP) GO TO 180
      DO 170 J=JSTRT,JSTOP
      NOD=JOES(J)
  170 LVSN(NOD)=LVSN(NOD)+1
  180 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Setup a list of the element connected to each node       |    **
C**   | Given LVSN(J) as the degree of node J                    |    **
C**   +----------------------------------------------------------+    **
      EOJP(1)=1
      DO 200 NOD=1,NODE
      EOJP(NOD+1)=EOJP(NOD)+LVSN(NOD)
  200 LVSN(NOD)=EOJP(NOD)
      DO 280 MEM=1,NELM
      JSTRT=JOEP(MEM)
      JSTOP=JOEP(MEM+1)-1
      IF(JSTRT.GT.JSTOP) GO TO 280
      DO 270 J=JSTRT,JSTOP
      NOD=JOES(J)
      K=LVSN(NOD)
      LVSN(NOD)=K+1
  270 EOJS(K)=MEM
  280 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Set EOJP(J) < 0 to mark node J as an un-re-numbered node |    **
C**   +----------------------------------------------------------+    **
      DO 500 NOD=1,NODE
  500 EOJP(NOD)=-EOJP(NOD)
C**   +----------------------------------------------------------+    **
C**   | Call EMRRCM to re-number node begin from ROOT = NOD      |    **
C**   +----------------------------------------------------------+    **
      NBGN=1
      DO 600 NOD=1,NODE
      IF(EOJP(NOD).GT.0) GO TO 600
      CALL EMRRCM(NOD,JOEP,JOES,EOJP,EOJS,LVSN(NBGN),NELM,NSIZ)
      NBGN=NBGN+NSIZ
      IF(NBGN.GT.NODE) RETURN
  600 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE EMRRCM(IROOT,JOEP,JOES,EOJP,EOJS,LVSN,NELM,NSIZ)
C**   ============================================================    **
      INTEGER JOEP(1),JOES(1),EOJP(1),EOJS(1),LVSN(1),LVBG(2),ROOT
      DATA KLVL/2/,MAXINT/32767/
C**   ============================================================    **
      ROOT=IROOT
      IROO=0
      NLVL=0
      NBND=MAXINT
      NENV=MAXINT
C**   +----------------------------------------------------------+    **
C**   | Single Root begin re-number from here                    |    **
C**   +----------------------------------------------------------+    **
  100 NSIZ=1
      LVSN(NSIZ)=ROOT
      EOJP(ROOT)=-EOJP(ROOT)
C**   +----------------------------------------------------------+    **
C**   | Multiple Roots begin re-number from here                 |    **
C**   +----------------------------------------------------------+    **
  110 NLVO=NLVL
      NBNO=NBND
      NENO=NENV
      NLVL=0
      NBND=0
      NENV=0
      LVEND=0
      DO 120 K=1,KLVL
  120 LVBG(K)=1
C**   +----------------------------------------------------------+    **
C**   | LVBGN & LVEND = Pointers to first & last of this level   |    **
C**   +----------------------------------------------------------+    **
  200 NLVL=NLVL+1
      LVBGN=LVEND+1
      LVEND=NSIZ
      DO 210 K=2,KLVL
  210 LVBG(K-1)=LVBG(K)
      LVBG(KLVL)=LVBGN
C**   +----------------------------------------------------------+    **
C**   | Generate the new level by finding all the masked         |    **
C**   | un-re-numbered neighbors of nodes in this level          |    **
C**   +----------------------------------------------------------+    **
      DO 400 I=LVBGN,LVEND
      NOD=LVSN(I)
      KSTRT=IABS(EOJP(NOD))
      KSTOP=IABS(EOJP(NOD+1))-1
      IF(KSTRT.GT.KSTOP) GO TO 400
      NSAV=NSIZ
      DO 240 K=KSTRT,KSTOP
      MEM=EOJS(K)
      IF(JOEP(MEM).LT.0) GO TO 240
      JOEP(MEM)=-JOEP(MEM)
      JSTRT=IABS(JOEP(MEM))
      JSTOP=IABS(JOEP(MEM+1))-1
C     IF(JSTRT.GT.JSTOP) GO TO 240
      DO 230 J=JSTRT,JSTOP
      NOD=JOES(J)
      IF(EOJP(NOD).GT.0) GO TO 230
      EOJP(NOD)=-EOJP(NOD)
      NSIZ=NSIZ+1
      LVSN(NSIZ)=NOD
  230 CONTINUE
  240 CONTINUE
C**
      IF(NSAV.GE.NSIZ) GO TO 320
      NDIF=NSIZ-I
      NJX=NSAV+1
      NJY=NSIZ
      IF(NJX.GE.NJY) GO TO 380
C**   +----------------------------------------------------------+    **
C**   | Shaker sorting LVSN(NJX) ... LVSN(NJY)                   |    **
C**   +----------------------------------------------------------+    **
      JYP=NJX
  250 JXP=JYP
      JYP=JYP+1
      JY=LVSN(JYP)
      JYT=IABS(EOJP(JY+1))-IABS(EOJP(JY))
C**   ------------------------------------------------------------    **
  260 JX=LVSN(JXP)
      JXT=IABS(EOJP(JX+1))-IABS(EOJP(JX))
      IF(JXT.LE.JYT) GO TO 270
      LVSN(JXP+1)=JX
      JXP=JXP-1
      IF(JXP.GE.NJX) GO TO 260
C**   ------------------------------------------------------------    **
  270 LVSN(JXP+1)=JY
      IF(JYP.LT.NJY) GO TO 250
      GO TO 380
C**   +----------------------------------------------------------+    **
C**   | Compute the exact envelope of RCM ordering               |    **
C**   | by reducing the envelope of CM ordering                  |    **
C**   +----------------------------------------------------------+    **
  320 IF(NSAV.LE.I) GO TO 360
      NOD=LVSN(NSAV)
      DO 340 K=KSTRT,KSTOP
      MEM=EOJS(K)
      JSTRT=IABS(JOEP(MEM))
      JSTOP=IABS(JOEP(MEM+1))-1
      DO 340 J=JSTRT,JSTOP
      IF(JOES(J).EQ.NOD) GO TO 360
  340 CONTINUE
      NSAV=NSAV-1
      GO TO 320
  360 NDIF=NSAV-I
  380 NBND=MAX0(NBND,NDIF)
      NENV=NENV+NDIF
  400 CONTINUE
      IF(NSIZ.GT.LVEND) GO TO 200
C**   +----------------------------------------------------------+    **
C**   | No more node in the new level(All nodes are re-numbered) |    **
C**   | Compare the Depth(Band,Envelope) of two level structures |    **
C**   +----------------------------------------------------------+    **
  405 FORMAT(/' ** ROOT,NLVL,NBND,NENV/LVSN(*)=',4I8/(10I8))
      WRITE(*,405) ROOT,NLVL,NBND,NENV,(LVSN(I),I=1,NSIZ)
      IF(ROOT.EQ.IABS(IROO).OR.NLVL.EQ.NSIZ) GO TO 800
      IF(NLVL-NLVO) 700,420,450
  420 IF(NBND-NBNO) 450,430,700
  430 IF(NENV-NENO) 450,800,700
C**   +----------------------------------------------------------+    **
C**   | Find new ROOT from the last level with minimum degree    |    **
C**   +----------------------------------------------------------+    **
  450 IROO=ROOT
      MINDEG=NSIZ
      DO 500 I=LVBGN,LVEND
      NOD=LVSN(I)
      NDEG=IABS(EOJP(NOD+1))-IABS(EOJP(NOD))
      IF(NDEG.GE.MINDEG) GO TO 500
      MINDEG=NDEG
      ROOT=NOD
  500 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Reset the mark to restart the re-numbering               |    **
C**   +----------------------------------------------------------+    **
  550 DO 600 I=1,NSIZ
      NOD=LVSN(I)
  600 EOJP(NOD)=-EOJP(NOD)
      DO 620 MEM=1,NELM
  620 JOEP(MEM)=IABS(JOEP(MEM))
      GO TO 100
C**   +----------------------------------------------------------+    **
C**   | Reconstruct previous level structure                     |    **
C**   +----------------------------------------------------------+    **
  700 ROOT=IROO
      GO TO 550
C**   +----------------------------------------------------------+    **
C**   | Reverse the Cuthill-Mckee ordering                       |    **
C**   +----------------------------------------------------------+    **
  800 NS=NSIZ/2
      L=NSIZ
      DO 900 I=1,NS
      NOD=LVSN(I)
      LVSN(I)=LVSN(L)
      LVSN(L)=NOD
  900 L=L-1
      DO 920 MEM=1,NELM
  920 JOEP(MEM)=IABS(JOEP(MEM))
      IF(NENV.GT.NENO) GO TO 550
      IF(IROO.LT.0) RETURN
      IROO=-ROOT
C**   +----------------------------------------------------------+    **
C**   | Use all nodes in the last KLVL levels as Multiple Roots  |    **
C**   +----------------------------------------------------------+    **
      I1=LVEND-LVBG(1)+2
      DO 960 I=I1,NSIZ
      NOD=LVSN(I)
  960 EOJP(NOD)=-EOJP(NOD)
      NSIZ=I1-1
      GO TO 110
      END
************************************************************************
