************************************************************************
      SUBROUTINE GENRCM(NODE,XADJ,ADJN,LVSN)
C**   ============================================================    **
      INTEGER XADJ(1),ADJN(1),LVSN(1)
C**   ============================================================    **
C**   Perform the RCM ordering to get LVSN(NODE)                      **
C**   ------------------------------------------------------------    **
C**   Input  : XADJ(NODE+1),ADJN(XADJ(NODE+1)-1),NODE                 **
C**   Output : LVSN(NODE),  ADJN : Sorted by degree in incr. order    **
C**   ------------------------------------------------------------    **
CIO   ADJN(XADJ(J)),...,ADJN(XADJ(J+1)-1) = Adjacency Nodes Of J      **
C*O   LVSN(I) = The original node number of the new I-th node         **
C*I   NODE    = Number of Nodes                                       **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Set LVSN(J) temporarily as the degree of node J          |    **
C**   +----------------------------------------------------------+    **
      DO 250 J=1,NODE
  250 LVSN(J)=XADJ(J+1)-XADJ(J)
C**   +----------------------------------------------------------+    **
C**   | Shaker sorting ADJN(NJX) ... ADJN(NJY)                   |    **
C**   +----------------------------------------------------------+    **
      DO 380 J=1,NODE
      NJX=XADJ(J)
      NJY=XADJ(J+1)-1
      IF(NJX.GE.NJY) GO TO 380
      JYP=NJX
  330 JXP=JYP
      JYP=JYP+1
      JY=ADJN(JYP)
      JYT=LVSN(JY)
C**   ------------------------------------------------------------    **
  340 JX=ADJN(JXP)
      JXT=LVSN(JX)
      IF(JXT.LE.JYT) GO TO 350
      ADJN(JXP+1)=JX
      JXP=JXP-1
      IF(JXP.GE.NJX) GO TO 340
C**   ------------------------------------------------------------    **
  350 ADJN(JXP+1)=JY
      IF(JYP.LT.NJY) GO TO 330
  380 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Set XADJ(J) < 0 to mark node J as an un-re-numbered node |    **
C**   +----------------------------------------------------------+    **
      DO 500 J=1,NODE
  500 XADJ(J)=-XADJ(J)
C**   +----------------------------------------------------------+    **
C**   | Call MRRCM to re-number node begin from ROOT = J         |    **
C**   +----------------------------------------------------------+    **
      NBGN=1
      DO 600 J=1,NODE
      IF(XADJ(J).GT.0) GO TO 600
      CALL MRRCM(J,XADJ,ADJN,LVSN(NBGN),NSIZ)
      NBGN=NBGN+NSIZ
      IF(NBGN.GT.NODE) RETURN
  600 CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE MRRCM(IROOT,XADJ,ADJN,LVSN,NSIZ)
C**   ============================================================    **
      INTEGER XADJ(1),ADJN(1),LVSN(1),LVBG(2),ROOT
      DATA KLVL/2/,MAXINT/32767/
C**   ============================================================    **
      ROOT=IROOT
      IROO=0
      NLVL=0
      NBND=MAXINT
      NENV=MAXINT
C**   +----------------------------------------------------------+    **
C**   | Single Root begin re-number from here                    |    **
C**   +----------------------------------------------------------+    **
  100 NSIZ=1
      LVSN(NSIZ)=ROOT
      XADJ(ROOT)=-XADJ(ROOT)
C**   +----------------------------------------------------------+    **
C**   | Multiple Roots begin re-number from here                 |    **
C**   +----------------------------------------------------------+    **
  110 NLVO=NLVL
      NBNO=NBND
      NENO=NENV
      NLVL=0
      NBND=0
      NENV=0
      LVEND=0
      DO 120 K=1,KLVL
  120 LVBG(K)=1
C**   +----------------------------------------------------------+    **
C**   | LVBGN & LVEND = Pointers to first & last of this level   |    **
C**   +----------------------------------------------------------+    **
  200 NLVL=NLVL+1
      LVBGN=LVEND+1
      LVEND=NSIZ
      DO 210 K=2,KLVL
  210 LVBG(K-1)=LVBG(K)
      LVBG(KLVL)=LVBGN
C**   +----------------------------------------------------------+    **
C**   | Generate the new level by finding all the masked         |    **
C**   | un-re-numbered neighbors of nodes in this level          |    **
C**   +----------------------------------------------------------+    **
      DO 400 I=LVBGN,LVEND
      NOD=LVSN(I)
      JSTRT=IABS(XADJ(NOD))
      JSTOP=IABS(XADJ(NOD+1))-1
      IF(JSTRT.GT.JSTOP) GO TO 400
      NSAV=NSIZ
      DO 300 J=JSTRT,JSTOP
      NOD=ADJN(J)
      IF(XADJ(NOD).GT.0) GO TO 300
      XADJ(NOD)=-XADJ(NOD)
      NSIZ=NSIZ+1
      LVSN(NSIZ)=NOD
  300 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Compute the exact envelope of RCM ordering               |    **
C**   | by reducing the envelope of CM ordering                  |    **
C**   +----------------------------------------------------------+    **
      NDIF=NSIZ-I
      IF(NSAV.LT.NSIZ) GO TO 380
  320 IF(NSAV.LE.I) GO TO 360
      NOD=LVSN(NSAV)
      DO 340 J=JSTRT,JSTOP
      IF(ADJN(J).EQ.NOD) GO TO 360
  340 CONTINUE
      NSAV=NSAV-1
      GO TO 320
  360 NDIF=NSAV-I
  380 NBND=MAX0(NBND,NDIF)
      NENV=NENV+NDIF
  400 CONTINUE
      IF(NSIZ.GT.LVEND) GO TO 200
C**   +----------------------------------------------------------+    **
C**   | No more node in the new level(All nodes are re-numbered) |    **
C**   | Compare the Depth(Band,Envelope) of two level structures |    **
C**   +----------------------------------------------------------+    **
  405 FORMAT(/' ** ROOT,NLVL,NBND,NENV/LVSN(*)=',4I8/(10I8))
      WRITE(*,405) ROOT,NLVL,NBND,NENV,(LVSN(I),I=1,NSIZ)
      IF(ROOT.EQ.IABS(IROO).OR.NLVL.EQ.NSIZ) GO TO 800
      IF(NLVL-NLVO) 700,420,450
  420 IF(NBND-NBNO) 450,430,700
  430 IF(NENV-NENO) 450,800,700
C**   +----------------------------------------------------------+    **
C**   | Find new ROOT from the last level with minimum degree    |    **
C**   +----------------------------------------------------------+    **
  450 IROO=ROOT
      MINDEG=NSIZ
      DO 500 I=LVBGN,LVEND
      NOD=LVSN(I)
      NDEG=IABS(XADJ(NOD+1))-IABS(XADJ(NOD))
      IF(NDEG.GE.MINDEG) GO TO 500
      MINDEG=NDEG
      ROOT=NOD
  500 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Reset the mark to restart the re-numbering               |    **
C**   +----------------------------------------------------------+    **
  550 DO 600 I=1,NSIZ
      NOD=LVSN(I)
  600 XADJ(NOD)=-XADJ(NOD)
      GO TO 100
C**   +----------------------------------------------------------+    **
C**   | Reconstruct previous level structure                     |    **
C**   +----------------------------------------------------------+    **
  700 ROOT=IROO
      GO TO 550
C**   +----------------------------------------------------------+    **
C**   | Reverse the Cuthill-Mckee ordering                       |    **
C**   +----------------------------------------------------------+    **
  800 NS=NSIZ/2
      L=NSIZ
      DO 900 I=1,NS
      NOD=LVSN(I)
      LVSN(I)=LVSN(L)
      LVSN(L)=NOD
  900 L=L-1
      IF(NENV.GT.NENO) GO TO 550
      IF(IROO.LT.0) RETURN
      IROO=-ROOT
C**   +----------------------------------------------------------+    **
C**   | Use all nodes in the last KLVL levels as Multiple Roots  |    **
C**   +----------------------------------------------------------+    **
      I1=LVEND-LVBG(1)+2
      DO 960 I=I1,NSIZ
      NOD=LVSN(I)
  960 XADJ(NOD)=-XADJ(NOD)
      NSIZ=I1-1
      GO TO 110
      END
************************************************************************
