************************************************************************
      FUNCTION GAUSS(AMEAN,SIGMA)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON/RAN000/ N,X(99),F(99),D(99),C(100)
      DATA IX/-1/
C**   ============================================================    **
C**   Use mixed method (invert method & reject-select method)         **
C**   to find the normal distributed number                           **
C**   ============================================================    **
      IF(IX.LT.0) THEN
         IX=11111
C**      +-------------------------------------------------------+    **
C**      | Set N,X(1:N),F(1:N),D(2:N),C(1:N+1)                   |    **
C**      | for unit std. div. zero mean normal dist.             |    **
C**      +-------------------------------------------------------+    **
         A=1.0
         N=2
         X(1)=-A
         X(2)=+A
         C(1)=DEXP(-0.5*X(1)**2)/(DSQRT(6.283185307179586D0)*DABS(X(1)))
         C(2)=1.0/DSQRT(6.283185307179586D0)
         C(3)=C(1)
         D(2)=EXP(-0.5*X(1)**2)
C**      +-------------------------------------------------------+    **
C**      | Calculate referential F(1:N) from X(1:N) and C(1:N+1) |    **
C**      | Follows are general procedure (for N>0)               |    **
C**      +-------------------------------------------------------+    **
C**      F(1)=C(1)                                                    **
C**      DO 30 I=2,N                                                  **
C**30    F(I)=F(I-1)+(X(I)-X(I-1))*C(I)                               **
C**      SNP1=F(N)+C(N+1)                                             **
C**      DO 50 I=1,N                                                  **
C**50    F(I)=F(I)/SNP1                                               **
C**      Next st. is to prevent choose from 1st section               **
C**      IF (F(1).EQ.0.0) F(1)=-0.00001                               **
C**      +-------------------------------------------------------+    **
C**      | But for this special case we use                      |    **
C**      +-------------------------------------------------------+    **
         F(1)=C(1)/(C(1)+2.*A*C(2)+C(3))
         F(2)=1.0-F(1)
      ENDIF
C**   ------------------------------------------------------------    **
      GAUSS = AMEAN + SIGMA * RANFUN(IX)
      RETURN
      END
************************************************************************
      FUNCTION DENFUN(X)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   COMMON/RAN000/ N,X(99),F(99),D(99),C(100)
      DATA RSQR2P/0.3989422804014327D0/
C**   ============================================================    **
C**   DENFUN = density function of a random number X                  **
C**   For unit std. div. zero mean normal dist.                       **
C**   ============================================================    **
      DENFUN = RSQR2P * DEXP(-0.5*X*X)
      RETURN
      END
************************************************************************
      SUBROUTINE FINVL(DIS,VAR,DEN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON/RAN000/ N,X(99),F(99),D(99),C(100)
C**   ============================================================    **
C**   Input : DIS = Distribution function                             **
C**   Output: VAR = Random number                                     **
C**           DEN = Density function                                  **
C**   ------------------------------------------------------------    **
C**   Partial Rayleigh distribution (VAR .GE. A) for SIGMA = 1        **
C**   For better result use A .GE. SIGMA                              **
C**   ============================================================    **
      ASQ = X(1)**2
      XSQ = ASQ-2.*DLOG(DIS)
      VAR = -DSQRT(XSQ)
      DEN = -VAR * DEXP(-0.5*(XSQ-ASQ))
      RETURN
      END
************************************************************************
      SUBROUTINE FINVR(DIS,VAR,DEN)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON/RAN000/ N,X(99),F(99),D(99),C(100)
C**   ============================================================    **
      ASQ = X(N)**2
      XSQ = ASQ-2.*DLOG(DIS)
      VAR = DSQRT(XSQ)
      DEN = VAR * DEXP(-0.5*(XSQ-ASQ))
      RETURN
      END
************************************************************************
