************************************************************************
      FUNCTION RANDOM(IX)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DATA MAXN/2147483647/
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Random number by linear congruential method              |    **
C**   | For 32 bits INTEGER*4: C=51605, M-1=2147483647           |    **
C**   | For 16 bits INTEGER*2: C=405,   M-1=32767                |    **
C**   | IX = MOD( 51605*IX+3, 2**32)                             |    **
C**   +----------------------------------------------------------+    **
      IX = 51605*IX + 3
C**   +----------------------------------------------------------+    **
C**   | Next statement changes the sign bit from 1 to 0          |    **
C**   | That is : IX = MOD( 51605*IX+3, 2**31)                   |    **
C**   |           for 1'S complement, cancel +1                  |    **
C**   +----------------------------------------------------------+    **
      IF(IX .LT. 0) IX = IX+MAXN+1
C**   +----------------------------------------------------------+    **
C**   | RANDOM = Uniform distributed number between 0 and 1      |    **
C**   +----------------------------------------------------------+    **
      RANDOM = IX/2147483647.0D0
C**   +----------------------------------------------------------+    **
C**   | If you can not avoid overflow error                      |    **
C**   | Use follows for 32 bits INTEGER*4:                       |    **
C**   +----------------------------------------------------------+    **
C     IX = MOD(405*IX+3,32768)
C     RANDOM = IX/32767.0D0
      RETURN
      END
************************************************************************
      FUNCTION RANFUN(IX)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      COMMON/RAN000/ N,X(99),F(99),D(99),C(100)
C**   ============================================================    **
C**   General program for any distributed random number               **
C**   By mixed method : invert method & reject-select method          **
C**   ------------------------------------------------------------    **
C**   Given : N,X(1:N),F(1:N),D(2:N),C(1:N+1) & following routines    **
C**   DENFUN(XX) = Density function of XX                             **
C**   FINVL(DIS,VAR,DEN) & FINVR(DIS,VAR,DEN)                         **
C**   = Referential distribution function for first & last sections   **
C**     (VAR can be find by invert method from DIS)                   **
C**   Input : DIS = Distribution function                             **
C**   Output: VAR = Random number                                     **
C**           DEN = Density function                                  **
C**   ===============================================                 **
C**    N   |    Conditions    |    Reqd. Functions                    **
C**   =====+==================+======================                 **
C**   N=0  |                  |  DENFUN, FINVL                        **
C**   -----+------------------+----------------------                 **
C**        |       F(1)=0     |  DENFUN,        FINVR                 **
C**   N=1  |   0 < F(1) < 1   |  DENFUN, FINVL, FINVR                 **
C**        |       F(1)=1     |  DENFUN, FINVL                        **
C**   -----+------------------+----------------------                 **
C**        |  0=F(1)  F(N)=1  |  DENFUN                               **
C**        |  0=F(1)  F(N)<1  |  DENFUN,        FINVR                 **
C**   N>1  |  0<F(1)  F(N)<1  |  DENFUN, FINVL, FINVR                 **
C**        |  0<F(1)  F(N)=1  |  DENFUN, FINVL                        **
C**   ===============================================                 **
C**   +----------------------------------------------------------+    **
C**   | IF F(I) = Referential distribution function              |    **
C**   | Here is for GO TO 50                                     |    **
C**   +----------------------------------------------------------+    **
   50 CONTINUE
C*1   ------------------------------------------------------------    **
      U0 = RANDOM(IX)
C*2   +----------------------------------------------------------+    **
C**   | Find I by binary search such that                        |    **
C**   | F(I-1) < U0 <= F(I)                                      |    **
C**   +----------------------------------------------------------+    **
      IBGN=1
      IEND=N+1
   10 IF (IBGN.GE.IEND) THEN
        I=IEND
      ELSE
        I=(IBGN+IEND)/2
        IF (U0.LT.F(I)) THEN
          IEND=I
          GO TO 10
        ELSE IF (U0.GT.F(I)) THEN
          IBGN=I+1
          GO TO 10
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | IF F(I) = Actual distribution function                   |    **
C**   | Here is for GO TO 50                                     |    **
C**   +----------------------------------------------------------+    **
C**50 CONTINUE
C*3   ------------------------------------------------------------    **
      U1 = RANDOM(IX)
      U2 = RANDOM(IX)
C*4   ------------------------------------------------------------    **
      IF (I.LT.2) THEN
        CALL FINVL (U1,XX,DEN)
        ZZ = C(1)   * DEN
      ELSE IF (I.GT.N) THEN
        CALL FINVR (U1,XX,DEN)
        ZZ = C(N+1) * DEN
      ELSE
        XX = X(I-1) + (X(I)-X(I-1)) * U1
        IF (D(I) .GE. U2) THEN
          RANFUN = XX
          RETURN
        ENDIF
        ZZ = C(I)
      ENDIF
C*5   ------------------------------------------------------------    **
      YY = DENFUN (XX)
C**   IF (YY .GT. ZZ) STOP 1111
C*6   +----------------------------------------------------------+    **
C**   | Reject XX                                                |    **
C**   +----------------------------------------------------------+    **
      IF (YY .LT. ZZ*U2) GO TO 50
C**   +----------------------------------------------------------+    **
C**   | Select XX                                                |    **
C**   +----------------------------------------------------------+    **
      RANFUN = XX
      RETURN
      END
************************************************************************
