************************************************************************
      SUBROUTINE BFGSLG(N,Xc,Xp,Gc,Gp,EPSMAH,ETA,ANGRAD,D,L,H,S,Y,T,VM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Xc(N),Xp(N),Gc(N),Gp(N),S(N),Y(N),T(N),D(N),L(N),H(1)
C**   ============================================================    **
C**   Update Hp from Hc                                               **
C**   ------------------------------------------------------------    **
C*I   N      = Number of variables                                    **
C*I   Xc(N)  = Last point                                             **
C*I   Xp(N)  = This point                                             **
C*I   Gc(N)  = Gradient at Xc                                         **
C*I   Gp(N)  = Gradient at Xp                                         **
C*I   EPSMAH = Machine epsilon                                        **
C**   ETA,ANGRAD = Dummy (Unused in this subroutine)                  **
C*I   D(N)   = Diagonal matrix for adaptive Lagrange multiplier       **
C*I   L(N)   = Location index of matrix H                             **
C*I   H(N,N) = Hessian matrix Hc                                      **
C*O          = Hessian matrix Hp                                      **
C*W   S(N)   = Xp(N)-Xc(N)  -> sqrt((1-VM)*TTS)*(Y(N)/YTS-T(N)/TTS)   **
C*W   Y(N)   = Gp(N)-Gc(N)  -> Y(N)/sqrt(YTS)                         **
C*W   T(N)   = Hc(N,N)*S(N) -> T(N)/sqrt(TTS)                         **
C*I   VM     = BFGS/(DFP+BFGS) : Proportion of update methods         **
C**            H = H_BFGS + (1-VM) * (H_DFP-H_BFGS)                   **
C**   Hp     = Hc + Y(I)*Y(J)/YTS - T(I)*T(J)/TTS                     **
C**            +(1-VM)*TTS*(Y(I)/YTS-T(I)/TTS)*(Y(J)/YTS-T(J)/TTS)    **
C**   ============================================================    **
      SS=0.0
      YY=0.0
      YS=0.0
      DO 10 I=1,N
      S(I)=Xp(I)-Xc(I)
      Y(I)=Gp(I)-Gc(I) + D(I)*S(I)
      SS=SS+S(I)*S(I)
      YY=YY+Y(I)*Y(I)
   10 YS=YS+Y(I)*S(I)
      IF(DABS(YS).LE.EPSMAH*DSQRT(SS*YY)) RETURN
C**   ------------------------------------------------------------    **
      TT=0.0
      TS=0.0
      DO 30 I=1,N
      T(I)=0.0 + D(I)*S(I)
      DO 20 J=1,I
   20 T(I)=T(I)+H(J+L(I))*S(J)
      DO 25 J=I+1,N
   25 T(I)=T(I)+H(I+L(J))*S(J)
      TT=TT+T(I)*T(I)
   30 TS=TS+T(I)*S(I)
      IF(DABS(TS).LE.EPSMAH*DSQRT(TT*SS)) RETURN
C**   ------------------------------------------------------------    **
      YS=DSQRT(YS)
      TS=DSQRT(TS)
      DO 50 I=1,N
      Y(I)=Y(I)/YS
      T(I)=T(I)/TS
   50 S(I)=(Y(I)*TS/YS-T(I))*DSQRT(1-VM)
C**   ------------------------------------------------------------    **
      DO 80 J=1,N
      DO 80 I=1,J
   80 H(I+L(J))=H(I+L(J))+Y(I)*Y(J)-T(I)*T(J)+S(I)*S(J)
      RETURN
      END
************************************************************************
      SUBROUTINE BFGS(N,Xc,Xp,Gc,Gp,EPSMAH,ETA,ANGRAD,L,B,S,Y,T,VM)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Xc(N),Xp(N),Gc(N),Gp(N),S(N),Y(N),T(N),L(N),B(1)
C**   ============================================================    **
C**   Update Bp from Bc  (B is the inverse of Hessian matrix H)       **
C**   ------------------------------------------------------------    **
C*I   N      = Number of variables                                    **
C*I   Xc(N)  = Last point                                             **
C*I   Xp(N)  = This point                                             **
C*I   Gc(N)  = Gradient at Xc                                         **
C*I   Gp(N)  = Gradient at Xp                                         **
C*I   EPSMAH = Machine epsilon                                        **
C**   ETA,ANGRAD = Dummy (Unused in this subroutine)                  **
C*I   L(N)   = Location index of matrix B                             **
C*I   B(N,N) = Inverse of Hessian matrix Hc                           **
C*O          = Inverse of Hessian matrix Hp                           **
C*W   Y(N)   = Gp(N)-Gc(N)  -> sqrt(  VM * TTY)*(S(N)/STY-T(N)/TTY)   **
C*W   S(N)   = Xp(N)-Xc(N)  -> S(N)/sqrt(STY)                         **
C*W   T(N)   = Bc(N,N)*Y(N) -> T(N)/sqrt(TTY)                         **
C*I   VM     = BFGS/(DFP+BFGS) : Proportion of update methods         **
C**            B = B_DFP + VM * (B_BFGS-B_DFP)                        **
C**   Bp     = Bc + S(I)*S(J)/STY - T(I)*T(J)/TTY                     **
C**               + VM*TTY*(S(I)/STY-T(I)/TTY)*(S(J)/STY-T(J)/TTY)    **
C**   ============================================================    **
C**   DO 5 I=1,N   ! Follows can be replaced by these 3 statements    **
C** 5 T(I)=0.0                                                        **
C**   CALL BFGSLG(N,Gc,Gp,Xc,Xp,EPSMAH,ETA,ANGRAD,T,L,B,Y,S,T,1-VM)   **
      SS=0.0
      YY=0.0
      SY=0.0
      DO 10 I=1,N
      S(I)=Xp(I)-Xc(I)
      Y(I)=Gp(I)-Gc(I)
      SS=SS+S(I)*S(I)
      YY=YY+Y(I)*Y(I)
   10 SY=SY+S(I)*Y(I)
      IF(DABS(SY).LE.EPSMAH*DSQRT(SS*YY)) RETURN
C**   ------------------------------------------------------------    **
      TT=0.0
      TY=0.0
      DO 30 I=1,N
      T(I)=0.0
      DO 20 J=1,I
   20 T(I)=T(I)+B(J+L(I))*Y(J)
      DO 25 J=I+1,N
   25 T(I)=T(I)+B(I+L(J))*Y(J)
      TT=TT+T(I)*T(I)
   30 TY=TY+T(I)*Y(I)
      IF(DABS(TY).LE.EPSMAH*DSQRT(TT*YY)) RETURN
C**   ------------------------------------------------------------    **
      SY=DSQRT(SY)
      TY=DSQRT(TY)
      DO 50 I=1,N
      S(I)=S(I)/SY
      T(I)=T(I)/TY
   50 Y(I)=(S(I)*TY/SY-T(I))*DSQRT(VM)
C**   ------------------------------------------------------------    **
      DO 80 J=1,N
      DO 80 I=1,J
   80 B(I+L(J))=B(I+L(J))+S(I)*S(J)-T(I)*T(J)+Y(I)*Y(J)
      RETURN
      END
************************************************************************
