************************************************************************
      SUBROUTINE FDGRAD(NX,NS,NE,N,X,PFAC,PPEN,C,Fp,ETA,G)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),PFAC(N),PPEN(6),C(1),G(N)
C**   ============================================================    **
C**   Compute Gradient G(N) by difference & partial analysis          **
C**   ------------------------------------------------------------    **
C*I   NX       = Number of original variables                         **
C*I   NS       = Number of constraints Gj(Xp(NX)) <= 0                **
C*I   NE       = Number of constraints Hi(Xp(NX))  = 0                **
C*I   N        = NX            : for Penalty function                 **
C*I            = NX+NS+NS+NE   : for Lagrange multiplier              **
C*I   X(N)     = Xp(NX)[,S(NS),Ls(NS),Le(NE)]                         **
C*I   PFAC(NX) = Scalar factor of Xp(NX)                              **
C*I   PPEN(6)  = R(1),R(2),R(3): Penalty parameters                   **
C*I              T,C1,C2       : Hyperbolic penalty function coef.    **
C*W   C(2*NC)  = Gj(Xp(NX)),Hi(Xp(NX)),dYsj/dGj,dYei/dHi : Penalty    **
C*W   C(NS+NE) = Gj(Xp(NX)),Hi(Xp(NX))                   : Lagrange   **
C*O   Fp       = F(X(N))                                              **
C*I   ETA      = Machine epsilon                                      **
C*O   G(N)     = Gradient at X(N)                                     **
C*S   GFNPN0,1 = Sub. to compute diff. Penalty obj. func. F(X(N))     **
C*S   GFNLAG   = Sub. to compute Lagrange objective func. F(X(N))     **
C**   ============================================================    **
      NC=NS+NE
      ETAH=DSQRT(ETA)
      IF(N.EQ.NX) CALL GFNPN0(NX,NS,NE,X,PFAC,PPEN,C,Fp,C(NC+1),C(NC+1))
      IF(N.NE.NX) CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,G(NX+1),Fp)
      DO 10 J=1,NX
      Xp=X(J)
      X(J)=Xp+ETAH*DMAX1(DABS(Xp),1.0D0)
      IF(N.EQ.NX) CALL GFNPN1(NX,NS,NE,X,PFAC,PPEN,C,FJ,C(NC+1),C(NC+1))
      IF(N.NE.NX) CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,FJ)
      G(J)=(FJ-Fp)/(X(J)-Xp)
   10 X(J)=Xp
C**   +----------------------------------------------------------+    **
C**   | For Lagrange multiplier method                           |    **
C**   +----------------------------------------------------------+    **
      DO 20 J=NX+NS+1,N-NE
      G(J-NS)=2*X(J-NS)*X(J)
   20 G(J)=G(J)+X(J-NS)**2
C     DO 25 I=NX+NS+NS+1,N
C  25 G(I)=G(I)
      RETURN
      END
************************************************************************
      SUBROUTINE FDHESS(NX,NS,NE,N,X,PFAC,PPEN,C,F,DX,ETA,L,H)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),PFAC(NX),PPEN(6),C(1),F(NX),DX(NX),L(N),H(1)
C**   ============================================================    **
C**   Compute Hessian matrix H(N,N) by difference & partial analysis  **
C**   ------------------------------------------------------------    **
C*I   NX       = Number of original variables                         **
C*I   NS       = Number of constraints Gj(Xp(NX)) <= 0                **
C*I   NE       = Number of constraints Hi(Xp(NX))  = 0                **
C*I   N        = NX            : for Penalty function                 **
C*I            = NX+NS+NS+NE   : for Lagrange multiplier              **
C*I   X(N)     = Xp(NX)[,S(NS),Ls(NS),Le(NE)]                         **
C*I   PFAC(NX) = Scalar factor of Xp(NX)                              **
C*I   PPEN(6)  = R(1),R(2),R(3): Penalty parameters                   **
C*I              T,C1,C2       : Hyperbolic penalty function coef.    **
C*W   C(NC*(NX+4)) = C(NC),HH(NC,NX),O(NC),D(NC),DD(NC) : Penalty     **
C*W   C(NC*2)      = C(NC),O(NC)                        : Lagrange    **
C*W   F(NX)    = F(X(N)+DX(I)*ei)                                     **
C*W   DX(NX)   = Step size                                            **
C*I   ETA      = Machine epsilon                                      **
C*I   L(N)     = Location index of matrix H(N,N)                      **
C*O   H(N+L(N))= Hessian matrix H(N,N) at X(N)                        **
C*S   GFNPN0,1 = Sub. to compute diff. Penalty obj. func. F(X(N))     **
C*S   GFNLAG   = Sub. to compute Lagrange objective func. F(X(N))     **
C**   ============================================================    **
      NC=NS+NE
      NO=NC+NC*NX
      ND=NO+NC+1
      NDD=NO+NC+NC
      ETAH=ETA**0.333333
      DO 10 I=1,N*(N+1)/2
   10 H(I)=0.0
      IF(N.EQ.NX) THEN
        CALL GFNPN0(NX,NS,NE,X,PFAC,PPEN,C(NO+1),Fp,C(ND),C(NDD+1))
      ELSE
        CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C(NC+1),Fp)
      ENDIF
      DO 30 I=1,NX
      XI=X(I)
      X(I)=XI+ETAH*DMAX1(DABS(XI),1.0D0)
      DX(I)=X(I)-XI
      IF(N.EQ.NX) THEN
        CALL GFNPN1(NX,NS,NE,X,PFAC,PPEN,C,F(I),C(ND),C(NDD+1))
        DO 20 J=1,NS+NE
   20   C(NC*I+J)=(C(J)-C(NO+J))/DX(I)
      ELSE
        CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,F(I))
        DO 25 J=1,NS+NE
   25   H(I+L(NX+NS+J))=(C(J)-C(NC+J))/DX(I)
      ENDIF
   30 X(I)=XI
C**   +----------------------------------------------------------+    **
C**   | Use forward difference for off-diagonal H(I,J)           |    **
C**   +----------------------------------------------------------+    **
      DO 40 J=1,NX
      XJ=X(J)
      X(J)=X(J)+DX(J)
      DO 35 I=1,J-1
      XI=X(I)
      X(I)=X(I)+DX(I)
      IF(N.EQ.NX) CALL GFNPN1(NX,NS,NE,X,PFAC,PPEN,C,FIJ,C(ND),C(NDD+1))
      IF(N.NE.NX) CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,FIJ)
      H(I+L(J))=((FIJ-F(J))-(F(I)-Fp))/(DX(I)*DX(J))
   35 X(I)=XI
C**   +----------------------------------------------------------+    **
C**   | Use central difference for diagonal H(J,J)               |    **
C**   +----------------------------------------------------------+    **
      X(J)=XJ-DX(J)
      IF(N.EQ.NX) CALL GFNPN1(NX,NS,NE,X,PFAC,PPEN,C,FIJ,C(ND),C(NDD+1))
      IF(N.NE.NX) CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,FIJ)
      H(J+L(J))=((F(J)-Fp)-(FIJ-Fp))/(DX(J)*DX(J))
   40 X(J)=XJ
C**   +----------------------------------------------------------+    **
C**   | For Lagrange multiplier method                           |    **
C**   +----------------------------------------------------------+    **
      IF(N.NE.NX) THEN
      DO 50 J=NX+1,N
      H(J+L(J+NS))=2*X(J)
   50 H(J+L(J))=2*X(J+NS)
      ELSE
C**   +----------------------------------------------------------+    **
C**   | For Penalty function method                              |    **
C**   +----------------------------------------------------------+    **
      DO 80 J=1,NX
      DO 80 I=1,J
      GIJ=0.0
      HIJ=0.0
      DO 60 K=1,NS
   60 GIJ=GIJ+C(NDD+K)*C(NC*I+K)*C(NC*J+K)
      DO 70 K=NS+1,NS+NE
   70 HIJ=HIJ+C(NDD+K)*C(NC*I+K)*C(NC*J+K)
   80 H(I+L(J))=H(I+L(J))+PPEN(2)*GIJ+PPEN(1)*HIJ
      ENDIF
      RETURN
      END
************************************************************************
      SUBROUTINE GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,W)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(1),PFAC(1),C(1)
C**   ============================================================    **
C**   Compute general objective function by Lagrange multiplier       **
C**   ------------------------------------------------------------    **
C*I   N        = NX+NS+NS+NE                                          **
C*I   X(N)     = Xp(NX),S(NS),Ls(NS),Le(NE)                           **
C**   PFAC(NX) = Scalar factor of Xp(NX)    (unused)                  **
C**   PPEN(6)  = R(3),T,C1,C2               (unused)                  **
C*W   C(NS+NE) = Gj(Xp(NX)),Hi(Xp(NX))                                **
C*O   W        = F + Ls(j)*(G(j) + S(j)**2)+ Le(i)*H(i)               **
C*S   FFF      = Sub. to compute F(Xp(NX)),Gj(Xp(NX)),Hi(Xp(NX))      **
C**   ============================================================    **
      CALL FFF(NX,NS,NE,X,F,C)
      W=0.0
      DO 20 J=1,NS
   20 W=W+X(NX+NS+J)*(C(J)+X(NX+J)**2)
      DO 30 I=NS+1,NS+NE
   30 W=W+X(NX+NS+I)*C(I)
      W=F+W
      RETURN
      END
************************************************************************
      SUBROUTINE GFMLAG(NX,NS,NE,X,PFAC,PPEN,C,W)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(1),PFAC(1),PPEN(6),C(1)
C**   ============================================================    **
C**   Compute Adaptive modified Lagrange function                     **
C**   ------------------------------------------------------------    **
C*I   N        = NX+NS+NS+NE                                          **
C*I   X(N)     = Xp(NX),S(NS),Ls(NS),Le(NE)                           **
C*I   PFAC(NX) = Scalar factor of Xp(NX)                              **
C*I   PFAC(NX+1  :NX+N  ) = Newton point Xn(N)                        **
C*I   PFAC(NX+N+1:NX+N+N) = Modified diagonal D(N)                    **
C**   PPEN(6)  = R(3),T,C1,C2               (unused)                  **
C*W   C(NS+NE) = Gj(Xp(NX)),Hi(Xp(NX))                                **
C*O   W        = FL + (X(i)-Xn(i))*D(i)*(X(i)-Xn(i)) / 2              **
C**   FL       = F(Xp(NX)) + Ls(j)*(G(j) + S(j)**2)+ Le(i)*H(i)       **
C*S   GFNLAG   = Sub. to compute Lagrange objective func. FL(X(N))    **
C**   ============================================================    **
      N=NX+NS+NS+NE
      CALL GFNLAG(NX,NS,NE,X,PFAC,PPEN,C,FL)
      W=0.0
      DO 20 I=1,N
      DX=X(I)-PFAC(I+NX)
   20 W=W+DX*PFAC(I+NX+N)*DX
      W=FL+0.5*W
      RETURN
      END
************************************************************************
      SUBROUTINE GFNPEN(NX,NS,NE,X,PFAC,PPEN,C,W)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(NX),PFAC(NX),PPEN(6),C(NS+NE)
C**   DATA T,C1,C2/0.1D-3,1.0D0,0.1D0/
C**   ============================================================    **
C**   Compute general objective function by Penalty function          **
C**   ------------------------------------------------------------    **
C*I   N        = NX                                                   **
C*I   X(NX)    = Xp(NX)                                               **
C*I   PFAC(NX) = Scalar factor of Xp(NX)    (unused)                  **
C*I   PPEN(6)  = R(3),T,C1,C2                                         **
C*W   C(NS+NE) = Gj(Xp(NX)),Hi(Xp(NX))                                **
C*O   W        = R(3)*F + R(2)*Ys(Gj;T,C1,C2) + R(1)*Ye(Hi;T,C1,C2)   **
C*S   FFF      = Sub. to compute F(Xp(NX)),Gj(Xp(NX)),Hi(Xp(NX))      **
C**   ============================================================    **
      CALL FFF(NX,NS,NE,X,F,C)
      PG=0.0
      PH=0.0
      DO 20 J=1,NS
   20 PG=PG+PF(C(J),PFAC(J),PPEN(4),PPEN(5),PPEN(6),-1)
      DO 30 I=NS+1,NS+NE
   30 PH=PH+PF(C(I),PFAC(I),PPEN(4),PPEN(5),PPEN(6), 0)
      W=PPEN(3)*F+PPEN(2)*PG+PPEN(1)*PH
      RETURN
      END
************************************************************************
      SUBROUTINE GFNPN0(NX,NS,NE,X,PFAC,PPEN,C,W,D,DD)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(NX),PFAC(NX),PPEN(6),C(NS+NE),D(NS+NE),DD(NS+NE)
C**   DATA T,C1,C2/0.1D-3,1.0D0,0.1D0/
C**   ============================================================    **
C**   Compute differential Penalty function for the gradient          **
C**   ------------------------------------------------------------    **
C*I   N         = NX                                                  **
C*I   X(NX)     = Xp(NX)                                              **
C*I   PFAC(NX)  = Scalar factor of Xp(NX)                             **
C*I   PPEN(6)   = R(3),T,C1,C2                                        **
C*W   C(NS+NE)  = Gj(Xp(NX)),Hi(Xp(NX))                               **
C*O   W         = R(3)*F + R(2)*DYj*Gj+ R(1)*DYi*Hi                   **
C*O   D(NS+NE)  = dYsj/dGj   or dYei/dHi                              **
C*O   DD(NS+NE) = d2Ysj/dGj2 or d2Yei/dHi2                            **
C*S   FFF       = Sub. to compute F(Xp(NX)),Gj(Xp(NX)),Hi(Xp(NX))     **
C**   ============================================================    **
      CALL FFF(NX,NS,NE,X,F,C)
      PG=0.0
      PH=0.0
      DO 20 J=1,NS
      CALL PD(C(J),PFAC(J),PPEN(4),PPEN(5),PPEN(6),-1,Y,D(J),DD(J))
   20 PG=PG+D(J)*C(J)
      DO 30 I=NS+1,NS+NE
      CALL PD(C(I),PFAC(I),PPEN(4),PPEN(5),PPEN(6), 0,Y,D(I),DD(I))
   30 PH=PH+D(I)*C(I)
      W=PPEN(3)*F+PPEN(2)*PG+PPEN(1)*PH
      RETURN
      END
************************************************************************
      SUBROUTINE GFNPN1(NX,NS,NE,X,PFAC,PPEN,C,W,D,DD)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(NX),PFAC(NX),PPEN(6),C(NS+NE),D(NS+NE),DD(NS+NE)
C**   DATA T,C1,C2/0.1D-3,1.0D0,0.1D0/
C**   ============================================================    **
C**   Compute differential Penalty function for the gradient          **
C**   ------------------------------------------------------------    **
C*I   N         = NX                                                  **
C*I   X(NX)     = Xp(NX)                                              **
C*I   PFAC(NX)  = Scalar factor of Xp(NX)                             **
C*I   PPEN(6)   = R(3),T,C1,C2                                        **
C*W   C(NS+NE)  = Gj(Xp(NX)),Hi(Xp(NX))                               **
C*O   W         = R(3)*F + R(2)*DYj*Gj + R(1)*DYi*Hi                  **
C1I   D(NS+NE)  = dYsj/dGj   or dYei/dHi   : input  for GRDPN1        **
C0O   D(NS+NE)  = dYsj/dGj   or dYei/dHi   : output for GRDPN0        **
C1I   DD(NS+NE) = d2Ysj/dGj2 or d2Yei/dHi2 : input  for GRDPN1        **
C0O   DD(NS+NE) = d2Ysj/dGj2 or d2Yei/dHi2 : output for GRDPN0        **
C*S   FFF       = Sub. to compute F(Xp(NX)),Gj(Xp(NX)),Hi(Xp(NX))     **
C**   ============================================================    **
      CALL FFF(NX,NS,NE,X,F,C)
      PG=0.0
      PH=0.0
      DO 20 J=1,NS
C0    CALL PD(C(J),PFAC(J),PPEN(4),PPEN(5),PPEN(6),-1,Y,D(J),DD(J))
   20 PG=PG+D(J)*C(J)                     ! D(1:NS) are given
      DO 30 I=NS+1,NS+NE
C0    CALL PD(C(I),PFAC(I),PPEN(4),PPEN(5),PPEN(6), 0,Y,D(I),DD(I))
   30 PH=PH+D(I)*C(I)                     ! D(NS+1:NS+NE) are given
      W=PPEN(3)*F+PPEN(2)*PG+PPEN(1)*PH
      RETURN
      END
************************************************************************
