************************************************************************
      SUBROUTINE PD(G,P,T,C1,C2,NCTP,F,DF,DDF)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C*I   G     = value of constraint                                     **
C*I   P     = positive scale factor of G                              **
C*I   T     = shape parameter of the hyperbola                        **
C*I   C1,C2 = coefficients for modified penalty function              **
C*I   NCTP  = type of constraint                                      **
C**         > 0 for constraint P*G>=0  :   S=-P ==> X = S*G <= 0      **
C**         < 0 for constraint P*G<=0  :   S=+P ==> X = S*G <= 0      **
C**         = 0 for constraint P*G =0  :   S=+P ==> X = S*G  = 0      **
C*O   F     = C1*Y + C2*Y*Y      = Penalty function for T>0           **
C**         = exp(C1*Y+C2*Y*Y)-1 = Penalty function for T<0           **
C*O   DF    = (dF/dY)*DY                     :  DY  =  dY/dG          **
C*O   DDF   = (dF/dY)*DDY + (d2F/dY2)*DY*DY  :  DDY = d2Y/dG2         **
C**   ============================================================    **
      S=P
      IF(NCTP.GT.0) S=-S
      X=S*G
C**   +----------------------------------------------------------+    **
C**   | Penalty function for X=0 constraint                      |    **
C**   | from  Y**2 - X**2 = T**2                 \        /      |    **
C**   | Y   = sqrt(X*X+T*T)                        \    /        |    **
C**   | DY  = S*X/sqrt(X*X+T*T) = S*X/Y              \/          |    **
C**   | DDY = (S*T)**2/sqrt(X*X+T*T)**3                          |    **
C**   +----------------------------------------------------------+    **
      Y=DSQRT(X*X+T*T)
      DY=S*X/Y
      DDY=(S*T/Y)**2/Y
C**   +----------------------------------------------------------+    **
C**   | Penalty function for X<=0 constraint                     |    **
C**   | from  (2Y)**2 - 2*X*(2Y) = T**2                     /    |    **
C**   | 2Y  = sqrt(X*X+T*T)+X                             /      |    **
C**   | 2DY = S*(sqrt(X*X+T*T)+X)/sqrt(X*X+T*T)   ______/        |    **
C**   | 2DDY= (S*T)**2/sqrt(X*X+T*T)**3                          |    **
C**   +----------------------------------------------------------+    **
      IF(NCTP.NE.0) THEN
      Y2=Y+DABS(X)
      IF(X.LT.0.0) Y2=T*T/Y2
      DDY=0.5*DDY
      DY=0.5*S*Y2/Y
      Y=0.5*Y2
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Modified penalty function                                |    **
C**   +----------------------------------------------------------+    **
      F=C1*Y+C2*Y*Y
      D1=C1+2*C2*Y
      D2=2*C2
      IF(T.LT.0) THEN
        F=DEXP(DMIN1(F,64.0D0))
        D2=(D1*D1+D2)*F
        D1=D1*F
        F=F-1
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | For gradient, only DF is required, Loc.DF=Loc.DDF is OK  |    **
C**   +----------------------------------------------------------+    **
      DDF=D1*DDY+D2*DY*DY
      DF=D1*DY
      RETURN
      END
************************************************************************
      SUBROUTINE PE(G,P,T,C1,C2,NCTP,F)
CD    SUBROUTINE PD(G,P,T,C1,C2,NCTP,F,DF,DDF)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
C*I   G     = value of constraint                                     **
C*I   P     = positive scale factor of G                              **
C*I   T     = shape parameter of the hyperbola                        **
C*I   C1,C2 = coefficients for modified penalty function              **
C*I   NCTP  = type of constraint                                      **
C**         > 0 for constraint P*G>=0  :   S=-P ==> X = S*G <= 0      **
C**         < 0 for constraint P*G<=0  :   S=+P ==> X = S*G <= 0      **
C**         = 0 for constraint P*G =0  :   S=+P ==> X = S*G  = 0      **
C*O   F     = C1*Y + C2*Y*Y      = Penalty function for T>0           **
C**         = exp(C1*Y+C2*Y*Y)-1 = Penalty function for T<0           **
C*O   DF    = (dF/dY)*DY                     :  DY  =  dY/dG          **
C*O   DDF   = (dF/dY)*DDY + (d2F/dY2)*DY*DY  :  DDY = d2Y/dG2         **
C**   ============================================================    **
      S=P
      IF(NCTP.GT.0) S=-S
      X=S*G
C**   +----------------------------------------------------------+    **
C**   | Penalty function for X=0 constraint                      |    **
C**   | from  Y**2 - X**2 = T**2                 \        /      |    **
C**   | Y   = sqrt(X*X+T*T)                        \    /        |    **
C**   | DY  = S*X/sqrt(X*X+T*T) = S*X/Y              \/          |    **
C**   | DDY = (S*T)**2/sqrt(X*X+T*T)**3                          |    **
C**   +----------------------------------------------------------+    **
      Y=DSQRT(X*X+T*T)
CD    DY=S*X/Y
CD    DDY=(S*T/Y)**2/Y
C**   +----------------------------------------------------------+    **
C**   | Penalty function for X<=0 constraint                     |    **
C**   | from  (2Y)**2 - 2*X*(2Y) = T**2                     /    |    **
C**   | 2Y  = sqrt(X*X+T*T)+X                             /      |    **
C**   | 2DY = S*(sqrt(X*X+T*T)+X)/sqrt(X*X+T*T)   ______/        |    **
C**   | 2DDY= (S*T)**2/sqrt(X*X+T*T)**3                          |    **
C**   +----------------------------------------------------------+    **
      IF(NCTP.NE.0) THEN
      Y2=Y+DABS(X)
      IF(X.LT.0.0) Y2=T*T/Y2
CD    DDY=0.5*DDY
CD    DY=0.5*S*Y2/Y
      Y=0.5*Y2
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | Modified penalty function                                |    **
C**   +----------------------------------------------------------+    **
      F=C1*Y+C2*Y*Y
CD    D1=C1+2*C2*Y
CD    D2=2*C2
      IF(T.LT.0) THEN
        F=DEXP(DMIN1(F,64.0D0))
CD      D2=(D1*D1+D2)*F
CD      D1=D1*F
        F=F-1
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | For gradient, only DF is required, Loc.DF=Loc.DDF is OK  |    **
C**   +----------------------------------------------------------+    **
CD    DDF=D1*DDY+D2*DY*DY
CD    DF=D1*DY
      RETURN
      END
************************************************************************
      FUNCTION PF(G,P,T,C1,C2,NCTP)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
C**   ============================================================    **
      CALL PE(G,P,T,C1,C2,NCTP,PF)
      RETURN
      END
************************************************************************
