************************************************************************
      SUBROUTINE FR(N,Xp,Gc,Gp,S,ISTEP)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Xp(N),Gc(N),Gp(N),S(N)
C**   ============================================================    **
C**   Find conjugate direction Sp(N) for given Gp(N),Sc(N)            **
C**   ------------------------------------------------------------    **
C*I   N     = Number of variables                                     **
C*I   Xp(N) = This point (Unused in this subroutine)                  **
C*I   Gc(N) = Gradient at Xc (Unused, for consistence with PR)        **
C*I   Gp(N) = Gradient at Xp                                          **
C*I   S (N) = Sc(N) : Conjugate direction at Xc (for ISTEP>0 only)    **
C*O         = Sp(N) : Conjugate direction at Xp                       **
C*I   ISTEP = 0 : Sp(I) = -Gp(I)   (Sc(N) unused)                     **
C*I         > 0 : Sp(I) = -Gp(I) + Sc(I) * (Gp.Gp)/(Gc.Gc)            **
C*I   CC    = Gc.Gc   (for ISTEP>0 only)                              **
C*O         = Gp.Gp                                                   **
C**   ============================================================    **
C**   CALL PR(N,Xp,Gp,Gp,S,-ISTEP)
      PP=0.0
      DO 10 I=1,N
   10 PP=PP+Gp(I)*Gp(I)
      IF(ISTEP.EQ.0) THEN
        DO 20 I=1,N
   20   S(I)=-Gp(I)
      ELSE
        POC=PP/CC
        DO 40 I=1,N
   40   S(I)=-Gp(I)+S(I)*POC
      ENDIF
      CC=PP
      RETURN
      END
************************************************************************
      SUBROUTINE PR(N,Xp,Gc,Gp,S,ISTEP)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION Xp(N),Gc(N),Gp(N),S(N)
C**   ============================================================    **
C**   Find conjugate direction Sp(N) for given Gc(N),Gp(N),Sc(N)      **
C**   ------------------------------------------------------------    **
C*I   N     = Number of variables                                     **
C*I   Xp(N) = This point (Unused in this subroutine)                  **
C*I   Gc(N) = Gradient at Xc (for ISTEP>0 only)                       **
C*I   Gp(N) = Gradient at Xp                                          **
C*I   S (N) = Sc(N) : Conjugate direction at Xc (for ISTEP<>0 only)   **
C*O         = Sp(N) : Conjugate direction at Xp                       **
C*I   ISTEP = 0 : Sp(I) = -Gp(I)   (Gc(N) and Sc(N) unused)           **
C*I         > 0 : Sp(I) = -Gp(I) + Sc(I) * (Gp.Gp-Gc.Gp)/(Gc.Gc)      **
C*I         < 0 : Sp(I) = -Gp(I) + Sc(I) * Gp.Gp/(Gc.Gc)              **
C*I   CC    = Gc.Gc   (for ISTEP<>0 only)                             **
C*O         = Gp.Gp                                                   **
C**   ============================================================    **
      PP=0.0
      DO 10 I=1,N
   10 PP=PP+Gp(I)*Gp(I)
      IF(ISTEP.EQ.0) THEN
        DO 20 I=1,N
   20   S(I)=-Gp(I)
      ELSE
        CP=PP
        IF(ISTEP.GT.0) THEN
          DO 30 I=1,N
   30     CP=CP-Gc(I)*Gp(I)
        ENDIF
        POC=CP/CC
        DO 40 I=1,N
   40   S(I)=-Gp(I)+S(I)*POC
      ENDIF
      CC=PP
      RETURN
      END
************************************************************************
