************************************************************************
      SUBROUTINE SPT(N,XOLD,SN,SC,XNEW,T,BEZTYP)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION XOLD(N),SN(N),SC(N),XNEW(N),BEZTYP(2)
C**   ============================================================    **
C**   Find a point on Bezier curve for given T and (Xo,Xc,Xn)         **
C**   ------------------------------------------------------------    **
C*I   N       = Number of variables                                   **
C*I   XOLD(N) = Xo      = Initial point                               **
C*I   SN(N)   = Xn - Xo = Newton direction                            **
C*I   SC(N)   = Xc - Xo = Cauchy direction                            **
C*O   XNEW(N) = Xo + t^Nb*(Xn-Xo) + (1-t^Nb-(1-t)^Nb) * (Xv-Xo)       **
C**           = Xo + t^Nb*(Xn-Xo) + (1-t^Nb-(1-t)^Nb)*v*(Xc-Xo)       **
C**             Xv = v*Xc + (1-v)*Xo                                  **
C*I   T = t   = Parameter of Bezier curve                             **
C*I   Nb      = DABS(BEZTYP(1)) = Order of Bezier curve               **
C**           = 0 : For straight line OC (SN,BEZTYP(2:2) Unuse)       **
C**           = 1 : For straight line ON (SC,BEZTYP(2:2) Unuse)       **
C**           > 1 : For Bezier curve of order Nb, BUT IF              **
C**                 BEZTYP(1) < 0 : Use line ON for T>=1 or T<=0      **
C*I   v       = DABS(BEZTYP(2)) <= 1                                  **
C**   ============================================================    **
      NB=DABS(BEZTYP(1))
      IF(NB.EQ.0) THEN
        DO  5 I=1,N
    5   XNEW(I)=XOLD(I)+T*SC(I)
      ELSE IF(NB.EQ.1.OR.BEZTYP(2).EQ.0
     *  .OR.(BEZTYP(1).LT.0.AND.(T.GE.1.OR.T.LE.0))) THEN
        DO 10 I=1,N
   10   XNEW(I)=XOLD(I)+T*SN(I)
      ELSE
        TN=T**NB
        TC=(1-TN-(1-T)**NB)*DABS(BEZTYP(2))
        DO 20 I=1,N
   20   XNEW(I)=XOLD(I)+TN*SN(I)+TC*SC(I)
      ENDIF
      RETURN
      END
************************************************************************
