************************************************************************
      FUNCTION XMin(X,F,dX,Istep,IVY,B)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION A(25),B(25)
      EQUIVALENCE (A(1),X1),(A(2),X2),(A(3),X3),(A(4),X4),(A(5),X5)
     *           ,(A(6),F1),(A(7),F2),(A(8),F3),(A(9),F4),(A(10),F5)
     *       ,(A(11),XB),(A(12),XL),(A(13),XM),(A(14),XR),(A(15),XE)
     *       ,(A(16),FB),(A(17),FL),(A(18),FM),(A(19),FR),(A(20),FE)
     *       ,(A(21),ST),(A(22),VY),(A(23),VL),(A(24),VR),(A(25),VN)
      DATA Pmax/1.0D30/
C**   ============================================================    **
C**   FIND  XMin                                                      **
C**   ------------------------------------------------------------    **
C**   XMin  = new guess of X for minimizing F(X)                      **
C**   X     = argument of function                                    **
C**   F     = F(X) = function value of X                              **
C**   dX    = return XMin=X+dX if no better formula to find XMin      **
C**   Istep = number of call to Function XMin                         **
C**   IVY   = 1/0 if valley set is found/not found                    **
C**   B(25) = buffer must NOT be changed between each calling Istep   **
C**   ------------------------------------------------------------    **
C**   for Istep = 1  :  (X1=X,F1=F) (ST=1) (VY=0)                     **
C**       XMin  = X1 + dX   if dX.NE.0                                **
C**       XMin  = X1 + 0.1  if dX.EQ.0                                **
C**   for Istep = 2  :  (X2=X,F2=F) (ST=2) (VY=0)                     **
C**       XMin  = X1 + (X1-X2)*2  if F1<F2                            **
C**       XMin  = X2 + (X2-X1)*2  if F2<F1                            **
C**   for Istep > 2  :  (X3=X,F3=F) (ST>2) (VY=1 if found valley)     **
C**       XMin  = Y1  from quad eq of newest pts (X1,X2,X3,X4,X5)     **
C**       XMin  = Y2  from quad eq of valley set (XB,XL,XM,XR,XE)     **
C**                   if VY=1 and when Y1 is NA or Outside (XL,XR)    **
C**   ============================================================    **
      IF (Istep.EQ.1) THEN
        DO 10 I=1,25
   10   A(I)=Pmax
        ST=0
        VY=0
        VL=0
        VR=0
        VN=6
      ELSE
        DO 20 I=1,25
   20   A(I)=B(I)
      ENDIF
      ST=ST+1
      IL=VL
      IR=VR
      IN=VN-1
      IF (IN.LT.1) IN=1
C**   +----------------------------------------------------------+    **
C**   | SAVE  the newest 5 points to (X1,X2,X3,X4,X5)            |    **
C**   +----------------------------------------------------------+    **
      DO 25 I=1,9
   25 A(I)=A(I+1)
      X5=X
      F5=F
C**   +----------------------------------------------------------+    **
C**   | FIND  new points (XL,XM,XR) that bracket the root        |    **
C**   +----------------------------------------------------------+    **
      IF(X.GT.XM) THEN
        IF(F.GT.FM.OR.(F.EQ.FM.AND.FR.GT.FL)) THEN
          XE=XR
          FE=FR
          XR=X
          FR=F
          IR=4
          IF(FM*(XE-XR)+FR*(XM-XE)+FE*(XR-XM).GT.0.0) IR=5
        ELSE
          XB=XL
          FB=FL
          XL=XM
          FL=FM
          XM=X
          FM=F
          IL=2
          IR=4
          IF(FB*(XM-XL)+FL*(XB-XM)+FM*(XL-XB).GT.0.0) IL=1
          IF(FM*(XE-XR)+FR*(XM-XE)+FE*(XR-XM).GT.0.0) IR=5
        ENDIF
      ELSE IF(X.LT.XM) THEN
        IF(F.GT.FM.OR.(F.EQ.FM.AND.FL.GT.FR)) THEN
          XB=XL
          FB=FL
          XL=X
          FL=F
          IL=2
          IF(FB*(XM-XL)+FL*(XB-XM)+FM*(XL-XB).GT.0.0) IL=1
        ELSE
          XE=XR
          FE=FR
          XR=XM
          FR=FM
          XM=X
          FM=F
          IR=4
          IL=2
          IF(FM*(XE-XR)+FR*(XM-XE)+FE*(XR-XM).GT.0.0) IR=5
          IF(FB*(XM-XL)+FL*(XB-XM)+FM*(XL-XB).GT.0.0) IL=1
        ENDIF
      ENDIF
      IF(A(IL+15).EQ.Pmax) IL=IL+1
      IF(A(IR+15).EQ.Pmax) IR=IR-1
      IF(FL.LT.Pmax.AND.FR.LT.Pmax) VY=1
      Iget=0
      XX=0.0
C**   +----------------------------------------------------------+    **
C**   | FIND a new point from qradratic eq on (X1,X2,X3,X4,X5)   |    **
C**   | if it is concave upward  d2F/dX2 = 1/ddF > 0             |    **
C**   | and make sure that ABS(XX-(X5+X3)/2) < 3*ABS(X5-X3)      |    **
C**   |                                                          |    **
C**   | X3  X4      X5    <---3.75*(X5-X4)--->    XXmax          |    **
C**   |  *---*---+---*---+---x---+---x---+---x---+-*             |    **
C**   |        *---+---+---x---+---+---x---+---+---*             |    **
C**   |        0        <---3*(X5-X3)--->        -ddX            |    **
C**   +----------------------------------------------------------+    **
      IF(ST.GT.2) THEN
        FMM=(F5-F4)*(X3-X4)-(F3-F4)*(X5-X4)
        IF(FMM.NE.0.0) THEN
          ddF=(X5-X3)*(X5-X4)*(X3-X4)/FMM
          IF(ddF.LE.0.0) IN=4
        ELSE
          IN=4
        ENDIF
        IF(IN.EQ.1) THEN
          CALL QUADFT(A(1),A(6),5,XX,Iget)
          ddX=0.5*(X5+X3)-XX
          IF(Iget.NE.0) Iget=6
        ENDIF
        IF(Iget.EQ.0.AND.IN.LE.3) THEN
          ddX=0.5*(F5-F3)/(X5-X3)*ddF
          Iget=5
        ENDIF
        IF(Iget.NE.0) THEN
          Rate=ABS(ddX/(X5-X3))/3.0
          IF(Rate.LE.1.0) Rate=1.0
          XX=0.5*(X5+X3)-ddX/Rate
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | if minimum is bracketed by (XL,XM,XR) and if no XX get   |    **
C**   | or XX goes outside (XL,XR) then use quadratic equation   |    **
C**   +----------------------------------------------------------+    **
      IF(VY.NE.0) THEN
        IF(Iget.EQ.0.OR.(XX-XL)*(XX-XR).GT.0.0) THEN
          IF(IR-IL.EQ.4) CALL QUADFT(A(11),A(16),5,XX,Iget)
          IF(Iget.EQ.0.OR.(XX-XL)*(XX-XR).GT.0.0) THEN
            FMM=(FR-FM)*(XL-XM)-(FL-FM)*(XR-XM)
            XX=0.5*(XR+XL)
            IF(FMM.NE.0.0) XX=XX-0.5*(FR-FL)*(XR-XM)*(XL-XM)/FMM
            Iget=3
          ENDIF
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | if none XX get so far, then:                             |    **
C**   | if known at least 2 points, go the downward side         |    **
C**   | if known only one point, set XX=X+dX or XX=X+0.1         |    **
C**   +----------------------------------------------------------+    **
      IF(Iget.EQ.0) THEN
        IF(ST.GE.2) THEN
          IF(F5.GT.F4) THEN
            XX=X4
            X4=X5
            X5=XX
            XX=F4
            F4=F5
            F5=XX
          ENDIF
          XX=X5+(X5-X4)*2.0
          Iget=2
        ELSE
          IF(DX.NE.0.0) THEN
            XX=X+DX
          ELSE
            XX=X+0.1D0
          ENDIF
          Iget=1
        ENDIF
      ENDIF
C**   +----------------------------------------------------------+    **
C**   | For debug                                                |    **
C**   +----------------------------------------------------------+    **
      X1=Iget
      F1=XX
C**   ------------------------------------------------------------    **
      VL=IL
      VR=IR
      VN=IN
      DO 30 I=1,25
   30 B(I)=A(I)
      IVY=VY
      XMin=XX
      RETURN
      END
************************************************************************
      SUBROUTINE QUADFT(X,Y,N,XM,Iget)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(N),Y(N),A(6,6),L(2,6),C(6)
C**   ============================================================    **
C**   Fit a quadratic curve for (X(I),Y(I),I=1,N), N=5                **
C**   C1 + C2*X + C3*Y + C4*X**2 +C5*X*Y + C6*Y**2 = 0                **
C**   Then call QUADXM to find the minimizer XM OF Y(X)               **
C**   where : CI = A(I,6)                                             **
C**   ------------------------------------------------------------    **
C**   Input  : X(N),Y(N) & N=5 (Iget=0)                               **
C**   Output : XM & Iget=4 if XM found                                **
C**   ============================================================    **
      DO 30 I=1,N
      A(I,1)=1.0
      A(I,2)=X(I)
      A(I,3)=Y(I)
      A(I,4)=X(I)*X(I)
      A(I,5)=X(I)*Y(I)
      A(I,6)=Y(I)*Y(I)
      A(6,I)=0.0
   30 CONTINUE
      A(6,6)=0.0
C**   ------------------------------------------------------------    **
      write(*,'('' 1a'',1p6e12.4)') a
      CALL LUFPDC(A,L,6,NSD,6)
      write(*,'('' nsd:'',i5)') nsd
      write(*,'('' 2a'',1p6e12.4)') a
      write(*,'('' 2l'',6I12)') l
      IF(NSD.GT.1) RETURN
      CALL LUFPSG(A,C,L,6,1,6)
      write(*,'('' 3a'',1p6e12.4)') a
      write(*,'('' 3c'',1p6e12.4)') c
      write(*,'('' 3l'',6I12)') l
      CALL QUADXM(C,XM,Iget)
      write(*,'('' 4c'',1p6e12.4)') c
      RETURN
      END
************************************************************************
      SUBROUTINE QUADXM(C,XM,IGET)
C**   ============================================================    **
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION C(6)
C**   ============================================================    **
C**   Find the minimizer XM of Y(X) from the quadratic eq             **
C*Q1  C1 + C2*X + C3*Y + C4*X**2 + C5*X*Y + C6*Y**2 = 0               **
C**   DEM = C5**2 - 4*C4*C6                                           **
C**   ------------------------------------------------------------    **
C**   For hyperbolic (DEM>0) or elliptic (DEM<0) :                    **
C*Q2  D1 + C4*(X-X0)**2 + C5*(X-X0)*(Y-Y0) + C6*(Y-Y0)**2 = 0         **
C**   where :                                                         **
C**   X0 = (2*C2*C6 - C3*C5)/DEM  :  C2 + 2*C4*X0 + C5*Y0 = 0         **
C**   Y0 = (2*C3*C4 - C2*C5)/DEM  :  C3 + C5*X0 + 2*C6*Y0 = 0         **
C**   D1 = C1 + C2*X0 + C3*Y0 + C4*X0**2 + C5*X0*Y0 + C6*Y0**2        **
C**      = C1                 - C4*X0**2 - C5*X0*Y0 - C6*Y0**2        **
C**      = C1 +(C2*X0 + C3*Y0)*0.5                                    **
C**   so :                                                            **
C**   dQ2/dX  :  2*C4*(X-X0) + C5*(Y-Y0) = 0                          **
C**   IF (D1*C4*DEM.GT.0) XM=X0-sqrt(D1*C4*DEM)*(C5/(C4*DEM))         **
C**         note that    (XM-X0) same Sign as  -(C5/(C4*DEM))         **
C**         therefore    (YM-Y0) same Sign as  DEM                    **
C**         that is  YM>Y0 for Hyperbolic; YM<Y0 for Elliptic         **
C**   since :                                                         **
C**   (Y-Y0) = -(2*C4/C5)*(X-X0) --> Q2                               **
C*Q3  D1 + (C4 - 2*C4 + 4*C6*C4**2/C5**2)*(X-X0)**2 = 0               **
C**   (X-X0)**2 = (D1*C5**2)/(C4*DEM)                                 **
C**   ------------------------------------------------------------    **
C**   For parabolic (DEM=0)                                           **
C**   dQ1/dX  :  C2 + 2*C4*X + C5*Y = 0                               **
C**   IF (C5.EQ.0) THEN : XM=-0.5*C2/C4  (AND IF(C3*C4.LT.0))         **
C**   IF (C5.NE.0) THEN : XM=D1/D2       (AND IF(D2.NE.0))            **
C**   where : D1 = C1 - C3*C2/C5 + C6*(C2/C5)**2                      **
C**           D2 = 2*C3*C4/C5 - 4*C6*C2*C4/C5**2                      **
C**   since :                                                         **
C**   for C5.NE.0                                                     **
C**   Y = - C2/C5 - (2*C4/C5)*X --> Q1                                **
C**   C1 - C3*C2/C5 + C6*(C2/C5)**2                                   **
C**      + (C2 - 2*C3*C4/C5 - C2 + 4*C6*C2*C4/C5**2)*X                **
C**      + (C4- 2*C4 + 4*C6*C4**2/C5**2)*X**2 = 0                     **
C*L4  D1 - D2*X = 0                                                   **
C**   for C5.EQ.0 & C3*C4.NE.0          (also C6=0, since DEM=0)      **
C*Q5  C1 + C2*X + C3*Y + C4*X**2 = 0    (Y is Min if C3*C4.LT.0)      **
C**   ============================================================    **
      WRITE(*,'('' C'',1P6E13.6)') C
      DEM=C(5)**2-4.0*C(4)*C(6)
      IF(DEM.NE.0) THEN
        X0=(2.0*C(2)*C(6)-C(3)*C(5))/DEM
        Y0=(2.0*C(3)*C(4)-C(2)*C(5))/DEM
        D1=C(1)+(C(2)*X0+C(3)*Y0)*0.5
        D2=D1*C(4)*DEM
        WRITE(*,'('' D'',1P6E13.6)') DEM,X0,Y0,D1,D2
        IF(D2.GT.0) THEN
          XM=X0-SQRT(D2)*C(5)/(C(4)*DEM)
          IGET=4
          WRITE(*,'('' D'',1PE13.6,I6)') XM,IGET
        ENDIF
      ELSE
        IF(C(5).NE.0) THEN
          D1=C(1)-C(3)*C(2)/C(5)+C(6)*(C(2)/C(5))**2
          D2=2.0*C(3)*C(4)/C(5)-4.0*C(6)*C(2)*C(4)/C(5)**2
          IF(D2.NE.0) THEN
            XM=D1/D2
            IGET=4
            WRITE(*,'('' E'',1P6E13.6)') DEM,D1,D2,XM
          ENDIF
        ELSE
          IF(C(3)*C(4).LT.0) THEN
            XM=-0.5*C(2)/C(4)
            IGET=4
            WRITE(*,'('' F'',1P6E13.6)') DEM,XM
          ENDIF
        ENDIF
      ENDIF
      RETURN
      END
************************************************************************
