************************************************************************
**    MAIN PROGRAM  BridgeIdent3D                                     **
**  ================================================================  **
**    System Identification analysis for Multiple Supported Inputs    **
**    of Bridge Structures by Modal Minimization Method ( Beck,J.L.,  **
**    "Determining Models of Structures from Earthquake Records",     **
**    CalTech, CA, 1978 )                                             **
**  ================================================================  **
      CALL  InputData
      CALL  Optimize
      CALL  OutputResult
      STOP
      END
************************************************************************
      SUBROUTINE  InputData
**  ================================================================  **
**    UserInput   computational parameters, initial guess of modal    **
**                parameters, pseudo influence matrix                 **
**    ReadData    of recorded ground and structure responses          **
**    Initialize  recorded and computed responses, normalization c    **
**  ================================================================  **
      CALL  UserInput
      CALL  ReadData
      CALL  Initialize
      RETURN
      END
************************************************************************
      SUBROUTINE  UserInput
**  ================================================================  **
**    READ  NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf                         **
**    READ  Omg,Ksi    INITIALIZE  Phi    READ  Inf                   **
**    ------------------------------------------------------------    **
**    NumM           = Number of Modes to be analyized                **
**    NumF           = Number of support Forcing inputs   = 1,2,3     **
**    NumG           = Number of instrumented Ground support DOFs     **
**    NumS           = Number of instrumented Superstructure DOFs     **
**    NumT           = Number of Time steps in input data file        **
**    Mi             = Initial number of Modes of the analysis        **
**    Ti,Tf          = Initial and Final Time step of the analysis    **
**    ............................................................    **
**    Omg(NumM)      = estimated modal frequencies                    **
**    Ksi(NumM)      = estimated modal damping coefficients           **
**    Phi(NumS,NumM) = estimated mode  shapes                  = 1    **
**    Inf(NumS,NumG) = pseudo Influence matrix                        **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf , Mode,Grnd,Stru
      REAL*8     Omg(MaxM),Ksi(MaxM),Phi(MaxS,MaxM),Inf(MaxS,MaxG)
      COMMON  /Moo/  NumM
      COMMON  /Foo/  NumF
      COMMON  /Goo/  NumG
      COMMON  /Soo/  NumS
      COMMON  /Too/  NumT
      COMMON  /Tif/  Ti,Tf
      COMMON  /Mii/  Mi
      COMMON  /Xoo/  Omg,Ksi
      COMMON  /Yoo/  Phi
      COMMON  /Ioo/  Inf
**  ================================================================  **
      WRITE(*,100)
 100  FORMAT(' Bridge Identification by ''Modal Minimization Method'' ')
      OPEN (1,FILE='BI3D1.In',FORM='FORMATTED',ACCESS='SEQUENTIAL')
**    +----------------------------------------------------------+    **
**    |  READ  NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf                 |    **
**    +----------------------------------------------------------+    **
      READ (1,'(8I5)')  NumM,NumF,NumG,NumS,NumT,Mi,Ti,Tf
      IF (NumM.GT.MaxM)           STOP ' UserInput : NumM > MaxM '
      IF (NumF.GT.MaxF)           STOP ' UserInput : NumF > MaxF '
      IF (NumG.GT.MaxG)           STOP ' UserInput : NumG > MaxG '
      IF (NumS.GT.MaxS)           STOP ' UserInput : NumS > MaxS '
      IF (NumT.GT.MaxT)           STOP ' UserInput : NumT > MaxT '
      IF (Mi.LT.1.OR.Mi.GT.NumM)  Mi = NumM
      IF (Ti.LT.1)                Ti = 1
      IF (Tf.GT.NumT)             Tf = NumT
      Tf = Ti + (Tf-Ti)/2*2
**    ............................................................    **
*      OPEN (6,FILE='BI3D1.Deb',FORM='FORMATTED')
      WRITE(*,'('' Max M F G S T = '',5I5)')  MaxM,MaxF,MaxG,MaxS,MaxT
      WRITE(*,'('' Num M F G S T = '',5I5)')  NumM,NumF,NumG,NumS,NumT
      WRITE(*,'(''     Mi  Ti Tf = '',3I5)')  Mi,Ti,Tf
**    +----------------------------------------------------------+    **
**    |  READ  Omg(NumM),Ksi(NumM)                               |    **
**    +----------------------------------------------------------+    **
      DO 10  Mode = 1,NumM
          READ (1,'(2F5.0)')  Omg(Mode),Ksi(Mode)
          WRITE(*,'('' Omg Ksi = '',2F8.4)')  Omg(Mode),Ksi(Mode)
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  INITIALIZE  Phi(NumS,NumM)                              |    **
**    +----------------------------------------------------------+    **
      DO 20  Mode = 1,NumM
      DO 20  Stru = 1,NumS
          Phi(Stru,Mode) = 1.D0
  20  CONTINUE
**    +----------------------------------------------------------+    **
**    |  READ  Inf(NumS,NumG)                                    |    **
**    +----------------------------------------------------------+    **
      DO 30  Grnd = 1,NumG
          READ (1,'(16F5.0)')  ( Inf(Stru,Grnd) , Stru=1,NumS )
          WRITE(*,'('' Inf = '')')
          WRITE(*,'(10F8.4)')  ( Inf(Stru,Grnd) , Stru=1,NumS )
  30  CONTINUE
      CLOSE(1)
      RETURN
      END
************************************************************************
      SUBROUTINE  ReadData
**  ================================================================  **
**    READ  Ag,At                                                     **
**    ------------------------------------------------------------    **
**    Ag(NumT,NumG,NumF) = recorded Ground support Acce,velo,displ    **
**    At(NumT,NumS)      = recorded Total superstructure Accel.       **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumF,NumG,NumS,NumT , Forc,Grnd,Stru,Time
      REAL*8     Ag(MaxT,MaxG,MaxF),At(MaxT,MaxS)
      REAL*4     Temp(MaxT)
      COMMON  /Foo/  NumF
      COMMON  /Goo/  NumG
      COMMON  /Soo/  NumS
      COMMON  /Too/  NumT
      COMMON  /Agoo/ Ag
      COMMON  /Atoo/ At
**  ================================================================  **
      OPEN (3,FILE='BI3D1.Rec',FORM='FORMATTED',ACCESS='SEQUENTIAL')
**    +----------------------------------------------------------+    **
**    |  READ  Ag(NumT,NumG,NumF)                                |    **
**    +----------------------------------------------------------+    **
      DO 10  Grnd = 1,NumG
      DO 10  Forc = 1,NumF
          READ (3,*)  ( Temp(Time) , Time=1,NumT )
      DO 10  Time = 1,NumT
          Ag(Time,Grnd,Forc) = Temp(Time)
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  READ  At(NumT,NumS)                                     |    **
**    +----------------------------------------------------------+    **
      DO 20  Stru = 1,NumS
          READ (3,*)  ( Temp(Time) , Time=1,NumT )
      DO 20  Time = 1,NumT
          At(Time,Stru) = Temp(Time)
  20  CONTINUE
      CLOSE(3)
      RETURN
      END
************************************************************************
      SUBROUTINE  Initialize
**  ================================================================  **
**    COMPUTE  Ar    INITIALIZE  Am    COMPUTE  Norm                  **
**    ------------------------------------------------------------    **
**    Ar(T,NumS) = recorded Relative Acceleration      = At-Inf*Ag    **
**    Am(T,NumM) = computed Modal relative Acceleration               **
**    Norm(NumS) = Normalization constant of Ar      = 1/Int(Ar^2)    **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumM,NumG,NumS,Ti,Tf , Mode,Grnd,Stru,Time
      REAL*8     Ag(MaxT,MaxG,MaxF),At(MaxT,MaxS)
      REAL*8     Ar(MaxT,MaxS),Am(MaxT,MaxM),Norm(MaxS)
      REAL*8     Inf(MaxS,MaxG),InfAg,Intg,dT/0.02D0/
      COMMON  /Moo/  NumM
      COMMON  /Goo/  NumG
      COMMON  /Soo/  NumS
      COMMON  /Tif/  Ti,Tf
      COMMON  /Agoo/ Ag
      COMMON  /Atoo/ At
      COMMON  /Aroo/ Ar
      COMMON  /Amoo/ Am
      COMMON  /Ioo/  Inf
      COMMON  /Nooo/ Norm
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  COMPUTE  Ar(T,NumS) = At(T,NumS)                        |    **
**    |                      - Inf(NumS,NumG)*Ag(T,NumG,1)       |    **
**    +----------------------------------------------------------+    **
      DO 10  Stru = 1,NumS
      DO 10  Time = Ti,Tf
          InfAg = 0.D0
          DO 20  Grnd = 1,NumG
              InfAg = InfAg + Inf(Stru,Grnd)*Ag(Time,Grnd,1)
  20      CONTINUE
          Ar(Time,Stru) = At(Time,Stru) - InfAg
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  INITIALIZE  Am(T,NumM) = 0                              |    **
**    +----------------------------------------------------------+    **
      DO 30  Mode = 1,NumM
      DO 30  Time = Ti,Tf
          Am(Time,Mode) = 0.D0
  30  CONTINUE
**    +----------------------------------------------------------+    **
**    |  COMPUTE  Norm(NumS) = 1/Int(Ar(T,NumS)^2)               |    **
**    +----------------------------------------------------------+    **
      DO 40  Stru = 1,NumS
          Norm(Stru) = 1 / Intg ( Ar(Ti,Stru),Ar(Ti,Stru),Ti,Tf,dT )
  40  CONTINUE
      WRITE(*,'('' Norm = '',1P8F10.4)')  ( Norm(Stru) , Stru=1,NumS )
      RETURN
      END
************************************************************************
      FUNCTION  Intg ( X,Y,Ti,Tf,dT )
**  ================================================================  **
**    COMPUTE  the numerical integration  Intg  of function  X*Y      **
**        over the time interval  Ti~Tf  with the fixed step size     **
**        dT  by using the Simpson's rule ( 1 4 2 4 2 ... 4 1 )       **
**    !!  (Tf-Ti) must be an even number which is multiple of 2       **
**  ================================================================  **
      INTEGER    Ti,Tf,Time
      REAL*8     Intg,X(Ti:Tf),Y(Ti:Tf),dT,A,B
**  ================================================================  **
      A=0.D0
      B=0.D0
      DO 10  Time = Ti+1,Tf,2
          A = A + X(Time)*Y(Time)
          B = B + X(Time+1)*Y(Time+1)
  10  CONTINUE
      Intg = (X(Ti)*Y(Ti)-X(Tf)*Y(Tf)+4.D0*A+2.D0*B) / 3.D0 * dT
      RETURN
      END
************************************************************************
      SUBROUTINE  Optimize
**  ================================================================  **
**    For each Swep of Mode = 1,NumM  :  ComputeArM  MinimizeErr      **
**    ------------------------------------------------------------    **
**    Err  = Error function to be minimized                           **
**    OmgO = Old Omg before MinimizeErr                               **
**    KsiO = Old Ksi before MinimizeErr                               **
**    Diff = relative Difference between OmgO,KsiO and Omg,Ksi        **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumM,Mi,Mf , Mode,Swep
      REAL*8     Omg(MaxM),Ksi(MaxM),Err,OmgO,KsiO,Diff
      COMMON  /Moo/  NumM
      COMMON  /Mii/  Mi
      COMMON  /Xoo/  Omg,Ksi
**  ================================================================  **
      DO 10  Mf = Mi,NumM
      DO 20  Swep = 1,10
          Diff = 0.D0
          DO 30  Mode = 1,Mf
**    +----------------------------------------------------------+    **
**    |  STORE  OmgO,KsiO                                        |    **
**    +----------------------------------------------------------+    **
              OmgO = Omg(Mode)
              KsiO = Ksi(Mode)
**    +----------------------------------------------------------+    **
**    |  OPTIMIZE  Omg,Ksi    by    MINIMIZE  Err                |    **
**    +----------------------------------------------------------+    **
              CALL  ComputeArM ( Mode )
              CALL  MinimizeErr ( Mode,Omg(Mode),Ksi(Mode),Err )
**    +----------------------------------------------------------+    **
**    |  COMPUTE  Diff                                           |    **
**    +----------------------------------------------------------+    **
              Diff = DMAX1 (DABS(OmgO/Omg(Mode)-1.D0)
     &                     ,DABS(KsiO/Ksi(Mode)-1.D0),Diff)
**    ............................................................    **
              WRITE(*,'('' Omg Ksi Err Diff OmgD KsiD = '',6F10.5)')
     &                     Omg(Mode),Ksi(Mode),Err,Diff,
     &                         1.D0-OmgO/Omg(Mode),1.D0-KsiO/Ksi(Mode)
  30      CONTINUE
          IF (Diff.LT.0.001D0)  GOTO 10
  20  CONTINUE
  10  CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE  ComputeArM ( M )
**  ================================================================  **
**    COMPUTE  ArM                                                    **
**    ------------------------------------------------------------    **
**    ArM(T,NumS) = recorded Relative Modal Accel = Ar-Sum(Am*Phi)    **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumM,NumS,Ti,Tf , M , Mode,Stru,Time
      REAL*8     Ar(MaxT,MaxS),Am(MaxT,MaxM),Phi(MaxS,MaxM)
      REAL*8     ArM(MaxT,MaxS)
      COMMON  /Moo/   NumM
      COMMON  /Soo/   NumS
      COMMON  /Tif/   Ti,Tf
      COMMON  /Aroo/  Ar
      COMMON  /Amoo/  Am
      COMMON  /ArMoo/ ArM
      COMMON  /Yoo/   Phi
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  COMPUTE  ArM(T,NumS) = Ar(T,NumS)                       |    **
**    |                       - Sum(Am(T,NumM)*Phi(NumS,NumM))   |    **
**    +----------------------------------------------------------+    **
      DO 10  Stru = 1,NumS
      DO 10  Time = Ti,Tf
          ArM(Time,Stru) = Ar(Time,Stru) + Am(Time,M)*Phi(Stru,M)
      DO 10  Mode = 1,NumM
          ArM(Time,Stru) = ArM(Time,Stru) - Am(Time,Mode)*Phi(Stru,Mode)
  10  CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE  MinimizeErr ( M,OmgM,KsiM,Err )
**  ================================================================  **
**    MINIMIZE  Err  to find Optimal OmgM,KsiM by using T.W.Lin's     **
**        nonlinear minimization procedures and  Xmin                 **
**    ............................................................    **
**    ComputeErr : COMPUTE  Err  for given values of OmgM and KsiM    **
**    Xmin       : FIND  Optimum values of OmgM or KsiM               **
**    ------------------------------------------------------------    **
**    Err       = Error function to be minimized                      **
**    OmgM,KsiM = variables of error function to be optimized         **
**    ............................................................    **
**    X(1:2) = initial and optimum values of variables OmgM,KsiM      **
**    F*     = Function values of X                          = Err    **
**    TypX   = Typical values of X                                    **
**    TolX   = Tolerance value for X/TypX                             **
**    TolF   = Tolerance value for dF(X)/dX                           **
**    dX     = new step size of X              = Max(|Xm-Xi|/TypX)    **
**           = maximum relative total moving distance                 **
**    Count  = number of Count over minimizing OmgM and KsiM <= 12    **
**    Step   = number of Step in minimizing OmgM or KsiM     <= 16    **
**  ================================================================  **
      INTEGER    M,I,Count,Step,Valley
      REAL*8     OmgM,KsiM,Err,X(2),TypX(2),TolX,TolF,dX,dXOld,Xmin
      REAL*8     Xo,Fo,Xx,Fx,Xm,Fm,Xn,Xi,Rate,Buf(14)
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  INITIALIZE  X(2),TypX(2),TolX,dX                        |    **
**    +----------------------------------------------------------+    **
      X(1)    = OmgM
      X(2)    = KsiM
      TypX(1) = 5.D0
      TypX(2) = 0.05D0
      TolX    = 0.0001D0
      dX      = 0.1D0
**    +----------------------------------------------------------+    **
**    |  ITERATE  minimizing OmgM and KsiM up to Count=12 times  |    **
**    +----------------------------------------------------------+    **
      Count = 0
  95  Count = Count + 1
      dXOld = dX
      dX    = 0.D0
      DO 10  I = 1,2
**    +----------------------------------------------------------+    **
**    |  INITIALIZE  current Xx,Fx , optimum Xm,Fm , next Xn     |    **
**    |  SAVE        initial Xi                                  |    **
**    +----------------------------------------------------------+    **
          Xx = 1.D30
          Fx = 1.D30
          Xm = Xx
          Fm = Fx
          Xn = X(I)
          Xi = X(I)
**    +----------------------------------------------------------+    **
**    |  ITERATE  minimizing OmgM or KsiM up to Step=16 times    |    **
**    +----------------------------------------------------------+    **
          Step = 0
  85      Step = Step + 1
**    +----------------------------------------------------------+    **
**    |  SAVE  previous Xo,Fo                                    |    **
**    +----------------------------------------------------------+    **
          Xo = Xx
          Fo = Fx
**    +----------------------------------------------------------+    **
**    |  USE  next Xn from Xmin as current Xx,X for ComputeErr   |    **
**    +----------------------------------------------------------+    **
          Xx   = Xn
          X(I) = Xx
          CALL  ComputeErr ( M,X(1),X(2),Fx )
**    +----------------------------------------------------------+    **
**    |  SAVE  optimum Xm,Fm during iteration Steps              |    **
**    +----------------------------------------------------------+    **
          IF (Fx.LT.Fm)  THEN
              Xm = Xx
              Fm = Fx
          ENDIF
**    +----------------------------------------------------------+    **
**    |  OBTAIN  next Xn from Xmin for next iteration Step       |    **
**    +----------------------------------------------------------+    **
          Xn = Xmin ( Xx,Fx,dXOld*TypX(I),Step,Valley,Buf )
**    +----------------------------------------------------------+    **
**    |  CHECK  convergence of X,F                               |    **
**    +----------------------------------------------------------+    **
          IF (Step.EQ.2)  TolF = TolX * DABS((Fx-Fo)/(Xx-Xo))
          IF (Step.LT.5 .AND. Valley.EQ.0)  GOTO 85
          IF (Step.LT.16 .AND.
     &        DABS(Xx-Xo).GT.TolX*TypX(I) .AND.
     &        DABS(Fx-Fo).GT.TolF*DABS(Xx-Xo))  GOTO 85
**    +----------------------------------------------------------+    **
**    |  UPDATE  X , dX = Max(|Xm-Xi|/TypX)                      |    **
**    +----------------------------------------------------------+    **
          X(I) = Xm
          Rate = 0.5 * DABS(Xm-Xi)/TypX(I)
          IF (Rate.GT.dX)  dX = Rate
          WRITE(*,'(I4,F7.4)')  Step,Rate
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  CHECK  convergence of OmgM or KsiM                      |    **
**    +----------------------------------------------------------+    **
      IF (dX.GT.dXOld)  dX = dXOld
      IF (dX.GT.TolX.AND.Count.LT.12)  GOTO 95
**    +----------------------------------------------------------+    **
**    |  RETURN  the minimum Err and the optimum OmgM,KsiM       |    **
**    +----------------------------------------------------------+    **
      OmgM = X(1)
      KsiM = X(2)
      CALL  ComputeErr ( M,OmgM,KsiM,Err )
      RETURN
      END
************************************************************************
      SUBROUTINE  ComputeErr ( M,OmgM,KsiM,Err )
**  ================================================================  **
**    COMPUTE  AmM,PhiM,Err                                           **
**    ------------------------------------------------------------    **
**    AmM(T)     = computed Modal Acceleration of the Mth mode        **
**    PhiM(NumS) = computed mode shapes of the Mth mode               **
**    Err        = Error function = Sum(Norm*Int((ArM-AmM*PhiM)^2))   **
**    PF         = Penalty Function to Err for computed OmgM,KsiM     **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumS,Ti,Tf , M , Stru,Time
      REAL*8     ArM(MaxT,MaxS),Am(MaxT,MaxM),Phi(MaxS,MaxM),Norm(MaxS)
      REAL*8     Err,OmgM,KsiM,Diff(MaxT),Intg,dT/0.02D0/,PF
      COMMON  /Soo/   NumS
      COMMON  /Tif/   Ti,Tf
      COMMON  /ArMoo/ ArM
      COMMON  /Amoo/  Am
      COMMON  /Yoo/   Phi
      COMMON  /Nooo/  Norm
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  COMPUTE  AmM(T),PhiM(NumS)                              |    **
**    +----------------------------------------------------------+    **
      CALL  ComputeAmM ( OmgM,KsiM,Phi(1,M),Am(1,M) )
**    +----------------------------------------------------------+    **
**    |  COMPUTE  Err = Sum ( Norm(NumS) * Int ( (ArM(T,NumS)    |    **
**    |               - AmM(T)*PhiM(NumS))^2 ) )                 |    **
**    +----------------------------------------------------------+    **
      Err = 0.D0
      DO 10  Stru = 1,NumS
          DO 20  Time = Ti,Tf
              Diff(Time) = ArM(Time,Stru) - Am(Time,M) * Phi(Stru,M)
  20      CONTINUE
          Err = Err + Norm(Stru) * Intg (Diff(Ti),Diff(Ti),Ti,Tf,dT)
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  ADD  PF to Err for OmgM<0 and KsiM<0 and KsiM>0.5       |    **
**    +----------------------------------------------------------+    **
      Err = Err + PF(    OmgM,100.D0,0.001D0,1.D0,1.D0,1)
      Err = Err + PF(    KsiM,100.D0,0.001D0,1.D0,1.D0,1)
      Err = Err + PF(0.5-KsiM,100.D0,0.001D0,1.D0,1.D0,1)
      WRITE(*,'('' OmgM,KsiM,Err = '',2F10.5,F7.3)')  OmgM,KsiM,Err
      RETURN
      END
************************************************************************
      SUBROUTINE  ComputeAmM ( OmgM,KsiM,PhiM,AmM )
**  ================================================================  **
**    COMPUTE  AmM,PhiM  by iteration of linear modal minimization    **
**        for linear parameter PhiM and participation factor B        **
**    ............................................................    **
**    COMPUTE S,Sjk,SjA << COMPUTE RHS , SOLVE B , COMPUTE BSB,BSA    **
**        , COMPUTE PhiM , CHECK convergence >> NORMALIZE PhiM,B ,    **
**        COMPUTE AmM                                                 **
**    ------------------------------------------------------------    **
**    ArM(T,NumS) = recorded Relative Modal Acceleration              **
**    AmM(T)      = computed Modal Acceleration         = Sum(B*S)    **
**    OmgM,KsiM   = Modal frequency & Modal damping ratio             **
**    PhiM(NumS)  = Mode shapes  = Int(SumBS*ArM)/Int(SumBS*SumBS)    **
**    Norm(NumS)  = Normalization constant of Ar                      **
**    ............................................................    **
**    B(NumV)        = participation factor    = soln of Sjk*B=RHS    **
**    S(T,NumV)      = unit accel response of SDOF linear oscil.      **
**    Sjk(NumV,NumV) = Int(Sj*Sk)                                     **
**    SjA(NumV,NumS) = Int(Sj*ArM)                                    **
**    RHS(NumV)      = Right Hand Side of linear equations            **
**                   = Sum(Norm*PhiM*SjA)/Sum(Norm*PhiM^2)            **
**    BSB            = Bj*Sjk*Bk = Int(Sum(B*S)*Sum(B*S))             **
**    BSA            = Bj*SjA = Int(Sum(B*S)*ArM)                     **
**    NumV           = Number of Variables of system = NumG*NumF+2    **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumS,Ti,Tf,NumV , Stru,Time , J,K,Count
      REAL*8     Ag(MaxT,MaxG,MaxF),ArM(MaxT,MaxS),AmM(MaxT)
      REAL*8     OmgM,KsiM,PhiM(MaxS),Norm(MaxS),PhiMax,PhiOld
      REAL*8     B(MaxV),S(MaxT,MaxV),Sjk(MaxV,MaxV),SjA(MaxV,MaxS)
      REAL*8     RHS(MaxV),BSB,BSA,Intg,dT/0.02D0/,Sum
      COMMON  /Soo/   NumS
      COMMON  /Tif/   Ti,Tf
      COMMON  /Voo/   NumV
      COMMON  /Agoo/  Ag
      COMMON  /ArMoo/ ArM
      COMMON  /Nooo/  Norm
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  COMPUTE  S(T,NumV)                                      |    **
**    +----------------------------------------------------------+    **
      CALL  ComputeS ( OmgM,KsiM,Ag,S )
**    +----------------------------------------------------------+    **
**    |  COMPUTE  Sjk = Skj = Int(Sj*Sk) , SjA = Int(Sj*ArM)     |    **
**    +----------------------------------------------------------+    **
      DO 10  J = 1,NumV
      DO 10  K = 1,J
          Sjk(J,K) = Intg ( S(Ti,J),S(Ti,K),Ti,Tf,dT )
          Sjk(K,J) = Sjk(J,K)
  10  CONTINUE
      DO 20  J = 1,NumV
      DO 20  Stru = 1,NumS
          SjA(J,Stru) = Intg ( S(Ti,J),ArM(Ti,Stru),Ti,Tf,dT )
  20  CONTINUE
**    ------------------------------------------------------------    **
      Count = 0
  95  Count = Count + 1
**    +----------------------------------------------------------+    **
**    |  COMPUTE  RHS = Sum(Norm*PhiM*SjA) / Sum(Norm*PhiM^2)    |    **
**    +----------------------------------------------------------+    **
      Sum = 0.D0
      DO 30  Stru = 1,NumS
          Sum = Sum + Norm(Stru) * PhiM(Stru)**2
  30  CONTINUE
      DO 40  J = 1,NumV
          RHS(J) = 0.D0
          DO 50  Stru = 1,NumS
              RHS(J) = RHS(J) + Norm(Stru) * PhiM(Stru) * SjA(J,Stru)
  50      CONTINUE
          RHS(J) = RHS(J) / Sum
  40  CONTINUE
**    +----------------------------------------------------------+    **
**    |  SOLVE  B from linear equations  Sjk * B = RHS           |    **
**    +----------------------------------------------------------+    **
      CALL  SolveSystem ( Sjk,RHS,B,NumV )
**    +----------------------------------------------------------+    **
**    |  COMPUTE  BSB = Bj*Sjk*Bk = Int(Sum(B*S)*Sum(B*S))       |    **
**    +----------------------------------------------------------+    **
      BSB =  0.D0
      DO 60  J = 1,NumV
      DO 60  K = 1,NumV
          BSB = BSB + B(J) * Sjk(J,K) * B(K)
  60  CONTINUE
**    ------------------------------------------------------------    **
      PhiMax = 0.D0
      Sum = 0.D0
      DO 70  Stru = 1,NumS
          PhiOld = PhiM(Stru)
**    +----------------------------------------------------------+    **
**    |  COMPUTE  BSA = Bj*SjA = Int(Sum(B*S)*ArM)               |    **
**    +----------------------------------------------------------+    **
          BSA = 0.D0
          DO 80  J = 1,NumV
              BSA = BSA + B(J) * SjA(J,Stru)
  80      CONTINUE
**    +----------------------------------------------------------+    **
**    |  COMPUTE  PhiM = BSA / BSB                               |    **
**    +----------------------------------------------------------+    **
          PhiM(Stru) = BSA / BSB
          IF (DABS(PhiM(Stru)).GT.PhiMax)  PhiMax = DABS(PhiM(Stru))
          Sum = Sum + DABS(PhiM(Stru)-PhiOld)
  70  CONTINUE
**    +----------------------------------------------------------+    **
**    |  CHECK  convergence of PhiM                              |    **
**    +----------------------------------------------------------+    **
      IF (Sum.GT.PhiMax*NumS*0.001D0.AND.Count.LT.10)  GOTO 95
**    +----------------------------------------------------------+    **
**    |  NORMALIZE  PhiM,B                                       |    **
**    +----------------------------------------------------------+    **
      DO 90  Stru = 1,NumS
          PhiM(Stru) = PhiM(Stru) / PhiMax
  90  CONTINUE
      DO 100  J = 1,NumV
          B(J) = B(J) * PhiMax
 100  CONTINUE
*      WRITE(*,'('' PhiM = '',10F8.4)')  ( PhiM(Stru) , Stru=1,NumS )
*      WRITE(*,'('' B = '',10F8.4)')  ( B(J) , J=1,NumV )
**    +----------------------------------------------------------+    **
**    |  COMPUTE  AmM = Sum(B*S)                                 |    **
**    +----------------------------------------------------------+    **
      DO 110  Time = Ti,Tf
          AmM(Time) = 0.D0
          DO 110  J = 1,NumV
              AmM(Time) = AmM(Time) + B(J) * S(Time,J)
 110  CONTINUE
      RETURN
      END
************************************************************************
      SUBROUTINE ComputeS ( Omg,Ksi,G,S )
**  ================================================================  **
**    COMPUTE  unit acceleration response S(T,NumV) of SDOF linear    **
**        oscillator  S + 2*Ksi*Omg*V + Omg^2*X = - G  by using       **
**        Duhamel's integral over the time interval Ti~Tf ( Beck,     **
**        J.L. & Dowling,M.J. , " Quick Algorithms for Computing      **
**        Either Displacement, Velocity or Acceleration of an         **
**        Oscillator " , Earthquake Engineering and Structural        **
**        Dynamics, Vol.16, P.245-253, 1988 )                         **
**    ------------------------------------------------------------    **
**    S(T,1:NumV-2) = response of 'forced vibration' due to ground    **
**                    accel,veloc,displ with X(To)=V(To)=0            **
**    S(T,NumV-1)   = response of 'free vibration' with X(To)=1       **
**    S(T,NumV)     = response of 'free vibration' with V(To)=1       **
**    ............................................................    **
**    Omg,Ksi        = natural frequency & damping ratio of oscil.    **
**    G(T,NumG,NumF) = recorded Ground support Accel,Veloc,Displ      **
**                   = Ag(T,NumG), Vg(T,NumG), Xg(T,NumG)             **
**    NumV           = Number of response Variables  = NumG*NumF+2    **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumF,NumG,Ti,Tf,NumV , F,I,T,V
      REAL*8     Omg,Ksi,G(MaxT,MaxG,MaxF),S(MaxT,MaxV),dT/0.02D0/
      REAL*8     A1,A2,A3,A4,B1,B2,B3,OmgT,KsiT,SKsiT,e,eSin,eCos
      COMMON  /Foo/   NumF
      COMMON  /Goo/   NumG
      COMMON  /Tif/   Ti,Tf
      COMMON  /Voo/   NumV
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  COMPUTE  the required repeat constants                  |    **
**    +----------------------------------------------------------+    **
      OmgT = Omg * dT
      KsiT = Ksi * OmgT
      SKsiT = DSQRT(1-Ksi**2) * OmgT
      e = DEXP(-KsiT)
      eSin = e * DSIN(SKsiT) / SKsiT
      eCos = e * DCOS(SKsiT)
**    +----------------------------------------------------------+    **
**    |  COMPUTE  the time independent coefficients As & Bs      |    **
**    +----------------------------------------------------------+    **
      A1 = 2.D0 * eCos
      A2 = - e**2
      A3 = Omg**2 * ( KsiT*eSin - eCos )
      A4 = - Omg * ( (1.D0-2.D0*Ksi**2)*OmgT*eSin + 2.D0*Ksi*eCos )
      B1 = eSin
      B2 = eCos - (1.D0+KsiT)*eSin
      B3 = 1.D0
**    +----------------------------------------------------------+    **
**    |  COMPUTE  responses of forced vibration  S(T,1:NumV-2)   |    **
**    +----------------------------------------------------------+    **
      V = 0
      DO 10  F = 1,NumF
          DO 20  I = 1,NumG
              V = V + 1
              S(Ti,V)   = - B3*G(Ti,I,F)
              S(Ti+1,V) = - B1*G(Ti+1,I,F) - B2*G(Ti,I,F)
          DO 20  T = Ti+2,Tf
              S(T,V) = A1*S(T-1,V)                 + A2*S(T-2,V)
     &               - B1*(G(T,I,F) - 2*G(T-1,I,F) + G(T-2,I,F))
  20      CONTINUE
**    +----------------------------------------------------------+    **
**    |  UPDATE  Bs  from G=Ag to G=2*Ksi*Omg*Vg and G=Omg^2*Xg  |    **
**    +----------------------------------------------------------+    **
          B1 = B1 * Omg
          B2 = B2 * Omg
          B3 = B3 * Omg
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  COMPUTE  responses of free vibration  S(T,NumV-1:NumV)  |    **
**    +----------------------------------------------------------+    **
      S(Ti,V+1) = - Omg**2
      S(Ti,V+2) = - 2.D0*Ksi*Omg
      S(Ti+1,V+1) = A3
      S(Ti+1,V+2) = A4
      DO 30  T = Ti+2,Tf
          S(T,V+1) = A1*S(T-1,V+1) + A2*S(T-2,V+1)
          S(T,V+2) = A1*S(T-1,V+2) + A2*S(T-2,V+2)
  30  CONTINUE
      NumV = V + 2
      RETURN
      END
************************************************************************
      SUBROUTINE  SolveSystem ( A,B,X,N )
**  ================================================================  **
**    SOLVE  system of N linear equations  A(N,N)*X(N)=B(N)  of X     **
**    ............................................................    **
**    LDLTDec : Lower-Diagonal-Lower-Transpose Decomposition for A    **
**              A = LDLT    AX = B    ->    LDLTX = B                 **
**    FDBSub  : Forward-Diagonal-Backward Substitution to solve X     **
**              LTX = Y    DY = Z    LZ = B                           **
**    ------------------------------------------------------------    **
**    A    : must be a symmetric positive definite square matrix      **
**    MaxV = Maximum dimension of matrix A                    >= N    **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    N
      REAL*8     A(MaxV,MaxV),B(MaxV),X(MaxV)
      REAL*8     L(MaxV,MaxV),D(MaxV)
**  ================================================================  **
      DO 10 J=1,N
      X(J)=B(J)
      DO 10 I=1,J
   10 L(I,J)=A(I,J)
      CALL  CBDECP (L,N,N,MaxV)
      CALL  CBSOLX (L,X,N,N,MaxV)
      RETURN
      END
************************************************************************
      SUBROUTINE  OutputResult
**  ================================================================  **
**    WRITE  Ar,Ac    WRITE  Omg,Ksi,Phi                              **
**    ------------------------------------------------------------    **
**    Ar(T,NumS)     = Recorded relative Acceleration                 **
**    Ac(T,NumS)     = Computed relative Acceleration = Sum(Am*Phi)   **
**    Omg(NumM)      = computed optimal modal frequencies             **
**    Ksi(NumM)      = computed optimal modal damping coefficients    **
**    Phi(NumS,NumM) = computed optimal mode shapes                   **
**  ================================================================  **
$     INCLUDE : 'BI3D1.Inc'
      INTEGER    NumM,NumS,Ti,Tf , Mode,Stru,Time
      REAL*8     Ar(MaxT,MaxS),Am(MaxT,MaxM),Ac
      REAL*8     Omg(MaxM),Ksi(MaxM),Phi(MaxS,MaxM)
      COMMON  /Moo/  NumM
      COMMON  /Soo/  NumS
      COMMON  /Tif/  Ti,Tf
      COMMON  /Aroo/ Ar
      COMMON  /Amoo/ Am
      COMMON  /Xoo/  Omg,Ksi
      COMMON  /Yoo/  Phi
**  ================================================================  **
**    +----------------------------------------------------------+    **
**    |  WRITE  Ar(T,NumS)                                       |    **
**    |         Ac(T,NumS) = Sum(Am(T,NumM)*Phi(NumS,NumM))      |    **
**    +----------------------------------------------------------+    **
      OPEN (4,FILE='BI3D1.Res',FORM='FORMATTED')
      DO 10  Stru = 1,NumS
      DO 10  Time = Ti,Tf
          Ac = 0.D0
          DO 20  Mode = 1,NumM
              Ac = Ac + Am(Time,Mode)*Phi(Stru,Mode)
  20      CONTINUE
          WRITE(4,110)  Stru,Time,Ar(Time,Stru),Ac
 110      FORMAT(2I5,1P2E12.4)
  10  CONTINUE
**    +----------------------------------------------------------+    **
**    |  WRITE  Omg(NumM),Ksi(NumM),Phi(NumS,NumM)               |    **
**    +----------------------------------------------------------+    **
      OPEN (2,FILE='BI3D1.Out',FORM='FORMATTED')
      DO 30  Mode = 1,NumM
          WRITE(2,120)  Mode,Omg(Mode),Mode,Ksi(Mode)
 120      FORMAT(2X,'Omg(',I1,') =',F12.6,10X,'Ksi(',I1,') =',F12.6)
      DO 30  Stru = 1,NumS
          WRITE(2,130)  Stru,Mode,Phi(Stru,Mode)
 130      FORMAT(2X,'Phi(',I2,',',I1,') =',F12.6)
  30  CONTINUE
      RETURN
      END
************************************************************************
