************************************************************************
      SUBROUTINE MKLOOP(NOPP,PONP,PONS,POLP,POLS
     *                 ,PONU,LVSN,LVSP,LVSB,NP,NN,NL)
C**   ============================================================    **
      INTEGER NOPP(2,NP),PONP(1),PONS(1),POLP(1),POLS(1)
      INTEGER PONU(NN),LVSN(NN),LVSP(NN),LVSB(NN)
C**   ============================================================    **
C**   Input  : NOPP(2,NP),NP,NN                                       **
C**   Oouput : PONP(NN+1),PONS(2*NP),POLP(NL+1),POLS(POLP(NL+1)-1),NL **
C**   Working: PONU(NN),LVSN(NN),LVSP(NN),LVSB(NN)                    **
C**   ------------------------------------------------------------    **
C**   NP,NN,NL   = Numbers of pipes, nodes, loops                     **
C**   NOPP(2,IP) = Positive pipe flow                                 **
C**                from node NOPP(1,IP) to NOPP(2,IP)                 **
C**   PONP,PONS  = Pipe list of nodes                                 **
C**   POLP,POLS  = Pipe list of loops                                 **
C**   PONU(IN)   = Index for the usage of pipes connected to node IN  **
C**              = PONP(IN+1) : No   pipe  used : unused node         **
C**              = PONP(IN)   : All  pipes used : used   node         **
C**              = In between : Some pipes used : used   node         **
C**   PONP(IN)   < 0 : To mark node IN as labeled                     **
C**   NOPP(1,IP) < 0 : To mark pipe IP as used                        **
C**   NOPP(2,IP) < 0 : Input value for a dummy pipe btwn 2 reservoirs **
C**                    It is used in the only loop rooted by itself   **
C**   LVSN,LVSP,LVSB = Node, pipe, backward_pntr in level structure   **
C**   ------------------------------------------------------------    **
C**   Connected pipes of node IN : PONS( PONP(IN) : PONP(IN+1)-1) )   **
C**   Connected pipes of loop IL : POLS( POLP(IL) : POLP(IL+1)-1) )   **
C**   ============================================================    **
C**   +----------------------------------------------------------+    **
C**   | Count the no. of pipes connecting to node IN : PONU(IN)  |    **
C*1   +----------------------------------------------------------+    **
      DO 100 IN=1,NN
  100 PONU(IN)=0
      DO 120 IP=1,NP
      DO 120 K=1,2
      IN=IABS(NOPP(K,IP))
  120 PONU(IN)=PONU(IN)+1
C**   +----------------------------------------------------------+    **
C**   | Setup pointers to the pipe list for the nodes            |    **
C**   +----------------------------------------------------------+    **
      PONP(1)=1
      DO 140 IN=1,NN
      PONP(IN+1)=PONP(IN)+PONU(IN)
  140 PONU(IN)=PONP(IN)
C**   +----------------------------------------------------------+    **
C**   | Setup pipe list of nodes                                 |    **
C**   +----------------------------------------------------------+    **
      DO 160 IP=1,NP
      DO 160 K=1,2
      IN=IABS(NOPP(K,IP))
      IT=PONU(IN)
      PONS(IT)=IP*(2*K-3)
  160 PONU(IN)=IT+1
C**   +----------------------------------------------------------+    **
C**   | Initialize a loop array                                  |    **
C**   +----------------------------------------------------------+    **
      NL=0
      POLP(NL+1)=1
      ML=NP-NN+1
C**   +----------------------------------------------------------+    **
C**   | Setup a level structure for an unused pipe: NOPP(1,IP)>0 |    **
C**   | The pipe must has at least one  used  node: node IR      |    **
C**   | Node IR is a used node but not labeled as a root node    |    **
C*2   +----------------------------------------------------------+    **
      IR=1
  200 IBG=PONP(IR)
      IED=PONP(IR+1)-1
      DO 700 IT=IBG,IED
      IP=IABS(PONS(IT))
      IF(NOPP(1,IP).LE.0) GO TO 700      ! Skip used pipes
C**   +----------------------------------------------------------+    **
C**   | Make IP as a root pipe and JR as a root node : JL = 1    |    **
C**   | Mark node JR as labeled : PONP(JR) < 0                   |    **
C*3   +----------------------------------------------------------+    **
      JR=NOPP(1,IP)+IABS(NOPP(2,IP))-IR
      PONP(JR)=-PONP(JR)
      JL=1
      LVSN(JL)=JR
      LVSP(JL)=PONS(IT)
      LVSB(JL)=0
      IL=1
C*4   ------------------------------------------------------------    **
  300 IN=LVSN(IL)
      JBG=IABS(PONP(IN))
      JED=IABS(PONP(IN+1))-1
      DO 400 JT=JBG,JED
      JP=IABS(PONS(JT))
C**   +----------------------------------------------------------+    **
C**   | Never go through unused pipe from used node              |    **
C**   +----------------------------------------------------------+    **
C**   | PONU(IN)=PONP(IN+1): unused node: OK for all unused pipe |    **
C**   | PONU(IN)=PONP(IN)  : used   node: OK for all  used  pipe |    **
C**   | PONU(IN)=In between: used   node: OK for used pipes only |    **
C**   +----------------------------------------------------------+    **
      IF(PONU(IN).LE.JED.AND.NOPP(1,JP).GT.0) GO TO 400
      IF(NOPP(2,JP).LT.0) GO TO 400
      JN=IABS(NOPP(1,JP))+NOPP(2,JP)-IN
      IF(PONP(JN).LT.0) GO TO 400
      IF(JP.EQ.IP) GO TO 400
C*5   ------------------------------------------------------------    **
      PONP(JN)=-PONP(JN)
      JL=JL+1
      LVSN(JL)=JN
      LVSP(JL)=PONS(JT)
      LVSB(JL)=IL
      IF(JN.EQ.IR) GO TO 500
  400 CONTINUE
C*6   ------------------------------------------------------------    **
      IL=IL+1
      IF(IL.LE.JL) GO TO 300
C**   +----------------------------------------------------------+    **
C**   | Reset index for labeled nodes                            |    **
C*6   | Always mark the root pipe and its end nodes as used      |    **
C*7   +----------------------------------------------------------+    **
  500 DO 550 IL=1,JL
      IN=LVSN(IL)
  550 PONP(IN)=IABS(PONP(IN))
C**   ------------------------------------------------------------    **
      PONU(IR)=PONU(IR)-1
      PONU(JR)=PONU(JR)-1
      NOPP(1,IP)=-NOPP(1,IP)
C**   ------------------------------------------------------------    **
      IF(IN.NE.IR) GO TO 700
C**   +----------------------------------------------------------+    **
C**   | Trace back the level structure for the pipes of new loop |    **
C*7   +----------------------------------------------------------+    **
      NL=NL+1
      KT=POLP(NL)
C**   ------------------------------------------------------------    **
  600 JP=IABS(LVSP(JL))
      IF(NOPP(1,JP).GT.0) THEN
         IN=NOPP(1,JP)
         JN=NOPP(2,JP)
         PONU(IN)=PONU(IN)-1
         PONU(JN)=PONU(JN)-1
         NOPP(1,JP)=-IN
      ENDIF
      POLS(KT)=LVSP(JL)
      KT=KT+1
      JL=LVSB(JL)
      IF(JL.GT.0) GO TO 600
C**   ------------------------------------------------------------    **
      POLP(NL+1)=KT
  700 CONTINUE
C**   +----------------------------------------------------------+    **
C**   | Find a used node which has at least one unused pipe : IR |    **
C**   |   or an unused node for another separate district   : IY |    **
C**   +----------------------------------------------------------+    **
C**   | PONU(IR)=PONP(IR+1): unused node: NG for all unused pipe |    **
C**   | PONU(IR)=PONP(IR)  : used   node: NG for all  used  pipe |    **
C*8   | PONU(IR)=In between: used   node: OK for any unused pipe |    **
C*9   +----------------------------------------------------------+    **
      IX=IR
      IY=NN+1
  750 IR=IR+1
      IF(IR.GT.NN) IR=1
      IF(IR.EQ.IX) THEN
         IF(IY.GT.NN) GO TO 900
         ML=ML+1
         IR=IY
         GO TO 200
      ENDIF
      IF(PONU(IR).EQ.PONP(IR)) GO TO 750
      IF(PONU(IR).NE.PONP(IR+1)) GO TO 200
      IY=MIN0(IR,IY)
      GO TO 750
C**   ------------------------------------------------------------    **
  900 IF(NL.NE.ML) WRITE(*,*) ' ERROR IN MKLOOP '
      DO 950 IP=1,NP
  950 NOPP(1,IP)=IABS(NOPP(1,IP))
      RETURN
      END
************************************************************************
